Package: glmnet
Type: Package
Title: Lasso and elastic-net regularized generalized linear models
Version: 1.8-5
Date: 2012-12-31
Author: Jerome Friedman, Trevor Hastie, Rob Tibshirani
Maintainer: Trevor Hastie <hastie@stanford.edu>
Depends: Matrix (>= 1.0-6), utils
Suggests: survival
Description: Extremely efficient procedures for fitting the entire
        lasso or elastic-net regularization path for linear regression,
        logistic and multinomial regression models, poisson regression
        and the Cox model. Two recent additions are the multiresponse
        gaussian, and the grouped multinomial. The algorithm uses
        cyclical coordinate descent in a pathwise fashion, as described
        in the paper listed below.
License: GPL-2
URL: http://www.jstatsoft.org/v33/i01/.
Packaged: 2012-12-31 19:26:08 UTC; hastie
Repository: CRAN
Date/Publication: 2013-01-04 09:21:44
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 10:17:55 UTC; unix

Package: glmnet
Type: Package
Title: Lasso and Elastic-Net Regularized Generalized Linear Models
Version: 5.0
Date: 2026-04-27
Authors@R: c(person("Jerome", "Friedman", role=c("aut")),
	     person("Trevor", "Hastie", role=c("aut", "cre"), email = "hastie@stanford.edu"),
	     person("Rob", "Tibshirani", role=c("aut")),
	     person("Balasubramanian", "Narasimhan", role=c("aut")),
	     person("Kenneth","Tay",role=c("aut")),
	     person("Noah", "Simon", role=c("aut")),
	     person("Junyang", "Qian", role=c("ctb")),
	     person("James", "Yang", role=c("aut")),
	     person("Jonathan","Taylor", role=c("aut")))
Depends: R (>= 3.6.0), Matrix (>= 1.0-6)
Imports: methods, utils, foreach, shape, survival, Rcpp
Suggests: knitr, lars, nnet, testthat, xfun, rmarkdown
SystemRequirements: C++17
Description: Extremely efficient procedures for fitting the entire lasso or elastic-net regularization path for linear regression, logistic and multinomial regression models, Poisson regression, Cox model,  multiple-response Gaussian, and the grouped multinomial regression; see <doi:10.18637/jss.v033.i01> and <doi:10.18637/jss.v039.i05>. There are two new and important additions. The family argument can be a GLM family object, which opens the door to any programmed family (<doi:10.18637/jss.v106.i01>). This comes with a modest computational cost, so when the built-in families suffice, they should be used instead.  The other novelty is the relax option, which refits each of the active sets in the path unpenalized. The algorithm uses cyclical coordinate descent in a path-wise fashion, as described in the papers cited.
License: GPL-2
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://glmnet.stanford.edu
RoxygenNote: 7.3.3
LinkingTo: RcppEigen, Rcpp
NeedsCompilation: yes
Packaged: 2026-04-30 21:22:11 UTC; hastie
Author: Jerome Friedman [aut],
  Trevor Hastie [aut, cre],
  Rob Tibshirani [aut],
  Balasubramanian Narasimhan [aut],
  Kenneth Tay [aut],
  Noah Simon [aut],
  Junyang Qian [ctb],
  James Yang [aut],
  Jonathan Taylor [aut]
Maintainer: Trevor Hastie <hastie@stanford.edu>
Repository: CRAN
Date/Publication: 2026-05-04 05:40:42 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-04 06:39:27 UTC; unix
Archs: glmnet.so.dSYM

useDynLib("glmnet")
importFrom(utils,head,tail,packageDescription)
import(Matrix)
export(glmnet,
       cv.glmnet,
       cv.elnet,cv.lognet,cv.multnet,cv.fishnet,cv.coxnet,cv.mrelnet,getmin,auc,auc.mat,cvcompute,coxnet.deviance,
       lambda.interp,response.coxnet,rmult,glmnet_softmax,na.mean,
       predict.elnet,predict.lognet,predict.fishnet,mrelnet,predict.multnet,predict.coxnet,predict.mrelnet,nonzeroCoef,plotCoef,jerr,jerr.elnet,jerr.lognet,jerr.fishnet,jerr.coxnet,elnet,lognet,fishnet,coxnet,getcoef,getcoef.multinomial,coefnorm)

S3method("print","glmnet")
S3method("plot","glmnet")
S3method("plot","multnet")
S3method("plot","mrelnet")
S3method("predict","glmnet")
S3method("coef","glmnet")
S3method("coef","cv.glmnet")
S3method("predict","cv.glmnet")
S3method("plot","cv.glmnet")
S3method("deviance","glmnet")

# Generated by roxygen2: do not edit by hand

S3method("[",stratifySurv)
S3method(buildPredmat,array)
S3method(buildPredmat,coxnetlist)
S3method(buildPredmat,default)
S3method(buildPredmat,glmnetfitlist)
S3method(buildPredmat,lognetlist)
S3method(buildPredmat,mrelnetlist)
S3method(buildPredmat,multnetlist)
S3method(coef,cv.glmnet)
S3method(coef,cv.relaxed)
S3method(coef,glmnet)
S3method(coef,relaxed)
S3method(deviance,glmnet)
S3method(family,cv.glmnet)
S3method(family,glmnet)
S3method(family,glmnetfit)
S3method(family,relaxed)
S3method(plot,cv.glmnet)
S3method(plot,cv.relaxed)
S3method(plot,glmnet)
S3method(plot,mrelnet)
S3method(plot,multnet)
S3method(plot,relaxed)
S3method(predict,coxnet)
S3method(predict,cv.glmnet)
S3method(predict,cv.relaxed)
S3method(predict,elnet)
S3method(predict,fishnet)
S3method(predict,glmnet)
S3method(predict,glmnetfit)
S3method(predict,lognet)
S3method(predict,mrelnet)
S3method(predict,multnet)
S3method(predict,relaxed)
S3method(print,bigGlm)
S3method(print,confusion.table)
S3method(print,cv.glmnet)
S3method(print,cv.relaxed)
S3method(print,glmnet)
S3method(print,relaxed)
S3method(survfit,coxnet)
S3method(survfit,cv.glmnet)
export(Cindex)
export(assess.glmnet)
export(bigGlm)
export(buildPredmat)
export(coef.glmnet)
export(coef.relaxed)
export(confusion.glmnet)
export(coxgrad)
export(coxnet.deviance)
export(cv.glmnet)
export(glmnet)
export(glmnet.control)
export(glmnet.measures)
export(makeX)
export(na.replace)
export(na_sparse_fix)
export(predict.glmnet)
export(predict.relaxed)
export(prepareX)
export(print.cv.glmnet)
export(relax.glmnet)
export(rmult)
export(roc.glmnet)
export(stratifySurv)
import(Matrix)
import(foreach)
import(methods)
importFrom(Matrix,sparse.model.matrix)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(shape,colorlegend)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,contrasts)
importFrom(stats,family)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,weighted.mean)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,coxph)
importFrom(survival,is.Surv)
importFrom(survival,strata)
importFrom(survival,survfit)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
useDynLib(glmnet)

# Changes in Mortran and R

## Tracing progress fixes in Mortran and R

1. Changed the function definition

```
subroutine get_int_parms(sml,eps,big,mnlam,rsqmax,pmin,exmx);
```
to 
```
subroutine get_int_parms(sml,eps,big,mnlam,rsqmax,pmin,exmx,itrace);
```

taking on itrace as a switch for tracing progress. Added an `entry
chg_itrace` to change `itrace`.

2. Changed all calls to `get_int_parms` to include extra parameter.

3. Corresponding changes to `glmnet.control.R` to include `itrace` as a
   parameter. 

4. Fixes in `relax.glmnet.R` for displaying the progress by
   temporarily turning off and on tracing, if it was on. 


## Unused variable fixes in Mortran. 

1. The subroutines `spstandard`, `spstandard1`, `multspstandard1` had `jerr` as an unused
   argument. 
2. The subroutines `coxnet1`, `fishnet1`, `spfishnet1` had `isd` as an
   unused argument.
3. The subroutine `risk` had `ni` as an unusued argument.

In all these cases, I merely added a _no-op_ statement to the
function/subroutine (e.g. `jerr = jerr * 1` or `ni = ni * 1`) to stop
compiler warnings. More advanced compilers may detect these
shenanigans. We can address that later if it comes up.

## Some extraneous tabs in Mortran.

1. The subroutine `multstandard1` has an extra tabl between the
   arguments `ju` and `xm` that I removed. This is most likely because
   Emacs replaces 8 spaces with tabs. 
   
## Uninitialized variable in Mortran

1. The `gfortran` compiler warns that the variable `alf` is
   uninitialized in `elnet1`. The logic is correct, but the compiler
   is unable to detect it.  So I initialized `alf=1` and combined it
   with a previous fix for `alm=0.0` in `elnet1`.  I am reasonably
   certain this will not affect any logic.
   
## Generating Fortran from Mortran

Note that it is Mortran processing that generates unusued labels. So
each time you run Mortran to generate the Fortran, you will have to
deal with the problem of unused labels. No worries, however, if you
follow this procedure.

1. Install the latest [`SUtools`](https://github.com/bnaras/SUtools)
   package, after deleting your old one, if you had it.
2. Execute the following, assuming `GLMNET_DIR` is your `glmnet`
   package directory.

```{r, eval = FALSE}
library(SUtools)
GLMNET_DIR  <- "~/GITHUB/glmnet"
mortran <- readLines(file.path(GLMNET_DIR, "inst", "mortran", "glmnet5dpclean.m"))
result  <- process_mortran(input_mortran_file = "glmnet5dpclean.m",
                           pkg_name = "glmnet",
                           control = sutools_control(fix_allocate = FALSE))
```

The fortran file can now be written out to a file. For example, you
may choose to overwrite the old fortran with the new fortran.

```{r, eval = FALSE}
writeLines(fixed_fortran, file.path(GLMNET_DIR, "src", "glmnet5dpclean.f"))
```

## Maxit bug fixed

Mortran would branch to :again: when convergence was not reached, and
not check if maxit had been reached. On the new mac M1 this could get
into a state with a degenerate example where it never
terminated. Added a maxit check immediately after again throughout.

<!-- README.md is generated from the source: README.Rmd -->

# Lasso and Elastic-Net Regularized Generalized Linear Models <img src="man/figures/logo.png" width="100" align="right" />

<!-- badges: start -->
<!-- NOTE on badges below: Manually comment out R_CMD_check badge as -->
<!-- the repo is private and will frail CRAN README.md checks -->
<!-- [![R-CMD-check](https://github.com/trevorhastie/glmnet/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/trevorhastie/glmnet/actions/workflows/R-CMD-check.yaml) -->
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/glmnet)](https://cran.r-project.org/package=glmnet)[![](https://cranlogs.r-pkg.org/badges/glmnet)](https://CRAN.R-project.org/package=glmnet)
<!-- badges: end -->

We provide extremely efficient procedures for fitting the entire lasso
or elastic-net regularization path for linear regression (gaussian),
multi-task gaussian, logistic and multinomial regression models (grouped
or not), Poisson regression and the Cox model. The algorithm uses
cyclical coordinate descent in a path-wise fashion. Details may be found
in Friedman, Hastie, and Tibshirani ([2010](#ref-glmnet)), Simon et al.
([2011](#ref-coxnet)), Tibshirani et al. ([2012](#ref-strongrules)),
Simon, Friedman, and Hastie ([2013](#ref-block)).

Version 3.0 is a major release with several new features, including:

- Relaxed fitting to allow models in the path to be refit without
  regularization. CV will select from these, or from specified mixtures
  of the relaxed fit and the regular fit;
- Progress bar to monitor computation;
- Assessment functions for displaying performance of models on test
  data. These include all the measures available via `cv.glmnet`, as
  well as confusion matrices and ROC plots for classification models;
- print methods for CV output;
- Functions for building the `x` input matrix for `glmnet` that allow
  for *one-hot-encoding* of factor variables, appropriate treatment of
  missing values, and an option to create a sparse matrix if
  appropriate.
- A function for fitting unpenalized a single version of any of the GLMs
  of `glmnet`.

Version 4.0 is a major release that allows for any GLM family, besides
the built-in families.

Version 4.1 is a major release that expands the scope for survival
modeling, allowing for (start, stop) data, strata, and sparse X inputs.
It also provides a much-requested method for `survival:survfit`.

## References

<div id="refs" class="references">

<div id="ref-glmnet">

Friedman, Jerome, Trevor Hastie, and Robert Tibshirani. 2010.
“Regularization Paths for Generalized Linear Models via Coordinate
Descent.” *Journal of Statistical Software, Articles* 33 (1): 1–22.
<https://doi.org/10.18637/jss.v033.i01>.

</div>

<div id="ref-block">

Simon, Noah, Jerome Friedman, and Trevor Hastie. 2013. “A Blockwise
Descent Algorithm for Group-Penalized Multiresponse and Multinomial
Regression.”

</div>

<div id="ref-coxnet">

Simon, Noah, Jerome Friedman, Trevor Hastie, and Robert Tibshirani.
2011. “Regularization Paths for Cox’s Proportional Hazards Model via
Coordinate Descent.” *Journal of Statistical Software, Articles* 39 (5):
1–13. <https://doi.org/10.18637/jss.v039.i05>.

</div>

<div id="ref-strongrules">

Tibshirani, Robert, Jacob Bien, Jerome Friedman, Trevor Hastie, Noah
Simon, Jonathan Taylor, and Ryan Tibshirani. 2012. “Strong Rules for
Discarding Predictors in Lasso-Type Problems.” *Journal of the Royal
Statistical Society: Series B (Statistical Methodology)* 74 (2): 245–66.
<https://doi.org/10.1111/j.1467-9868.2011.01004.x>.

</div>

<div id="ref-glm">

Kenneth Tay, J, Narasimhan, Balasubramanian, Hastie, Trevor. 2023.
“Elastic Net Regularization Paths for All Generalized Linear Models.”
*Journal of Statistical Software, Articles* 106 (1): 1–31.
<https://doi.org/10.18637/jss.v106.i01>.

</div>

</div>

# glmnetpp C++ Core Engine

## Overview

The `glmnetpp` C++ core engine implements the core components of the
system.

## Dependencies

If you have set up a conda environment following the instructions in the main repo README, you should already have a C++ toolchain installed along with Bazel. If not, we require a C++ toolchain that supports C++-14 and [OpenMP](https://www.openmp.org/) and an installation of [Bazel](https://bazel.build/)

Suggested compilers:
- [GCC >= 9.3.0](https://gcc.gnu.org/)
- [Clang >= 10.0.0](https://clang.llvm.org/)

## Build

```
mamba update -y conda
mamba env create
conda activate glmnetpp
```

__Note__: On Linux, it's best to specify whether you want to use `clang` or `gcc`.
Add the appropriate flag to each `bazel` call below:
```
# For gcc
# For clang
bazel ... --config=gcc
bazel ... --config=clang
```

__To build `glmnetpp`__:
```
bazel build //:glmnetpp 
```
Note that `glmnetpp` is a header-only library,
so this will simply collect all the headers and register its dependencies.
For release mode, add the flag `-c opt` after `build`.
For debug mode, add the flag `-c dbg` after `build`.

__To run all tests__:
```
bazel test -c dbg //test/... 
```

__To run a particular test__:
```
bazel test -c dbg //test:name-of-test
```

__To run the benchmarks__:
```
bazel run -c opt //benchmark:name-of-benchmark
```

# Design Documentation for `glmnetpp`

## Table of Content
- [Motivation](#motivation)
- [Current Design](#current-design)
  - [Library Structure](#library-structure)
  - [Elnet Driver](#elnet-driver)
  - [Elnet Path](#elnet-path)
    - [Why is there an `ElnetPathCRTPBase` and an `ElnetPathBase`?](#why-is-there-an-elnetpathcrtpbase-and-an-elnetpathbase)
    - [Is CRTP really needed?](#is-crtp-really-needed)
    - [What are these `...Pack` nested class templates?](#what-are-these-pack-nested-class-templates)
  - [Elnet Point](#elnet-point)
    - [Why do we need yet another policy for `ElnetPointInternalPolicy`?](#why-do-we-need-yet-another-policy-for-elnetpointinternalpolicy)
    - [Why does `ElnetPointCRTPBase` inherit the internal policy?](#why-does-elnetpointcrtpbase-inherit-the-internal-policy)
  - [Elnet Point Internal](#elnet-point-internal)
    - [What is `ElnetPointInternalStaticBase`?](#what-is-elnetpointinternalstaticbase)
- [Other Design Ideas](#other-design-ideas)
  - [Policy-Based Design](#policy-based-design)
  - [Dynamic Polymorphism (Virtual Mechanism)](#dynamic-polymorphism-virtual-mechanism)

This is a documentation for the design choices of the C++ implementation of `glmnet` original Fortran routines, `glmnetpp`.

## Motivation

For context, [`glmnet5dpclean.m`](../../../../inst/mortran/glmnet5dpclean.m) 
(or [`glmnet5dpclean.f`](../../src/legacy/glmnet5dpclean.f), the generated Fortran code)
is the MORTRAN code that was the workhorse for `glmnet` up until version `4.1-2`.
This code contains all the implementation of the IRLS algorithm for the various GLMs (Gaussian, Binomial, Poisson, etc.).
As seen in the MORTRAN code, the biggest drawback with the code is the difficulty in maintenance.
We believe that this backend code of `glmnet` 
should arguably be the _easiest_ to maintain, if anything,
as it is the most crucial part of the library.
Hence, the primary goal of this C++ implementation is to use better design principles 
to ease maintainability and extendability (see [Current Design](#current-design) for more detail).
We note in passing that in the process of converting the MORTRAN code, however, to our surprise, 
we actually found very tangible speed-ups as well
(at least 2x speed-up for dense versions and around 20% speed-up for sparse versions).

The biggest drawback with the MORTRAN code is the large code duplication.
All versions (for different GLMS) of the IRLS algorithm are _essentially_ the same algorithm,
yet the algorithm is copied and pasted as separate subroutines for each GLM and dense/sparse version,
leading to about 16 copies in total.
While this is tough for maintenance and future extendability, 
we should note that it may be more beneficial performance-wise.
A refactored code would potentially abstract a common block into a function
and reuse this function in the different versions, which would require a function call.
If the Fortran compiler cannot inline this function, we would pay a large cost in performance
if that function call were made frequently.
However, a copy-paste would be one way to manually force inlining of the function.
In C++, we have ways to strong-inline functions (force the compiler to inline a function if possible)
(see [macros.hpp](../../include/glmnetpp_bits/util/macros.hpp))
and we are unaware of similar capabilities of the Fortran compiler.

The second drawback of the MORTRAN code is that the linear algebra routines are not as optimized 
as those provided by some matrix libraries in C++.
Our implementation relies on the widely-used, industry-standard
matrix library, [Eigen](https://eigen.tuxfamily.org/index.php?title=Main_Page).
We noticed that `Eigen` allows for more vectorized code, leading to performance boosts.

From the above, the problem is clear:
_how do we write a refactored library where we reuse abstracted code 
as much as possible to remove code duplications,
write cache-friendly code, and use more vectorized code
without incurring the cost of function calls and generally avoiding costs that come from abstraction?_

Let's dive into the problem a little bit more.
At first glance, it may seem like a simple task to refactor the MORTRAN code,
however, the abstraction is quite subtle.
Usually, when we refactor, we have the following problem:
```cpp
void foo1() 
{
    // some stuff..
    // BEGIN COMMON BLOCK
    // ...
    // END COMMON BLOCK
    // some stuff..
}

void foo2() 
{
    // some different stuff..
    // BEGIN COMMON BLOCK
    // ...
    // END COMMON BLOCK
    // some different stuff..
}
```
and we would like to abstract the common block into a separate function from two macroscopic functions like:
```cpp
void common_block() { /*...*/ }
void foo1() 
{
    // some stuff..
    common_block();
    // some stuff..
}

void foo2() 
{
    // some different stuff..
    common_block();
    // some different stuff..
}
```
Indeed, we do see these kinds of problems where the common block could be 
the formula for gradient computation,
residual updates, and beta updates.

This is, indeed, a simple task, however, this is not the __only__ problem we see in the MORTRAN code.
We also have the following kind of problem:
```cpp
void foo(...) 
{
    // 1. set-up quantities needed for the elastic net path-solver

    // 2. loop through each lambda in the regularization path
    for (int m = 0; m < max_n_lambdas; ++m) {
        // 3. compute the current lambda at iteration m
        // 4. solve the elastic net problem at the current lambda
        // 5. save the result into output
    }

    // 6. do any final computations after solving for the whole path
}
```
Essentially, there is a _common macroscopic function_ `foo` and some of the _internal details_ (numbered steps) are different
depending on the GLM and dense/sparse storage of the data matrix.
Moreover, step 4 also has further common code such as the weighted-least-squares (WLS) algorithm of the form:
```cpp
void wls(...)
{
    while (1) {
        double convergence_measure = 0.0;
        // 1. iterate through each feature and do coordinate descent.
        for (int k = 0; k < p; ++k) {
            // update beta...
            if (new_beta == old_beta) continue;
            // update convergence_measure...
            // update other quantities...
        }

        // 2. check for convergence
        if (convergence_measure < tolerance) {
            // check KKT...
            if (kkt_passed) return;
            continue; 
        }

        // 3. do coordinate descent on only the active variables until convergence
    }
}
```
All versions of the subroutines reuse this general algorithm.
Only the exact details of how we update `beta`, `convergence_measure`, and `other quantities` are different
across the different versions of the algorithm.

In the [next section](#current-design),
we explain our current design of the library and how it tries to solve this very problem.

## Current Design

In this section, we go over the current design of `glmnetpp`.
First, we summarize the library structure at a high-level.
Then, we delve into each of the key components of the library.
Of course, we cannot explain every minute detail,
so we only explain the big _key_ design choices. 
We do not discuss the correctness of the implementation,
as we did not change the algorithm itself (except bug fixes).

### Library Structure

The following file structure outlines the library structure:
```
glmnetpp
    \_ include
        \_ glmnetpp_bits
            \_ elnet_driver
            \_ elnet_path
            \_ elnet_point
                \_ internal
            \_ util
            \_ elnet_driver.hpp
            \_ elnet_path.hpp
            \_ elnet_point.hpp
        \_ glmnetpp
    \_ src
        \_ legacy
```
Recall that `glmnetpp` is a header-only template library, 
so the whole library is defined in `include`.
`src` exists purely for legacy reasons to store the old Fortran code.
`glmnetpp_bits` contains the bits and pieces of the library
and the full library is exported into the final header file `glmnetpp`.
Inside `glmnetpp_bits`, there are four major components:

- [**elnet_driver**](#elnet-driver): contains driver classes that are at the highest-level of the hierarchy of classes.
    The main job is to delegate to the correct elastic net path-solver depending on the input.
    The exported header file is `elnet_driver.hpp`.
- [**elnet_path**](#elnet-path): contains elastic net path-solver classes for the different GLMs (dense and sparse).
    These classes are responsible for solving the elastic net problem for the whole regularization path.
    The exported header file is `elnet_path.hpp`.
- [**elnet_point**](#elnet-point): contains generic elastic net point-solver classes for the different GLMs (dense and sparse).
    These classes are responsible for solving the elastic net problem for a _fixed_ regularization parameter.
    The exported header file is `elnet_point.hpp`.
- [**elnet_point/internal**](#elnet-point-internal): contains the internal details that define the GLM-specific routines
    for the point-solvers.

Finally, `util` contains some utility functions/macros that are not intended to be exported for direct use,
but rather as tools in the implementation of the library.

### Elnet Driver

All drivers have very similar implementation details, so we only discuss one of the drivers, `gaussian.hpp`.
These drivers mimic the Fortran functions such as `elnet, lognet, fishnet`, 
the highest-level functions that get exported to `R`.
The class declaration is of the form
```cpp
template <>
struct ElnetDriver<util::glm_type::gaussian>
    : ElnetDriverBase
{
private:
    static constexpr util::glm_type glm = util::glm_type::gaussian;
    using mode_t = util::mode_type<glm>;

public:
    template <...>
    void fit(...);
};
```
that is, we have a single class template `ElnetDriver`
which takes in an `enum class` value of type `util::glm_type`
defined in [types.hpp](../../include/glmnetpp_bits/util/types.hpp).
The idea is that we only specialize this class template only
for particular GLMs,
since logically, there is no "default" definition of a driver 
for a general GLM.
The same idea holds in `elnet_path` and `elnet_point`.
Note that for each `glm_type`, there is an associated `Mode` class
which defines flags to indicate further differences in the algorithm
for a given GLM.
As an example, we will see that Gaussian GLM 
allows for two different algorithms: naive and covariance.
These flags along with dense/sparse storage type of the data matrix
uniquely define each of the different versions of IRLS algorithm.

The `fit` function simply prepares the inputs 
for the path solvers such as 
standardizing features and normalizing weights if necessary.
Note that `fit` is heavily templatized! 
We could have easily written it non-templated and take in
specific `Eigen` matrix objects, since that is the only use-case currently.
However, looking ahead, we plan to export `glmnetpp` not just to `R`
but to other higher-level languages such as `Python` and `Matlab`.
It is not clear what objects we would need to pass to the `fit` function
in that case, as these languages may not support bindings for `Eigen` objects.
The only cost in templatizing is compile-time,
which is abundant for our use-case since we build this library once
when exporting in `R`.

`fit` also checks _at compile time_ whether `X` data matrix is dense or sparse.
Hence, at compile-time, we choose the correct version of the path solver.
The technical details of this "choosing" is slightly nuanced 
since we must be able to support C++14 standard and do not have the convenient `if constexpr` in C++17.
In `gaussian.hpp`, you will see `FitPathGaussian` class template in `namespace details`,
which is a metaprogramming tool to choose the dense or sparse version.
The problem is that if we write something like:
```cpp
template <...>
void fit(...) 
{
    constexpr bool is_dense = ...;
    if (is_dense) {
        call_dense_version(...);
    } else {
        call_sparse_version(...)
    }
}
```
the compiler has to be able to generate the code for _both_ sparse and dense versions 
_for the same given input_.
For example, if `X` were sparse, the compiler would still need to be able to generate code for
`call_dense_version` with `X`, which is not going to be case for us since
there are, naturally, operations that are not defined for sparse matrices
but are defined for dense matrices that get used in `call_dense_version` (such as `operator+=`).
Instead we add a layer of dispatch with `FitPathGaussian` like
```
FitPathGaussian<is_dense>::eval(...);
```
such that the compiler will first choose the specialization of `FitPathGaussian` depending on the boolean `is_dense`
and then generate the code for `eval` only for that specialization.

### Elnet Path

`ElnetPath` is a class template responsible for fitting an elastic net model for the full regularization path.
The declaration is as follows:
```cpp
template <util::glm_type glm
        , util::mode_type<glm> mode
        , class ElnetPointPolicy=ElnetPoint<glm, mode> >
struct ElnetPath;
```
As before, `glm` denotes the GLM enum value and `mode` is the `glm` corresponding mode value.
The third template parameter `ElnetPointPolicy` is the type of the point-solver.
A priori, `ElnetPath` does not depend on any specific implementation of a point-solver.
By default, we use our own implementation given in `ElnetPoint<glm, mode>`,
but we may substitute that type with a different one, so long as it satisfies a certain interface
(for now, read the code to see how `ElnetPoint` is used in `ElnetPath`, TODO: maybe write-up an interface).
The structure is hopefully clear: the most derived classes are the header files that don't end with `_base.hpp`
and the sparse versions are prefixed with `sp_`.
Each GLM has its own base class, and all such classes derive from the base class(es) in `base.hpp`.

#### Why is there an ElnetPathCRTPBase and an ElnetPathBase?

[Curiously Recurring Template Pattern (CRTP)](https://en.wikipedia.org/wiki/Curiously_recurring_template_pattern#:~:text=The%20curiously%20recurring%20template%20pattern,form%20of%20F%2Dbounded%20quantification.)
is a design pattern where a base class template takes in the derived type as a template parameter
and casts itself to that derived type to access its (public) interface.
A derived class `D` is expected to inherit from the CRTP base class `B<D>`, where `B` is the CRTP base class template.
As a result, for all derived classes `D_i` that inherit from `B<D_i>`, they all inherit from __different instances__ of `B`,
that is, `B<D_i>` are all __different class types__.
So, if there are any common functions defined in the class template `B` _that do not actually depend on the derived type_,
the compiler generates _copies_ of them in every `B<D_i>`, which leads to both increase in compile-time and code-bloat.
__Only__ the code that depends on the derived class, i.e. those that __must__ be differently generated per derived class,
should be defined inside `B`.
The rest should be in a non-CRTP base class (like `ElnetPathBase`) so that the compiler only generates one version of these member functions
for all derived classes.

#### Is CRTP really needed?

CRTP is not necessary for our purposes, but it does encapsulate and abstract very well so it is convenient.
All the CRTP base member functions could have been free function templates,
which would've also reduced compile-time,
but it does expose this function to the user technically, which isn't something we need to do.
We viewed these members as implementation detail and wanted to encapsulate them in a class.

#### What are these `...Pack` nested class templates?

All `ElnetPath` classes define three "pack" class templates: `FitPack, PathConfigPack, PointConfigPack`.
They all perform one job: pack a bunch of arguments into a single object.

- `FitPack` packs all the arguments from the user into a single object.
- `PathConfigPack`: packs any configuration variables needed for the path-solver.
- `PointConfigPack`: packs any configuration variables needed for the point-solver.

Not surprisingly, these packs have an inheritance hierarchy as there are many shared quantities across the different versions.
We decided to nest them rather than having them as separate classes because:

- Logically, they should not exist outside of `ElnetPath` -- they are structures that only have meaning in the context of `ElnetPath`.
- Their hierarchy structure directly follows that of `ElnetPath` and its base class(es), so it is convenient to nest.

But why create these packs in the first place? 
Let's consider the `FitPack` as an example.
All versions of the path solvers have different set of inputs.
For example, some take in an `offset` parameter whereas some don't.
Some take in a residual vector `r` where as others take instead a gradient vector `g`.
And then there are some that take both `r` and `g`.
It would be a massive headache to find a super-set of these arguments
and write our path-solvers assuming this one big set of arguments for the following reasons:

- We would have to dynamically check which arguments are used.
- It would make it difficult to see which version depends on what set of arguments.

Hence, we resolve this at compile-time by having each version define its own set of `FitPack`,
containing __only the arguments it needs__.
We now have a generic interface where the path-solvers simply work with one `FitPack`-like object.
See [base.hpp](../../include/glmnetpp_bits/elnet_path/base.hpp) to see how this generic `fit` function works
in `ElnetPathCRTPBase`.

Similarly, `PathConfigPack` and `PointConfigPack` define each version-specific set of values
that are needed for the path-solver and point-solver, respectively.

### Elnet Point

`ElnetPoint` is a class template responsible for fitting an elastic net model for a fixed regularization parameter.
The declaration is as follows:
```cpp
template <util::glm_type glm
        , util::mode_type<glm> mode
        , class ElnetPointInternalPolicy=ElnetPointInternal<glm, mode> >
struct ElnetPoint;
```
A priori, `ElnetPoint` does not depend on the internal implementation for a point solver.
By default, we use our policy `ElnetPointInternal` defined in [elnet_point/internal](../../include/glmnetpp_bits/elnet_point/internal),
but we may swap it for a different class so long as it satisfies the interface needed by `ElnetPoint`
(for now, read the code to see how `ElnetPoint` interacts with `ElnetPointInternal`.
Note that different specializations require a different interface).

#### Why do we need yet another policy for ElnetPointInternalPolicy?

It may seem like an over-generalization to have yet another policy for the internal implementation detail,
but once we explain what we mean by "internal", the design will make more sense.
As explained in [Motivation](#motivation),
there are two types of abstraction problems:

- Generalizing a small common block that is reused in many bigger functions.
- Generalizing the control-flow of many smaller functions, which can differ across different versions.

`ElnetPoint` aims to solve the second problem.
As an example, in [base.hpp](../../include/glmnetpp_bits/elnet_point/base.hpp),
`ElnetPointCRTPBase` contains some common routines used by all point solvers.
For example, the `fit` function is really the coordinate-descent algorithm.
Note that `fit` simply "manages" the smaller functions such as `increment_passes`,
`coord_desc_reset`, `for_each_with_skip`, `all_begin`, etc.
In particular, it defines the order and conditions under which these smaller functions are to be executed.
However, `fit` __does not__ rely on the __implementation-specific detail for the smaller functions__.
It is precisely these smaller functions that we refer to as "internal".
For each different GLM, we would obviously want a different internal policy 
that defines each of these smaller functions,
but for future development as well, we may want to try different internal policies for the same GLM.
This design grants us this flexibility.

#### Why does `ElnetPointCRTPBase` inherit the internal policy?

This was purely out of coding convenience.
The internal policies have constructors that are very cumbersome to type out,
so it was very convenient to simply inherit the policy and expose its constructor.
Because this doesn't break any logic, we stuck with this design.

### Elnet Point Internal

`ElnetPointInternal` is a class template responsible for defining the internal details for each `ElnetPoint` specialization.
The declaration is as follows:
```cpp
template <util::glm_type g
        , util::mode_type<g> mode
        , class ValueType = double
        , class IndexType = int
        , class BoolType = bool>
struct ElnetPointInternal;
```
Currently, the only special behavior that occurs is when `BoolType` is `bool`.
Internally, we keep an inclusion vector that contains which features may be included in the model
and which are always forbidden from being included.
If `BoolType` is `bool` we use `std::vector<bool>` to represent this vector,
as it is a more specialized and performant data structure.
Otherwise, we simply use an `Eigen::Matrix<BoolType, -1, 1>` (column vector).

This class simply tries to solve the first problem in the 
[previous section](#why-do-we-need-yet-another-policy-for-elnetpointinternalpolicy):
generalizing the smaller common blocks.
These classes contain the meat of the algorithm.
They were written with the intention that the members would be called in a certain order
by `ElnetPoint` classes, so when trying to read the code,
__read it together with the corresponding `ElnetPoint`__.

Currently, the most derived classes really only contain logic related to the `X` data matrix,
i.e. defining dense or sparse routines.
Otherwise, the two versions reuse the rest of the logic defined in their GLM-specific base class.
TODO: some work remains in abstracting the sparse logic across the different versions.

A lot of these base classes will reuse static members or further base class members.
Hopefully, this sheds some light into how much was being duplicated in the original MORTRAN code!

#### What is `ElnetPointInternalStaticBase`?

For encapsulation purposes and to associate routines under a certain name,
we place commonly used routines as static member functions in `ElnetPointInternalStaticBase`.
Other base classes may define more static members on top of these for the derived classes `ElnetPointInternal`.
The point of these static members is mainly to define formulas once.
A very common source of bugs stems from copying and pasting a common block multiple times.
Once an edit must be made in one, we have to manually update the copied-and-pasted places as well.
Our approach removes this problem from ever happening.
See the following issues on GitHub regarding such bugs:
- [Issue 39](https://github.com/trevorhastie/glmnet/issues/39)
- [Issue 40](https://github.com/trevorhastie/glmnet/issues/40)
- [Issue 41](https://github.com/trevorhastie/glmnet/issues/41)
- [Issue 42](https://github.com/trevorhastie/glmnet/issues/42)
- [Issue 43](https://github.com/trevorhastie/glmnet/issues/43)

## Other Design Ideas

The current design of `glmnetpp` is by no means a perfect design.
In this section, we go over some other ideas we thought about in the process.

### Policy-Based Design

Another idea we thought about was to use policy-based design.
We noticed that in [elnet_point/internal](../../include/glmnetpp_bits/elnet_point/internal),
we're borrowing from multiple base class members sometimes (such as Gaussian multi-response and Binomial multi-response grouped).
It is possible to remove this intricate dependency by defining a set of policies for each `ElnetPointInternal`.
We didn't go with this idea because it felt like the number of policies could get out-of-hand a priori.

### Dynamic Polymorphism (Virtual Mechanism)

In many places, we used CRTP and compile-time tricks to abstract away certain logic,
such as the `Pack` objects in `ElnetPack`.
However, one could have also used dynamic polymorphism without relying on template magic.
The big issue we saw was that dynamic polymorphism completely removes the benefit of inlining for us, which is detrimental!
Although it reduces binary size and compile-time,
once again, compile-time is abundant for us and code-bloat is not an issue
because this is not meant to be a generic library used by others for larger programs.
We only compile our export functions once using this library and create a dynamic library.

## ----include=FALSE------------------------------------------------------------
# the code in this chunk enables us to truncate the print output for each
# chunk using the `out.lines` option
# save the built-in output hook
hook_output <- knitr::knit_hooks$get("output")

# set a new output hook to truncate text output
knitr::knit_hooks$set(output = function(x, options) {
  if (!is.null(n <- options$out.lines)) {
    x <- xfun::split_lines(x)
    if (length(x) > n) {
      # truncate the output
      x <- c(head(x, n), "....\n")
    }
    x <- paste(x, collapse = "\n")
  }
  hook_output(x, options)
})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = FALSE, message = FALSE, warning = FALSE,
                      fig.align = "center")

## ----timeline, fig.width=7, fig.height=3.8, out.width="\\textwidth", fig.cap="glmnet release milestones. Top row runs left-to-right; the path folds down at the right and the bottom row runs right-to-left, ending at v5.0. Evenly spaced for legibility rather than to reflect calendar time."----
par(mar = c(0.4, 0.4, 0.4, 0.4))
milestones <- data.frame(
  ver = c("Initial release (2008)", "1.2 (2010)", "1.6 (2011)",
          "2.0-1 (2015)", "3.0 (2019)",
          "4.0 (2020)", "4.1 (2021)", "4.1-3 (2021)", "5.0 (2026)"),
  lbl = c("coordinate descent;\nFortran kernel\n(Friedman, Hastie,\nTibshirani)",
          "coxnet\n(Simon)",
          "strong rules screen\n(Tibshirani et al.)",
          "CV rewrite\n(Hastie)",
          "relax; assess;\nbigGlm\n(Hastie,\nNarasimhan)",
          "GLM extension\n(Tay, Hastie,\nNarasimhan)",
          "(start, stop] Cox;\nstrata; sparse Cox\n(Tay, Narasimhan,\nHastie)",
          "Fortran to C++\n(Yang)",
          "coxdev submodules;\nCox unified\n(Taylor,\nNarasimhan, Hastie)")
)
n <- nrow(milestones); ntop <- 5; nbot <- n - ntop
top_y <- 1.4; bot_y <- -1.4
col_x <- seq_len(ntop)
px <- numeric(n); py <- numeric(n)
px[1:ntop]     <- col_x;                   py[1:ntop]     <- top_y
px[(ntop+1):n] <- rev(col_x[(ntop-nbot+1):ntop]); py[(ntop+1):n] <- bot_y

plot.new(); plot.window(xlim = c(0.4, ntop + 0.45), ylim = c(-2.3, 2.3))

seg <- function(x0, y0, x1, y1)
  segments(x0, y0, x1, y1, lwd = 1.4, col = "grey35")
arr <- function(x0, y0, x1, y1)
  arrows(x0, y0, x1, y1, length = 0.09, angle = 25, lwd = 1.4,
         col = "grey35")
pad <- 0.18
# top row: left -> right
for (i in 1:(ntop-1)) arr(px[i] + pad, py[i], px[i+1] - pad, py[i+1])
# fold down at right: tiny side step to clear labels on the shared column
fold_x <- ntop + 0.3
seg(px[ntop] + pad, py[ntop], fold_x, py[ntop])        # right from last top node
seg(fold_x, py[ntop], fold_x, py[ntop+1])              # down along margin
arr(fold_x, py[ntop+1], px[ntop+1] + pad, py[ntop+1])  # left into first bottom node
# bottom row: right -> left
for (i in (ntop+1):(n-1)) arr(px[i] - pad, py[i], px[i+1] + pad, py[i+1])

points(px, py, pch = 21, bg = "grey90", col = "grey20",
       cex = 2.0, lwd = 1.3)
text(px, py, labels = seq_len(n), cex = 0.72, col = "grey15",
     font = 2)

# version labels: above top row, below bottom row
text(px[1:ntop], py[1:ntop] + 0.28, milestones$ver[1:ntop],
     cex = 0.72, col = "grey25", pos = 3, offset = 0)
text(px[(ntop+1):n], py[(ntop+1):n] - 0.28, milestones$ver[(ntop+1):n],
     cex = 0.72, col = "grey25", pos = 1, offset = 0)

# description labels: below top row, above bottom row (interior)
text(px[1:ntop], py[1:ntop] - 0.28, milestones$lbl[1:ntop],
     cex = 0.66, pos = 1, offset = 0)
text(px[(ntop+1):n], py[(ntop+1):n] + 0.28, milestones$lbl[(ntop+1):n],
     cex = 0.66, pos = 3, offset = 0)

glmnet_softmax <-
  function (x,ignore_labels=FALSE)
{
  d <- dim(x)
  dd <- dimnames(x)[[2]]
  if(is.null(dd) || !length(dd)) ignore_labels=TRUE

  nas=apply(is.na(x),1,any)
  if(any(nas)){
    pclass=rep(NA,d[1])
    if(sum(nas)<d[1]){
      pclass2=glmnet_softmax(x[!nas,],ignore_labels)
      pclass[!nas]=pclass2
      if(is.factor(pclass2))pclass=factor(pclass,levels = seq(d[2]),labels=levels(pclass2))
    }
  }
  else{
    maxdist <- x[, 1]
    pclass <- rep(1, d[1])
    for (i in seq(2, d[2])) {
      l <- x[, i] > maxdist
      pclass[l] <- i
      maxdist[l] <- x[l, i]
    }
    dd <- dimnames(x)[[2]]
    if(!ignore_labels) pclass=factor(pclass, levels = seq(d[2]), labels = dd)
  }
  pclass
}

#' Compute a confusion matrix (or many) for a binomial or multinomial glmnet model
#'
#' @rdname assess.glmnet
#' @export confusion.glmnet
confusion.glmnet=function(object,newx=NULL,newy,family=c("binomial","multinomial"),...){
### object must be either a glmnet or cv.glmnet object fit with family binomial or multinomial
### or else a matrix/array of predictions of a glmnet model of these families
###    (the last dimension can be 1)
###    (not on the mean scale, but natural parameter scale)
    oclass=grep("glmnet",class(object))
    if(length(oclass)){# either a glmnet or cv.glmnet
        if(!match(family(object),c("binomial","multinomial"),FALSE))
            stop("confusion available only for binomial or multinomial family")
        classmat=predict(object,newx=newx,type="class",...)
    }
    else{
        fam=match.arg(family)
        predmat=object
        if(fam=="binomial"){
            classnames=attr(predmat,"classnames")#if it was created by cv with keep
            if(is.null(classnames))classnames=c("1","2")
            if(is.vector(predmat))predmat=as.matrix(predmat)
            cnum=ifelse(predmat>0,2,1)
            classmat=classnames[cnum]
            if(is.matrix(predmat))classmat=array(classmat,dim(predmat),dimnames(predmat))
        }
        else{
            classmat=apply(predmat, 3, glmnet_softmax)
        }
    }

### newy can be a matrix or a factor
      nc = dim(newy)
    if (is.null(nc)) {
        newy = as.factor(newy)
    }
    else {
        nc = nc[2]
        yc=newy%*%(1:nc)
        cn=colnames(newy)
        if(!is.null(cn))
            newy=factor(yc,labels=cn)
        else newy=factor(yc)
    }
    ctable=function(...){
        tab=table(...)
        class(tab)=c("confusion.table",class(tab))
        tab
        }
    if(ncol(classmat)>1)
        ## convert to a dataframe, to prevent apply simplifying
        lapply(data.frame(classmat,stringsAsFactors=FALSE,check.names=FALSE),
               function(x,y)ctable(Predicted=x,True=y),y=newy)
    else(ctable(Predicted=classmat,True=newy))
}

zeromat=function(nvars,lmu,vnames,stepnames){
  ca=rep(0,lmu)
  ia=seq(lmu+1)
  ja=rep(1,lmu)
  dd=c(nvars,lmu)
  new("dgCMatrix", Dim = dd,
      Dimnames = list(vnames,stepnames),
      x = as.vector(ca),
      p = as.integer(ia - 1), i = as.integer(ja - 1))
}

#' method plot cv.relaxed
#' @importFrom shape colorlegend
#' @param se.bands Should shading be produced to show standard-error bands;
#' default is \code{TRUE}
#' @rdname plot.cv.glmnet
#' @export
plot.cv.relaxed <- function (x, se.bands=TRUE,...)
{
    xr=x$relaxed
    oldpar = par(mar = c(4, 4, 3, 4))
    on.exit(par(oldpar))
    statlist=xr$statlist
    gamma=xr$gamma
    ngamma=length(gamma)
    ylim=range(unlist(lapply(statlist,"[[","cvm")))
    if(se.bands){
        cvup=lapply(statlist,"[[","cvup")
        cvlo=lapply(statlist,"[[","cvlo")
        ylim=range(ylim,unlist(cvup),unlist(cvlo))
        }
    xlim=log(range(unlist(lapply(statlist,"[[","lambda")))+0.00001)
    cvcolors = rainbow(ngamma, start = .1, end = 1)
     with(statlist[[ngamma]],plot(log(lambda), cvm, type = "n", xlab = expression(Log(lambda)),
                                 ylab = x$name,ylim=ylim,xlim=xlim))
    if(se.bands){
        for (i in seq(ngamma))
            with(statlist[[i]],polygon(c(log(lambda),rev(log(lambda))),c(cvup,rev(cvlo)),
                                       col="floralwhite",border="antiquewhite"))
    }
    for (i in seq(ngamma))
        with(statlist[[i]],lines(log(lambda), cvm, lwd = 2, col = cvcolors[i]))
    mins=log(c(xr$lambda.min,xr$lambda.1se))
    abline(v=mins,lty=3)
    dof= statlist[[1]]$nzero
    lambda=statlist[[1]]$lambda
    axis(side = 3, at = log(lambda), labels = paste(dof),
         tick = FALSE, line = 0)

#    dof=c(x$nzero.min,x$nzero.1se)
#    axis(3, at = mins, labels = format(dof), tick=FALSE, line = 0,cex.axis=.9)
    colorlegend(posy = c(0.2, 0.8), posx = c(0.93, 0.945)-.03, col = rainbow(ngamma,
        start = 0.1, end = 1), zlim = c(0, 1), zval = gamma , main = expression(gamma),digit=2)
    invisible()
}

cv.glmnetfit <-function(predmat,y,type.measure,weights,foldid,grouped){
    family=attr(predmat,"family")
    mumat=family$linkinv(predmat)
    nobs=nrow(predmat)# was nrow(mumat), which failed for tweedie instance
    ## initialize from family function. Makes y a vector in case of binomial, and possibly changes weights
    ## Expects nobs to be defined, and creates n and mustart (neither used here)
    ## Some cases expect to see things, so we set it up just to make it work
    etastart=0;mustart=NULL;start=NULL
    eval(family$initialize)
    ##
    ## Just in case this was not done in initialize()
    y <- drop(y)  # we don't like matrix responses
    N = nobs - apply(is.na(predmat), 2, sum)
  cvraw = switch(type.measure,
                 mse = (y - mumat)^2,
                 mae = abs(y - mumat),
                 deviance = family$dev.resids(array(y,dim(mumat)), mumat,1)
                 )
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)
}

lambda.interp=function(lambda,s){
### lambda is the index sequence that is produced by the model
### s is the new vector at which evaluations are required.
### the value is a vector of left and right indices, and a vector of fractions.
### the new values are interpolated bewteen the two using the fraction
### Note: lambda decreases. you take:
### sfrac*left+(1-sfrac*right)

  if(length(lambda)==1){# degenerate case of only one lambda
    nums=length(s)
    left=rep(1,nums)
    right=left
    sfrac=rep(1,nums)
  }
  else{
      ## s[s > max(lambda)] = max(lambda)
      ## s[s < min(lambda)] = min(lambda)
      k=length(lambda)
      sfrac <- (lambda[1]-s)/(lambda[1] - lambda[k])
      lambda <- (lambda[1] - lambda)/(lambda[1] - lambda[k])
      sfrac[sfrac < min(lambda)] <- min(lambda)
      sfrac[sfrac > max(lambda)] <- max(lambda)
      coord <- approx(lambda, seq(lambda), sfrac)$y
      left <- floor(coord)
      right <- ceiling(coord)
      sfrac=(sfrac-lambda[right])/(lambda[left] - lambda[right])
      sfrac[left==right]=1
      sfrac[abs(lambda[left]-lambda[right])<.Machine$double.eps]=1

    }
list(left=left,right=right,frac=sfrac)
}

## CHECK WITH TREVOR: why alias here for predict.cv.relaxed as well as in plot.cv.relaxed?
##

#' make predictions from a "cv.glmnet" object.
#'
#' This function makes predictions from a cross-validated glmnet model, using
#' the stored \code{"glmnet.fit"} object, and the optimal value chosen for
#' \code{lambda} (and \code{gamma} for a 'relaxed' fit.
#'
#' This function makes it easier to use the results of cross-validation to make
#' a prediction.
#'
#' @aliases coef.cv.glmnet coef.cv.relaxed predict.cv.glmnet
#' @param object Fitted \code{"cv.glmnet"} or \code{"cv.relaxed"} object.
#' @param newx Matrix of new values for \code{x} at which predictions are to be
#' made. Must be a matrix; can be sparse as in \code{Matrix} package. See
#' documentation for \code{predict.glmnet}.
#' @param s Value(s) of the penalty parameter \code{lambda} at which
#' predictions are required. Default is the value \code{s="lambda.1se"} stored
#' on the CV \code{object}. Alternatively \code{s="lambda.min"} can be used. If
#' \code{s} is numeric, it is taken as the value(s) of \code{lambda} to be
#' used. (For historical reasons we use the symbol 's' rather than 'lambda' to
#' reference this parameter)
#' @param \dots Not used. Other arguments to predict.
#' @return The object returned depends on the \dots{} argument which is passed
#' on to the \code{predict} method for \code{glmnet} objects.
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}, and \code{print}, and \code{coef} methods, and
#' \code{cv.glmnet}.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent, Journal of Statistical Software, Vol. 33, Issue 1, Feb 2010}\cr
#' \url{https://www.jstatsoft.org/v33/i01/}
#' \url{https://arxiv.org/abs/1707.08692}\cr Hastie, T., Tibshirani, Robert,
#' Tibshirani, Ryan (2019) \emph{Extended Comparisons of Best Subset Selection,
#' Forward Stepwise Selection, and the Lasso}
#' @keywords models regression
#' @examples
#'
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' cv.fit = cv.glmnet(x, y)
#' predict(cv.fit, newx = x[1:5, ])
#' coef(cv.fit)
#' coef(cv.fit, s = "lambda.min")
#' predict(cv.fit, newx = x[1:5, ], s = c(0.001, 0.002))
#' cv.fitr = cv.glmnet(x, y, relax = TRUE)
#' predict(cv.fit, newx = x[1:5, ])
#' coef(cv.fit)
#' coef(cv.fit, s = "lambda.min", gamma = "gamma.min")
#' predict(cv.fit, newx = x[1:5, ], s = c(0.001, 0.002), gamma = "gamma.min")
#'
#' @method predict cv.glmnet
#' @export
predict.cv.glmnet=function(object,newx,s=c("lambda.1se","lambda.min"),...){
  if(is.numeric(s))lambda=s
  else
    if(is.character(s)){
      s=match.arg(s)
      lambda=object[[s]]
    }
    else stop("Invalid form for s")
  predict(object$glmnet.fit,newx,s=lambda,...)
}

#' @method buildPredmat coxnetlist
#' @export
buildPredmat.coxnetlist <-
    function(outlist, lambda, x, offset, foldid, alignment,y,weights,grouped,type.measure="deviance",...){
        nfolds = max(foldid)
        if ((length(weights)/nfolds < 10) && !grouped) grouped = TRUE
        devtrue=type.measure=="deviance"
        cvraw = if(devtrue) matrix(NA, nfolds, length(lambda)) else NULL
        nlambda = length(lambda)
        predmat=matrix(NA, nrow(x), length(lambda))
        rn=rownames(x)
        sn=paste("s",seq(0,length=nlambda),sep="")
        dimnames(predmat)=list(rn,sn)
    predmat

    for (i in seq(nfolds)) {
        which = foldid == i
        fitobj = outlist[[i]]
        coefmat = switch(alignment,
                         fraction = predict(fitobj,type = "coefficients",...),
                         lambda = predict(fitobj, type = "coefficients",s = lambda,...)
                         )
        nlami = min(ncol(coefmat), nlambda)
        if(devtrue){
            if (grouped) {
                plfull = coxnet.deviance(x = x, y = y, offset = offset,
                                         weights = weights, beta = coefmat)
                plminusk = coxnet.deviance(x = x[!which, ], y = y[!which, ], 
                                           offset = offset[!which],
                                           weights = weights[!which], 
                                           beta = coefmat)
                cvraw[i, seq(nlami)] = (plfull - plminusk)[seq(nlami)]
            }
            else {
                plk = coxnet.deviance(x = x[which, ], y = y[which, ], 
                                      offset = offset[which], 
                                      weights = weights[which], beta = coefmat)
                cvraw[i, seq(nlami)] = plk[seq(nlami)]
            }
        }
        predmat[which, seq(nlami)] = as.matrix(x[which,]%*% coefmat)
        if (nlami < nlambda){
            if(devtrue)cvraw[i, seq(from = nlami, to = nlambda)] = cvraw[i, nlami]
            predmat[which,seq(from=nlami,to=nlambda)]=predmat[which,nlami]
        }
    }
        if(devtrue) attr(predmat,"cvraw")=cvraw
        predmat
    }

#' @method plot relaxed
#' @param gamma Value of the mixing parameter for a "relaxed" fit
#' @rdname plot.glmnet
#' @export
plot.relaxed <-
    function(x,xvar = c("lambda", "dev"), label = FALSE, gamma=1,
             ...) {
        xvar=match.arg(xvar)
        if(any(wh<-gamma<0)){
            warning("negative gamma values ignored")
            gamma=gamma[!wh]
        }
        if(any(wh<-gamma>1)){
            warning("gamma values larger than 1 ignored")
            gamma=gamma[!wh]
        }
        if(!length(gamma))stop("no valid values of gamma")
        for(i in seq(along=length(gamma))){
            bfit=blend.relaxed(x,gamma=gamma[i],extend=FALSE)
            plot(bfit,xvar=xvar,label=label,...)
            }
    }


#' @method print relaxed
#' @export
print.relaxed <-
function (x, digits = max(3, getOption("digits") - 3), ...)
{
    cat("\nCall: ", deparse(x$call), "\n")
    cat("Relaxed\n\n")
    which=match(x$relaxed$lambda,x$lambda,0)
    rdev=rep(NA,length(x$lambda))
    rdev[which]=round(x$relaxed$dev.ratio*100, 2)
    out=data.frame(Df = x$df, `%Dev` = round(x$dev.ratio*100, 2), `%Dev R`=rdev,
                    Lambda = signif(x$lambda, digits),check.names=FALSE,row.names=seq(along=rdev))
    class(out)=c("anova",class(out))
    print(out)
}

jerr=function(n,maxit,pmax,family){
  if(n==0) list(n=0,fatal=FALSE,msg="")
  else {
    errlist=switch(family,
              "gaussian"=jerr.elnet(n,maxit,pmax),
               "binomial"=jerr.lognet(n,maxit,pmax),
              "multinomial"=jerr.lognet(n,maxit,pmax),
              "poisson"=jerr.fishnet(n,maxit,pmax),
              "cox"=jerr.coxnet(n,maxit,pmax),
              "mrelnet"=jerr.mrelnet(n,maxit,pmax)
      )
    names(errlist)=c("n","fatal","msg")
    errlist$msg=paste("from glmnet C++ code (error code ",n, "); ",errlist$msg,sep="")
    errlist
  }
}
                  
                  
     

#' @method print cv.relaxed
#' @export
print.cv.relaxed <- function(x, digits = max(3, getOption("digits") - 3), ...)
{
    cat("\nCall: ", deparse(x$call), "\n\n")
    cat("Measure:", x$name, "\n\n")
    x = x$relaxed
    optlams = c(x$lambda.min, x$lambda.1se)
    wg1 = match(x$gamma.min, x$gamma)
    wl1 = match(x$lambda.min, x$statlist[[wg1]]$lambda)
    s1 = with(x$statlist[[wg1]], c(x$gamma.min,wg1, x$lambda.min,wl1,
        cvm[wl1], cvsd[wl1], x$nzero.min))
    wg2 = match(x$gamma.1se, x$gamma)
    wl2 = match(x$lambda.1se, x$statlist[[wg2]]$lambda)
    s2 = with(x$statlist[[wg2]], c(x$gamma.1se,wg2, x$lambda.1se,wl2,
        cvm[wl2], cvsd[wl2], x$nzero.1se))
    mat = rbind(s1, s2)
    dimnames(mat) = list(c("min", "1se"), c("Gamma","Index", "Lambda","Index",
        "Measure", "SE", "Nonzero"))
    mat = data.frame(mat, check.names = FALSE)
    class(mat) = c("anova", class(mat))
    print(mat, digits = digits)
}

cv.lognet <- function(predmat,y,type.measure,weights,foldid,grouped){
    prob_min = 1e-05
    prob_max = 1 - prob_min
    nc = dim(y)
    if (is.null(nc)) {
        y = as.factor(y)
        ntab = table(y)
        nc = as.integer(length(ntab))
        y = diag(nc)[as.numeric(y), , drop=FALSE]
    }
    N = nrow(y)
    nfolds = max(foldid)
    if ((N/nfolds < 10) && type.measure == "auc") {
        warning("Too few (< 10) observations per fold for type.measure='auc' in cv.lognet; changed to type.measure='deviance'. Alternatively, use smaller value for nfolds",
                call. = FALSE)
        type.measure = cvtype("deviance", "lognet")
    }
    predmat=1/(1+exp(-predmat))
    nlambda=ncol(predmat)
    nlams=rep(nlambda,nfolds)
    if (type.measure == "auc") {
        cvraw = matrix(NA, nfolds, nlambda)
        good = matrix(0, nfolds, nlambda)
        for (i in seq(nfolds)) {
            good[i, seq(nlams[i])] = 1
            which = foldid == i
            for (j in seq(nlams[i])) {
                cvraw[i, j] = auc.mat(y[which, ], predmat[which,
                                                          j], weights[which])
            }
        }
        N = apply(good, 2, sum)
        weights = tapply(weights, foldid, sum)
        grouped=FALSE
    }
    else {
        ywt = apply(y, 1, sum)
        y = y/ywt
        weights = weights * ywt
        N = nrow(y) - apply(is.na(predmat), 2, sum)
        cvraw = switch(type.measure,
                       mse = (y[, 1] - (1 - predmat))^2 + (y[, 2] - predmat)^2,
                       mae = abs(y[, 1] - (1 - predmat)) + abs(y[, 2] - predmat),
                       deviance = {
                           predmat = pmin(pmax(predmat, prob_min), prob_max)
                           lp = y[, 1] * log(1 - predmat) + y[, 2] * log(predmat)
                           ly = log(y)
                           ly[y == 0] = 0
                           ly = drop((y * ly) %*% c(1, 1))
                           2 * (ly - lp)
                       },
                       class = y[, 1] * (predmat > 0.5) + y[, 2] * (predmat <= 0.5)
                       )
        }
list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)
}

cv.mrelnet <- function(predmat,y,type.measure,weights,foldid,grouped){
    ndim = dim(y)
    nc = ndim[2]
    nobs = ndim[1]
    N = nobs - apply(is.na(predmat[, 1,,drop=FALSE ]), 2, sum)# dimensions could be lost if third dim=1
    bigY = array(y, dim(predmat))
    cvraw = switch(type.measure,
                   mse = apply((bigY - predmat)^2,c(1, 3), sum),
                   deviance = apply((bigY - predmat)^2, c(1,3), sum),
                   mae = apply(abs(bigY - predmat), c(1, 3), sum)
                   )
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)
}

#' Compute deviance for Cox model
#' 
#' Compute the deviance (-2 log partial likelihood) for Cox model.
#'
#' Computes the deviance for a single set of predictions, or for a matrix 
#' of predictions. The user can either supply the predictions
#' directly through the \code{pred} option, or by supplying the \code{x} matrix
#' and \code{beta} coefficients. Uses the Breslow approach to ties.
#' 
#' The function first checks if \code{pred} is passed: if so, it is used as
#' the predictions. If \code{pred} is not passed but \code{x} and \code{beta}
#' are passed, then these values are used to compute the predictions. If 
#' neither \code{x} nor \code{beta} are passed, then the predictions are all
#' taken to be 0.
#' 
#' \code{coxnet.deviance()} is a wrapper: it calls \code{coxnet.deviance0()}
#' if the response is right-censored data, and calls \code{coxnet.deviance3()} 
#' if the response is (start, stop] survival data. 
#' 
#' \code{coxnet.deviance2()} gives the same output as \code{coxnet.deviance0()}
#' but is written completely in R. It is not called by 
#' \code{coxnet.deviance()}, and is kept in the package for completeness.
#' 
#' @aliases coxnet.deviance
#' @param pred Fit vector or matrix (usually from glmnet at a particular 
#' lambda or a sequence of lambdas).
#' @param y Survival response variable, must be a \code{Surv} or 
#' \code{stratifySurv} object.
#' @param x Optional \code{x} matrix, to be supplied if \code{pred = NULL}.
#' @param offset Optional offset vector.
#' @param weights Observation weights (default is all equal to 1).
#' @param std.weights If TRUE (default), observation weights are standardized
#' to sum to 1.
#' @param beta Optional coefficient vector/matrix, to be supplied if 
#' \code{pred = NULL}.
#' 
#' @return A vector of deviances, one for each column of predictions.
#' 
#' @examples
#' set.seed(1)
#' eta <- rnorm(10)
#' time <- runif(10, min = 1, max = 10)
#' d <- ifelse(rnorm(10) > 0, 1, 0)
#' y <- survival::Surv(time, d)
#' coxnet.deviance(pred = eta, y = y)
#' 
#' # if pred not provided, it is set to zero vector
#' coxnet.deviance(y = y)
#' 
#' # example with x and beta
#' x <- matrix(rnorm(10 * 3), nrow = 10)
#' beta <- matrix(1:3, ncol = 1)
#' coxnet.deviance(y = y, x = x, beta = beta)
#' 
#' # example with (start, stop] data
#' y2 <- survival::Surv(time, time + runif(10), d)
#' coxnet.deviance(pred = eta, y = y2)
#' 
#' # example with strata
#' y2 <- stratifySurv(y, rep(1:2, length.out = 10))
#' coxnet.deviance(pred = eta, y = y2)
#' 
#' @seealso \code{coxgrad}
#' @keywords Cox model
#'
#' @export
coxnet.deviance <- function(pred = NULL, y, x = NULL, offset = NULL, 
                            weights = NULL, std.weights = TRUE, beta = NULL) {
  y <- response.coxnet(y)
  
  # if y has 2 columns, it is right-censored data
  # if y has 3 columns, it is (start, stop] data
  # otherwise, throw error
  if (ncol(y) == 2) {
    return(coxnet.deviance0(pred = pred, y = y, x = x, offset = offset,
                            weights = weights, std.weights = std.weights,
                            beta = beta))
  } else if (ncol(y) == 3) {
    return(coxnet.deviance3(pred = pred, y = y, x = x, offset = offset, 
                            weights = weights, std.weights = std.weights,
                            beta = beta))
  } else {
    stop("Response y should have 2 or 3 columns")
  }
}

#' @rdname coxnet.deviance
coxnet.deviance0 <- function(pred = NULL, y, x = NULL, offset = NULL, 
                             weights = NULL, std.weights = TRUE, beta = NULL) {
  ty <- y[, "time"]
  tevent <- y[, "status"]
  ty <- ty + (1 - tevent) * 100 * .Machine$double.eps
  nobs <- as.integer(length(ty))
  
  # hack for the case where user passes in x as sparse matrix
  if (!is.null(x) && inherits(x, "sparseMatrix")) {
    if (is.null(beta))
      stop("if x is passed, beta must also be passed")
    pred <- as.matrix(x %*% beta)
    return(coxnet.deviance0(pred = pred, y = y, offset = offset,
                            weights = weights, std.weights = std.weights))
  }
  
  # Sort out the pred, x and beta options.
  # If user provided `pred`, we let x = pred and beta = identity matrix.
  # This allows us to use the loglike Fortran routine to compute the
  # partial log likelihood.
  # In the end, only x and beta are passed to the Fortran routine.
  if (!is.null(pred)) {
    x <- as.matrix(pred)
    nvec <- ncol(x)
    beta <- diag(nvec)
    nvars <- as.integer(nvec)
  } else if (is.null(x) && is.null(beta)) {
    x <- matrix(0, nrow = nobs, ncol = 1)
    beta <- double(0)
    nvec <- 1
    nvars <- as.integer(0)
  } else if (!is.null(x) && !is.null(beta)) {
    x <- as.matrix(x)
    beta <- as.matrix(beta)
    nvec <- ncol(beta)
    nvars <- nrow(beta)
  } else {
    stop("user must pass either `pred`, or both `x` and `beta`")
  }
  storage.mode(x) <- "double"
  storage.mode(beta) <- "double"
  nvec <- as.integer(nvec)
  nvars <- as.integer(nvars)
  
  # normalize weights to sum to nobs
  if (is.null(weights))
    weights <- rep(1, nobs)
  else {
    if (std.weights) weights <- nobs * weights / sum(weights)
    weights <- as.double(weights)
  }
  
  if (is.null(offset))
    offset <- rep(0, nobs)
  else offset <- as.double(offset)
  
  # extract strata (if any)
  if ("strata" %in% names(attributes(y))) {
    strata <- attr(y, "strata")
  } else {
    strata <- rep(1, nobs)
  }
  if (length(strata) != nobs) stop("length of strata != nobs")
  
  # if all in same strata, do the deviance computation
  # if not, take the sum of the strata-level deviances
  if (length(unique(strata)) == 1) {
    ### Compute saturated loglikelihood
    wd <- weights[tevent == 1]
    tyd <- ty[tevent == 1]
    if (any(duplicated(tyd))) {
      wd <- tapply(wd, tyd, sum)
    }
    wd <- wd[wd > 0]
    lsat <- -sum(wd * log(wd))
    ####
    
    fit <- .Fortran("loglike", nobs, nvars, x, ty, tevent, offset,
                    weights, nvec, beta, flog = double(nvec), jerr = integer(1),
                    PACKAGE = "glmnet")
    if (fit$jerr != 0) {
      errmsg <- jerr(fit$jerr, maxit = 0, pmax = 0, family = "cox")
      if (errmsg$fatal)
        stop(errmsg$msg, call. = FALSE)
      else warning(errmsg$msg, call. = FALSE)
    }
    return(2 * (lsat - fit$flog))
  } else {
    # more than one strata provided: return the sum of strata-level deviances
    tot_dev <- 0
    for (i in unique(strata)) {
      ii <- which(strata == i)
      tot_dev <- tot_dev + 
        coxnet.deviance0(y = y[ii, , drop = FALSE], x = x[ii, , drop = FALSE], 
                         beta = beta, offset = offset[ii], 
                         weights = weights[ii], std.weights = FALSE)
    }
    return(tot_dev)
  }
}

#' @rdname coxnet.deviance
coxnet.deviance2 <- function(pred = NULL, y, x = NULL, offset = NULL, 
                             weights = NULL, std.weights = TRUE, beta = NULL) {
  if (!is.Surv(y)) stop("y must be a Surv object")
  nobs <- nrow(y)
  
  # if pred is NULL, use beta and x to compute pred
  # if beta is NULL too, set pred to all zeros
  if (is.null(pred)) {
    if ((!is.null(x) && is.null(beta)) || (is.null(x) && !is.null(beta)))
      stop("user must pass either `pred`, or both `x` and `beta`")
    if (is.null(beta)) {
      pred <- rep(0, times = nobs)
    } else {
      pred <- x %*% beta
    }
  }
  
  # if more than one column of predictions is passed, run coxnet.deviance2()
  # for each column
  if (!is.null(ncol(pred)) && ncol(pred) > 1) {
    return(sapply(seq(ncol(pred)),
                  function(j) coxnet.deviance2(
                    pred = pred[, j], y = y, offset = offset, 
                    weights = weights, std.weights = std.weights)))
  } else {
    # check that pred is of the right length
    if(length(pred) != nobs) stop("pred and y must have the same length")
    
    # normalize weights to sum to nobs
    if (is.null(weights))
      w <- rep(1, nobs)
    else {
      if (length(weights) != nobs) stop("weights and y must have the same length")
      if (std.weights) {
        w <- nobs * weights / sum(weights)
      } else {
        w <- weights
      }
    }
    
    # if there's an offset, add it to the pred vector
    if (is.null(offset)) {
      offset <- rep(0, nobs)
    } else {
      if (length(offset) != nobs) stop("offset and y must have the same length")
      pred <- pred + offset
    }
    
    # extract strata (if any)
    if ("strata" %in% names(attributes(y))) {
      strata <- attr(y, "strata")
    } else {
      strata <- rep(1, nobs)
    }
    if (length(strata) != nobs) stop("length of strata != nobs")
    
    # if all in same strata, do the deviance computation
    # if not, take the sum of the strata-level deviances
    if (length(unique(strata)) == 1) {
      time <- y[, "time"]
      d <- y[, "status"]
      
      ### Compute saturated loglikelihood
      wd <- w[d == 1]
      tyd <- time[d == 1]
      if (any(duplicated(tyd))) {
        wd <- tapply(wd, tyd, sum)
      }
      wd <- wd[wd > 0]
      lsat <- -sum(wd * log(wd))
      ####
      
      # order time, d, pred and w in ascending time order
      # for tied times, all deaths come before censored observations
      if ("stop_time" %in% names(attributes(y))) {
        o <- attr(y, "stop_time")
      } else {
        o <- order(time, d, decreasing = c(FALSE, TRUE))
      }
      time <- time[o]
      d <- d[o]
      pred <- pred[o]
      w <- w[o]
      
      ### See if there are dups in death times
      dups <- fid(time[d==1],seq(length(d))[d==1])
      dd <- d
      ww <- w
      
      ### next code replaces each sequence of tied death indicators by a new
      ### sequence where only the first is a 1 and the rest are zero. This
      ### makes the accounting in the following step work properly we also
      ### sums the weights in each of the tied death sets, and assign that
      ### weight to the first
      if(!is.null(ties<-dups$index_ties)){
        dd[unlist(ties)]=0
        dd[dups$index_first]=1
        wsum=sapply(ties,function(i,w)sum(w[i]),ww)
        tie1=sapply(ties,function(i)i[1])
        ww[tie1]=wsum
      }
      
      # compute the sum inside the log term of the partial likelihood
      w_exp_pred <- w * exp(pred)
      rsk <- rev(cumsum(rev(w_exp_pred)))
      
      # take just the terms related to actual death times
      log_terms <- (ww * log(rsk))[dd > 0]
      loglik <- sum((w * pred)[d > 0]) - sum(log_terms)
      
      return(2 * (lsat -loglik))
    } else {
      # more than one strata provided: return the sum of strata-level 
      # deviances
      tot_dev <- 0
      for (i in unique(strata)) {
        ii <- which(strata == i)
        tot_dev <- tot_dev + 
          coxnet.deviance2(pred = pred[ii], y = y[ii, , drop = FALSE], 
                           offset = NULL, weights = w[ii], std.weights = FALSE)
      }
      return(tot_dev)
    }
  }
}

#' @rdname coxnet.deviance
coxnet.deviance3 <- function(pred = NULL, y, x = NULL, offset = NULL, 
                             weights = NULL, std.weights = TRUE, beta = NULL) {
  if (!is.Surv(y)) stop("y must be a Surv object")
  nobs <- nrow(y)
  
  # if pred is NULL, use beta and x to compute pred
  # if beta is NULL too, set pred to all zeros
  if (is.null(pred)) {
    if ((!is.null(x) && is.null(beta)) || (is.null(x) && !is.null(beta)))
      stop("user must pass either `pred`, or both `x` and `beta`")
    if (is.null(beta)) {
      pred <- rep(0, times = nobs)
    } else {
      pred <- x %*% beta
    }
  }
  
  # if more than one column of predictions is passed, run coxnet.deviance3()
  # for each column
  if (!is.null(ncol(pred)) && ncol(pred) > 1) {
    return(sapply(seq(ncol(pred)),
                  function(j) coxnet.deviance3(
                    pred = pred[, j], y = y, offset = offset, 
                    weights = weights, std.weights = std.weights)))
  } else {
    # check that pred is of the right length
    if(length(pred) != nobs) stop("pred and y must have the same length")
    
    # normalize weights to sum to nobs
    if (is.null(weights))
      w <- rep(1, nobs)
    else {
      if (length(weights) != nobs) stop("weights and y must have the same length")
      if (std.weights) {
        w <- nobs * weights / sum(weights)
      } else {
        w <- weights
      }
    }
    
    # if there's an offset, add it to the pred vector
    if (is.null(offset)) {
      offset <- rep(0, nobs)
    } else {
      if (length(offset) != nobs) stop("offset and y must have the same length")
      pred <- pred + offset
    }
    
    # extract strata (if any)
    if ("strata" %in% names(attributes(y))) {
      strata <- attr(y, "strata")
    } else {
      strata <- rep(1, nobs)
    }
    if (length(strata) != nobs) stop("length of strata != nobs")
    
    # if all in same strata, do the deviance computation
    # if not, take the sum of the strata-level deviances
    if (length(unique(strata)) == 1) {
      start_time <- y[, "start"]
      stop_time <- y[, "stop"]
      d <- y[, "status"]
      
      ### Compute saturated loglikelihood
      wd <- w[d == 1]
      tyd <- stop_time[d == 1]
      if (any(duplicated(tyd))) {
        wd <- tapply(wd, tyd, sum)
      }
      wd <- wd[wd > 0]
      lsat <- -sum(wd * log(wd))
      ####
      
      # get ordering for stop time (ascending, deaths before censored), and
      # start time (ascending)
      if ("stop_time" %in% names(attributes(y))) {
        stop_o <- attr(y, "stop_time")
      } else {
        stop_o <- order(stop_time, d, decreasing = c(FALSE, TRUE))
      }
      if ("start_time" %in% names(attributes(y))) {
        start_o <- attr(y, "start_time")
      } else {
        start_o <- order(start_time, decreasing = c(FALSE))
      }
      
      # keep a set of values which are ordered by start time
      w_exp_pred_start <- (w * exp(pred))[start_o]
      start_time_start <- start_time[start_o]
      
      # reorder everything by stop time
      start_time <- start_time[stop_o]
      stop_time  <- stop_time[stop_o]
      d          <- d[stop_o]
      pred       <- pred[stop_o]
      w          <- w[stop_o]
      
      ### See if there are dups in death times
      dups <- fid(stop_time[d == 1], seq(length(d))[d == 1])
      dd <- d
      ww <- w
      
      ### next code replaces each sequence of tied death indicators by a new
      ### sequence where only the first is a 1 and the rest are zero. This
      ### makes the accounting in the following step work properly we also
      ### sums the weights in each of the tied death sets, and assign that
      ### weight to the first
      if(!is.null(ties<-dups$index_ties)){
        dd[unlist(ties)]=0
        dd[dups$index_first]=1
        wsum=sapply(ties,function(i,w)sum(w[i]),ww)
        tie1=sapply(ties,function(i)i[1])
        ww[tie1]=wsum
      }
      
      # compute risk set sums rsk[i] = \sum_{j in R_i} w_j exp(eta_j)
      # where i indexes the observations. (In the end, we will only care
      # about the indices i which have actual death times.)
      rsk <- rev(cumsum(rev(w * exp(pred))))
      current_sum <- 0
      stop_idx <- nobs; start_idx <- nobs
      while (stop_idx > 0 && start_idx > 0) {
        if (start_time_start[start_idx] < stop_time[stop_idx]) {
          # current start time belongs in risk set ending in stop time,
          # so we should remove the current cumulative sum and consider
          # the next risk set
          rsk[stop_idx] <- rsk[stop_idx] - current_sum
          stop_idx <- stop_idx - 1
        } else {
          # current start time does not belong in risk set ending in stop
          # time, so we should add it to current_sum and check if the
          # start time before it should also be added
          current_sum <- current_sum + w_exp_pred_start[start_idx]
          start_idx <- start_idx - 1
        }
      }
      
      log_terms <- ww[dups$index_first] * (log(rsk[dd == 1]))
      loglik <- sum((w * pred)[d > 0]) - sum(log_terms)
      return(2 * (lsat -loglik))
    } else {
      # more than one strata provided: return the sum of strata-level 
      # deviances
      tot_dev <- 0
      for (i in unique(strata)) {
        ii <- which(strata == i)
        tot_dev <- tot_dev + 
          coxnet.deviance3(pred = pred[ii], y = y[ii, , drop = FALSE], 
                           offset = NULL, weights = w[ii], std.weights = FALSE)
      }
      return(tot_dev)
    }
  }
}

#' make predictions from a "glmnet" object.
#'
#' Similar to other predict methods, this functions predicts fitted values,
#' logits, coefficients and more from a fitted \code{"glmnet"} object.
#'
#' The shape of the objects returned are different for \code{"multinomial"}
#' objects. This function actually calls \code{NextMethod()}, and the
#' appropriate predict method is invoked for each of the three model types.
#' \code{coef(...)} is equivalent to \code{predict(type="coefficients",...)}
#'
#' @aliases coef.glmnet coef.relaxed predict.glmnet predict.relaxed
#' predict.elnet predict.lognet predict.multnet predict.mrelnet predict.fishnet
#' predict.coxnet
#' @param object Fitted \code{"glmnet"} model object or a \code{"relaxed"}
#' model (which inherits from class "glmnet").
#' @param newx Matrix of new values for \code{x} at which predictions are to be
#' made. Must be a matrix; can be sparse as in \code{Matrix} package. This
#' argument is not used for \code{type=c("coefficients","nonzero")}
#' @param s Value(s) of the penalty parameter \code{lambda} at which
#' predictions are required. Default is the entire sequence used to create the
#' model.
#' @param type Type of prediction required. Type \code{"link"} gives the linear
#' predictors for \code{"binomial"}, \code{"multinomial"}, \code{"poisson"} or
#' \code{"cox"} models; for \code{"gaussian"} models it gives the fitted
#' values. Type \code{"response"} gives the fitted probabilities for
#' \code{"binomial"} or \code{"multinomial"}, fitted mean for \code{"poisson"}
#' and the fitted relative-risk for \code{"cox"}; for \code{"gaussian"} type
#' \code{"response"} is equivalent to type \code{"link"}. Type
#' \code{"coefficients"} computes the coefficients at the requested values for
#' \code{s}.  Note that for \code{"binomial"} models, results are returned only
#' for the class corresponding to the second level of the factor response.
#' Type \code{"class"} applies only to \code{"binomial"} or
#' \code{"multinomial"} models, and produces the class label corresponding to
#' the maximum probability. Type \code{"nonzero"} returns a list of the indices
#' of the nonzero coefficients for each value of \code{s}.
#' @param exact This argument is relevant only when predictions are made at
#' values of \code{s} (lambda) \emph{different} from those used in the fitting
#' of the original model. Not available for \code{"relaxed"} objects. If
#' \code{exact=FALSE} (default), then the predict function uses linear
#' interpolation to make predictions for values of \code{s} (lambda) that do
#' not coincide with those used in the fitting algorithm. While this is often a
#' good approximation, it can sometimes be a bit coarse.  With
#' \code{exact=TRUE}, these different values of \code{s} are merged (and
#' sorted) with \code{object$lambda}, and the model is refit before predictions
#' are made. In this case, it is required to supply the original data \code{x=}
#' and \code{y=} as additional named arguments to \code{predict()} or
#' \code{coef()}.  The workhorse \code{predict.glmnet()} needs to \code{update}
#' the model, and so needs the data used to create it. The same is true of
#' \code{weights}, \code{offset}, \code{penalty.factor}, \code{lower.limits},
#' \code{upper.limits} if these were used in the original call. Failure to do
#' so will result in an error.
#' @param newoffset If an offset is used in the fit, then one must be supplied
#' for making predictions (except for \code{type="coefficients"} or
#' \code{type="nonzero"})
#' @param \dots This is the mechanism for passing arguments like \code{x=} when
#' \code{exact=TRUE}; see\code{exact} argument.
#' @return The object returned depends on type.
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}, and \code{print}, and \code{coef} methods, and
#' \code{cv.glmnet}.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}, \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}\cr
#' \emph{Journal of Statistical Software, Vol. 33(1), 1-22 Feb 2010}\cr
#' \url{https://www.jstatsoft.org/v33/i01/}\cr Simon, N., Friedman, J., Hastie,
#' T., Tibshirani, R. (2011) \emph{Regularization Paths for Cox's Proportional
#' Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
#' 39(5) 1-13}\cr \url{https://www.jstatsoft.org/v39/i05/}
#' @keywords models regression
#'
#' @examples
#' x=matrix(rnorm(100*20),100,20)
#' y=rnorm(100)
#' g2=sample(1:2,100,replace=TRUE)
#' g4=sample(1:4,100,replace=TRUE)
#' fit1=glmnet(x,y)
#' predict(fit1,newx=x[1:5,],s=c(0.01,0.005))
#' predict(fit1,type="coef")
#' fit2=glmnet(x,g2,family="binomial")
#' predict(fit2,type="response",newx=x[2:5,])
#' predict(fit2,type="nonzero")
#' fit3=glmnet(x,g4,family="multinomial")
#' predict(fit3,newx=x[1:3,],type="response",s=0.01)
#' @method predict glmnet
#' @export
#' @export predict.glmnet
predict.glmnet=function(object,newx,s=NULL,type=c("link","response","coefficients","nonzero","class"),exact=FALSE,newoffset,...){
 type=match.arg(type)
  if(missing(newx)){
    if(!match(type,c("coefficients","nonzero"),FALSE))stop("You need to supply a value for 'newx'")
     }
  if(exact&&(!is.null(s))){
###we augment the lambda sequence with the new values, if they are different,and refit the model using update
    lambda=object$lambda
    which=match(s,lambda,FALSE)
    if(!all(which>0)){
        lambda=unique(rev(sort(c(s,lambda))))
        check_dots(object,...)# This fails if you don't supply the crucial arguments
        object=update(object,lambda=lambda,...)
    }
  }
  a0=t(as.matrix(object$a0))
  rownames(a0)="(Intercept)"
  nbeta=methods::rbind2(a0,object$beta)#was rbind2
  if(!is.null(s)){
    vnames=dimnames(nbeta)[[1]]
    dimnames(nbeta)=list(NULL,NULL)
    lambda=object$lambda
    lamlist=lambda.interp(lambda,s)

    nbeta=nbeta[,lamlist$left,drop=FALSE]%*%Diagonal(x=lamlist$frac) +nbeta[,lamlist$right,drop=FALSE]%*%Diagonal(x=1-lamlist$frac)
    dimnames(nbeta)=list(vnames,paste(seq(along=s)))
  }
  if(type=="coefficients")return(nbeta)
  if(type=="nonzero")return(nonzeroCoef(nbeta[-1,,drop=FALSE],bystep=TRUE))
  ###Check on newx
 if(inherits(newx, "sparseMatrix"))newx=as(newx,"dgCMatrix")
  nfit=as.matrix(cbind2(1,newx)%*%nbeta)
   if(object$offset){
    if(missing(newoffset))stop("No newoffset provided for prediction, yet offset used in fit of glmnet",call.=FALSE)
    if(is.matrix(newoffset)&&inherits(object,"lognet")&&dim(newoffset)[[2]]==2)newoffset=newoffset[,2]
    nfit=nfit+array(newoffset,dim=dim(nfit))
  }
nfit
  }

getcoef.multinomial=function(fit,nvars,nx,vnames,nc,classnames,center.intercept=TRUE){
  lmu=fit$lmu
  nin=fit$nin[seq(lmu)]
  ninmax=max(nin)
  lam=fit$alm[seq(lmu)]
  stepnames=paste("s",seq(lmu)-1,sep="")
  beta.list=as.list(seq(nc))
  names(beta.list)=classnames
    a0=matrix(fit$a0[seq(lmu*nc)],nc,lmu,dimnames=list(classnames,stepnames))
      if(center.intercept){
        a0=scale(a0,TRUE,FALSE)
        attr(a0,"scaled:center")=NULL
      }
      dfmat=a0
      dd=c(nvars,lmu)
      if(ninmax>0){
        ca=array(fit$ca[seq(nx*lmu*nc)],c(nx,nc,lmu))[seq(ninmax),,,drop=FALSE]
        ja=fit$ia[seq(ninmax)]#confusing but too hard to change
        oja=order(ja)
        ja=rep(ja[oja],lmu)
        ia=cumsum(c(1,rep(ninmax,lmu)))
        df=apply(abs(ca)>0,c(1,3),any)
        df=apply(df,2,sum)
        for(k in seq(nc)){
          cak=ca[oja,k, ,drop=FALSE]
          dfmat[k,]=apply(abs(cak)>0,3,sum)
          beta=new("dgCMatrix",Dim=dd,Dimnames=list(vnames,stepnames),x=as.vector(cak),p=as.integer(ia-1),i=as.integer(ja-1))
          beta.list[[k]]=drop0(beta)
        }
   } else{
    for (k in seq(nc)) {
      dfmat[k, ] = rep(0,lmu)
      beta.list[[k]] = zeromat(nvars,lmu,vnames,stepnames)
    }
  }
       
  list(a0=a0,beta=beta.list,dfmat=dfmat,df=df,dim=dd,lambda=lam)
}

c     mortran 2.0     (version of 7/04/75 mod 7/4/87 (ajc))             
      subroutine wls(alm0,almc,alpha,m,no,ni,x,r,v,intr,ju,vp,cl,nx,thr,
     *maxit,  a,aint,g,ia,iy,iz,mm,nino,rsqc,nlp,jerr)
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),r(no),a(ni),vp(ni),cl(2,ni)             
      double precision v(no),g(ni)                                      
      integer iy(ni),ia(nx),ju(ni),mm(ni)                               
      double precision, dimension (:), allocatable :: xv                
      allocate(xv(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      do 10011 j=1,ni                                                   
      if(ju(j).eq.0)goto 10011                                          
      g(j)=abs(dot_product(r,x(:,j)))                                   
10011 continue                                                          
      continue                                                          
      do 10021 j=1,ni                                                   
      if(iy(j).gt.0) xv(j)=dot_product(v,x(:,j)**2)                     
10021 continue                                                          
      continue                                                          
      xmz = sum(v)                                                      
      ab=almc*alpha                                                     
      dem=almc*(1.0-alpha)                                              
      tlam=alpha*(2.0*almc-alm0)                                        
      do 10031 k=1,ni                                                   
      if(iy(k).eq.1)goto 10031                                          
      if(ju(k).eq.0)goto 10031                                          
      if(g(k) .le. tlam*vp(k))goto 10051                                
      iy(k)=1                                                           
      xv(k)=dot_product(v,x(:,k)**2)                                    
10051 continue                                                          
10031 continue                                                          
      continue                                                          
      jz = 1                                                            
      continue                                                          
10061 continue                                                          
      if(iz*jz.ne.0) go to 10070                                        
10080 continue                                                          
      nlp=nlp+1                                                         
      dlx=0.0                                                           
      do 10091 k=1,ni                                                   
      if(iy(k).eq.0)goto 10091                                          
      gk=dot_product(r,x(:,k))                                          
      ak=a(k)                                                           
      u=gk+ak*xv(k)                                                     
      au=abs(u)-vp(k)*ab                                                
      if(au .gt. 0.0)goto 10111                                         
      a(k)=0.0                                                          
      goto 10121                                                        
10111 continue                                                          
      a(k)=max(cl(1,k),min(cl(2,k),sign(au,u)/(xv(k)+vp(k)*dem)))       
10121 continue                                                          
      continue                                                          
      if(a(k).eq.ak)goto 10091                                          
      if(mm(k) .ne. 0)goto 10141                                        
      nino=nino+1                                                       
      if(nino.gt.nx)goto 10092                                          
      mm(k)=nino                                                        
      ia(nino)=k                                                        
10141 continue                                                          
      d=a(k)-ak                                                         
      rsqc=rsqc+d*(2.0*gk-d*xv(k))                                      
      r=r-d*v*x(:,k)                                                    
      dlx=max(xv(k)*d**2,dlx)                                           
10091 continue                                                          
10092 continue                                                          
      if(nino.gt.nx)goto 10062                                          
      d=0.0                                                             
      if(intr.ne.0) d=sum(r)/xmz                                        
      if(d .eq. 0.0)goto 10161                                          
      aint=aint+d                                                       
      rsqc=rsqc+d*(2.0*sum(r)-d*xmz)                                    
      dlx=max(dlx,xmz*d**2)                                             
      r=r-d*v                                                           
10161 continue                                                          
      if(dlx .ge. thr)goto 10181                                        
      ixx=0                                                             
      do 10191 k=1,ni                                                   
      if(iy(k).eq.1)goto 10191                                          
      if(ju(k).eq.0)goto 10191                                          
      g(k)=abs(dot_product(r,x(:,k)))                                   
      if(g(k) .le. ab*vp(k))goto 10211                                  
      iy(k)=1                                                           
      xv(k)=dot_product(v,x(:,k)**2)                                    
      ixx=1                                                             
10211 continue                                                          
10191 continue                                                          
      continue                                                          
      if(ixx.eq.1) go to 10080                                          
      goto 10062                                                        
10181 continue                                                          
      if(nlp .le. maxit)goto 10231                                      
      jerr=-m                                                           
      return                                                            
10231 continue                                                          
10070 continue                                                          
      iz = 1                                                            
      continue                                                          
10241 continue                                                          
      nlp=nlp+1                                                         
      dlx=0.0                                                           
      do 10251 l=1,nino                                                 
      k=ia(l)                                                           
      gk=dot_product(r,x(:,k))                                          
      ak=a(k)                                                           
      u=gk+ak*xv(k)                                                     
      au=abs(u)-vp(k)*ab                                                
      if(au .gt. 0.0)goto 10271                                         
      a(k)=0.0                                                          
      goto 10281                                                        
10271 continue                                                          
      a(k)=max(cl(1,k),min(cl(2,k),sign(au,u)/(xv(k)+vp(k)*dem)))       
10281 continue                                                          
      continue                                                          
      if(a(k).eq.ak)goto 10251                                          
      d=a(k)-ak                                                         
      rsqc=rsqc+d*(2.0*gk-d*xv(k))                                      
      r=r-d*v*x(:,k)                                                    
      dlx=max(xv(k)*d**2,dlx)                                           
10251 continue                                                          
      continue                                                          
      d=0.0                                                             
      if(intr.ne.0) d=sum(r)/xmz                                        
      if(d .eq. 0.0)goto 10301                                          
      aint=aint+d                                                       
      rsqc=rsqc+d*(2.0*sum(r)-d*xmz)                                    
      dlx=max(dlx,xmz*d**2)                                             
      r=r-d*v                                                           
10301 continue                                                          
      if(dlx.lt.thr)goto 10242                                          
      if(nlp .le. maxit)goto 10321                                      
      jerr=-m                                                           
      return                                                            
10321 continue                                                          
      goto 10241                                                        
10242 continue                                                          
      jz=0                                                              
      goto 10061                                                        
10062 continue                                                          
      deallocate(xv)                                                    
      return                                                            
      end                                                               
      subroutine spwls(alm0,almc,alpha,m,no,ni,x,ix,jx,xm,xs,r,v,intr,ju
     *,  vp,cl,nx,thr,maxit,a,aint,g,ia,iy,iz,mm,nino,rsqc,nlp,jerr)
      implicit double precision(a-h,o-z)                                
      double precision x(*),xm(ni),xs(ni),r(no),a(ni),vp(ni),cl(2,ni)   
      double precision v(no),g(ni)                                      
      integer ix(*),jx(*),iy(ni),ia(nx),ju(ni),mm(ni)                   
      double precision, dimension (:), allocatable :: xv                
      allocate(xv(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      xmz = sum(v)                                                      
      rsum = sum(r)                                                     
      do 10331 j=1,ni                                                   
      if(ju(j).eq.0)goto 10331                                          
      jb=ix(j)                                                          
      je=ix(j+1)-1                                                      
      g(j)=abs(dot_product(r(jx(jb:je)),x(jb:je))-rsum*xm(j))/xs(j)     
10331 continue                                                          
      continue                                                          
      do 10341 j=1,ni                                                   
      if(iy(j) .le. 0)goto 10361                                        
      jb=ix(j)                                                          
      je=ix(j+1)-1                                                      
      xv(j)=dot_product(v(jx(jb:je)),x(jb:je)**2)                       
      xv(j)=xv(j)-2*xm(j)*dot_product(v(jx(jb:je)),x(jb:je))            
      xv(j)=(xv(j)+xmz*xm(j)**2)/xs(j)**2                               
10361 continue                                                          
10341 continue                                                          
      continue                                                          
      ab=almc*alpha                                                     
      dem=almc*(1.0-alpha)                                              
      tlam=alpha*(2.0*almc-alm0)                                        
      do 10371 k=1,ni                                                   
      if(iy(k).eq.1)goto 10371                                          
      if(ju(k).eq.0)goto 10371                                          
      if(g(k) .le. tlam*vp(k))goto 10391                                
      iy(k)=1                                                           
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      xv(k)=dot_product(v(jx(jb:je)),x(jb:je)**2)                       
      xv(k)=xv(k)-2*xm(k)*dot_product(v(jx(jb:je)),x(jb:je))            
      xv(k)=(xv(k)+xmz*xm(k)**2)/xs(k)**2                               
10391 continue                                                          
10371 continue                                                          
      continue                                                          
      jz = 1                                                            
      continue                                                          
10401 continue                                                          
      if(iz*jz.ne.0) go to 10070                                        
10080 continue                                                          
      nlp=nlp+1                                                         
      dlx=0.0                                                           
      do 10411 k=1,ni                                                   
      if(iy(k).eq.0)goto 10411                                          
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      gk=(dot_product(r(jx(jb:je)),x(jb:je))-rsum*xm(k))/xs(k)          
      ak=a(k)                                                           
      u=gk+ak*xv(k)                                                     
      au=abs(u)-vp(k)*ab                                                
      if(au .gt. 0.0)goto 10431                                         
      a(k)=0.0                                                          
      goto 10441                                                        
10431 continue                                                          
      a(k)=max(cl(1,k),min(cl(2,k),sign(au,u)/(xv(k)+vp(k)*dem)))       
10441 continue                                                          
      continue                                                          
      if(a(k).eq.ak)goto 10411                                          
      if(mm(k) .ne. 0)goto 10461                                        
      nino=nino+1                                                       
      if(nino.gt.nx)goto 10412                                          
      mm(k)=nino                                                        
      ia(nino)=k                                                        
10461 continue                                                          
      d=a(k)-ak                                                         
      rsqc=rsqc+d*(2.0*gk-d*xv(k))                                      
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      r(jx(jb:je))=r(jx(jb:je))-d*v(jx(jb:je))*x(jb:je)/xs(k)           
      r=r+d*v*xm(k)/xs(k)                                               
      rsum=sum(r)                                                       
      dlx=max(xv(k)*d**2,dlx)                                           
10411 continue                                                          
10412 continue                                                          
      if(nino.gt.nx)goto 10402                                          
      d=0.0                                                             
      if(intr.ne.0) d=rsum/xmz                                          
      if(d .eq. 0.0)goto 10481                                          
      aint=aint+d                                                       
      rsqc=rsqc+d*(2.0*rsum-d*xmz)                                      
      dlx=max(dlx,xmz*d**2)                                             
      r=r-d*v                                                           
      rsum=sum(r)                                                       
10481 continue                                                          
      if(dlx .ge. thr)goto 10501                                        
      ixx=0                                                             
      do 10511 k=1,ni                                                   
      if(iy(k).eq.1)goto 10511                                          
      if(ju(k).eq.0)goto 10511                                          
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      g(k)=dot_product(r(jx(jb:je)),x(jb:je))                           
      g(k)=abs(g(k)-rsum*xm(k))/xs(k)                                   
      if(g(k) .le. ab*vp(k))goto 10531                                  
      iy(k)=1                                                           
      xv(k)=dot_product(v(jx(jb:je)),x(jb:je)**2)                       
      vx=dot_product(v(jx(jb:je)),x(jb:je))                             
      xv(k)=xv(k)-2*xm(k)*vx                                            
      xv(k)=(xv(k)+xmz*xm(k)**2)/xs(k)**2                               
      ixx=1                                                             
10531 continue                                                          
10511 continue                                                          
      continue                                                          
      if(ixx.eq.1) go to 10080                                          
      goto 10402                                                        
10501 continue                                                          
      if(nlp .le. maxit)goto 10551                                      
      jerr=-m                                                           
      return                                                            
10551 continue                                                          
10070 continue                                                          
      iz = 1                                                            
      continue                                                          
10561 continue                                                          
      nlp=nlp+1                                                         
      dlx=0.0                                                           
      do 10571 l=1,nino                                                 
      k=ia(l)                                                           
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      gk=(dot_product(r(jx(jb:je)),x(jb:je))-rsum*xm(k))/xs(k)          
      ak=a(k)                                                           
      u=gk+ak*xv(k)                                                     
      au=abs(u)-vp(k)*ab                                                
      if(au .gt. 0.0)goto 10591                                         
      a(k)=0.0                                                          
      goto 10601                                                        
10591 continue                                                          
      a(k)=max(cl(1,k),min(cl(2,k),sign(au,u)/(xv(k)+vp(k)*dem)))       
10601 continue                                                          
      continue                                                          
      if(a(k).eq.ak)goto 10571                                          
      d=a(k)-ak                                                         
      rsqc=rsqc+d*(2.0*gk-d*xv(k))                                      
      jb=ix(k)                                                          
      je=ix(k+1)-1                                                      
      r(jx(jb:je))=r(jx(jb:je))-d*v(jx(jb:je))*x(jb:je)/xs(k)           
      r=r+d*v*xm(k)/xs(k)                                               
      rsum=sum(r)                                                       
      dlx=max(xv(k)*d**2,dlx)                                           
10571 continue                                                          
      continue                                                          
      d=0.0                                                             
      if(intr.ne.0) d=rsum/xmz                                          
      if(d .eq. 0.0)goto 10621                                          
      aint=aint+d                                                       
      rsqc=rsqc+d*(2.0*rsum-d*xmz)                                      
      dlx=max(dlx,xmz*d**2)                                             
      r=r-d*v                                                           
      rsum=rsum-d*xmz                                                   
10621 continue                                                          
      if(dlx.lt.thr)goto 10562                                          
      if(nlp .le. maxit)goto 10641                                      
      jerr=-m                                                           
      return                                                            
10641 continue                                                          
      goto 10561                                                        
10562 continue                                                          
      jz=0                                                              
      goto 10401                                                        
10402 continue                                                          
      deallocate(xv)                                                    
      return                                                            
      end                                                               
      subroutine get_int_parms2(epsnr,mxitnr)                           
      implicit double precision(a-h,o-z)                                
      data epsnr0,mxitnr0  /1.0d-6,25/                                  
      epsnr=epsnr0                                                      
      mxitnr=mxitnr0                                                    
      return                                                            
      entry chg_epsnr(arg)                                              
      epsnr0=arg                                                        
      return                                                            
      entry chg_mxitnr(irg)                                             
      mxitnr0=irg                                                       
      return                                                            
      end                                                               

#' @export
buildPredmat.default=function(outlist, lambda, x, offset, foldid, alignment,...){
    if (!is.null(offset)) {
        is.offset = TRUE
        offset = drop(offset)
    }
    else is.offset = FALSE
    predmat = matrix(NA, nrow(x), length(lambda))
    nfolds = max(foldid)
    nlams = double(nfolds)
    nlambda=length(lambda)
    for (i in seq(nfolds)) {
        which = foldid == i
        fitobj = outlist[[i]]
        if (is.offset)
            off_sub = offset[which]
        preds = switch(alignment,
                   fraction=predict(fitobj, x[which, , drop = FALSE], newoffset = off_sub,...),
                   lambda=predict(fitobj, x[which, , drop = FALSE], s=lambda, newoffset = off_sub,...)
                   )
    nlami = min(ncol(preds),nlambda)
    predmat[which, seq(nlami)] = preds[,seq(nlami)]
    if(nlami<nlambda)predmat[which,seq(from=nlami,to=nlambda)]=preds[,nlami]
    }
    rn=rownames(x)
    sn=paste("s",seq(0,length=nlambda),sep="")
    dimnames(predmat)=list(rn,sn)

    predmat
    }

#' @export
buildPredmat=function(outlist, lambda, x, offset, foldid, alignment,...)
    UseMethod("buildPredmat")

#' @method print bigGlm
#' @export
print.bigGlm <-
function (x, digits = max(3, getOption("digits") - 3), ...)
{
    cat("\nCall: ", deparse(x$call), "\n\n")
     out=data.frame(Df = x$df, `%Dev` = round(x$dev.ratio*100, 2),
                    Lambda = 0,check.names=FALSE,row.names=seq(along=x$df))
class(out)=c("anova",class(out))
     print(out)
}

plotCoef=function(beta,norm,lambda,df,dev,label=FALSE,xvar=c("norm","lambda","dev"),xlab=iname,ylab="Coefficients",...){
  ##beta should be in "dgCMatrix" format
  which=nonzeroCoef(beta)
  nwhich=length(which)
  switch(nwhich+1,#we add one to make switch work
         "0"={
           warning("No plot produced since all coefficients zero")
           return()
         },
         "1"=warning("1 or less nonzero coefficients; glmnet plot is not meaningful")
         )
  beta=as.matrix(beta[which,,drop=FALSE])
  xvar=match.arg(xvar)
  switch(xvar,
    "norm"={
      index=if(missing(norm))apply(abs(beta),2,sum)else norm
      iname="L1 Norm"
      approx.f=1
    },
    "lambda"={
      index=log(lambda)
      iname="Log Lambda"
      approx.f=0
    },
    "dev"= {
      index=dev
      iname="Fraction Deviance Explained"
      approx.f=1
    }
         )
  dotlist=list(...)
  type=dotlist$type
  if(is.null(type))
    matplot(index,t(beta),lty=1,xlab=xlab,ylab=ylab,type="l",...)
  else matplot(index,t(beta),lty=1,xlab=xlab,ylab=ylab,...)
  atdf=pretty(index)
  ### compute df by interpolating to df at next smaller lambda
  ### thanks to Yunyang Qian
 prettydf=approx(x=index,y=df,xout=atdf,rule=2,method="constant",f=approx.f)$y
# prettydf=ceiling(approx(x=index,y=df,xout=atdf,rule=2)$y)
 axis(3,at=atdf,labels=prettydf,tcl=NA)
 if(label){
   nnz=length(which)
   xpos=max(index)
   pos=4
   if(xvar=="lambda"){
     xpos=min(index)
     pos=2
   }
   xpos=rep(xpos,nnz)
   ypos=beta[,ncol(beta)]
   text(xpos,ypos,paste(which),cex=.5,pos=pos)
 }

}

jerr.lognet=function(n,maxit,pmax){
    outlist=jerr.elnet(n,maxit,pmax)
    if(n<  -20000)outlist$msg=paste("Max(p(1-p),1.0e-6 at ",-n-20000,"th value of lambda; solutions for larger values of lambda returned")
    if(outlist$msg!="Unknown error")return(outlist)
    if((8000<n) & (n<9000))msg=paste("Null probability for class",n-8000, "< 1.0e-5")
    else if((9000<n) & (n<10000))msg=paste("Null probability for class",n-9000, "> 1.0 - 1.0e-5")
    else msg="Unknown error"
    list(n=n,fatal=TRUE,msg=msg)
}

jerr.fishnet=function(n,maxit,pmax){
  outlist=jerr.elnet(n,maxit,pmax)
  if(outlist$msg!="Unknown error")return(outlist)
  if(n==8888)msg="Negative response values - should be counts"  
  else if(n==9999)msg="No positive observation weights"
  else 	 msg="Unknown error"
  list(n=n,fatal=TRUE,msg=msg)
}

cv.relaxed.raw <-
    function (x, y, weights, offset, lambda, type.measure, nfolds, foldid, alignment,grouped, keep,
              parallel, trace.it, glmnet.call, cv.call, gamma, ...)
{

###This next function may be redundant
  relaxglmnet=function(x,...){
    fit=glmnet(x,...)
    relax.glmnet(fit,x,...,check.args=FALSE)
     }
    gamma=sort(checkgamma.relax(gamma))
### The next bit is to make sure we always include lasso in CV, as well as the gamma series
    lengamma=length(gamma)
    if(max(gamma)<1.0){
        gamma=c(gamma,1.0)
        }

    glmnet.call$relax=TRUE
  if (trace.it) cat("Training\n")
  glmnet.object = relaxglmnet(x=x, y=y, weights = weights, offset = offset,
    lambda = lambda, trace.it=trace.it,...)
  glmnet.object$call = glmnet.call
  subclass=class(glmnet.object)[[2]]# it is of class c("relaxed","subtype","glmnet")
  type.measure=cvtype(type.measure,subclass)
  is.offset = glmnet.object$offset
###Next line is commented out so each call generates its own lambda sequence
# lambda=glmnet.object$lambda
 if (inherits(glmnet.object, "multnet") && !glmnet.object$grouped) {
    nz = predict(glmnet.object, type = "nonzero")
    nz = sapply(nz, function(x) sapply(x, length))
    nz = ceiling(apply(nz, 1, median))
  }
  else nz = sapply(predict(glmnet.object, type = "nonzero"),
         length)

  outlist = as.list(seq(nfolds))
  N=nrow(x)
    if (parallel) {
#  if (parallel && require(foreach)) {
    outlist = foreach(i = seq(nfolds), .packages = c("glmnet")) %dopar%
    {
      which = foldid == i
#      if (is.matrix(y))
      if (length(dim(y))>1)
        y_sub = y[!which, ]
      else y_sub = y[!which]
      if (is.offset)
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      relaxglmnet(x=x[!which, , drop = FALSE], y=y_sub, lambda = lambda,
             offset = offset_sub, weights = weights[!which],
             ...)
    }
  }
  else {
      for (i in seq(nfolds)) {
      if (trace.it) cat(sprintf("Fold: %d/%d\n", i, nfolds))
      which = foldid == i
      if (is.matrix(y))
        y_sub = y[!which, ]
      else y_sub = y[!which]
      if (is.offset)
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      outlist[[i]] = relaxglmnet(x=x[!which, , drop = FALSE],
               y=y_sub, lambda = lambda, offset = offset_sub,
               weights = weights[!which],trace.it=trace.it, ...)
    }
  }
  lambda = glmnet.object$lambda
  class(outlist)=paste0(subclass,"list")
  predmatlist=as.list(gamma)
  names(predmatlist)=paste("g",gamma,sep=":")
  outstuff=cvstufflist=predmatlist
### Even though the following is innefficient, It makes the code more modular
###  fun = paste("cv", subclass, sep = ".")
  for(i in seq(along=gamma)){
      predob=buildPredmat(outlist,
                         lambda=lambda,x=x,offset=offset,foldid=foldid,alignment=alignment,
                         y=y,weights=weights,grouped=grouped,
                         gamma=gamma[i],family=family(glmnet.object)
                         )
#      if(subclass=="glmnetfit")attr{predob,"family")=glmnet.object$family #special case
      predmatlist[[i]]=predob
      fun = paste("cv", subclass, sep = ".")
      cvstufflist[[i]] = do.call(fun, list(predmatlist[[i]],y,type.measure,weights,foldid,grouped))
  }
  grouped=all(sapply(cvstufflist,"[[","grouped"))
    if ((N/nfolds < 3) && grouped) {
    warning("Option grouped=FALSE enforced in cv.relaxed, since < 3 observations per fold",
            call. = FALSE)
    grouped = FALSE
    }
  for(i in seq(along=gamma)) outstuff[[i]]=cvstats(cvstufflist[[i]],foldid,nfolds,lambda,nz,grouped)
  cvn = cvstufflist[[1]]$type.measure
    cvname=names(cvn);names(cvname)=cvn# to be compatible with earlier version; silly, I know
    out=outstuff[[length(gamma)]] # lasso stats
    out = c(out,list(call=cv.call,name = cvname, glmnet.fit = glmnet.object))
    if(lengamma<length(gamma)){
        outstuff=outstuff[1:lengamma]
        gamma=gamma[1:lengamma]
        }
    relaxed=list(statlist=outstuff,gamma=gamma)
    lamin=getOptcv.relaxed(outstuff,cvname,gamma)
    relaxed=  c(relaxed, as.list(lamin))# for relaxed part
    lamin=with(out,getOptcv.glmnet(lambda, cvm, cvsd, cvname))
    out = c(out, as.list(lamin)) # for lasso part
  if (keep)
      out = c(out, list(fit.preval = predmatlist, foldid = foldid))
    out$relaxed=relaxed
  class(out) = c("cv.relaxed","cv.glmnet")
  out
}

#' @method predict fishnet
#' @export
predict.fishnet=function(object,newx,s=NULL,type=c("link","response","coefficients","nonzero"),exact=FALSE,newoffset,...){
  type=match.arg(type)
nfit=NextMethod("predict")
  switch(type,
         response=exp(nfit),
         nfit
         )
}

cvstats=function(cvstuff,foldid,nfolds,lambda,nz,grouped,...){
    if (grouped){
      nlams=rep(dim(cvstuff$cvraw)[2],nfolds) ## All the same - this should go
      cvstuff= cvcompute(cvstuff, foldid, nlams)
}
  cvm=with(cvstuff,apply(cvraw, 2, weighted.mean, w = weights, na.rm = TRUE))
  cvsd=with(cvstuff, sqrt(apply(scale(cvraw, cvm, FALSE)^2, 2, weighted.mean,
                               w = weights, na.rm = TRUE)/(N - 1)))
   nas=is.na(cvsd)
  if(any(nas)){
    lambda=lambda[!nas]
    cvm=cvm[!nas]
    cvsd=cvsd[!nas]
    nz=nz[!nas]
  }
list(lambda = lambda, cvm = cvm, cvsd = cvsd, cvup = cvm +
                                                  cvsd, cvlo = cvm - cvsd, nzero = nz)
}

#' fit a glm with all the options in \code{glmnet}
#'
#' Fit a generalized linear model as in \code{glmnet} but unpenalized. This
#' allows all the features of \code{glmnet} such as sparse x, bounds on
#' coefficients, offsets, and so on.
#'
#' This is essentially the same as fitting a "glmnet" model with a single value
#' \code{lambda=0}, but it avoids some edge cases. CAVEAT: If the user tries a
#' problem with N smaller than or close to p for some models, it is likely to
#' fail (and maybe not gracefully!) If so, use the \code{path=TRUE} argument.
#'
#' @param x input matrix
#' @param ... Most other arguments to glmnet that make sense
#' @param path  Since \code{glmnet} does not do stepsize optimization, the Newton
#' algorithm can get stuck and not converge, especially with unpenalized fits. With \code{path=TRUE},
#' the  fit computed with pathwise lasso regularization. The current implementation does this twice:
#' the first time to get the lambda sequence, and the second time with a zero attached to the end).
#' Default is \code{path=FALSE}.
#' @return It returns an object of class "bigGlm" that inherits from class
#' "glmnet". That means it can be predicted from, coefficients extracted via
#' \code{coef}. It has its own print method.
#' @author Trevor Hastie\cr Maintainer: Trevor Hastie
#' \email{hastie@@stanford.edu}
#' @seealso \code{print}, \code{predict}, and \code{coef} methods.
#' @keywords models regression
#' @examples
#'
#' # Gaussian
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' fit1 = bigGlm(x, y)
#' print(fit1)
#'
#' fit2=bigGlm(x,y>0,family="binomial")
#' print(fit2)
#' fit2p=bigGlm(x,y>0,family="binomial",path=TRUE)
#' print(fit2p)
#'
#' @export bigGlm
bigGlm=function (x,..., path=FALSE){
### fit a model using x, y  and the other glmnet specs in ... unpenalized
    thiscall = match.call(glmnet)
### Next few lines simply in case x is a vector, in which case glmnet complains
    bx=cbind(x,1)
    d=dim(bx)
    n=d[1]
    p=d[2]-1
    bx[1,p+1]=0 # so its not a constant
    arglist = list(...)
    arglist$x = bx
    arglist$exclude=p+1
    fixbeta <- function(beta,p,nlam=1){
        beta <- beta[1:p, nlam , drop = FALSE]
        dn <- dimnames(beta)
        dn <- list(dn[[1]],NULL)
        dimnames(beta) <- dn
        return(beta)
        }
    if(!path){
        arglist$lambda=0
        fit = do.call("glmnet", arglist)
        if (is.list(fit$beta)){
            fit$beta = lapply(fit$beta, fixbeta, p=p)
            dimnames(fit$a0) <- NULL
            }
        else {
                fit$beta = fixbeta(fit$beta, p=p)
                names(fit$a0) <- NULL
                }
    }
    else
    {
        fit = do.call("glmnet", arglist)# get the lambda sequence
        lam0=c(fit$lambda,0)
        arglist$lambda=lam0
        fit = do.call("glmnet", arglist)
        nlam=length(fit$lambda)#may have stopped early
        if (is.list(fit$beta)){
            fit$a0=fit$a0[,nlam,drop=FALSE]
            dimnames(fit$a0) <- NULL
            fit$beta = lapply(fit$beta, fixbeta, p=p, nlam=nlam)
            dfmat=fit$dfmat
            if(!is.null(dfmat))fit$dfmat=dfmat[,nlam,drop=FALSE]
        }
        else {
            fit$a0=fit$a0[nlam]
            names(fit$a0) <- NULL
            fit$beta = fixbeta(fit$beta, p=p, nlam=nlam)
            }
        fit$df=p
        fit$lambda=0
        fit$dev.ratio=fit$dev.ratio[nlam]
    }
    fit$dim=c(p,1)
    fit$call = thiscall
    class(fit) = c("bigGlm", class(fit))
    fit
}

getOptcv.relaxed <-
    function (statlist,cvname,gamma)
{
    index=matrix(NA,2,2,dimnames=list(c("min","1se"),c("Lambda","Gamma")))
    cvm=lapply(statlist,"[[","cvm")
    nlams=sapply(cvm,length)
    index.lambda=unlist(lapply(nlams,seq))
    cvm=unlist(cvm)
    lambdas=unlist(lapply(statlist,"[[","lambda"))
    cvsd=unlist(lapply(statlist,"[[","cvsd"))
    nzero=unlist(lapply(statlist,"[[","nzero"))
    gammas=rep(gamma,nlams)
    index.gamma = rep(seq(along=gamma),nlams)
    names(lambdas)=NULL
    names(gammas)=NULL
    if(match(cvname,c("AUC","C-index"),0))cvm=-cvm

    cvmin = min(cvm, na.rm = TRUE)
    idmin = cvm <= cvmin
    which=order(lambdas[idmin],gammas[idmin],decreasing=TRUE)[1]
    lambda.min = lambdas[idmin][which]
    gamma.min= gammas[idmin][which]
    index["min","Lambda"]=index.lambda[idmin][which]
    index["min","Gamma"]=index.gamma[idmin][which]
    nzero.min=nzero[idmin][which]
    idmin=seq(along=lambdas)[idmin][which]
    semin = (cvm + cvsd)[idmin]
    idmin = cvm <= semin
    which=order(lambdas[idmin],gammas[idmin],decreasing=TRUE)[1]
    lambda.1se = lambdas[idmin][which]
    gamma.1se= gammas[idmin][which]
    index["1se","Lambda"]=index.lambda[idmin][which]
    index["1se","Gamma"]=index.gamma[idmin][which]
    nzero.1se=nzero[idmin][which]
    list(lambda.min = lambda.min, lambda.1se = lambda.1se, gamma.min=gamma.min, gamma.1se=gamma.1se,nzero.min=nzero.min,nzero.1se=nzero.1se, index=index)
}

#' convert a data frame to a data matrix with one-hot encoding
#'
#' Converts a data frame to a data matrix suitable for input to \code{glmnet}.
#' Factors are converted to dummy matrices via "one-hot" encoding. Options deal
#' with missing values and sparsity.
#'
#' The main function is to convert factors to dummy matrices via "one-hot"
#' encoding. Having the 'train' and 'test' data present is useful if some
#' factor levels are missing in either. Since a factor with k levels leads to a
#' submatrix with 1/k entries zero, with large k the \code{sparse=TRUE} option
#' can be helpful; a large matrix will be returned, but stored in sparse matrix
#' format. Finally, the function can deal with missing data. The current
#' version has the option to replace missing observations with the mean from
#' the training data. For dummy submatrices, these are the mean proportions at
#' each level.
#'
#' @param train Required argument. A dataframe consisting of vectors, matrices
#' and factors
#' @param test Optional argument. A dataframe matching 'train' for use as
#' testing data
#' @param na.impute Logical, default \code{FALSE}. If \code{TRUE}, missing
#' values for any column in the resultant 'x' matrix are replaced by the means
#' of the nonmissing values derived from 'train'
#' @param sparse Logical, default \code{FALSE}. If \code{TRUE} then the
#' returned matrice(s) are converted to matrices of class "CsparseMatrix".
#' Useful if some factors have a large number of levels, resulting in very big
#' matrices, mostly zero
#' @param ... additional arguments, currently unused
#' @return If only 'train' was provided, the function returns a matrix 'x'. If
#' missing values were imputed, this matrix has an attribute containing its
#' column means (before imputation). If 'test' was provided as well, a list
#' with two components is returned: 'x' and 'xtest'.
#' @author Trevor Hastie\cr Maintainer: Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}
#' @keywords models
#' @examples
#'
#' set.seed(101)
#' ### Single data frame
#' X = matrix(rnorm(20), 10, 2)
#' X3 = sample(letters[1:3], 10, replace = TRUE)
#' X4 = sample(LETTERS[1:3], 10, replace = TRUE)
#' df = data.frame(X, X3, X4)
#' makeX(df)
#' makeX(df, sparse = TRUE)
#'
#' ### Single data freame with missing values
#' Xn = X
#' Xn[3, 1] = NA
#' Xn[5, 2] = NA
#' X3n = X3
#' X3n[6] = NA
#' X4n = X4
#' X4n[9] = NA
#' dfn = data.frame(Xn, X3n, X4n)
#'
#' makeX(dfn)
#' makeX(dfn, sparse = TRUE)
#' makeX(dfn, na.impute = TRUE)
#' makeX(dfn, na.impute = TRUE, sparse = TRUE)
#'
#' ### Test data as well
#' X = matrix(rnorm(10), 5, 2)
#' X3 = sample(letters[1:3], 5, replace = TRUE)
#' X4 = sample(LETTERS[1:3], 5, replace = TRUE)
#' dft = data.frame(X, X3, X4)
#'
#' makeX(df, dft)
#' makeX(df, dft, sparse = TRUE)
#'
#' ### Missing data in test as well
#' Xn = X
#' Xn[3, 1] = NA
#' Xn[5, 2] = NA
#' X3n = X3
#' X3n[1] = NA
#' X4n = X4
#' X4n[2] = NA
#' dftn = data.frame(Xn, X3n, X4n)
#'
#' makeX(dfn, dftn)
#' makeX(dfn, dftn, sparse = TRUE)
#' makeX(dfn, dftn, na.impute = TRUE)
#' makeX(dfn, dftn, sparse = TRUE, na.impute = TRUE)
#'
#' @export makeX
makeX=function(train,test=NULL, na.impute=FALSE,sparse=FALSE,...){
    df=train
    istest=!is.null(test)
    ntr=nrow(train)
    if(istest){
        nte=nrow(test)
        df=rbind(df,test)
    }
    ### bug fix because of change of default behavior of data.frame
    ## check if any character columns
    classes = sapply(df,class)
    if (any(classes == "character"))
        df <- as.data.frame(unclass(df), stringsAsFactors = TRUE)
    ###
    x=prepareX(df,sparse=sparse)
    if(na.impute){
        xbar=colMeans(x[seq(ntr),],na.rm=TRUE)
        x=na.replace(x,xbar)
    }
    if(istest){
        xt=x[seq(ntr+1,ntr+nte),]
        x=x[seq(ntr),]
        }
        if(na.impute)attr(x,"means")=xbar
    if(istest)list(x=x,xtest=xt) else x
}

#' @importFrom stats contrasts model.matrix na.pass
#' @importFrom Matrix sparse.model.matrix
#' @export prepareX
prepareX <-
    function(df,sparse=FALSE,...){
        if(!inherits(df,"data.frame"))stop("first argument must be of class `data.frame`")
        whichfac=sapply(df,inherits,"factor")
### bug fix if factor has one level only
        ## Cannot get contrasts to do the job here, so do it manually
        df_level <- lapply(df[,whichfac,drop = FALSE], levels)
        nlevels <- sapply(df_level,length)
        which <- nlevels == 1
        if(any(which)){
            whichfac1 = whichfac
            whichfac1[whichfac] = which
            dn <- names(df)
            cx <- as.matrix(df[,whichfac1],drop=FALSE)
            unimat <- array(1,dim=dim(cx))
            cnames <- paste0(dn[whichfac1],unlist(df_level[which]))
            unimat[is.na(cx)] <- NA
            df[,whichfac1] <- data.frame(unimat)
            dn[whichfac1] <- cnames
            names(df) <- dn
            whichfac[whichfac] = !which
            warning(call. = FALSE,paste("Column(s) ",paste(cnames,collapse=", "), "are all 1, due to factors with a single level"))
        }
###
        oldna=options()$na.action
        cna=as.character(substitute(na.action))
        options(na.action=na.pass)
        on.exit(options(na.action=oldna))
        if(any(whichfac))
            ctr=lapply(df[,whichfac,drop=FALSE], contrasts,contrast=FALSE)
        else ctr=NULL
        if(sparse){
            m=sparse.model.matrix(~.-1,data=df,contrasts.arg=ctr,...)
            m=na_sparse_fix(m,names(df)) # sparse.model.matrix is faulty
            }
           else m = model.matrix(~.-1,data=df,contrasts.arg=ctr,...)
        if(any(whichfac))attr(m,"contrasts")=NULL
        attr(m,"assign")=NULL
        m
    }

#' @export na_sparse_fix
na_sparse_fix=function(x,dfnames){
    a=attributes(x)
    ac=a$contrasts
    as=a$assign
    if(is.null(ac))return(x)
    acn=names(ac)
    whichn=match(acn,dfnames)
    for(i in whichn){
        xi=x[,as==i]
        rowtot=rowSums(xi)
        if(sum(rowtot)<length(rowtot)){# Nas present
            x[rowtot==0,as==i]=NA
        }
    }
    x
}


#' Replace the missing entries in a matrix columnwise with the entries in a
#' supplied vector
#'
#' Missing entries in any given column of the matrix are replaced by the column
#' means or the values in a supplied vector.
#'
#' This is a simple imputation scheme. This function is called by \code{makeX}
#' if the \code{na.impute=TRUE} option is used, but of course can be used on
#' its own. If 'x' is sparse, the result is sparse, and the replacements are
#' done so as to maintain sparsity.
#'
#' @param x A matrix with potentially missing values, and also potentially in
#' sparse matrix format (i.e. inherits from "sparseMatrix")
#' @param m Optional argument. A vector of values used to replace the missing
#' entries, columnwise. If missing, the column means of 'x' are used
#' @return A version of 'x' is returned with the missing values replaced.
#' @author Trevor Hastie\cr Maintainer: Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{makeX} and \code{glmnet}
#' @keywords models
#' @examples
#'
#' set.seed(101)
#' ### Single data frame
#' X = matrix(rnorm(20), 10, 2)
#' X[3, 1] = NA
#' X[5, 2] = NA
#' X3 = sample(letters[1:3], 10, replace = TRUE)
#' X3[6] = NA
#' X4 = sample(LETTERS[1:3], 10, replace = TRUE)
#' X4[9] = NA
#' dfn = data.frame(X, X3, X4)
#'
#' x = makeX(dfn)
#' m = rowSums(x, na.rm = TRUE)
#' na.replace(x, m)
#'
#' x = makeX(dfn, sparse = TRUE)
#' na.replace(x, m)
#'
#' @export na.replace
na.replace=function(x,m=rowSums(x,na.rm=TRUE)){
    if(inherits(x,"sparseMatrix")){
        x=as(x,"CsparseMatrix")
        ccount=diff(x@p)
        cindex=rep(1:length(ccount),ccount)
        nas=is.na(x@x)
        if(any(nas))x@x[nas]=m[cindex[nas]]
        }
    else{
            d=dim(x)
            cols=rep(1:d[2],rep(d[1],d[2]))
            nas=is.na(x)
            if(any(nas))x[nas]=m[cols[nas]]
            }
    x
}

coefnorm=function(coeflist,q=1){
  ###coeflist comes from mrelnet or multnet
  nmat=coeflist[[1]]*0
  nclass=length(coeflist)
  for(i in seq(nclass)){
    nmat=nmat+abs(coeflist[[i]])^q
  }
  nmat^(1/q)
}

elnet=function(x,is.sparse,y,weights,offset,type.gaussian=c("covariance","naive"),alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,pb){
  maxit=as.integer(maxit)
  weights=as.double(weights)
  type.gaussian=match.arg(type.gaussian)

  ka=as.integer(switch(type.gaussian,
    covariance=1,
    naive=2,
    ))


 storage.mode(y)="double"
   if(is.null(offset)){
    is.offset=FALSE}
  else{
    storage.mode(offset)="double"
    is.offset=TRUE
    y=y-offset
  }
### compute the null deviance
  ybar=if(intr)weighted.mean(y,weights)else 0
  nulldev=sum(weights* (y-ybar)^2)
if(nulldev==0)stop("y is constant; gaussian glmnet fails at standardization step")

fit=if(is.sparse) spelnet_exp(
        ka,parm=alpha,x,y,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,pb,
        lmu=integer(1),
        a0=double(nlam),
        ca=matrix(0.0, nrow=nx, ncol=nlam),
        ia=integer(nx),
        nin=integer(nlam),
        rsq=double(nlam),
        alm=double(nlam),
        nlp=integer(1),
        jerr=integer(1)
        )
else elnet_exp(
        ka,parm=alpha,x,y,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,pb,
        lmu=integer(1),
        a0=double(nlam),
        ca=matrix(0.0, nrow=nx, ncol=nlam),
        ia=integer(nx),
        nin=integer(nlam),
        rsq=double(nlam),
        alm=double(nlam),
        nlp=integer(1),
        jerr=integer(1)
          )
if(fit$jerr!=0){
  errmsg=jerr(fit$jerr,maxit,pmax=nx,family="gaussian")
  if(errmsg$fatal)stop(errmsg$msg,call.=FALSE)
  else warning(errmsg$msg,call.=FALSE)
}
  outlist=getcoef(fit,nvars,nx,vnames)
  dev=fit$rsq[seq(fit$lmu)]
  outlist=c(outlist,list(dev.ratio=dev,nulldev=nulldev,npasses=fit$nlp,jerr=fit$jerr,offset=is.offset))
  class(outlist)="elnet"
  outlist
}

#' Fit a GLM with elastic net regularization for a path of lambda values
#'
#' Fit a generalized linear model via penalized maximum likelihood for a path of
#' lambda values. Can deal with any GLM family.
#'
#' \code{glmnet.path} solves the elastic net problem for a path of lambda values.
#' It generalizes \code{glmnet::glmnet} in that it works for any GLM family.
#'
#' Sometimes the sequence is truncated before \code{nlambda} values of lambda
#' have been used. This happens when \code{glmnet.path} detects that the decrease
#' in deviance is marginal (i.e. we are near a saturated fit).
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. Can be a sparse matrix.
#' @param y Quantitative response variable.
#' @param weights Observation weights. Default is 1 for each observation.
#' @param lambda A user supplied lambda sequence. Typical usage is to have the
#' program compute its own lambda sequence based on \code{nlambda} and
#' \code{lambda.min.ratio}. Supplying a value of lambda overrides this.
#' @param nlambda The number of lambda values, default is 100.
#' @param lambda.min.ratio Smallest value for lambda as a fraction of lambda.max,
#' the (data derived) entry value (i.e. the smallest value for which all
#' coefficients are zero). The default depends on the sample size \code{nobs}
#' relative to the number of variables \code{nvars}. If \code{nobs >= nvars}, the
#' default is 0.0001, close to zero. If \code{nobs < nvars}, the default is 0.01.
#' A very small value of \code{lambda.min.ratio} will lead to a saturated fit
#' in the \code{nobs < nvars} case. This is undefined for some families of
#' models, and the function will exit gracefully when the percentage deviance
#' explained is almost 1.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
#' \code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.
#' @param offset A vector of length \code{nobs} that is included in the linear
#' predictor. Useful for the "poisson" family (e.g. log of exposure time), or
#' for refining a model by starting at a current fit. Default is NULL. If
#' supplied, then values must also be supplied to the \code{predict} function.
#' @param family A description of the error distribution and link function to be
#' used in the model. This is the result of a call to a family function. Default
#' is \code{gaussian()}. (See \code{\link[stats:family]{family}} for details on
#' family functions.)
#' @param standardize Logical flag for x variable standardization, prior to
#' fitting the model sequence. The coefficients are always returned on the
#' original scale. Default is \code{standardize=TRUE}. If variables are in the
#' same units already, you might not wish to standardize.
#' @param intercept Should intercept be fitted (default=TRUE) or set to zero (FALSE)?
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than thresh times the null deviance.
#' Default value is \code{1e-10}.
#' @param maxit Maximum number of passes over the data; default is \code{10^5}.
#' @param penalty.factor Separate penalty factors can be applied to each
#' coefficient. This is a number that multiplies \code{lambda} to allow differential
#' shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
#' variable is always included in the model. Default is 1 for all variables (and
#' implicitly infinity for variables listed in exclude). Note: the penalty
#' factors are internally rescaled to sum to \code{nvars}.
#' @param exclude Indices of variables to be excluded from the model. Default is
#' none. Equivalent to an infinite penalty factor.
#' @param lower.limits Vector of lower limits for each coefficient; default
#' \code{-Inf}. Each of these must be non-positive. Can be presented as a single
#' value (which will then be replicated), else a vector of length \code{nvars}.
#' @param upper.limits Vector of upper limits for each coefficient; default
#' \code{Inf}. See \code{lower.limits}.
#' @param trace.it Controls how much information is printed to screen. Default is
#' \code{trace.it=0} (no information printed). If \code{trace.it=1}, a progress
#' bar is displayed. If \code{trace.it=2}, some information about the fitting
#' procedure is printed to the console as the model is being fitted.
#'
#' @return An object with class "glmnetfit" and "glmnet".
#' \item{a0}{Intercept sequence of length \code{length(lambda)}.}
#' \item{beta}{A \code{nvars x length(lambda)} matrix of coefficients, stored in
#' sparse matrix format.}
#' \item{df}{The number of nonzero coefficients for each value of lambda.}
#' \item{dim}{Dimension of coefficient matrix.}
#' \item{lambda}{The actual sequence of lambda values used. When alpha=0, the
#' largest lambda reported does not quite give the zero coefficients reported
#' (lambda=inf would in principle). Instead, the largest lambda for alpha=0.001
#' is used, and the sequence of lambda values is derived from this.}
#' \item{dev.ratio}{The fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
#' for the saturated model (a model with a free parameter per observation).
#' Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
#' \item{npasses}{Total passes over the data summed over all lambda values.}
#' \item{jerr}{Error flag, for warnings and errors (largely for internal
#' debugging).}
#' \item{offset}{A logical variable indicating whether an offset was included
#' in the model.}
#' \item{call}{The call that produced this object.}
#' \item{family}{Family used for the model.}
#' \item{nobs}{Number of observations.}
#'
#' @examples
#' set.seed(1)
#' x <- matrix(rnorm(100 * 20), nrow = 100)
#' y <- ifelse(rnorm(100) > 0, 1, 0)
#'
#' # binomial with probit link
#' fit1 <- glmnet:::glmnet.path(x, y, family = binomial(link = "probit"))
glmnet.path <- function(x, y, weights=NULL, lambda = NULL, nlambda = 100,
                        lambda.min.ratio = ifelse(nobs<nvars, 0.01, 0.0001),
                        alpha = 1.0, offset = NULL, family = gaussian(),
                        standardize = TRUE, intercept = TRUE, thresh = 1e-10, maxit = 100000,
                        penalty.factor = rep(1.0, nvars), exclude = integer(0), lower.limits = -Inf,
                        upper.limits = Inf, trace.it = 0) {

    ### Check on family argument
    if(is.function(family))family=family()
    if(!inherits(family,"family"))
        stop("Invalid family argument; must be either character, function or family object")

    ### Prepare all the generic arguments
    if (alpha > 1) {
        warning("alpha > 1; set to 1")
        alpha = 1
    } else if (alpha < 0) {
        warning("alpha < 0; set to 0")
        alpha = 0
    }
    alpha = as.double(alpha)

    this.call <- match.call()

    np = dim(x)
    if(is.null(np) || (np[2] <= 1)) stop("x should be a matrix with 2 or more columns")
    nobs = as.integer(np[1]); nvars = as.integer(np[2])

    # get feature variable names
    vnames=colnames(x)
    if(is.null(vnames))vnames=paste("V",seq(nvars),sep="")

    # check weights
    if(is.null(weights)) weights = rep(1,nobs)
    else if (length(weights) != nobs)
        stop(paste("Number of elements in weights (",length(weights),
                   ") not equal to the number of rows of x (",nobs,")",sep=""))
    weights <- as.double(weights)

    ## initialize from family function. Makes y a vector in case of binomial, and possibly changes weights
    ## Expects nobs to be defined, and creates n and mustart (neither used here)
    ## Some cases expect to see things, so we set it up just to make it work
    etastart=0;mustart=NULL;start=NULL
    eval(family$initialize)
    ##
    ## Just in case this was not done in initialize()
    y <- drop(y)  # we don't like matrix responses

    is.offset <- !(is.null(offset))
    if (is.offset == FALSE) {
        offset <- as.double(y * 0) #keeps the shape of y
    }

    # check and standardize penalty factors (to sum to nvars)
    if(any(penalty.factor == Inf)) {
        exclude = c(exclude, seq(nvars)[penalty.factor == Inf])
        exclude = sort(unique(exclude))
    }
    if(length(exclude) > 0) {
        jd = match(exclude, seq(nvars), 0)
        if(!all(jd > 0)) stop ("Some excluded variables out of range")
        penalty.factor[jd] = 1 # ow can change lambda sequence
    }
    vp = pmax(0, penalty.factor)
    if (max(vp) <= 0) stop("All penalty factors are <= 0")
    vp = as.double(vp * nvars / sum(vp))

    # if all the non-excluded variables have zero variance, throw error
    isconst <- function(x) 1 - (max(x) == min(x)) * 1
    if (inherits(x, "sparseMatrix")) {
        xt <- as(t(x), "dgCMatrix")
        lx <- split(xt@x, xt@i)
        const_vars <- sapply(lx, isconst)
    }
    else const_vars <- apply(x, 2, isconst)
    nzvar <- setdiff(which(const_vars == 1), exclude)
    if (length(nzvar) == 0) stop("All used predictors have zero variance")

    ### check on limits
    control <- glmnet.control()
    if (thresh >= control$epsnr)
        warning("thresh should be smaller than glmnet.control()$epsnr",
                call. = FALSE)

    if(any(lower.limits > 0)){ stop("Lower limits should be non-positive") }
    if(any(upper.limits < 0)){ stop("Upper limits should be non-negative") }
    lower.limits[lower.limits == -Inf] = -control$big
    upper.limits[upper.limits == Inf] = control$big
    if (length(lower.limits) < nvars) {
        if(length(lower.limits) == 1) lower.limits = rep(lower.limits, nvars) else
            stop("Require length 1 or nvars lower.limits")
    } else lower.limits = lower.limits[seq(nvars)]
    if (length(upper.limits) < nvars) {
        if(length(upper.limits) == 1) upper.limits = rep(upper.limits, nvars) else
            stop("Require length 1 or nvars upper.limits")
    } else upper.limits = upper.limits[seq(nvars)]

    if (any(lower.limits == 0) || any(upper.limits == 0)) {
        ###Bounds of zero can mess with the lambda sequence and fdev;
        ###ie nothing happens and if fdev is not zero, the path can stop
        fdev <- glmnet.control()$fdev
        if(fdev!= 0) {
            glmnet.control(fdev = 0)
            on.exit(glmnet.control(fdev = fdev))
        }
    }
    ### end check on limits
    ### end preparation of generic arguments

    # standardize x if necessary
    if (intercept) {
        xm <- apply(x, 2, function(r) weighted.mean(r, weights))
    } else {
        xm <- rep(0.0, times = nvars)
    }
    if (standardize) {
        xs <- apply(x, 2, function(r) sqrt(weighted.mean(r^2, weights) -
                                               weighted.mean(r, weights)^2))
    } else {
        xs <- rep(1.0, times = nvars)
    }
    if (!inherits(x, "sparseMatrix")) {
        x <- t((t(x) - xm) / xs)
    } else {
        attr(x, "xm") <- xm
        attr(x, "xs") <- xs
    }
    lower.limits <- lower.limits * xs
    upper.limits <- upper.limits * xs

    # get null deviance and lambda max
    start_val <- get_start(x, y, weights, family, intercept, is.offset,
                           offset, exclude, vp, alpha)

    # work out lambda values
    nlam = as.integer(nlambda)
    user_lambda = FALSE   # did user provide their own lambda values?
    if (is.null(lambda)) {
        if (lambda.min.ratio >= 1) stop("lambda.min.ratio should be less than 1")

        # compute lambda max: to add code here
        lambda_max <- start_val$lambda_max

        # compute lambda sequence
        ulam <- exp(seq(log(lambda_max), log(lambda_max * lambda.min.ratio),
                        length.out = nlam))
    } else {  # user provided lambda values
        user_lambda = TRUE
        if (any(lambda < 0)) stop("lambdas should be non-negative")
        ulam = as.double(rev(sort(lambda)))
        nlam = as.integer(length(lambda))
    }

    # start progress bar
    if (trace.it == 1) pb <- utils::txtProgressBar(min = 0, max = nlam, style = 3)

    a0 <- rep(NA, length = nlam)
    beta <- matrix(0, nrow = nvars, ncol = nlam)
    dev.ratio <- rep(NA, length = nlam)
    fit <- NULL
    mnl <- min(nlam, control$mnlam)
    for (k in 1:nlam) {
        # get the correct lambda value to fit
        if (k > 1) {
            cur_lambda <- ulam[k]
        } else {
            cur_lambda <- ifelse(user_lambda, ulam[k], control$big)
        }

        if (trace.it == 2) cat("Fitting lambda index", k, ":", ulam[k], fill = TRUE)
        fit <- glmnet.fit(x, y, weights / sum(weights), cur_lambda, alpha = alpha, offset = offset,
                          family = family, intercept = intercept, thresh = thresh,
                          maxit = maxit, penalty.factor = vp, exclude = exclude,
                          lower.limits = lower.limits, upper.limits = upper.limits,
                          warm = fit, from.glmnet.path = TRUE, save.fit = TRUE,
                          trace.it = trace.it)
        if (trace.it == 1) utils::setTxtProgressBar(pb, k)
        # if error code non-zero, a non-fatal error must have occurred
        # print warning, ignore this lambda value and return result
        # for all previous lambda values
        if (fit$jerr != 0) {
            errmsg <- jerr.glmnetfit(fit$jerr, maxit, k)
            warning(errmsg$msg, call. = FALSE)
            k <- k - 1
            break
        }

        a0[k] <- fit$a0
        beta[, k] <- as.matrix(fit$beta)
        dev.ratio[k] <- fit$dev.ratio

        # early stopping if dev.ratio almost 1 or no improvement
        if (k >= mnl && user_lambda == FALSE) {
            if (dev.ratio[k] > control$devmax) break
            else if (k > 1) {
                if (family$family == "gaussian") {
                    if (dev.ratio[k] - dev.ratio[k-1] < control$fdev * dev.ratio[k])
                        break
                } else if (family$family == "poisson") {
                    if (dev.ratio[k] - dev.ratio[k - mnl + 1] <
                        10 * control$fdev * dev.ratio[k])
                        break
                } else if (dev.ratio[k] - dev.ratio[k-1] < control$fdev) break
            }
        }
    }
    if (trace.it == 1) {
        utils::setTxtProgressBar(pb, nlam)
        cat("", fill = TRUE)
    }

    # truncate a0, beta, dev.ratio, lambda if necessary
    if (k < nlam) {
        a0 <- a0[1:k]
        beta <- beta[, 1:k, drop = FALSE]
        dev.ratio <- dev.ratio[1:k]
        ulam <- ulam[1:k]
    }

    # return coefficients to original scale (because of x standardization)
    beta <- beta / xs
    a0 <- a0 - colSums(beta * xm)

    # output
    stepnames <- paste0("s", 0:(length(ulam) - 1))
    out <- list()
    out$a0 <- a0
    names(out$a0) <- stepnames
    out$beta <- Matrix::Matrix(beta, sparse = TRUE,
                               dimnames = list(vnames, stepnames))
    out$df <- colSums(abs(beta) > 0)
    out$dim <- dim(beta)
    out$lambda <- ulam
    out$dev.ratio <- dev.ratio
    out$nulldev <- start_val$nulldev
    out$npasses <- fit$npasses
    out$jerr <- fit$jerr
    out$offset <- is.offset
    out$call <- this.call
    out$family <- family
    out$nobs <- nobs
    class(out) <- c("glmnetfit", "glmnet")

    return(out)
}
#' Fit a GLM with elastic net regularization for a single value of lambda
#'
#' Fit a generalized linear model via penalized maximum likelihood for a single
#' value of lambda. Can deal with any GLM family.
#'
#' WARNING: Users should not call \code{glmnet.fit} directly. Higher-level functions
#' in this package call \code{glmnet.fit} as a subroutine. If a warm start object
#' is provided, some of the other arguments in the function may be overriden.
#'
#' \code{glmnet.fit} solves the elastic net problem for a single, user-specified
#' value of lambda. \code{glmnet.fit} works for any GLM family. It solves the
#' problem using iteratively reweighted least squares (IRLS). For each IRLS
#' iteration, \code{glmnet.fit} makes a quadratic (Newton) approximation of the
#' log-likelihood, then calls \code{elnet.fit} to minimize the resulting
#' approximation.
#'
#' In terms of standardization: \code{glmnet.fit} does not standardize \code{x}
#' and \code{weights}. \code{penalty.factor} is standardized so that they sum up
#' to \code{nvars}.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed that any standardization needed
#' has already been done.
#' @param y Quantitative response variable.
#' @param weights Observation weights. \code{glmnet.fit} does NOT standardize
#' these weights.
#' @param lambda A single value for the \code{lambda} hyperparameter.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
#' \code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.
#' @param offset A vector of length \code{nobs} that is included in the linear
#' predictor. Useful for the "poisson" family (e.g. log of exposure time), or
#' for refining a model by starting at a current fit. Default is NULL. If
#' supplied, then values must also be supplied to the \code{predict} function.
#' @param family A description of the error distribution and link function to be
#' used in the model. This is the result of a call to a family function. Default
#' is \code{gaussian()}. (See \code{\link[stats:family]{family}} for details on
#' family functions.)
#' @param intercept Should intercept be fitted (default=TRUE) or set to zero (FALSE)?
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than thresh times the null deviance.
#' Default value is \code{1e-10}.
#' @param maxit Maximum number of passes over the data; default is \code{10^5}.
#' (If a warm start object is provided, the number of passes the warm start object
#' performed is included.)
#' @param penalty.factor Separate penalty factors can be applied to each
#' coefficient. This is a number that multiplies \code{lambda} to allow differential
#' shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
#' variable is always included in the model. Default is 1 for all variables (and
#' implicitly infinity for variables listed in exclude). Note: the penalty
#' factors are internally rescaled to sum to \code{nvars}.
#' @param exclude Indices of variables to be excluded from the model. Default is
#' none. Equivalent to an infinite penalty factor.
#' @param lower.limits Vector of lower limits for each coefficient; default
#' \code{-Inf}. Each of these must be non-positive. Can be presented as a single
#' value (which will then be replicated), else a vector of length \code{nvars}.
#' @param upper.limits Vector of upper limits for each coefficient; default
#' \code{Inf}. See \code{lower.limits}.
#' @param warm Either a \code{glmnetfit} object or a list (with names \code{beta}
#' and \code{a0} containing coefficients and intercept respectively) which can
#' be used as a warm start. Default is \code{NULL}, indicating no warm start.
#' For internal use only.
#' @param from.glmnet.path Was \code{glmnet.fit()} called from \code{glmnet.path()}?
#' Default is FALSE.This has implications for computation of the penalty factors.
#' @param save.fit Return the warm start object? Default is FALSE.
#' @param trace.it Controls how much information is printed to screen. If
#' \code{trace.it=2}, some information about the fitting procedure is printed to
#' the console as the model is being fitted. Default is \code{trace.it=0}
#' (no information printed). (\code{trace.it=1} not used for compatibility with
#' \code{glmnet.path}.)
#'
#' @return An object with class "glmnetfit" and "glmnet". The list
#' returned contains more keys than that of a "glmnet" object.
#' \item{a0}{Intercept value.}
#' \item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
#' format.}
#' \item{df}{The number of nonzero coefficients.}
#' \item{dim}{Dimension of coefficient matrix.}
#' \item{lambda}{Lambda value used.}
#' \item{dev.ratio}{The fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
#' for the saturated model (a model with a free parameter per observation).
#' Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
#' \item{npasses}{Total passes over the data.}
#' \item{jerr}{Error flag, for warnings and errors (largely for internal
#' debugging).}
#' \item{offset}{A logical variable indicating whether an offset was included
#' in the model.}
#' \item{call}{The call that produced this object.}
#' \item{nobs}{Number of observations.}
#' \item{warm_fit}{If \code{save.fit=TRUE}, output of FORTRAN routine, used for
#' warm starts. For internal use only.}
#' \item{family}{Family used for the model.}
#' \item{converged}{A logical variable: was the algorithm judged to have
#' converged?}
#' \item{boundary}{A logical variable: is the fitted value on the boundary of
#' the attainable values?}
#' \item{obj_function}{Objective function value at the solution.}
#'
glmnet.fit <- function(x, y, weights, lambda, alpha = 1.0,
                       offset = rep(0, nobs), family = gaussian(),
                       intercept = TRUE, thresh = 1e-10, maxit = 100000,
                       penalty.factor = rep(1.0, nvars), exclude = c(), lower.limits = -Inf,
                       upper.limits = Inf, warm = NULL, from.glmnet.path = FALSE,
                       save.fit = FALSE, trace.it = 0) {
    this.call <- match.call()
    control <- glmnet.control()

    ### Prepare all the generic arguments
    nobs <- nrow(x)
    nvars <- ncol(x)
    is.offset <- !(missing(offset))
    if (is.offset == FALSE) {
        offset <- as.double(y * 0) #keeps the shape of y
    } else if (is.null(offset)) {
        offset <- rep(0, nobs)
        is.offset = FALSE
    }
    # add xm and xs attributes if they are missing for sparse x
    # glmnet.fit assumes that x is already standardized. Any standardization 
    # the user wants should be done beforehand.
    if (inherits(x, "sparseMatrix")) {
        if ("xm" %in% names(attributes(x)) == FALSE) 
            attr(x, "xm") <- rep(0.0, times = nvars)
        if ("xs" %in% names(attributes(x)) == FALSE) 
            attr(x, "xs") <- rep(0.0, times = nvars)
    }

    # if calling from glmnet.path(), we do not need to check on exclude
    # and penalty.factor arguments as they have been prepared by glmnet.path()
    if (!from.glmnet.path) {
        # check and standardize penalty factors (to sum to nvars)
        if(any(penalty.factor == Inf)) {
            exclude = c(exclude, seq(nvars)[penalty.factor == Inf])
            exclude = sort(unique(exclude))
        }
        if(length(exclude) > 0) {
            jd = match(exclude, seq(nvars), 0)
            if(!all(jd > 0)) stop ("Some excluded variables out of range")
            penalty.factor[jd] = 1 # ow can change lambda sequence
        }
        vp = pmax(0, penalty.factor)
        vp = as.double(vp * nvars / sum(vp))
    } else {
        vp <- as.double(penalty.factor)
    }

    ### check on limits
    lower.limits[lower.limits == -Inf] = -control$big
    upper.limits[upper.limits == Inf] = control$big
    if (length(lower.limits) < nvars)
        lower.limits = rep(lower.limits, nvars) else
            lower.limits = lower.limits[seq(nvars)]
    if (length(upper.limits) < nvars)
        upper.limits = rep(upper.limits, nvars) else
            upper.limits = upper.limits[seq(nvars)]
    ### end check on limits
    ### end preparation of generic arguments

    # get the relevant family functions
    variance <- family$variance
    linkinv <- family$linkinv
    if (!is.function(variance) || !is.function(linkinv))
        stop("'family' argument seems not to be a valid family object",
             call. = FALSE)
    mu.eta <- family$mu.eta
    unless.null <- function(x, if.null) if (is.null(x))
        if.null
    else x
    valideta <- unless.null(family$valideta, function(eta) TRUE)
    validmu <- unless.null(family$validmu, function(mu) TRUE)



    # computation of null deviance (get mu in the process)
    if (is.null(warm)) {
        start_val <- get_start(x, y, weights, family, intercept, is.offset,
                               offset, exclude, vp, alpha)
        nulldev <- start_val$nulldev
        mu <- start_val$mu
        fit <- NULL
        coefold <- rep(0, nvars)   # initial coefs = 0
        eta <- family$linkfun(mu)
        intold <- (eta - offset)[1]
    } else {
        if ("glmnetfit" %in% class(warm)) {
            if (class(warm$warm_fit) != "warmfit") stop("Invalid warm start object")
            fit <- warm
            nulldev <- fit$nulldev
            coefold <- fit$warm_fit$a   # prev value for coefficients
            intold <- fit$warm_fit$aint    # prev value for intercept
            eta <- get_eta(x, coefold, intold)
            mu <- linkinv(eta <- eta + offset)
        } else if ("list" %in% class(warm) && "a0" %in% names(warm) &&
                   "beta" %in% names(warm)) {
            nulldev <- get_start(x, y, weights, family, intercept, is.offset,
                                   offset, exclude, vp, alpha)$nulldev
            fit <- warm
            coefold <- fit$beta   # prev value for coefficients
            intold <- fit$a0    # prev value for intercept
            eta <- get_eta(x, coefold, intold)
            mu <- linkinv(eta <- eta + offset)
        } else {
            stop("Invalid warm start object")
        }
    }

    if (!(validmu(mu) && valideta(eta)))
        stop("cannot find valid starting values: please specify some",
             call. = FALSE)

    start <- NULL     # current value for coefficients
    start_int <- NULL # current value for intercept
    obj_val_old <- obj_function(y, mu, weights, family, lambda, alpha, coefold, vp)
    if (trace.it == 2) {
        cat("Warm Start Objective:", obj_val_old, fill = TRUE)
    }
    conv <- FALSE      # converged?

    # IRLS loop
    for (iter in 1L:control$mxitnr) {
        # some checks for NAs/zeros
        varmu <- variance(mu)
        if (anyNA(varmu)) stop("NAs in V(mu)")
        if (any(varmu == 0)) stop("0s in V(mu)")
        mu.eta.val <- mu.eta(eta)
        if (any(is.na(mu.eta.val))) stop("NAs in d(mu)/d(eta)")

        # compute working response and weights
        z <- (eta - offset) + (y - mu)/mu.eta.val
        w <- (weights * mu.eta.val^2)/variance(mu)

        # have to update the weighted residual in our fit object
        # (in theory g and iy should be updated too, but we actually recompute g
        # and iy anyway in wls.f)
        if (!is.null(fit)) {
            fit$warm_fit$r <- w * (z - eta + offset)
        }

        # do WLS with warmstart from previous iteration
        fit <- elnet.fit(x, z, w, lambda, alpha, intercept,
                         thresh = thresh, maxit = maxit, penalty.factor = vp,
                         exclude = exclude, lower.limits = lower.limits,
                         upper.limits = upper.limits, warm = fit,
                         from.glmnet.fit = TRUE, save.fit = TRUE)
        if (fit$jerr != 0) return(list(jerr = fit$jerr))

        # update coefficients, eta, mu and obj_val
        start <- fit$warm_fit$a
        start_int <- fit$warm_fit$aint
        eta <- get_eta(x, start, start_int)
        mu <- linkinv(eta <- eta + offset)
        obj_val <- obj_function(y, mu, weights, family, lambda, alpha, start, vp)
        if (trace.it == 2) cat("Iteration", iter, "Objective:", obj_val, fill = TRUE)

        boundary <- FALSE
        halved <- FALSE  # did we have to halve the step size?
        # if objective function is not finite, keep halving the stepsize until it is finite
        # for the halving step, we probably have to adjust fit$g as well?
        if (!is.finite(obj_val) || obj_val > control$big) {
            warning("Infinite objective function!", call. = FALSE)
            if (is.null(coefold) || is.null(intold))
                stop("no valid set of coefficients has been found: please supply starting values",
                     call. = FALSE)
            warning("step size truncated due to divergence", call. = FALSE)
            ii <- 1
            while (!is.finite(obj_val) || obj_val > control$big) {
                if (ii > control$mxitnr)
                    stop("inner loop 1; cannot correct step size", call. = FALSE)
                ii <- ii + 1
                start <- (start + coefold)/2
                start_int <- (start_int + intold)/2
                eta <- get_eta(x, start, start_int)
                mu <- linkinv(eta <- eta + offset)
                obj_val <- obj_function(y, mu, weights, family, lambda, alpha, start, vp)
                if (trace.it == 2) cat("Iteration", iter, " Halved step 1, Objective:",
                               obj_val, fill = TRUE)
            }
            boundary <- TRUE
            halved <- TRUE
        }
        # if some of the new eta or mu are invalid, keep halving stepsize until valid
        if (!(valideta(eta) && validmu(mu))) {
            warning("Invalid eta/mu!", call. = FALSE)
            if (is.null(coefold) || is.null(intold))
                stop("no valid set of coefficients has been found: please supply starting values",
                     call. = FALSE)
            warning("step size truncated: out of bounds", call. = FALSE)
            ii <- 1
            while (!(valideta(eta) && validmu(mu))) {
                if (ii > control$mxitnr)
                    stop("inner loop 2; cannot correct step size", call. = FALSE)
                ii <- ii + 1
                start <- (start + coefold)/2
                start_int <- (start_int + intold)/2
                eta <- get_eta(x, start, start_int)
                mu <- linkinv(eta <- eta + offset)
            }
            boundary <- TRUE
            halved <- TRUE
            obj_val <- obj_function(y, mu, weights, family, lambda, alpha, start, vp)
            if (trace.it == 2) cat("Iteration", iter, " Halved step 2, Objective:", obj_val, fill = TRUE)
        }

        # if we did any halving, we have to update the coefficients, intercept
        # and weighted residual in the warm_fit object
        if (halved) {
            fit$warm_fit$a <- start
            fit$warm_fit$aint <- start_int
            fit$warm_fit$r <- w * (z - eta)
        }

        # test for convergence
        if (abs(obj_val - obj_val_old)/(0.1 + abs(obj_val)) < control$epsnr) {
            conv <- TRUE
            break
        }
        else {
            coefold <- start
            intold <- start_int
            obj_val_old <- obj_val
        }
    }
    # end of IRLS loop

    # checks on convergence and fitted values
    if (!conv)
        warning("glmnet.fit: algorithm did not converge", call. = FALSE)
    if (boundary)
        warning("glmnet.fit: algorithm stopped at boundary value", call. = FALSE)

    # some extra warnings, printed only if trace.it == 2
    if (trace.it == 2) {
        eps <- 10 * .Machine$double.eps
        if ((family$family == "binomial") && (any(mu > 1 - eps) || any(mu < eps)))
                warning("glm.fit: fitted probabilities numerically 0 or 1 occurred",
                        call. = FALSE)
        if ((family$family == "poisson") && (any(mu < eps)))
                warning("glm.fit: fitted rates numerically 0 occurred",
                        call. = FALSE)
    }

    # prepare output object
    if (save.fit == FALSE) {
        fit$warm_fit <- NULL
    }
    # overwrite values from elnet.fit object
    fit$call <- this.call
    fit$offset <- is.offset
    fit$nulldev <- nulldev
    fit$dev.ratio <- 1 - dev_function(y, mu, weights, family) / nulldev

    # add new key-value pairs to list
    fit$family <- family
    fit$converged <- conv
    fit$boundary <- boundary
    fit$obj_function <- obj_val

    class(fit) <- c("glmnetfit", "glmnet")
    fit
}

#' Solve weighted least squares (WLS) problem for a single lambda value
#'
#' Solves the weighted least squares (WLS) problem for a single lambda value. Internal
#' function that users should not call directly.
#'
#' WARNING: Users should not call \code{elnet.fit} directly. Higher-level functions
#' in this package call \code{elnet.fit} as a subroutine. If a warm start object
#' is provided, some of the other arguments in the function may be overriden.
#'
#' \code{elnet.fit} is essentially a wrapper around a FORTRAN subroutine which
#' minimizes
#'
#' \deqn{1/2 \sum w_i (y_i - X_i^T \beta)^2 + \sum \lambda \gamma_j
#' [(1-\alpha)/2 \beta^2+\alpha|\beta|],}
#'
#' over \eqn{\beta}, where \eqn{\gamma_j} is the relative penalty factor on the
#' jth variable. If \code{intercept = TRUE}, then the term in the first sum is
#' \eqn{w_i (y_i - \beta_0 - X_i^T \beta)^2}, and we are minimizing over both
#' \eqn{\beta_0} and \eqn{\beta}.
#'
#' None of the inputs are standardized except for \code{penalty.factor}, which
#' is standardized so that they sum up to \code{nvars}.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed that any standardization needed
#' has already been done.
#' @param y Quantitative response variable.
#' @param weights Observation weights. \code{elnet.fit} does NOT standardize
#' these weights.
#' @param lambda A single value for the \code{lambda} hyperparameter.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
#' \code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.
#' @param intercept Should intercept be fitted (default=TRUE) or set to zero (FALSE)?
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than thresh times the null deviance.
#' Default value is \code{1e-7}.
#' @param maxit Maximum number of passes over the data; default is \code{10^5}.
#' (If a warm start object is provided, the number of passes the warm start object
#' performed is included.)
#' @param penalty.factor Separate penalty factors can be applied to each
#' coefficient. This is a number that multiplies \code{lambda} to allow differential
#' shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
#' variable is always included in the model. Default is 1 for all variables (and
#' implicitly infinity for variables listed in exclude). Note: the penalty
#' factors are internally rescaled to sum to \code{nvars}.
#' @param exclude Indices of variables to be excluded from the model. Default is
#' none. Equivalent to an infinite penalty factor.
#' @param lower.limits Vector of lower limits for each coefficient; default
#' \code{-Inf}. Each of these must be non-positive. Can be presented as a single
#' value (which will then be replicated), else a vector of length \code{nvars}.
#' @param upper.limits Vector of upper limits for each coefficient; default
#' \code{Inf}. See \code{lower.limits}.
#' @param warm Either a \code{glmnetfit} object or a list (with names \code{beta}
#' and \code{a0} containing coefficients and intercept respectively) which can
#' be used as a warm start. Default is \code{NULL}, indicating no warm start.
#' For internal use only.
#' @param from.glmnet.fit Was \code{elnet.fit()} called from \code{glmnet.fit()}?
#' Default is FALSE.This has implications for computation of the penalty factors.
#' @param save.fit Return the warm start object? Default is FALSE.
#'
#' @return An object with class "glmnetfit" and "glmnet". The list returned has
#' the same keys as that of a \code{glmnet} object, except that it might have an
#' additional \code{warm_fit} key.
#' \item{a0}{Intercept value.}
#' \item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
#' format.}
#' \item{df}{The number of nonzero coefficients.}
#' \item{dim}{Dimension of coefficient matrix.}
#' \item{lambda}{Lambda value used.}
#' \item{dev.ratio}{The fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
#' for the saturated model (a model with a free parameter per observation).
#' Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
#' \item{npasses}{Total passes over the data.}
#' \item{jerr}{Error flag, for warnings and errors (largely for internal
#' debugging).}
#' \item{offset}{Always FALSE, since offsets do not appear in the WLS problem.
#' Included for compability with glmnet output.}
#' \item{call}{The call that produced this object.}
#' \item{nobs}{Number of observations.}
#' \item{warm_fit}{If \code{save.fit=TRUE}, output of FORTRAN routine, used for
#' warm starts. For internal use only.}
#'
elnet.fit <- function(x, y, weights, lambda, alpha = 1.0, intercept = TRUE,
                      thresh = 1e-7, maxit = 100000,
                      penalty.factor = rep(1.0, nvars), exclude = c(),
                      lower.limits = -Inf, upper.limits = Inf, warm = NULL,
                      from.glmnet.fit = FALSE, save.fit = FALSE) {
    this.call <- match.call()
    internal.parms <- glmnet.control()

    # compute null deviance
    ybar <- weighted.mean(y, weights)
    nulldev <- sum(weights * (y - ybar)^2)

    # if class "glmnetfit" warmstart object provided, pull whatever we want out of it
    # else, prepare arguments, then check if coefs provided as warmstart
    # (if only coefs are given as warmstart, we prepare the other arguments
    # as if no warmstart was provided)
    if (!is.null(warm) && "glmnetfit" %in% class(warm)) {
        warm <- warm$warm_fit
        if (class(warm) != "warmfit") stop("Invalid warm start object")

        a <- warm$a
        aint <- warm$aint
        alm0 <- warm$almc
        cl <- warm$cl
        g <- warm$g
        ia <- warm$ia
        iy <- warm$iy
        iz <- warm$iz
        ju <- warm$ju
        m <- warm$m
        mm <- warm$mm
        nino <- warm$nino
        nobs <- warm$no
        nvars <- warm$ni
        nlp <- warm$nlp
        nx <- warm$nx
        r <- warm$r
        rsqc <- warm$rsqc
        vp <- warm$vp
    } else {
        nobs <- as.integer(nrow(x))
        nvars <- as.integer(ncol(x))

        # if calling from glmnet.fit(), we do not need to check on exclude
        # and penalty.factor arguments as they have been prepared by glmnet.fit()
        if (!from.glmnet.fit) {
            # check and standardize penalty factors (to sum to nvars)
            if(any(penalty.factor == Inf)) {
                exclude = c(exclude, seq(nvars)[penalty.factor == Inf])
                exclude = sort(unique(exclude))
            }
            if(length(exclude) > 0) {
                jd = match(exclude, seq(nvars), 0)
                if(!all(jd > 0)) stop ("Some excluded variables out of range")
                penalty.factor[jd] = 1 # ow can change lambda sequence
            }
            vp = pmax(0, penalty.factor)
            vp = as.double(vp * nvars / sum(vp))
        } else {
            vp <- as.double(penalty.factor)
        }
        # compute ju
        isconst <- function(x) 1 - (max(x) == min(x)) * 1
        if (inherits(x, "sparseMatrix")) {
            xt <- as(t(x), "dgCMatrix")
            lx <- split(xt@x, xt@i)
            ju <- sapply(lx, isconst)
        }
        else ju <- apply(x, 2, isconst)
        ju[exclude] <- 0
        ju <- as.integer(ju)

        # compute cl from lower.limits and upper.limits
        lower.limits[lower.limits == -Inf] <- -internal.parms$big
        upper.limits[upper.limits == Inf] <- internal.parms$big
        if (length(lower.limits) < nvars)
            lower.limits = rep(lower.limits, nvars) else
                lower.limits = lower.limits[seq(nvars)]
        if (length(upper.limits) < nvars)
            upper.limits = rep(upper.limits, nvars) else
                upper.limits = upper.limits[seq(nvars)]
        cl <- rbind(lower.limits, upper.limits)
        storage.mode(cl) = "double"

        nx <- as.integer(nvars)

        a <- double(nvars)
        aint <- double(1)
        alm0 <- double(1)
        g <- double(nvars)
        ia <- integer(nx)
        iy <- integer(nvars)
        iz <- integer(1)
        m <- as.integer(1)
        mm <- integer(nvars)
        nino <- integer(1)
        nlp <- integer(1)
        r <- weights * y
        rsqc <- double(1)

        # check if coefs were provided as warmstart: if so, use them
        if (!is.null(warm)) {
            if ("list" %in% class(warm) && "a0" %in% names(warm) &&
                "beta" %in% names(warm)) {
                a <- as.double(warm$beta)
                aint <- as.double(warm$a0)
                mu <- drop(x %*% a + aint)
                r <- weights * (y - mu)
                rsqc <- 1 - sum(weights * (y - mu)^2) / nulldev
            } else {
                stop("Invalid warm start object")
            }
        }
    }

    # for the parameters here, we are overriding the values provided by the
    # warmstart object
    alpha <- as.double(alpha)
    almc <- as.double(lambda)
    intr <- as.integer(intercept)
    jerr <- integer(1)
    maxit <- as.integer(maxit)
    thr <- as.double(thresh)
    v <- as.double(weights)

    # take out components of x and run FORTRAN subroutine
    if (inherits(x, "sparseMatrix")) {
        xm <- as.double(attr(x, "xm"))
        xs <- as.double(attr(x, "xs"))
        ix <- as.integer(x@p + 1)
        jx <- as.integer(x@i + 1)
        x <- as.double(x@x)
        wls_fit <- .Fortran("spwls",
                            alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
                            x=x,ix=ix,jx=jx,xm=xm,xs=xs,r=r,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
                            maxit=maxit,a=a,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
                            nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
    } else {
        x <- as.double(x)
        wls_fit <- .Fortran("wls",
                            alm0=alm0,almc=almc,alpha=alpha,m=m,no=nobs,ni=nvars,
                            x=x,r=r,v=v,intr=intr,ju=ju,vp=vp,cl=cl,nx=nx,thr=thr,
                            maxit=maxit,a=a,aint=aint,g=g,ia=ia,iy=iy,iz=iz,mm=mm,
                            nino=nino,rsqc=rsqc,nlp=nlp,jerr=jerr)
    }

    # if error code > 0, fatal error occurred: stop immediately
    # if error code < 0, non-fatal error occurred: return error code
    if (wls_fit$jerr > 0) {
        errmsg <- jerr.glmnetfit(wls_fit$jerr, maxit)
        stop(errmsg$msg, call. = FALSE)
    } else if (wls_fit$jerr < 0)
        return(list(jerr = wls_fit$jerr))
    warm_fit <- wls_fit[c("almc", "m", "no", "ni", "r", "ju", "vp", "cl", "nx",
                          "a", "aint", "g", "ia", "iy", "iz", "mm", "nino",
                          "rsqc", "nlp")]
    class(warm_fit) <- "warmfit"

    beta <- Matrix::Matrix(wls_fit$a, sparse = TRUE)

    out <- list(a0 = wls_fit$aint, beta = beta, df = sum(abs(beta) > 0),
                dim = dim(beta), lambda = lambda, dev.ratio = wls_fit$rsqc,
                nulldev = nulldev, npasses = wls_fit$nlp, jerr = wls_fit$jerr,
                offset = FALSE, call = this.call, nobs = nobs, warm_fit = warm_fit)
    if (save.fit == FALSE) {
        out$warm_fit <- NULL
    }
    class(out) <- c("glmnetfit", "glmnet")
    out
}

#' Get null deviance, starting mu and lambda max
#'
#' Return the null deviance, starting mu and lambda max values for
#' initialization. For internal use only.
#'
#' This function is called by \code{glmnet.path} for null deviance, starting mu
#' and lambda max values. It is also called by \code{glmnet.fit} when used
#' without warmstart, but they only use the null deviance and starting mu values.
#'
#' When \code{x} is not sparse, it is expected to already by centered and scaled.
#' When \code{x} is sparse, the function will get its attributes \code{xm} and
#' \code{xs} for its centering and scaling factors.
#'
#' Note that whether \code{x} is centered & scaled or not, the values of \code{mu}
#' and \code{nulldev} don't change. However, the value of \code{lambda_max} does
#' change, and we need \code{xm} and \code{xs} to get the correct value.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed to be standardized.
#' @param y Quantitative response variable.
#' @param weights Observation weights.
#' @param family A description of the error distribution and link function to be
#' used in the model. This is the result of a call to a family function.
#' (See \code{\link[stats:family]{family}} for details on family functions.)
#' @param intercept Does the model we are fitting have an intercept term or not?
#' @param is.offset Is the model being fit with an offset or not?
#' @param offset Offset for the model. If \code{is.offset=FALSE}, this should be
#' a zero vector of the same length as \code{y}.
#' @param exclude Indices of variables to be excluded from the model.
#' @param vp Separate penalty factors can be applied to each coefficient.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
get_start <- function(x, y, weights, family, intercept, is.offset, offset,
                      exclude, vp, alpha) {
    nobs <- nrow(x); nvars <- ncol(x)

    # compute mu and null deviance
    # family = binomial() gives us warnings due to non-integer weights
    # to avoid, suppress warnings
    if (intercept) {
        if (is.offset) {
            suppressWarnings(tempfit <- glm(y ~ 1, family = family,
                                            weights = weights, offset = offset))
            mu <- tempfit$fitted.values
        } else {
            mu <- rep(sum(weights * y) / sum(weights), times = nobs)
        }
    } else {
        mu <- family$linkinv(offset)
    }
    nulldev <- dev_function(y, mu, weights, family)

    # if some penalty factors are zero, we have to recompute mu
    vp_zero <- setdiff(which(vp == 0), exclude)
    if (length(vp_zero) > 0) {
        tempx <- x[, vp_zero, drop = FALSE]
        if (intercept) {
            tempx <- cbind(tempx, 1)
        }
        tempfit <- glm.fit(tempx, y, family = family, weights = weights, offset = offset)
        mu <- tempfit$fitted.values
    }

    # compute lambda max
    isconst <- function(x) 1 - (max(x) == min(x)) * 1
    if (inherits(x, "sparseMatrix")) {
        xt <- as(t(x), "dgCMatrix")
        lx <- split(xt@x, xt@i)
        ju <- sapply(lx, isconst)
    }
    else ju <- apply(x, 2, isconst)
    ju[exclude] <- 0
    r <- y - mu
    eta <- family$linkfun(mu)
    v <- family$variance(mu)
    m.e <- family$mu.eta(eta)
    weights <- weights / sum(weights)
    rv <- r / v * m.e * weights
    if (inherits(x, "sparseMatrix")) {
        xm <- attr(x, "xm")
        xs <- attr(x, "xs")
        g <- abs((drop(t(rv) %*% x) - sum(rv) * xm) / xs)
    } else {
        g <- abs(drop(t(rv) %*% x))
    }
    g <- g * ju / ifelse(vp > 0, vp, 1)
    lambda_max <- max(g) / max(alpha, 1e-3)

    list(nulldev = nulldev, mu = mu, lambda_max = lambda_max)
}
#' Elastic net objective function value
#'
#' Returns the elastic net objective function value.
#'
#' @param y Quantitative response variable.
#' @param mu Model's predictions for \code{y}.
#' @param weights Observation weights.
#' @param family A description of the error distribution and link function to be
#' used in the model. This is the result of a call to a family function.
#' @param lambda A single value for the \code{lambda} hyperparameter.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' @param coefficients The model's coefficients (excluding intercept).
#' @param vp Penalty factors for each of the coefficients.
obj_function <- function(y, mu, weights, family,
                         lambda, alpha, coefficients, vp) {
    dev_function(y, mu, weights, family) / 2 +
        lambda * pen_function(coefficients, alpha, vp)
}

#' Elastic net penalty value
#'
#' Returns the elastic net penalty value without the \code{lambda} factor.
#'
#' The penalty is defined as
#' \deqn{(1-\alpha)/2 \sum vp_j \beta_j^2 + \alpha \sum vp_j |\beta|.}
#' Note the omission of the multiplicative \code{lambda} factor.
#'
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' @param coefficients The model's coefficients (excluding intercept).
#' @param vp Penalty factors for each of the coefficients.
pen_function <- function(coefficients, alpha = 1.0, vp = 1.0) {
    sum(vp * (alpha * abs(coefficients) + (1-alpha)/2 * coefficients^2))
}

#' Elastic net deviance value
#'
#' Returns the elastic net deviance value.
#'
#' @param y Quantitative response variable.
#' @param mu Model's predictions for \code{y}.
#' @param weights Observation weights.
#' @param family A description of the error distribution and link function to be
#' used in the model. This is the result of a call to a family function.
dev_function <- function(y, mu, weights, family) {
    sum(family$dev.resids(y, mu, weights))
}


jerr.glmnetfit <- function (n, maxit, k = NULL) {
    if (n == 0) {
        list(n = 0, fatal = FALSE, msg = "")
    } else if (n > 0) {
        # fatal error
        fatal <- TRUE
        msg <- ifelse(n < 7777,
                      "Memory allocation error; contact package maintainer",
                      "Unknown error")
    } else {
        # non-fatal error
        fatal <- FALSE
        msg <- paste("Convergence for ", k, "th lambda value not reached after maxit=",
                     maxit, " iterations; solutions for larger lambdas returned",
                     sep = "")
    }
    list(n = n, fatal = fatal, msg = msg)
}
#' Get predictions from a \code{glmnetfit} fit object
#'
#' Gives fitted values, linear predictors, coefficients and number of non-zero
#' coefficients from a fitted \code{glmnetfit} object.
#'
#' @param object Fitted "glmnetfit" object.
#' @param newx Matrix of new values for \code{x} at which predictions are to be
#' made. Must be a matrix. This argument is not used for \code{type =
#' c("coefficients","nonzero")}.
#' @param s Value(s) of the penalty parameter lambda at which predictions are
#' required. Default is the entire sequence used to create the model.
#' @param type Type of prediction required. Type "link" gives the linear
#' predictors (eta scale); Type "response" gives the fitted values (mu scale).
#' Type "coefficients" computes the coefficients at the requested values for s.
#' Type "nonzero" returns a list of the indices of the nonzero coefficients for
#' each value of s.
#' @param exact This argument is relevant only when predictions are made at values
#' of \code{s} (lambda) \emph{different} from those used in the fitting of the
#' original model. If \code{exact=FALSE} (default), then the predict function
#' uses linear interpolation to make predictions for values of \code{s} (lambda)
#' that do not coincide with those used in the fitting algorithm. While this is
#' often a good approximation, it can sometimes be a bit coarse. With
#' \code{exact=TRUE}, these different values of \code{s} are merged (and sorted)
#' with \code{object$lambda}, and the model is refit before predictions are made.
#' In this case, it is required to supply the original data x= and y= as additional
#' named arguments to predict() or coef(). The workhorse \code{predict.glmnet()}
#' needs to update the model, and so needs the data used to create it. The same
#' is true of weights, offset, penalty.factor, lower.limits, upper.limits if
#' these were used in the original call. Failure to do so will result in an error.
#' @param newoffset If an offset is used in the fit, then one must be supplied for
#' making predictions (except for type="coefficients" or type="nonzero").
#' @param ... This is the mechanism for passing arguments like \code{x=} when
#' \code{exact=TRUE}; see \code{exact} argument.
#'
#' @return The object returned depends on type.
#'
#' @method predict glmnetfit
#' @export
predict.glmnetfit <- function(object, newx, s = NULL,
                              type = c("link", "response", "coefficients", "nonzero"),
                              exact = FALSE, newoffset, ...) {
    type = match.arg(type)
    nfit <- NextMethod("predict")
    if (type == "response") {
        object$family$linkinv(nfit)
    } else {
        nfit
    }
}

#' Helper function to get etas (linear predictions)
#'
#' Given x, coefficients and intercept, return linear predictions. Wrapper that
#' works with both regular and sparse x. Only works for single set of coefficients
#' and intercept.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed to be standardized.
#' @param beta Feature coefficients.
#' @param a0 Intercept.
get_eta <- function(x, beta, a0) {
    if (inherits(x, "sparseMatrix")) {
        x@x <- x@x / rep.int(attr(x, "xs"), times = diff(x@p))
        drop(x %*% beta - sum(beta * attr(x, "xm") / attr(x, "xs")) + a0)
    } else {
        drop(x %*% beta + a0)
    }
}

#' @method plot mrelnet
#' @param type.coef If \code{type.coef="2norm"} then a single curve per
#' variable, else if \code{type.coef="coef"}, a coefficient plot per response
#' @rdname plot.glmnet
#' @export
plot.mrelnet <-
  function(x, xvar=c("norm","lambda","dev"),label=FALSE,type.coef=c("coef","2norm"),...){
    type.coef=match.arg(type.coef)
    xvar=match.arg(xvar)
    beta=x$beta
    if(xvar=="norm") {
      cnorm1=function(beta){
        which=nonzeroCoef(beta)
        beta=as.matrix(beta[which,])
        apply(abs(beta),2,sum)
      }
      norm=apply(sapply(x$beta,cnorm1),1,sum)
    } else norm = NULL
    dfmat=x$dfmat
    if(type.coef=="coef"){
      ncl=nrow(dfmat)
      clnames=rownames(dfmat)
      for( i in seq(ncl)){
        plotCoef(beta[[i]],norm,x$lambda,dfmat[i,],x$dev.ratio,label=label,xvar=xvar,ylab=paste("Coefficients: Response",clnames[i]),...)
      }
    }
    else
        plotCoef(coefnorm(beta,2),norm,x$lambda,dfmat[1,],x$dev.ratio,label=label,xvar=xvar,ylab="Coefficient 2Norms",...)

  }

#' @method predict coxnet
#' @export
predict.coxnet=function(object,newx,s=NULL,type=c("link","response","coefficients","nonzero"),exact=FALSE,newoffset,...){
  type=match.arg(type)
  ###coxnet has no intercept, so we treat it separately
  if(missing(newx)){
    if(!match(type,c("coefficients","nonzero"),FALSE))stop("You need to supply a value for 'newx'")
  }
  if(exact&&(!is.null(s))){
    lambda=object$lambda
    which=match(s,lambda,FALSE)
    if(!all(which>0)){
      lambda=unique(rev(sort(c(s,lambda))))
        check_dots(object,...)# This fails if you don't supply the crucial arguments
        object=update(object,lambda=lambda,...)
    }
  }

  nbeta=object$beta
   if(!is.null(s)){
    vnames=dimnames(nbeta)[[1]]
    dimnames(nbeta)=list(NULL,NULL)
    lambda=object$lambda
    lamlist=lambda.interp(lambda,s)
    nbeta=nbeta[,lamlist$left,drop=FALSE]%*%Diagonal(x=lamlist$frac) +nbeta[,lamlist$right,drop=FALSE]%*%Diagonal(x=1-lamlist$frac)
    dimnames(nbeta)=list(vnames,paste(seq(along=s)))
  }
  if(type=="coefficients")return(nbeta)
  if(type=="nonzero")return(nonzeroCoef(nbeta,bystep=TRUE))
  nfit=as.matrix(newx%*%nbeta)
  if(object$offset){
    if(missing(newoffset))stop("No newoffset provided for prediction, yet offset used in fit of glmnet",call.=FALSE)
    nfit=nfit+array(newoffset,dim=dim(nfit))
  }
  switch(type,
         response=exp(nfit),
         link=nfit
         )
}

cvtype <- function(type.measure="mse",subclass="elnet"){
    type.measures = c("mse","deviance", "class", "auc", "mae","C")
    devname=switch(subclass,
                   elnet="Mean-squared Error",
                   lognet="Binomial Deviance",
                   fishnet="Poisson Deviance",
                   coxnet="Partial Likelihood Deviance",
                   multnet="Multinomial Deviance",
                   mrelnet="Mean-squared Error",
                   glmnetfit="GLM Deviance"
                   )
    typenames = c(deviance = devname, mse = "Mean-Squared Error",
    mae = "Mean Absolute Error",auc = "AUC", class = "Misclassification Error",C="C-index")
    subclass.ch=switch(subclass,
                   elnet=c(1,2,5),
                   lognet=c(2,3,4,1,5),
                   fishnet=c(2,1,5),
                   coxnet=c(2,6),
                   multnet=c(2,3,1,5),
                   mrelnet=c(1,2,5),
                   glmnetfit=c(2,1,5)
                   )
   subclass.type=type.measures[subclass.ch]
   if(type.measure=="default")type.measure=subclass.type[1]
    model.name=switch(subclass,
                   elnet="Gaussian",
                   lognet="Binomial",
                   fishnet="Poisson",
                   coxnet="Cox",
                   multnet="Multinomial",
                   mrelnet="Multi-response Gaussian",
                   glmnetfit="GLM"
                   )
    if(!match(type.measure,subclass.type,FALSE)){
        type.measure=subclass.type[1]
        warning(paste("Only ",paste(subclass.type,collapse=", ")," available as type.measure for ",model.name," models; ", type.measure," used instead",sep=""),call.=FALSE)
    }
    names(type.measure)=typenames[type.measure]
type.measure
    }

#' print a glmnet object
#'
#' Print a summary of the glmnet path at each step along the path.
#' @details
#' The call that produced the object `x` is printed, followed by a
#' three-column matrix with columns `Df`, `%Dev` and `Lambda`.
#' The `Df` column is the number of nonzero coefficients (Df is a
#' reasonable name only for lasso fits). `%Dev` is the percent deviance
#' explained (relative to the null deviance).  In the case of a 'relaxed' fit,
#' an additional column is inserted, `%Dev R` which gives the percent
#' deviance explained by the relaxed model. For a "bigGlm" model, a simpler
#' summary is printed.
#'
#' @aliases print.glmnet print.relaxed print.bigGlm
#' @param x fitted glmnet object
#' @param digits significant digits in printout
#' @param \dots additional print arguments
#' @return The matrix above is silently returned
#' @seealso \code{glmnet}, \code{predict} and \code{coef} methods.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008). Regularization Paths for Generalized Linear Models via Coordinate Descent
#' @keywords models regression
#' @examples
#'
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' fit1 = glmnet(x, y)
#' print(fit1)
#' @method print glmnet
#' @export
print.glmnet <-
    function (x, digits = max(3, getOption("digits") - 3), ...)
{
    cat("\nCall: ", deparse(x$call), "\n\n")
     out=data.frame(Df = x$df, `%Dev` = round(x$dev.ratio*100, 2),
                    Lambda = signif(x$lambda, digits),check.names=FALSE,row.names=seq(along=x$df))
class(out)=c("anova",class(out))
     print(out)
}

jerr.elnet=function(n,maxit,pmax){
  if(n>0){#fatal error
    if(n<7777)msg="Memory allocation error; contact package maintainer"
    else if(n==7777)msg="All used predictors have zero variance"
    else if(n==10000)msg="All penalty factors are <= 0"
    else msg="Unknown error"
    list(n=n,fatal=TRUE,msg=msg)
  }
  else if(n<0){# non-fatal error
           if(n>-10000)msg=paste("Convergence for ",-n,"th lambda value not reached after maxit=",maxit," iterations; solutions for larger lambdas returned",sep="")
    if(n < -10000)msg=paste("Number of nonzero coefficients along the path exceeds pmax=",pmax, " at ",-n-10000,"th lambda value; solutions for larger lambdas returned",sep="")
list(n=n,fatal=FALSE,msg=msg)
     }

}

blend.relaxed=function(fit,gamma,extend=TRUE,epsilon=0.00001){
###
###gamma must be a single value
    if(gamma==1){
        class(fit)=class(fit)[-1] #drop the relaxed status
        fit$relaxed=NULL
        return(fit)
    }
    gamma=max(gamma,epsilon)
### We do the above so that even of gamma=0, and extend=TRUE, we fill in the coefficients
### First see which if any of glmnet coefficients are missing on relaxed
    which=match(fit$lambda,fit$relaxed$lambda,0)
    if(extend)which[which==0]=max(which)
    beta=fit$beta
    a0=fit$a0
    fitr=fit$relaxed
    betar=fitr$beta
    a0r=fitr$a0
    islistbeta=is.list(beta)
    if(!islistbeta){
        fitr$beta=gamma*beta[,which>0] +(1-gamma)*betar[,which]
        fitr$a0=gamma*a0[which>0]+(1-gamma)*a0r[which]
    }
    else{
        ngroups=length(beta)
        for(i in 1:ngroups)
            beta[[i]]=gamma*beta[[i]][,which>0]+(1-gamma)*betar[[i]][,which]
        fitr$beta=beta
        fitr$a0=gamma*a0[,which>0]+(1-gamma)*a0r[,which]
    }
    fitr$dev.ratio=gamma*fit$dev.ratio[which>0]+(1-gamma)*fitr$dev.ratio[which]
    fitr$df=fit$df[which>0]
    fitr$lambda=fit$lambda[which>0]
    fitr
    }

#' @method coef cv.relaxed
#' @export
coef.cv.relaxed <-
function (object, s = c("lambda.1se", "lambda.min"), gamma=c("gamma.1se","gamma.min"),...)
 {
     relaxed=object$relaxed
    if (is.numeric(s))
        lambda = s
    else if (is.character(s)) {
        s = match.arg(s)
        lambda = relaxed[[s]]
        if(missing(gamma)){
            gamma=switch(s,
                         lambda.1se=relaxed[["gamma.1se"]],
                         lambda.min=relaxed[["gamma.min"]]
                         )
            }
    }
else stop("Invalid form for s")
    if (is.character(gamma)) {
        gamma = match.arg(gamma)
        gamma = relaxed[[gamma]]
    }
    if(!is.numeric(gamma))stop("Invalid form for gamma")

    coef(object$glmnet.fit, s = lambda, gamma=gamma, ...)
}

cv.elnet <-function(predmat,y,type.measure,weights,foldid,grouped){

    N = length(y) - apply(is.na(predmat), 2, sum)
    cvraw = switch(type.measure,
                   mse = (y - predmat)^2,
                   deviance = (y - predmat)^2,
                   mae = abs(y - predmat))
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)

}

#' Display the names of the measures used in CV for different "glmnet" families
#'
#' Produces a list of names of measures
#'
#' Try it and see. A very simple function to provide information
#'
#' @param family If a "glmnet" family is supplied, a list of the names of
#' measures available for that family are produced. Default is "all", in which
#' case the names of measures for all families are produced.
#' @author Trevor Hastie\cr Maintainer: Trevor Hastie
#' \email{hastie@@stanford.edu}
#' @seealso \code{cv.glmnet} and \code{assess.glmnet}.
#' @keywords models
#' @export glmnet.measures
glmnet.measures=function(family=c("all","gaussian", "binomial", "poisson",
                                  "multinomial", "cox", "mgaussian","GLM")){
    family=match.arg(family)
    type.measures = c("mse", "deviance", "class", "auc", "mae",
                      "C")
    family.ch = list( gaussian = c(1, 5), binomial = c(2,
                                                          3, 4, 1, 5), poisson = c(2, 1, 5), cox = c(2, 6),
                     multinomial = c(2, 3, 1, 5), mgaussian = c(1, 5),GLM=c(2,1,5))
    if(family=="all")lapply(family.ch,function(x)type.measures[x])
    else         type.measures[family.ch[[family]]]

        }

check_dots <- function(object, ..., 
                       need = c("x", "y", "weights", "offset", "penalty.factor",
                                "lower.limits", "upper.limits"), 
                       error_start = "used coef.glmnet() or predict.glmnet() with `exact=TRUE`",
                       error_end = " in order to safely rerun glmnet",
                       prefix = NULL) {
    if (is.null(need)) return(invisible())
    
    # extract the function options we need from the object's call
    thiscall = object$call
    ncall = names(thiscall)[-1]
    w = match(ncall, need, 0)
    need = need[w]
    if (length(need) == 0) return(invisible())
    
    # check that ... indeed has those function options
    if (!is.null(prefix)) need <- paste0(prefix, need)
    nargs = names(list(...))
    w = match(need, nargs, 0) > 0
    if(!all(w)) {
        margs = need[!w]
        stop(paste(error_start, 
                   "so must in addition supply original argument(s) ",
                   paste(margs,collapse=" and "), 
                   error_end), call.=FALSE)
    }
    invisible()
}

#' fit a GLM with lasso or elasticnet regularization
#'
#' Fit a generalized linear model via penalized maximum likelihood.  The
#' regularization path is computed for the lasso or elasticnet penalty at a
#' grid of values for the regularization parameter lambda. Can deal with all
#' shapes of data, including very large sparse data matrices. Fits linear,
#' logistic and multinomial, poisson, and Cox regression models.
#'
#' The sequence of models implied by \code{lambda} is fit by coordinate
#' descent. For \code{family="gaussian"} this is the lasso sequence if
#' \code{alpha=1}, else it is the elasticnet sequence.
#'
#' The objective function for \code{"gaussian"} is \deqn{1/2 RSS/nobs +
#' \lambda*penalty,} and for the other models it is \deqn{-loglik/nobs +
#' \lambda*penalty.} Note also that for \code{"gaussian"}, \code{glmnet}
#' standardizes y to have unit variance (using 1/n rather than 1/(n-1) formula)
#' before computing its lambda sequence (and then unstandardizes the resulting
#' coefficients); if you wish to reproduce/compare results with other software,
#' best to supply a standardized y. The coefficients for any predictor
#' variables with zero variance are set to zero for all values of lambda.
#'
#' ## Details on `family` option
#'
#' From version 4.0 onwards, glmnet supports both the original built-in families,
#' as well as \emph{any} family object as used by `stats:glm()`.
#' This opens the door to a wide variety of additional models. For example
#' `family=binomial(link=cloglog)` or `family=negative.binomial(theta=1.5)` (from the MASS library).
#' Note that the code runs faster for the built-in families.
#'
#' The built in families are specifed via a character string. For all families,
#' the object produced is a lasso or elasticnet regularization path for fitting the
#' generalized linear regression paths, by maximizing the appropriate penalized
#' log-likelihood (partial likelihood for the "cox" model). Sometimes the
#' sequence is truncated before \code{nlambda} values of \code{lambda} have
#' been used, because of instabilities in the inverse link functions near a
#' saturated fit. \code{glmnet(...,family="binomial")} fits a traditional
#' logistic regression model for the log-odds.
#' \code{glmnet(...,family="multinomial")} fits a symmetric multinomial model,
#' where each class is represented by a linear model (on the log-scale). The
#' penalties take care of redundancies. A two-class \code{"multinomial"} model
#' will produce the same fit as the corresponding \code{"binomial"} model,
#' except the pair of coefficient matrices will be equal in magnitude and
#' opposite in sign, and half the \code{"binomial"} values.
#' Two useful additional families are the \code{family="mgaussian"} family and
#' the \code{type.multinomial="grouped"} option for multinomial fitting. The
#' former allows a multi-response gaussian model to be fit, using a "group
#' -lasso" penalty on the coefficients for each variable. Tying the responses
#' together like this is called "multi-task" learning in some domains. The
#' grouped multinomial allows the same penalty for the
#' \code{family="multinomial"} model, which is also multi-responsed. For both
#' of these the penalty on the coefficient vector for variable j is
#' \deqn{(1-\alpha)/2||\beta_j||_2^2+\alpha||\beta_j||_2.} When \code{alpha=1}
#' this is a group-lasso penalty, and otherwise it mixes with quadratic just
#' like elasticnet. A small detail in the Cox model: if death times are tied
#' with censored times, we assume the censored times occurred just
#' \emph{before} the death times in computing the Breslow approximation; if
#' users prefer the usual convention of \emph{after}, they can add a small
#' number to all censoring times to achieve this effect.
#'
#' ## Details on response for `family="cox"`
#'
#' For Cox models, the response should preferably be a \code{Surv} object,
#' created by the \code{Surv()} function in \pkg{survival} package. For
#' right-censored data, this object should have type "right", and for
#' (start, stop] data, it should have type "counting". To fit stratified Cox
#' models, strata should be added to the response via the \code{stratifySurv()}
#' function before passing the response to \code{glmnet()}. (For backward
#' compatibility, right-censored data can also be passed as a
#' two-column matrix with columns named 'time' and 'status'. The
#' latter is a binary variable, with '1' indicating death, and '0' indicating
#' right censored.)
#'
#' ## Details on `relax` option
#'
#' If \code{relax=TRUE}
#' a duplicate sequence of models is produced, where each active set in the
#' elastic-net path is refit without regularization. The result of this is a
#' matching \code{"glmnet"} object which is stored on the original object in a
#' component named \code{"relaxed"}, and is part of the glmnet output.
#' Generally users will not call \code{relax.glmnet} directly, unless the
#' original 'glmnet' object took a long time to fit. But if they do, they must
#' supply the fit, and all the original arguments used to create that fit. They
#' can limit the length of the relaxed path via 'maxp'.
#'
#' @param x input matrix, of dimension nobs x nvars; each row is an observation
#' vector. Can be in sparse matrix format (inherit from class
#' \code{"sparseMatrix"} as in package \code{Matrix})
#' @param y response variable. Quantitative for \code{family="gaussian"}, or
#' \code{family="poisson"} (non-negative counts). For \code{family="binomial"}
#' should be either a factor with two levels, or a two-column matrix of counts
#' or proportions (the second column is treated as the target class; for a
#' factor, the last level in alphabetical order is the target class). For
#' \code{family="multinomial"}, can be a \code{nc>=2} level factor, or a matrix
#' with \code{nc} columns of counts or proportions. For either
#' \code{"binomial"} or \code{"multinomial"}, if \code{y} is presented as a
#' vector, it will be coerced into a factor. For \code{family="cox"}, preferably
#' a \code{Surv} object from the survival package: see Details section for
#' more information. For \code{family="mgaussian"}, \code{y} is a matrix
#' of quantitative responses.
#' @param family Either a character string representing
#' one of the built-in families, or else a `glm()` family object. For more
#' information, see Details section below or the documentation for response
#' type (above).
#' @param weights observation weights. Can be total counts if responses are
#' proportion matrices. Default is 1 for each observation
#' @param offset A vector of length \code{nobs} that is included in the linear
#' predictor (a \code{nobs x nc} matrix for the \code{"multinomial"} family).
#' Useful for the \code{"poisson"} family (e.g. log of exposure time), or for
#' refining a model by starting at a current fit. Default is \code{NULL}. If
#' supplied, then values must also be supplied to the \code{predict} function.
#' @param alpha The elasticnet mixing parameter, with \eqn{0\le\alpha\le 1}.
#' The penalty is defined as
#' \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.} \code{alpha=1} is the
#' lasso penalty, and \code{alpha=0} the ridge penalty.
#' @param nlambda The number of \code{lambda} values - default is 100.
#' @param lambda.min.ratio Smallest value for \code{lambda}, as a fraction of
#' \code{lambda.max}, the (data derived) entry value (i.e. the smallest value
#' for which all coefficients are zero). The default depends on the sample size
#' \code{nobs} relative to the number of variables \code{nvars}. If \code{nobs
#' > nvars}, the default is \code{0.0001}, close to zero.  If \code{nobs <
#' nvars}, the default is \code{0.01}.  A very small value of
#' \code{lambda.min.ratio} will lead to a saturated fit in the \code{nobs <
#' nvars} case. This is undefined for \code{"binomial"} and
#' \code{"multinomial"} models, and \code{glmnet} will exit gracefully when the
#' percentage deviance explained is almost 1.
#' @param lambda A user supplied \code{lambda} sequence. Typical usage is to
#' have the program compute its own \code{lambda} sequence based on
#' \code{nlambda} and \code{lambda.min.ratio}. Supplying a value of
#' \code{lambda} overrides this. WARNING: use with care. Avoid supplying a
#' single value for \code{lambda} (for predictions after CV use
#' \code{predict()} instead).  Supply instead a decreasing sequence of
#' \code{lambda} values. \code{glmnet} relies on its warms starts for speed,
#' and its often faster to fit a whole path than compute a single fit.
#' @param standardize Logical flag for x variable standardization, prior to
#' fitting the model sequence. The coefficients are always returned on the
#' original scale. Default is \code{standardize=TRUE}.  If variables are in the
#' same units already, you might not wish to standardize. See details below for
#' y standardization with \code{family="gaussian"}.
#' @param intercept Should intercept(s) be fitted (default=TRUE) or set to zero
#' (FALSE)
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than \code{thresh} times the null
#' deviance. Defaults value is \code{1E-7}.
#' @param dfmax Limit the maximum number of variables in the model. Useful for
#' very large \code{nvars}, if a partial path is desired.
#' @param pmax Limit the maximum number of variables ever to be nonzero
#' @param exclude Indices of variables to be excluded from the model. Default
#' is none. Equivalent to an infinite penalty factor (next item).
#' @param penalty.factor Separate penalty factors can be applied to each
#' coefficient. This is a number that multiplies \code{lambda} to allow
#' differential shrinkage. Can be 0 for some variables, which implies no
#' shrinkage, and that variable is always included in the model. Default is 1
#' for all variables (and implicitly infinity for variables listed in
#' \code{exclude}). Note: the penalty factors are internally rescaled to sum to
#' nvars, and the lambda sequence will reflect this change.
#' @param lower.limits Vector of lower limits for each coefficient; default
#' \code{-Inf}. Each of these must be non-positive. Can be presented as a
#' single value (which will then be replicated), else a vector of length
#' \code{nvars}
#' @param upper.limits Vector of upper limits for each coefficient; default
#' \code{Inf}. See \code{lower.limits}
#' @param maxit Maximum number of passes over the data for all lambda values;
#' default is 10^5.
#' @param type.gaussian Two algorithm types are supported for (only)
#' \code{family="gaussian"}. The default when \code{nvar<500} is
#' \code{type.gaussian="covariance"}, and saves all inner-products ever
#' computed. This can be much faster than \code{type.gaussian="naive"}, which
#' loops through \code{nobs} every time an inner-product is computed. The
#' latter can be far more efficient for \code{nvar >> nobs} situations, or when
#' \code{nvar > 500}.
#' @param type.logistic If \code{"Newton"} then the exact hessian is used
#' (default), while \code{"modified.Newton"} uses an upper-bound on the
#' hessian, and can be faster.
#' @param standardize.response This is for the \code{family="mgaussian"}
#' family, and allows the user to standardize the response variables
#' @param type.multinomial If \code{"grouped"} then a grouped lasso penalty is
#' used on the multinomial coefficients for a variable. This ensures they are
#' all in our out together. The default is \code{"ungrouped"}
#' @param relax If \code{TRUE} then for each \emph{active set} in the path of
#' solutions, the model is refit without any regularization. See \code{details}
#' for more information. This argument is new, and users may experience convergence issues
#' with small datasets, especially with non-gaussian families. Limiting the
#' value of 'maxp' can alleviate these issues in some cases.
#' @param trace.it If \code{trace.it=1}, then a progress bar is displayed;
#' useful for big models that take a long time to fit.
#' @param ... Additional argument used in \code{relax.glmnet}. These include
#' some of the original arguments to 'glmnet', and each must be named if used.
#' @return An object with S3 class \code{"glmnet","*" }, where \code{"*"} is
#' \code{"elnet"}, \code{"lognet"}, \code{"multnet"}, \code{"fishnet"}
#' (poisson), \code{"coxnet"} or \code{"mrelnet"} for the various types of
#' models. If the model was created with \code{relax=TRUE} then this class has
#' a prefix class of \code{"relaxed"}.  \item{call}{the call that produced this
#' object} \item{a0}{Intercept sequence of length \code{length(lambda)}}
#' \item{beta}{For \code{"elnet"}, \code{"lognet"}, \code{"fishnet"} and
#' \code{"coxnet"} models, a \code{nvars x length(lambda)} matrix of
#' coefficients, stored in sparse column format (\code{"CsparseMatrix"}). For
#' \code{"multnet"} and \code{"mgaussian"}, a list of \code{nc} such matrices,
#' one for each class.} \item{lambda}{The actual sequence of \code{lambda}
#' values used. When \code{alpha=0}, the largest lambda reported does not quite
#' give the zero coefficients reported (\code{lambda=inf} would in principle).
#' Instead, the largest \code{lambda} for \code{alpha=0.001} is used, and the
#' sequence of \code{lambda} values is derived from this.} \item{dev.ratio}{The
#' fraction of (null) deviance explained (for \code{"elnet"}, this is the
#' R-square). The deviance calculations incorporate weights if present in the
#' model. The deviance is defined to be 2*(loglike_sat - loglike), where
#' loglike_sat is the log-likelihood for the saturated model (a model with a
#' free parameter per observation). Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)); The NULL model refers to the intercept
#' model, except for the Cox, where it is the 0 model.} \item{df}{The number of
#' nonzero coefficients for each value of \code{lambda}. For \code{"multnet"},
#' this is the number of variables with a nonzero coefficient for \emph{any}
#' class.} \item{dfmat}{For \code{"multnet"} and \code{"mrelnet"} only. A
#' matrix consisting of the number of nonzero coefficients per class}
#' \item{dim}{dimension of coefficient matrix (ices)} \item{nobs}{number of
#' observations} \item{npasses}{total passes over the data summed over all
#' lambda values} \item{offset}{a logical variable indicating whether an offset
#' was included in the model} \item{jerr}{error flag, for warnings and errors
#' (largely for internal debugging).} \item{relaxed}{If \code{relax=TRUE}, this
#' additional item is another glmnet object with different values for
#' \code{beta} and \code{dev.ratio}}
#' @author Jerome Friedman, Trevor Hastie, Balasubramanian Narasimhan, Noah
#' Simon, Kenneth Tay and Rob Tibshirani\cr Maintainer: Trevor Hastie
#' \email{hastie@@stanford.edu}
#' @seealso \code{print}, \code{predict}, \code{coef} and \code{plot} methods,
#' and the \code{cv.glmnet} function.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent (2010), Journal of Statistical Software, Vol. 33(1), 1-22},
#' \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}.\cr
#' Simon, N., Friedman, J., Hastie, T. and Tibshirani, R. (2011)
#' \emph{Regularization Paths for Cox's Proportional
#' Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
#' 39(5), 1-13}, \url{https://www.jstatsoft.org/v39/i05/}.\cr Tibshirani,
#' Robert, Bien, J., Friedman, J., Hastie, T.,Simon, N.,Taylor, J. and
#' Tibshirani, Ryan. (2012) \emph{Strong Rules for Discarding Predictors in
#' Lasso-type Problems, JRSSB, Vol. 74(2), 245-266},
#' \url{https://statweb.stanford.edu/~tibs/ftp/strong.pdf}.\cr
#' Hastie, T., Tibshirani, Robert and Tibshirani, Ryan. \emph{Extended
#' Comparisons of Best Subset Selection, Forward Stepwise Selection, and the
#' Lasso (2017), Stanford Statistics Technical Report},
#' \url{https://arxiv.org/abs/1707.08692}.\cr
#' Glmnet webpage with four vignettes, \url{https://glmnet.stanford.edu}.
#' @keywords models regression
#' @examples
#'
#' # Gaussian
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' fit1 = glmnet(x, y)
#' print(fit1)
#' coef(fit1, s = 0.01)  # extract coefficients at a single value of lambda
#' predict(fit1, newx = x[1:10, ], s = c(0.01, 0.005))  # make predictions
#'
#' # Relaxed
#' fit1r = glmnet(x, y, relax = TRUE)  # can be used with any model
#'
#' # multivariate gaussian
#' y = matrix(rnorm(100 * 3), 100, 3)
#' fit1m = glmnet(x, y, family = "mgaussian")
#' plot(fit1m, type.coef = "2norm")
#'
#' # binomial
#' g2 = sample(c(0,1), 100, replace = TRUE)
#' fit2 = glmnet(x, g2, family = "binomial")
#' fit2n = glmnet(x, g2, family = binomial(link=cloglog))
#' fit2r = glmnet(x,g2, family = "binomial", relax=TRUE)
#' fit2rp = glmnet(x,g2, family = "binomial", relax=TRUE, path=TRUE)
#'
#' # multinomial
#' g4 = sample(1:4, 100, replace = TRUE)
#' fit3 = glmnet(x, g4, family = "multinomial")
#' fit3a = glmnet(x, g4, family = "multinomial", type.multinomial = "grouped")
#' # poisson
#' N = 500
#' p = 20
#' nzc = 5
#' x = matrix(rnorm(N * p), N, p)
#' beta = rnorm(nzc)
#' f = x[, seq(nzc)] %*% beta
#' mu = exp(f)
#' y = rpois(N, mu)
#' fit = glmnet(x, y, family = "poisson")
#' plot(fit)
#' pfit = predict(fit, x, s = 0.001, type = "response")
#' plot(pfit, y)
#'
#' # Cox
#' set.seed(10101)
#' N = 1000
#' p = 30
#' nzc = p/3
#' x = matrix(rnorm(N * p), N, p)
#' beta = rnorm(nzc)
#' fx = x[, seq(nzc)] %*% beta/3
#' hx = exp(fx)
#' ty = rexp(N, hx)
#' tcens = rbinom(n = N, prob = 0.3, size = 1)  # censoring indicator
#' y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
#' fit = glmnet(x, y, family = "cox")
#' plot(fit)
#'
#' # Cox example with (start, stop] data
#' set.seed(2)
#' nobs <- 100; nvars <- 15
#' xvec <- rnorm(nobs * nvars)
#' xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
#' x <- matrix(xvec, nrow = nobs)
#' start_time <- runif(100, min = 0, max = 5)
#' stop_time <- start_time + runif(100, min = 0.1, max = 3)
#' status <- rbinom(n = nobs, prob = 0.3, size = 1)
#' jsurv_ss <- survival::Surv(start_time, stop_time, status)
#' fit <- glmnet(x, jsurv_ss, family = "cox")
#'
#' # Cox example with strata
#' jsurv_ss2 <- stratifySurv(jsurv_ss, rep(1:2, each = 50))
#' fit <- glmnet(x, jsurv_ss2, family = "cox")
#'
#' # Sparse
#' n = 10000
#' p = 200
#' nzc = trunc(p/10)
#' x = matrix(rnorm(n * p), n, p)
#' iz = sample(1:(n * p), size = n * p * 0.85, replace = FALSE)
#' x[iz] = 0
#' sx = Matrix(x, sparse = TRUE)
#' inherits(sx, "sparseMatrix")  #confirm that it is sparse
#' beta = rnorm(nzc)
#' fx = x[, seq(nzc)] %*% beta
#' eps = rnorm(n)
#' y = fx + eps
#' px = exp(fx)
#' px = px/(1 + px)
#' ly = rbinom(n = length(px), prob = px, size = 1)
#' system.time(fit1 <- glmnet(sx, y))
#' system.time(fit2n <- glmnet(x, y))
#'
#' @export glmnet
glmnet=function(x,y,family=c("gaussian","binomial","poisson","multinomial","cox","mgaussian"),weights=NULL,offset=NULL,alpha=1.0,nlambda=100,lambda.min.ratio=ifelse(nobs<nvars,1e-2,1e-4),lambda=NULL,standardize=TRUE,intercept=TRUE,thresh=1e-7,dfmax=nvars+1,pmax=min(dfmax*2+20,nvars),exclude=NULL,penalty.factor=rep(1,nvars),lower.limits=-Inf,upper.limits=Inf,maxit=100000,type.gaussian=ifelse(nvars<500,"covariance","naive"),type.logistic=c("Newton","modified.Newton"),standardize.response=FALSE,type.multinomial=c("ungrouped","grouped"),relax=FALSE,trace.it=0,...){

    this.call=match.call()
### Need to do this first so defaults in call can be satisfied
    np=dim(x)
    ##check dims
    if(is.null(np)|(np[2]<=1))stop("x should be a matrix with 2 or more columns")
    nobs=as.integer(np[1])
    nvars=as.integer(np[2])

### See whether its a call to glmnet or to glmnet.path, based on family arg
    if(!is.character(family)){
        ## new.call=this.call
        ## new.call[[1]]=as.name("glmnet.path")
        ## fit=eval(new.call, parent.frame())
        fit=glmnet.path(x,y,weights,lambda,nlambda,lambda.min.ratio,alpha,offset,family,
                        standardize,intercept,thresh=thresh,maxit,penalty.factor,exclude,lower.limits,
                        upper.limits,trace.it=trace.it)
        fit$call=this.call
    } else {
      family=match.arg(family)
      if (family == "cox" && use.cox.path(x, y)) {
      # we should call the new cox.path()
      fit <- cox.path(x,y,weights,offset,alpha,nlambda,lambda.min.ratio,
                      lambda,standardize,thresh,exclude,penalty.factor,
                      lower.limits,upper.limits,maxit,trace.it,...)
      fit$call <- this.call
    } else {
      ### Must have been a call to old glmnet
      ### Prepare all the generic arguments, then hand off to family functions
      if(alpha>1){
        warning("alpha >1; set to 1")
        alpha=1
      }
      if(alpha<0){
        warning("alpha<0; set to 0")
        alpha=0
      }
      alpha=as.double(alpha)
      nlam=as.integer(nlambda)
      y=drop(y) # we dont like matrix responses unless we need them
      if(is.null(weights))weights=rep(1,nobs)
      else if(length(weights)!=nobs)stop(paste("number of elements in weights (",length(weights),") not equal to the number of rows of x (",nobs,")",sep=""))
      dimy=dim(y)
      nrowy=ifelse(is.null(dimy),length(y),dimy[1])
      if(nrowy!=nobs)stop(paste("number of observations in y (",nrowy,") not equal to the number of rows of x (",nobs,")",sep=""))
      vnames=colnames(x)
      if(is.null(vnames))vnames=paste("V",seq(nvars),sep="")
      ne=as.integer(dfmax)
      nx=as.integer(pmax)
      if(is.null(exclude))exclude=integer(0)
      if(any(penalty.factor==Inf)){
        exclude=c(exclude,seq(nvars)[penalty.factor==Inf])
        exclude=sort(unique(exclude))
      }
      if(length(exclude)>0){
        jd=match(exclude,seq(nvars),0)
        if(!all(jd>0))stop("Some excluded variables out of range")
        penalty.factor[jd]=1 #ow can change lambda sequence
        jd=as.integer(c(length(jd),jd))
      }else jd=as.integer(0)
      vp=as.double(penalty.factor)
      internal.parms=glmnet.control()
      if(internal.parms$itrace)trace.it=1
      else{
        if(trace.it){
          glmnet.control(itrace=1)
          on.exit(glmnet.control(itrace=0))
        }
      }
      ###check on limits
      if(any(lower.limits>0)){stop("Lower limits should be non-positive")}
      if(any(upper.limits<0)){stop("Upper limits should be non-negative")}
      lower.limits[lower.limits==-Inf]=-internal.parms$big
      upper.limits[upper.limits==Inf]=internal.parms$big
      if(length(lower.limits)<nvars){
        if(length(lower.limits)==1)lower.limits=rep(lower.limits,nvars)else stop("Require length 1 or nvars lower.limits")
      }
      else lower.limits=lower.limits[seq(nvars)]
      if(length(upper.limits)<nvars){
        if(length(upper.limits)==1)upper.limits=rep(upper.limits,nvars)else stop("Require length 1 or nvars upper.limits")
      }
      else upper.limits=upper.limits[seq(nvars)]
      cl=rbind(lower.limits,upper.limits)
      if(any(cl==0)){
        ###Bounds of zero can mess with the lambda sequence and fdev; ie nothing happens and if fdev is not
        ###zero, the path can stop
        fdev=glmnet.control()$fdev
        if(fdev!=0) {
          glmnet.control(fdev=0)
          on.exit(glmnet.control(fdev=fdev))
        }
      }
      storage.mode(cl)="double"
      ### end check on limits

      isd=as.integer(standardize)
      intr=as.integer(intercept)
      if(!missing(intercept)&&family=="cox")warning("Cox model has no intercept")
      jsd=as.integer(standardize.response)
      thresh=as.double(thresh)
      if(is.null(lambda)){
        if(lambda.min.ratio>=1)stop("lambda.min.ratio should be less than 1")
        flmin=as.double(lambda.min.ratio)
        ulam=double(1)
      }
      else{
        flmin=as.double(1)
        if(any(lambda<0))stop("lambdas should be non-negative")
        ulam=as.double(rev(sort(lambda)))
        nlam=as.integer(length(lambda))
      }
      is.sparse=FALSE
      ix=jx=NULL
      if(inherits(x,"sparseMatrix")){##Sparse case
        is.sparse=TRUE
        x=as(x,"CsparseMatrix")
        x=as(x,"dgCMatrix")
        ix=as.integer(x@p+1)
        jx=as.integer(x@i+1)
        xd=x@x
      }
      else xd <- x
      storage.mode(xd) <- "double"
      if (trace.it) {
        if (relax) cat("Training Fit\n")
        pb  <- createPB(min = 0, max = nlam, initial = 0, style = 3)
      }
      kopt=switch(match.arg(type.logistic),
                  "Newton"=0,#This means to use the exact Hessian
                  "modified.Newton"=1 # Use the upper bound
                  )
      if(family=="multinomial"){
        type.multinomial=match.arg(type.multinomial)
        if(type.multinomial=="grouped")kopt=2 #overrules previous kopt
      }
      kopt=as.integer(kopt)

      fit=switch(family,
                 "gaussian"=elnet(xd,is.sparse,ix,jx,y,weights,offset,type.gaussian,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit),
                 "poisson"=fishnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit),
                 "binomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family),
                 "multinomial"=lognet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family),
                 "cox"=coxnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,vnames,maxit),
                 "mgaussian"=mrelnet(xd,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,vnames,maxit)
                 )
      if (trace.it) {
        utils::setTxtProgressBar(pb, nlam)
        close(pb)
      }
      if(is.null(lambda))fit$lambda=fix.lam(fit$lambda)##first lambda is infinity; changed to entry point
      fit$call=this.call
      fit$nobs=nobs
      class(fit)=c(class(fit),"glmnet")
    }
    }

  if(relax)
    relax.glmnet(fit, x=x,y=y,weights=weights,offset=offset,
                 lower.limits=lower.limits,upper.limits=upper.limits,penalty.factor=penalty.factor,
                 check.args=FALSE,...)
  else
    fit
}

#' compute C index for a Cox model
#'
#' Computes Harrel's C index for predictions from a \code{"coxnet"} object.
#'
#' Computes the concordance index, taking into account censoring.
#'
#' @param pred Predictions from a \code{"coxnet"} object
#' @param y a survival response object - a matrix with two columns "time" and
#' "status"; see documentation for "glmnet"
#' @param weights optional observation weights
#' @author Trevor Hastie  <hastie@@stanford.edu>
#' @seealso \code{cv.glmnet}
#' @references Harrel Jr, F. E. and Lee, K. L. and Mark, D. B. (1996)
#' \emph{Tutorial in biostatistics: multivariable prognostic models: issues in
#' developing models, evaluating assumptions and adequacy, and measuring and
#' reducing error}, Statistics in Medicine, 15, pages 361--387.
#' @keywords Cox models cross-validation
#' @examples
#'
#' set.seed(10101)
#' N = 1000
#' p = 30
#' nzc = p/3
#' x = matrix(rnorm(N * p), N, p)
#' beta = rnorm(nzc)
#' fx = x[, seq(nzc)] %*% beta/3
#' hx = exp(fx)
#' ty = rexp(N, hx)
#' tcens = rbinom(n = N, prob = 0.3, size = 1)  # censoring indicator
#' y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
#' fit = glmnet(x, y, family = "cox")
#' pred = predict(fit, newx = x)
#' apply(pred, 2, Cindex, y=y)
#' cv.glmnet(x, y, family = "cox", type.measure = "C")
#'
#' @export Cindex
Cindex=function(pred,y,weights=rep(1,nrow(y))){
###  This function links to the concordance function in the survival package
    if(!is.Surv(y))y=Surv(y[,"time"],y[,"status"])
    f=-pred
    if(missing(weights))
        concordance(y~f)$concordance
    else
        concordance(y~f,weights=weights)$concordance
}

fishnet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit){
  if(any(y<0))stop("negative responses encountered;  not permitted for Poisson family")
  maxit=as.integer(maxit)
  weights=as.double(weights)
  storage.mode(y)="double"
   if(is.null(offset)){
    offset=y*0 #keeps the shape of y
    is.offset=FALSE}
  else{
    storage.mode(offset)="double"
    
    is.offset=TRUE
  }
fit=if(is.sparse).Fortran("spfishnet",
  parm=alpha,nobs,nvars,x,ix,jx,y,offset,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,
  lmu=integer(1),
  a0=double(nlam),
  ca=double(nx*nlam),
  ia=integer(nx),
  nin=integer(nlam),
  nulldev=double(1),
  dev=double(nlam),
  alm=double(nlam),
  nlp=integer(1),
  jerr=integer(1),PACKAGE="glmnet"
  )
else .Fortran("fishnet",
              parm=alpha,nobs,nvars,as.double(x),y,offset,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,
              lmu=integer(1),
              a0=double(nlam),
              ca=double(nx*nlam),
              ia=integer(nx),
              nin=integer(nlam),
              nulldev=double(1),
              dev=double(nlam),
              alm=double(nlam),
              nlp=integer(1),
              jerr=integer(1),PACKAGE="glmnet"
              )
if(fit$jerr!=0){
  errmsg=jerr(fit$jerr,maxit,pmax=nx,family="poisson")
  if(errmsg$fatal)stop(errmsg$msg,call.=FALSE)
  else warning(errmsg$msg,call.=FALSE)
}
  outlist=getcoef(fit,nvars,nx,vnames)
  dev=fit$dev[seq(fit$lmu)]
outlist=c(outlist,list(dev.ratio=dev,nulldev=fit$nulldev,npasses=fit$nlp,jerr=fit$jerr,offset=is.offset))
class(outlist)="fishnet"
outlist
}

#' @export
`[.stratifySurv` <- function(x, i, j, drop = FALSE) {
  strata <- attr(x, "strata")
  stop_time <- NULL; start_time <- NULL; ss_match <- NULL
  if ("stop_time" %in% names(attributes(x)))  stop_time  <- attr(x, "stop_time")
  if ("start_time" %in% names(attributes(x))) start_time <- attr(x, "start_time")
  if ("ss_match" %in% names(attributes(x)))   ss_match   <- attr(x, "ss_match")
  obj <- NextMethod(`[`)
  # !missing(i) && missing(j)?
  if (!missing(i) && is.matrix(obj)) {
    attr(obj, "strata") <- strata[i]
    if (!is.null(stop_time))  attr(obj, "stop_time")  <- stop_time[i]
    if (!is.null(start_time)) attr(obj, "start_time") <- start_time[i]
    if (!is.null(ss_match))   attr(obj, "ss_match")   <- ss_match[i]
    class(obj) <- class(x)
  }
  return(obj)
}

#' Add strata to a Surv object
#'
#' Helper function to add strata as an attribute to a Surv object. The
#' output of this function can be used as the response in \code{glmnet()}
#' for fitting stratified Cox models.
#'
#' When fitting a stratified Cox model with \code{glmnet()}, strata should
#' be added to a \code{Surv} response with this helper function. Note that
#' it is not sufficient to add strata as an attribute to the \code{Surv}
#' response manually: if the result does not have class \code{stratifySurv},
#' subsetting of the response will not work properly.
#'
#' @param y A Surv object.
#' @param strata A vector of length equal to the number of observations in
#' y, indicating strata membership. Default is all belong to same strata.
#'
#' @return An object of class \code{stratifySurv} (in addition to all the
#' classes \code{y} belonged to).
#'
#' @examples
#' y <- survival::Surv(1:10, rep(0:1, length.out = 10))
#' strata <- rep(1:3, length.out = 10)
#' y2 <- stratifySurv(y, strata)  # returns stratifySurv object
#'
#' @importFrom survival is.Surv
#' @export
stratifySurv <- function(y, strata = rep(1, length(y))) {
  y <- response.coxnet(y)
  if (length(y) != length(strata))
    stop("y and strata must have the same length (=nobs)")
  attr(y, "strata") <- strata
  y_class <- class(y)
  if (!("stratifySurv" %in% y_class)) class(y) <- c("stratifySurv", y_class)
  return(y)
}

#' @method family glmnet
#' @export
family.glmnet=function(object,...){
    families=c(elnet = "gaussian", lognet = "binomial", fishnet = "poisson",
               multnet = "multinomial", coxnet = "cox", mrelnet = "mgaussian")
    cl=class(object)[1]
    families[cl]
}

#' @method family glmnetfit
#' @export
family.glmnetfit=function(object,...) object$family


#' @method family relaxed
#' @export
family.relaxed=function(object,...)family(object$relaxed)

#' @method family cv.glmnet
#' @export
family.cv.glmnet=function(object,...)family(object$glmnet.fit)

jerr.mrelnet=function(n,maxit,pmax){
    if(n==90000){
      msg=paste("Newton stepping for bounded multivariate response did not converge")
      list(n=n,fatal=FALSE,msg=msg)
    }
    else jerr.elnet(n,maxit,pmax)
}

#' Cross-validation for glmnet
#'
#' Does k-fold cross-validation for glmnet, produces a plot, and returns a
#' value for \code{lambda} (and \code{gamma} if \code{relax=TRUE})
#'
#' The function runs \code{glmnet} \code{nfolds}+1 times; the first to get the
#' \code{lambda} sequence, and then the remainder to compute the fit with each
#' of the folds omitted. The error is accumulated, and the average error and
#' standard deviation over the folds is computed.  Note that \code{cv.glmnet}
#' does NOT search for values for \code{alpha}. A specific value should be
#' supplied, else \code{alpha=1} is assumed by default. If users would like to
#' cross-validate \code{alpha} as well, they should call \code{cv.glmnet} with
#' a pre-computed vector \code{foldid}, and then use this same fold vector in
#' separate calls to \code{cv.glmnet} with different values of \code{alpha}.
#' Note also that the results of \code{cv.glmnet} are random, since the folds
#' are selected at random. Users can reduce this randomness by running
#' \code{cv.glmnet} many times, and averaging the error curves.
#'
#' If \code{relax=TRUE} then the values of \code{gamma} are used to mix the
#' fits. If \eqn{\eta} is the fit for lasso/elastic net, and \eqn{\eta_R} is
#' the relaxed fit (with unpenalized coefficients), then a relaxed fit mixed by
#' \eqn{\gamma} is \deqn{\eta(\gamma)=(1-\gamma)\eta_R+\gamma\eta.} There is
#' practically no extra cost for having a lot of values for \code{gamma}.
#' However, 5 seems sufficient for most purposes. CV then selects both
#' \code{gamma} and \code{lambda}.
#'
#' @param x \code{x} matrix as in \code{glmnet}.
#' @param y response \code{y} as in \code{glmnet}.
#' @param weights Observation weights; defaults to 1 per observation
#' @param offset Offset vector (matrix) as in \code{glmnet}
#' @param lambda Optional user-supplied lambda sequence; default is
#' \code{NULL}, and \code{glmnet} chooses its own sequence
#' @param type.measure loss to use for cross-validation. Currently five
#' options, not all available for all models. The default is
#' \code{type.measure="deviance"}, which uses squared-error for gaussian models
#' (a.k.a \code{type.measure="mse"} there), deviance for logistic and poisson
#' regression, and partial-likelihood for the Cox model.
#' \code{type.measure="class"} applies to binomial and multinomial logistic
#' regression only, and gives misclassification error.
#' \code{type.measure="auc"} is for two-class logistic regression only, and
#' gives area under the ROC curve. \code{type.measure="mse"} or
#' \code{type.measure="mae"} (mean absolute error) can be used by all models
#' except the \code{"cox"}; they measure the deviation from the fitted mean to
#' the response.
#' \code{type.measure="C"} is Harrel's concordance measure, only available for \code{cox} models.
#' @param nfolds number of folds - default is 10. Although \code{nfolds} can be
#' as large as the sample size (leave-one-out CV), it is not recommended for
#' large datasets. Smallest value allowable is \code{nfolds=3}
#' @param foldid an optional vector of values between 1 and \code{nfold}
#' identifying what fold each observation is in. If supplied, \code{nfold} can
#' be missing.
#' @param alignment This is an experimental argument, designed to fix the
#' problems users were having with CV, with possible values \code{"lambda"}
#' (the default) else \code{"fraction"}. With \code{"lambda"} the \code{lambda}
#' values from the master fit (on all the data) are used to line up the
#' predictions from each of the folds. In some cases this can give strange
#' values, since the effective \code{lambda} values in each fold could be quite
#' different. With \code{"fraction"} we line up the predictions in each fold
#' according to the fraction of progress along the regularization. If in the
#' call a \code{lambda} argument is also provided, \code{alignment="fraction"}
#' is ignored (with a warning).
#' @param grouped This is an experimental argument, with default \code{TRUE},
#' and can be ignored by most users. For all models except the \code{"cox"},
#' this refers to computing \code{nfolds} separate statistics, and then using
#' their mean and estimated standard error to describe the CV curve. If
#' \code{grouped=FALSE}, an error matrix is built up at the observation level
#' from the predictions from the \code{nfold} fits, and then summarized (does
#' not apply to \code{type.measure="auc"}). For the \code{"cox"} family,
#' \code{grouped=TRUE} obtains the CV partial likelihood for the Kth fold by
#' \emph{subtraction}; by subtracting the log partial likelihood evaluated on
#' the full dataset from that evaluated on the on the (K-1)/K dataset. This
#' makes more efficient use of risk sets. With \code{grouped=FALSE} the log
#' partial likelihood is computed only on the Kth fold
#' @param keep If \code{keep=TRUE}, a \emph{prevalidated} array is returned
#' containing fitted values for each observation and each value of
#' \code{lambda}. This means these fits are computed with this observation and
#' the rest of its fold omitted. The \code{foldid} vector is also returned.
#' Default is keep=FALSE.  If \code{relax=TRUE}, then a list of such arrays is
#' returned, one for each value of 'gamma'. Note: if the value 'gamma=1' is
#' omitted, this case is included in the list since it corresponds to the
#' original 'glmnet' fit.
#' @param parallel If \code{TRUE}, use parallel \code{foreach} to fit each
#' fold.  Must register parallel before hand, such as \code{doMC} or others.
#' See the example below.
#' @param gamma The values of the parameter for mixing the relaxed fit with the
#' regularized fit, between 0 and 1; default is \code{gamma = c(0, 0.25, 0.5,
#' 0.75, 1)}
#' @param relax If \code{TRUE}, then CV is done with respect to the mixing
#' parameter \code{gamma} as well as \code{lambda}. Default is
#' \code{relax=FALSE}
#' @param trace.it If \code{trace.it=1}, then progress bars are displayed;
#' useful for big models that take a long time to fit. Limited tracing if
#' \code{parallel=TRUE}
#' @param \dots Other arguments that can be passed to \code{glmnet}
#' @return an object of class \code{"cv.glmnet"} is returned, which is a list
#' with the ingredients of the cross-validation fit.  If the object was created
#' with \code{relax=TRUE} then this class has a prefix class of
#' \code{"cv.relaxed"}.  \item{lambda}{the values of \code{lambda} used in the
#' fits.} \item{cvm}{The mean cross-validated error - a vector of length
#' \code{length(lambda)}.} \item{cvsd}{estimate of standard error of
#' \code{cvm}.} \item{cvup}{upper curve = \code{cvm+cvsd}.} \item{cvlo}{lower
#' curve = \code{cvm-cvsd}.} \item{nzero}{number of non-zero coefficients at
#' each \code{lambda}.} \item{name}{a text string indicating type of measure
#' (for plotting purposes).} \item{glmnet.fit}{a fitted glmnet object for the
#' full data.} \item{lambda.min}{value of \code{lambda} that gives minimum
#' \code{cvm}.} \item{lambda.1se}{largest value of \code{lambda} such that
#' error is within 1 standard error of the minimum.} \item{fit.preval}{if
#' \code{keep=TRUE}, this is the array of prevalidated fits. Some entries can
#' be \code{NA}, if that and subsequent values of \code{lambda} are not reached
#' for that fold} \item{foldid}{if \code{keep=TRUE}, the fold assignments used}
#' \item{index}{a one column matrix with the indices of \code{lambda.min} and \code{lambda.1se} in the sequence of coefficients, fits etc.}
#' \item{relaxed}{if \code{relax=TRUE}, this additional item has the CV info
#' for each of the mixed fits. In particular it also selects \code{lambda,
#' gamma} pairs corresponding to the 1se rule, as well as the minimum error. It also has a component \code{index}, a two-column matrix  which contains the \code{lambda} and \code{gamma} indices corresponding to the "min" and "1se" solutions.}
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Noah Simon
#' helped develop the 'coxnet' function.\cr Jeffrey Wong and B. Narasimhan
#' helped with the parallel option\cr Maintainer: Trevor Hastie
#' \email{hastie@@stanford.edu}
#' @seealso \code{glmnet} and \code{plot}, \code{predict}, and \code{coef}
#' methods for \code{"cv.glmnet"} and \code{"cv.relaxed"} objects.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}, \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}\cr
#' \emph{Journal of Statistical Software, Vol. 33(1), 1-22 Feb 2010}\cr
#' \url{https://www.jstatsoft.org/v33/i01/}\cr Simon, N., Friedman, J., Hastie,
#' T., Tibshirani, R. (2011) \emph{Regularization Paths for Cox's Proportional
#' Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
#' 39(5) 1-13}\cr \url{https://www.jstatsoft.org/v39/i05/}
#' @keywords models regression
#' @examples
#'
#' set.seed(1010)
#' n = 1000
#' p = 100
#' nzc = trunc(p/10)
#' x = matrix(rnorm(n * p), n, p)
#' beta = rnorm(nzc)
#' fx = x[, seq(nzc)] %*% beta
#' eps = rnorm(n) * 5
#' y = drop(fx + eps)
#' px = exp(fx)
#' px = px/(1 + px)
#' ly = rbinom(n = length(px), prob = px, size = 1)
#' set.seed(1011)
#' cvob1 = cv.glmnet(x, y)
#' plot(cvob1)
#' coef(cvob1)
#' predict(cvob1, newx = x[1:5, ], s = "lambda.min")
#' title("Gaussian Family", line = 2.5)
#' set.seed(1011)
#' cvob1a = cv.glmnet(x, y, type.measure = "mae")
#' plot(cvob1a)
#' title("Gaussian Family", line = 2.5)
#' set.seed(1011)
#' par(mfrow = c(2, 2), mar = c(4.5, 4.5, 4, 1))
#' cvob2 = cv.glmnet(x, ly, family = "binomial")
#' plot(cvob2)
#' title("Binomial Family", line = 2.5)
#' frame()
#' set.seed(1011)
#' cvob3 = cv.glmnet(x, ly, family = "binomial", type.measure = "class")
#' plot(cvob3)
#' title("Binomial Family", line = 2.5)
#' \dontrun{
#' cvob1r = cv.glmnet(x, y, relax = TRUE)
#' plot(cvob1r)
#' predict(cvob1r, newx = x[, 1:5])
#' set.seed(1011)
#' cvob3a = cv.glmnet(x, ly, family = "binomial", type.measure = "auc")
#' plot(cvob3a)
#' title("Binomial Family", line = 2.5)
#' set.seed(1011)
#' mu = exp(fx/10)
#' y = rpois(n, mu)
#' cvob4 = cv.glmnet(x, y, family = "poisson")
#' plot(cvob4)
#' title("Poisson Family", line = 2.5)
#'
#' # Multinomial
#' n = 500
#' p = 30
#' nzc = trunc(p/10)
#' x = matrix(rnorm(n * p), n, p)
#' beta3 = matrix(rnorm(30), 10, 3)
#' beta3 = rbind(beta3, matrix(0, p - 10, 3))
#' f3 = x %*% beta3
#' p3 = exp(f3)
#' p3 = p3/apply(p3, 1, sum)
#' g3 = glmnet:::rmult(p3)
#' set.seed(10101)
#' cvfit = cv.glmnet(x, g3, family = "multinomial")
#' plot(cvfit)
#' title("Multinomial Family", line = 2.5)
#' # Cox
#' beta = rnorm(nzc)
#' fx = x[, seq(nzc)] %*% beta/3
#' hx = exp(fx)
#' ty = rexp(n, hx)
#' tcens = rbinom(n = n, prob = 0.3, size = 1)  # censoring indicator
#' y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
#' foldid = sample(rep(seq(10), length = n))
#' fit1_cv = cv.glmnet(x, y, family = "cox", foldid = foldid)
#' plot(fit1_cv)
#' title("Cox Family", line = 2.5)
#' # Parallel
#' require(doMC)
#' registerDoMC(cores = 4)
#' x = matrix(rnorm(1e+05 * 100), 1e+05, 100)
#' y = rnorm(1e+05)
#' system.time(cv.glmnet(x, y))
#' system.time(cv.glmnet(x, y, parallel = TRUE))
#' }
#'
#' @export cv.glmnet
cv.glmnet <-
  function (x, y, weights=NULL, offset = NULL, lambda = NULL, type.measure = c("default","mse",
                                                           "deviance", "class", "auc", "mae","C"), nfolds = 10, foldid=NULL,  alignment=c("lambda","fraction"),grouped = TRUE, keep = FALSE, parallel = FALSE, gamma=c(0,.25,.5,.75,1),relax=FALSE,trace.it=0, ...)
{
    type.measure = match.arg(type.measure)
    alignment=match.arg(alignment)
  if (!is.null(lambda) && length(lambda) < 2)
      stop("Need more than one value of lambda for cv.glmnet")
  if (!is.null(lambda) && alignment=="fraction"){
      warning("fraction of path alignment not available if lambda given as argument; switched to alignment=`lambda`")
      alignment="lambda"
  }
   N = nrow(x)
 if (is.null(weights))
    weights = rep(1, N)
  else weights = as.double(weights)
  y = drop(y)
  cv.call=glmnet.call = match.call(expand.dots = TRUE)
  which = match(c("type.measure", "nfolds", "foldid", "grouped",
    "keep"), names(glmnet.call), FALSE)
  if (any(which))
    glmnet.call = glmnet.call[-which]
    glmnet.call[[1]] = as.name("glmnet")
    if(glmnet.control()$itrace)trace.it=1
    else{
        if(trace.it){
            glmnet.control(itrace=1)
            on.exit(glmnet.control(itrace=0))
            }
        }
   if (is.null(foldid))
    foldid = sample(rep(seq(nfolds), length = N))
  else nfolds = max(foldid)
  if (nfolds < 3)
    stop("nfolds must be bigger than 3; nfolds=10 recommended")
### Now we switch depending on relax
    if(relax)
        cv.relaxed.raw(x,y,weights,offset,lambda,type.measure,nfolds,foldid,
                       alignment,grouped,keep,parallel,trace.it,glmnet.call,cv.call,gamma,...)
    else
        cv.glmnet.raw(x,y,weights,offset,lambda,type.measure,nfolds,foldid,
                       alignment,grouped,keep,parallel,trace.it,glmnet.call,cv.call,...)
 }

#' Compute a survival curve from a cv.glmnet object
#'
#' Computes the predicted survivor function for a Cox proportional hazards
#' model with elastic net penalty from a cross-validated glmnet model.
#'
#' This function makes it easier to use the results of cross-validation
#' to compute a survival curve.
#'
#' @param formula A class \code{cv.glmnet} object. The object should have
#' been fit with \code{family = "cox"}.
#' @param s Value(s) of the penalty parameter lambda at which predictions
#' are required. Default is the value s="lambda.1se" stored on the CV object.
#' Alternatively s="lambda.min" can be used. If s is numeric, it is taken
#' as the value(s) of lambda to be used.
#' @param ... Other arguments to be passed to \code{survfit.coxnet}.
#'
#' @return If \code{s} is a single value, an object of class "survfitcox"
#' and "survfit" containing one or more survival curves. Otherwise, a list
#' of such objects, one element for each value in \code{s}.
#' Methods defined for survfit objects are print, summary and plot.
#'
#' @examples
#' set.seed(2)
#' nobs <- 100; nvars <- 15
#' xvec <- rnorm(nobs * nvars)
#' x <- matrix(xvec, nrow = nobs)
#' beta <- rnorm(nvars / 3)
#' fx <- x[, seq(nvars / 3)] %*% beta / 3
#' ty <- rexp(nobs, exp(fx))
#' tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
#' y <- survival::Surv(ty, tcens)
#' cvfit <- cv.glmnet(x, y, family = "cox")
#' # default: s = "lambda.1se"
#' survival::survfit(cvfit, x = x, y = y)
#'
#' # s = "lambda.min"
#' survival::survfit(cvfit, s = "lambda.min", x = x, y = y)
#' @importFrom survival survfit
#' @method survfit cv.glmnet
#' @export
survfit.cv.glmnet <- function(formula, s = c("lambda.1se", "lambda.min"), ...) {
  this.call <- match.call()
  object <- formula

  # check that a coxnet model was fit
  if (!("coxnet" %in% class(object$glmnet.fit)))
    stop("survfit only available for Cox models")

  # if user didn't specify s, pick it out from its lambda sequence
  if (is.numeric(s)) lambda <- s
  else
    if (is.character(s)) {
      s <- match.arg(s)
      lambda <- object[[s]]
    }
  else stop("Invalid form for s")
  sf <- survfit.coxnet(object$glmnet.fit, s = lambda, ...)
  sf$call <- this.call
  return(sf)
}

#' @method predict relaxed
#' @param gamma Single value of \code{gamma} at which predictions are required,
#' for "relaxed" objects.
#' @rdname predict.glmnet
#' @export
#' @export predict.relaxed
predict.relaxed=function (object, newx, s = NULL, gamma=1,type = c("link", "response",
    "coefficients", "nonzero", "class"), exact = FALSE, newoffset,
    ...) {
    exact=FALSE # we cannot accommodate exact here
    type=match.arg(type)
    gamma=checkgamma.relax(gamma)
    if(length(gamma)>1){
        ng=length(gamma)
        outlist=as.list(length(ng))
        names(outlist(format(round(gamma,2))))
        for( i in 1:ng)outlist[[i]]=predict(object, newx, s, gamma=gamma[i], exact, newoffset,  ...)
        return(outlist)
    }
    if(gamma==1)return(NextMethod("predict"))
    predict(blend.relaxed(object,gamma),newx, s,type, exact, newoffset,  ...)
}

checkgamma.relax=function(gamma){
    if(any(wh<-gamma<0)){
        warning("negative gamma values ignored")
        gamma=gamma[!wh]
    }
    if(any(wh<-gamma>1)){
        warning("gamma values larger than 1 ignored")
        gamma=gamma[!wh]
    }
    if(!length(gamma))stop("no valid values of gamma")
    gamma
}

#' @method coef cv.glmnet
#' @export
coef.cv.glmnet=function(object,s=c("lambda.1se","lambda.min"),...){
    if(is.numeric(s))lambda=s
  else
    if(is.character(s)){
      s=match.arg(s)
      lambda=object[[s]]
    }
    else stop("Invalid form for s")
  coef(object$glmnet.fit,s=lambda,...)
}

#' @method predict multnet
#' @export
predict.multnet <-
  function (object, newx, s = NULL, type = c("link", "response",
                                      "coefficients", "class", "nonzero"), exact = FALSE, newoffset,
            ...)
{
  type = match.arg(type)
    ###multnet  is very different, so we treat it separately
  if(missing(newx)){
    if(!match(type,c("coefficients","nonzero"),FALSE))stop("You need to supply a value for 'newx'")
  }
  if(exact&&(!is.null(s))){
    lambda=object$lambda
    which=match(s,lambda,FALSE)
    if(!all(which>0)){
      lambda=unique(rev(sort(c(s,lambda))))
      check_dots(object,...)# This fails if you don't supply the crucial arguments
      object=update(object,lambda=lambda,...)
    }
  }

  a0 = object$a0
  rownames(a0) = rep("(Intercept)", nrow(a0))
  nbeta = object$beta
  klam = dim(a0)
  nclass = klam[[1]]
  nlambda = length(s)
  if (!is.null(s)) {
    lambda = object$lambda
    lamlist = lambda.interp(lambda, s)
    for (i in seq(nclass)) {
      kbeta = methods::rbind2(a0[i, , drop = FALSE], nbeta[[i]])#was rbind2
      vnames = dimnames(kbeta)[[1]]
      dimnames(kbeta) = list(NULL, NULL)
      kbeta = kbeta[, lamlist$left, drop = FALSE] %*% Diagonal(x=lamlist$frac) +
        kbeta[, lamlist$right, drop = FALSE] %*% Diagonal(x=1 - lamlist$frac)
      dimnames(kbeta) = list(vnames, paste(seq(along = s)))
      nbeta[[i]] = kbeta
    }
  }
  else {
    for (i in seq(nclass)) nbeta[[i]] = methods::rbind2(a0[i, ], nbeta[[i]])#was rbind2
    nlambda=length(object$lambda)
  }
  if (type == "coefficients")
    return(nbeta)
  if (type == "nonzero")
#    if(object$grouped)return(nonzeroCoef(object$beta[[1]],bystep=TRUE))
    if(object$grouped)return(nonzeroCoef(nbeta[[1]],bystep=TRUE))
    else
      return(lapply(nbeta, function(x) nonzeroCoef(x[-1, ,
                                                   drop = FALSE], bystep = TRUE)))
  dd = dim(newx)
   if (inherits(newx, "sparseMatrix"))
    newx = as(newx, "dgCMatrix")
  npred = dd[[1]]
  dn = list(names(nbeta), dimnames(nbeta[[1]])[[2]], dimnames(newx)[[1]])
   dp = array(0, c(nclass, nlambda, npred), dimnames = dn)
  for (i in seq(nclass)) {
    fitk = cbind2(1, newx) %*% (nbeta[[i]])
    dp[i, , ] = dp[i, , ] + t(as.matrix(fitk))
  }
  if (object$offset) {
    if (missing(newoffset))
      stop("No newoffset provided for prediction, yet offset used in fit of glmnet",
           call. = FALSE)
    if (!is.matrix(newoffset) || dim(newoffset)[[2]] != nclass)
      stop(paste("Dimension of newoffset should be", npred, "x",
                 nclass), call. = FALSE)
    toff = t(newoffset)
    for (i in seq(nlambda)) {
      dp[, i, ] = dp[, i, ] + toff
    }
  }
  switch(type, response = {
    pp = exp(dp)
    psum = apply(pp, c(2, 3), sum)
    aperm(pp/rep(psum, rep(nclass, nlambda * npred)), c(3,
                                                        1, 2))
  }, link = aperm(dp, c(3, 1, 2)), class = {
    dp = aperm(dp, c(3, 1, 2))
    apply(dp, 3, glmnet_softmax)
  })
}

#' plot coefficients from a "glmnet" object
#'
#' Produces a coefficient profile plot of the coefficient paths for a fitted
#' \code{"glmnet"} object.
#'
#' A coefficient profile plot is produced. If \code{x} is a multinomial model,
#' a coefficient plot is produced for each class.
#'
#' @aliases plot.glmnet plot.multnet plot.mrelnet plot.relaxed
#' @param x fitted \code{"glmnet"} model
#' @param xvar What is on the X-axis. \code{"norm"} plots against the L1-norm
#' of the coefficients, \code{"lambda"} against the log-lambda sequence, and
#' \code{"dev"} against the percent deviance explained.
#' @param label If \code{TRUE}, label the curves with variable sequence
#' numbers.
#' @param \dots Other graphical parameters to plot
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}, and \code{print}, \code{predict} and \code{coef}
#' methods.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}
#' @keywords models regression
#'
#' @examples
#' x=matrix(rnorm(100*20),100,20)
#' y=rnorm(100)
#' g2=sample(1:2,100,replace=TRUE)
#' g4=sample(1:4,100,replace=TRUE)
#' fit1=glmnet(x,y)
#' plot(fit1)
#' plot(fit1,xvar="lambda",label=TRUE)
#' fit3=glmnet(x,g4,family="multinomial")
#' plot(fit3,pch=19)
#' @method plot glmnet
#' @export
plot.glmnet=function(x, xvar=c("norm","lambda","dev"),label=FALSE,...){
  xvar=match.arg(xvar)
 plotCoef(x$beta,lambda=x$lambda,df=x$df,dev=x$dev.ratio,label=label,xvar=xvar,...)
}

coxnet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,vnames,maxit){
  if(!is.matrix(y)||!all(match(c("time","status"),dimnames(y)[[2]],0)))stop("Cox model requires a matrix with columns 'time' (>0) and 'status'  (binary) as a response; a 'Surv' object suffices",call.=FALSE)
  ty=as.double(y[,"time"])
  tevent=as.double(y[,"status"])
  ty=ty+(1-tevent)*100*.Machine$double.eps## ties issue
  if(any(ty<=0))stop("negative event times encountered;  not permitted for Cox family")
  maxit=as.integer(maxit)
  weights=as.double(weights)
     if(is.null(offset)){
    offset=ty*0
    is.offset=FALSE}
  else{
    storage.mode(offset)="double"
    is.offset=TRUE
  }
  fit=if(is.sparse)stop("Cox model mot implemented for sparse x in glmnet",call.=FALSE)
  else .Fortran("coxnet",
              parm=alpha,nobs,nvars,as.double(x),ty,tevent,offset,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,
              maxit,isd,# need to get JHF to reverse these
              lmu=integer(1),
              ca=double(nx*nlam),
              ia=integer(nx),
              nin=integer(nlam),
              nulldev=double(1),
              dev=double(nlam),
              alm=double(nlam),
              nlp=integer(1),
              jerr=integer(1),PACKAGE="glmnet"
              )
 if(fit$jerr!=0){
  errmsg=jerr(fit$jerr,maxit,pmax=nx,family="cox")
  if(errmsg$fatal)stop(errmsg$msg,call.=FALSE)
  else warning(errmsg$msg,call.=FALSE)
}
  outlist=getcoef(fit,nvars,nx,vnames)
  dev=fit$dev[seq(fit$lmu)]
outlist=c(outlist,list(dev.ratio=dev,nulldev=fit$nulldev,npasses=fit$nlp,jerr=fit$jerr,offset=is.offset))
class(outlist)="coxnet"
outlist
}

getOptcv.glmnet <-
    function (lambda, cvm, cvsd,cvname)
{
    if(match(cvname,c("AUC","C-index"),0))cvm=-cvm
    cvmin = min(cvm, na.rm = TRUE)
    idmin = cvm <= cvmin
    lambda.min = max(lambda[idmin], na.rm = TRUE)
    idmin = match(lambda.min, lambda)
    semin = (cvm + cvsd)[idmin]
    id1se = cvm <= semin
    lambda.1se = max(lambda[id1se], na.rm = TRUE)
    id1se = match(lambda.1se, lambda)
    index=matrix(c(idmin,id1se),2,1,dimnames=list(c("min","1se"),"Lambda"))
    list(lambda.min = lambda.min, lambda.1se = lambda.1se, index = index)
}

cvcompute <-
    function (cvstuff, foldid, nlams)
{
    weights=cvstuff$weights
    mat=cvstuff$cvraw
    wisum = tapply(weights, foldid, sum)
    nfolds = max(foldid)
    outmat = matrix(NA, nfolds, ncol(mat))
    good = matrix(0, nfolds, ncol(mat))
    mat[is.infinite(mat)] = NA
    for (i in seq(nfolds)) {
        mati = mat[foldid == i, , drop = FALSE]
        wi = weights[foldid == i]
        outmat[i, ] = apply(mati, 2, weighted.mean, w = wi, na.rm = TRUE)
        good[i, seq(nlams[i])] = 1
    }
    N = apply(good, 2, sum)
    list(cvraw = outmat, weights = wisum, N = N, type.measure=cvstuff$type.measure)
}

error.bars <-
function(x, upper, lower, width = 0.02, ...)
{
	xlim <- range(x)
	barw <- diff(xlim) * width
	segments(x, upper, x, lower, ...)
	segments(x - barw, upper, x + barw, upper, ...)
	segments(x - barw, lower, x + barw, lower, ...)
	range(upper, lower)
}

cv.multnet <-function(predmat,y,type.measure,weights,foldid,grouped){
    prob_min = 1e-05
    prob_max = 1 - prob_min
    nc = dim(y)
    if (is.null(nc)) {
        y = as.factor(y)
        ntab = table(y)
        nc = as.integer(length(ntab))
        y = diag(nc)[as.numeric(y), ,drop=FALSE]
    }
    else nc = nc[2]

    ywt = apply(y, 1, sum)
    y = y/ywt
    weights = weights * ywt
    N = nrow(y) - apply(is.na(predmat[, 1, ,drop=FALSE]), 2, sum)## dimensions could be lost if third dim=1
    bigY = array(y, dim(predmat))
    predmat=exp(predmat)
    predtot=apply(predmat,c(1,3),sum)
    for(i in 1:nc)predmat[,i,]=predmat[,i,]/predtot
    cvraw = switch(type.measure,
                   mse = apply((bigY - predmat)^2,c(1, 3), sum),
                   mae = apply(abs(bigY - predmat), c(1,3), sum),
                   deviance = {                                                                                                              predmat = pmin(pmax(predmat, prob_min), prob_max)
                         lp = bigY * log(predmat)
                         ly = bigY * log(bigY)
                         ly[bigY == 0] = 0
                         apply(2 * (ly - lp), c(1, 3), sum)
                   },
                   class = {
                       classid = as.numeric(apply(predmat, 3, glmnet_softmax,ignore_labels=TRUE))
                       yperm = matrix(aperm(bigY, c(1, 3, 2)), ncol = nc)
                       matrix(1 - yperm[cbind(seq(classid), classid)], ncol = ncol(predtot))
                   }
                   )
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)
}

jerr.coxnet=function(n,maxit,pmax){

  if(n>0){#fatal error
    outlist=jerr.elnet(n)
    if(outlist$msg!="Unknown error")return(outlist)
    if(n==8888)msg="All observations censored - cannot proceed"  
    else if(n==9999)msg="No positive observation weights"
    else if(match(n,c(20000,30000),FALSE)) msg="Inititialization numerical error; probably too many censored observations"
    else 	 msg="Unknown error"
    list(n=n,fatal=TRUE,msg=msg)
  } else if(n<0){ # non-fatal error
    if(n<= -30000){
      msg=paste("Numerical error at ",-n-30000,"th lambda value; solutions for larger values of lambda returned",sep="")
      list(n=n,fatal=FALSE,msg=msg)
    }
    else
      jerr.elnet(n,maxit,pmax)
  }
}

#' print a cross-validated glmnet object
#'
#' Print a summary of the results of cross-validation for a glmnet model.
#'
#' A summary of the cross-validated fit is produced, slightly different for a
#' 'cv.relaxed' object than for a 'cv.glmnet' object.  Note that a 'cv.relaxed'
#' object inherits from class 'cv.glmnet', so by directly invoking
#' \code{print.cv.glmnet(object)} will print the summary as if
#' \code{relax=TRUE} had not been used.
#'
#' @aliases print.cv.glmnet print.cv.relaxed
#' @param x fitted 'cv.glmnet' object
#' @param digits significant digits in printout
#' @param \dots additional print arguments
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}, \code{predict} and \code{coef} methods.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}\cr \url{https://arxiv.org/abs/1707.08692}\cr Hastie, T.,
#' Tibshirani, Robert, Tibshirani, Ryan (2019) \emph{Extended Comparisons of
#' Best Subset Selection, Forward Stepwise Selection, and the Lasso}
#' @keywords models regression
#' @examples
#'
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' fit1 = cv.glmnet(x, y)
#' print(fit1)
#' fit1r = cv.glmnet(x, y, relax = TRUE)
#' print(fit1r)
#' ## print.cv.glmnet(fit1r)  ## CHECK WITH TREVOR
#' @method print cv.glmnet
#' @export
#' @export print.cv.glmnet
print.cv.glmnet <- function(x, digits = max(3, getOption("digits") - 3), ...)
{
    cat("\nCall: ", deparse(x$call), "\n\n")

    optlams=c(x$lambda.min,x$lambda.1se)
    which=match(optlams,x$lambda)
    mat = with(x, cbind(optlams, which, cvm[which], cvsd[which], nzero[which]))
    dimnames(mat) = list(c("min", "1se"), c("Lambda", "Index","Measure",
                                            "SE", "Nonzero"))
    cat("Measure:", x$name,"\n\n")

    mat=data.frame(mat,check.names=FALSE)
    class(mat)=c("anova",class(mat))
    print(mat,digits=digits)
}

#' Make response for coxnet
#'
#' Internal function to make the response y passed to glmnet suitable
#' for coxnet (i.e. glmnet with family = "cox"). Sanity checks are performed
#' here too.
#'
#' If y is a class "Surv" object, this function returns y with no changes. If
#' y is a two-column matrix with columns named 'time' and 'status', it is
#' converted into a "Surv" object.
#'
#' @param y Response variable. Either a class "Surv" object or a two-column
#' matrix with columns named 'time' and 'status'.
#'
#' @return A class "Surv" object.
#'
#' @importFrom survival Surv
response.coxnet <- function(y) {
  if (any(is.na(y))) stop(paste0("NAs encountered in response, not allowed"))

  # if Surv object, check that it is of correct type and perform sanity checks
  # One sanity check is that it have column names. If Surv() is called with a one-column matrix for
  # time, the name is lost
  # if all good, return with no changes
  if (is.Surv(y)) {
    if (attr(y, "type") == "right") {
      if (any(y[, 1] <= 0))
          stop("Non-positive event times encountered; not permitted for Cox family")
      colnames(y) <- c("time","status")
      return(y)
    } else if (attr(y, "type") == "counting") {
      if (any(y[, 1] < 0) || any(y[, 2] <= 0))
        stop(paste("Negative start/non-positive stop times encountered;",
                   "not permitted for Cox family"))
      if (any(y[, 1] >= y[, 2]))
        stop("Some rows have start time >= stop time; not permitted")
      colnames(y) <- c("start","stop","status")
      return(y)
    } else {
      stop("cox.path() only supports 'Surv' objects of type 'right' or 'counting'")
    }
  }

  # if two-column matrix passed, make it into a Surv object
  if (!is.matrix(y) || !all(match(c("time","status"),dimnames(y)[[2]],0)))
    stop(paste0("Cox model requires a matrix with columns 'time' (>0) and ",
                "'status' (binary) as a response; a 'Surv' object suffices"),
         call. = FALSE)
  ty <- as.double(y[,"time"])
  tevent <- as.double(y[,"status"])
  if (any(ty <= 0))
    stop("negative event times encountered; not permitted for Cox family")
  yob <- Surv(ty, tevent)
  colnames(yob) <- c("time","status")
  return(yob)
}

#' Generate multinomial samples from a probability matrix
#'
#' Generate multinomial samples
#'
#' Simple function that calls the \code{rmultinom} function. It generates a class label
#' for each row of its input matrix of class probabilities.
#'
#' @param p matrix of probabilities, with number of columns the number of classes
#' @return a vector of class memberships
#' @author Trevor Hastie \cr Maintainer: Trevor Hastie <hastie@@stanford.edu>
#' @export rmult
rmult <-
  function(p){
    x=t(apply(p,1,function(x)rmultinom(1,1,x)))
    x=x%*%seq(ncol(p))
    drop(x)
  }

#' @method predict mrelnet
#' @export
predict.mrelnet=function(object, newx, s = NULL, type = c("link", "response",
                                      "coefficients", "nonzero"), exact = FALSE, newoffset, ...)
  {
  type=match.arg(type)
  if(type=="response")type="link"
  object$grouped=TRUE
   predict.multnet(object,newx,s,type,exact,newoffset,...)
}

#' Extract coefficients from a glmnet object
#'
#' @method coef glmnet
#' @rdname predict.glmnet
#' @export
#' @export coef.glmnet
coef.glmnet=function(object,s=NULL,exact=FALSE,...)
  predict(object,s=s,type="coefficients",exact=exact,...)

#' assess performance of a 'glmnet' object using test data.
#'
#' Given a test set, produce summary performance measures for the glmnet
#' model(s)
#'
#' \code{assess.glmnet} produces all the different performance measures
#' provided by \code{cv.glmnet} for each of the families. A single vector, or a
#' matrix of predictions can be provided, or fitted model objects or CV
#' objects. In the case when the predictions are still to be made, the
#' \code{...} arguments allow, for example, 'offsets' and other prediction
#' parameters such as values for 'gamma' for 'relaxed' fits.  \code{roc.glmnet}
#' produces for a single vector a two column matrix with columns TPR and FPR
#' (true positive rate and false positive rate). This object can be plotted to
#' produce an ROC curve. If more than one predictions are called for, then a
#' list of such matrices is produced.  \code{confusion.glmnet} produces a
#' confusion matrix tabulating the classification results. Again, a single
#' table or a list, with a print method.
#'
#' @aliases assess.glmnet confusion.glmnet roc.glmnet
#' @param object Fitted \code{"glmnet"} or \code{"cv.glmnet"}, \code{"relaxed"}
#' or \code{"cv.relaxed"} object, OR a matrix of predictions (for
#' \code{roc.glmnet} or \code{assess.glmnet}). For \code{roc.glmnet} the model
#' must be a 'binomial', and for \code{confusion.glmnet} must be either
#' 'binomial' or 'multinomial'
#' @param newx If predictions are to made, these are the 'x' values. Required
#' for \code{confusion.glmnet}
#' @param newy required argument for all functions; the new response values
#' @param weights For observation weights for the test observations
#' @param family The family of the model, in case predictions are passed in as
#' 'object'
#' @param ... additional arguments to \code{predict.glmnet} when "object" is a
#' "glmnet" fit, and predictions must be made to produce the statistics.
#' @return \code{assess.glmnet} produces a list of vectors of measures.
#' \code{roc.glmnet} a list of 'roc' two-column matrices, and
#' \code{confusion.glmnet} a list of tables. If a single prediction is
#' provided, or predictions are made from a CV object, the latter two drop the
#' list status and produce a single matrix or table.
#' @author Trevor Hastie and Rob Tibshirani\cr Maintainer: Trevor Hastie
#' <hastie@@stanford.edu>
#' @seealso \code{cv.glmnet}, \code{glmnet.measures} and \code{vignette("relax",package="glmnet")}
#' @keywords models classification
#' @examples
#'
#' data(QuickStartExample)
#' x <- QuickStartExample$x; y <- QuickStartExample$y
#' set.seed(11)
#' train = sample(seq(length(y)),70,replace=FALSE)
#' fit1 = glmnet(x[train,], y[train])
#' assess.glmnet(fit1, newx = x[-train,], newy = y[-train])
#' preds = predict(fit1, newx = x[-train, ], s = c(1, 0.25))
#' assess.glmnet(preds, newy = y[-train], family = "gaussian")
#' fit1c = cv.glmnet(x, y, keep = TRUE)
#' fit1a = assess.glmnet(fit1c$fit.preval, newy=y,family="gaussian")
#' plot(fit1c$lambda, log="x",fit1a$mae,xlab="Log Lambda",ylab="Mean Absolute Error")
#' abline(v=fit1c$lambda.min, lty=2, col="red")
#' data(BinomialExample)
#' x <- BinomialExample$x; y <- BinomialExample$y
#' fit2 = glmnet(x[train,], y[train], family = "binomial")
#' assess.glmnet(fit2,newx = x[-train,], newy=y[-train], s=0.1)
#' plot(roc.glmnet(fit2, newx = x[-train,], newy=y[-train])[[10]])
#' fit2c = cv.glmnet(x, y, family = "binomial", keep=TRUE)
#' idmin = match(fit2c$lambda.min, fit2c$lambda)
#' plot(roc.glmnet(fit2c$fit.preval, newy = y)[[idmin]])
#' data(MultinomialExample)
#' x <- MultinomialExample$x; y <- MultinomialExample$y
#' set.seed(103)
#' train = sample(seq(length(y)),100,replace=FALSE)
#' fit3 = glmnet(x[train,], y[train], family = "multinomial")
#' confusion.glmnet(fit3, newx = x[-train, ], newy = y[-train], s = 0.01)
#' fit3c = cv.glmnet(x, y, family = "multinomial", type.measure="class", keep=TRUE)
#' idmin = match(fit3c$lambda.min, fit3c$lambda)
#' confusion.glmnet(fit3c$fit.preval, newy = y, family="multinomial")[[idmin]]
#'
#' @export assess.glmnet
assess.glmnet=function(object,newx=NULL,newy,weights=NULL,
                       family=c("gaussian","binomial","poisson","multinomial","cox","mgaussian"),
                       ...){
### object must be either a glmnet or cv.glmnet object, or else a matrix/vector of predictions of a glmnet model  (not on the mean scale, but natural parameter scale)
### It must either be a matrix or an array. The last dimension can be 1.
    oclass=grep("glmnet",class(object))
    if(length(oclass)){# either a glmnet or cv.glmnet
        predmat=predict(object,newx=newx,...)
        fam=family(object)
    }
    else{
        if(is.character(family))fam=match.arg(family)
        else {
            if(is.function(family))family=family()
            if(!inherits(family,"family"))
                stop("Invalid family argument; must be either character, function or family object")
            fam=family
            }

        predmat=object
        if(is.vector(predmat))predmat=as.matrix(predmat)
    }
    famtype=fam
    attr(predmat,"family")=fam
    if(inherits(fam,"family"))famtype="GLM"
    type.measures = glmnet.measures(famtype)
    y = drop(newy)
    dimy=dim(y)
    nrowy=ifelse(is.null(dimy),length(y),dimy[1])
    if (is.null(weights))
        weights = rep(1, nrowy)
    subclass=switch(famtype,gaussian="elnet",binomial="lognet",poisson="fishnet",multinomial="multnet",
                    cox="coxnet",mgaussian="mrelnet",GLM="glmnetfit")
    fun = paste("cv", subclass, sep = ".")
    if(subclass=="coxnet")fun="assess.coxnet"
    outlist=as.list(type.measures)
    names(outlist)=type.measures
    for(type.measure in type.measures){
    type.measure=cvtype(type.measure,subclass)
    teststuff = do.call(fun, list(predmat,y,type.measure,weights,foldid=rep(1,nrowy),grouped=FALSE))
    out=drop(with(teststuff, apply(cvraw, 2, weighted.mean, w = weights, na.rm = TRUE)))
    attr(out,"measure")=names(type.measure)
    outlist[[type.measure]]=out
    }
    outlist
}

assess.coxnet <- function(predmat,y,type.measure,weights,foldid,grouped){
### This is a hack, because CV for coxnet is complex
if(type.measure=="C")return(cv.coxnet(predmat,y,type.measure,weights,foldid,grouped))
    nlambda=ncol(predmat)
    N=nrow(predmat)
    cvraw = matrix(NA, 1, nlambda)
    for (j in seq(nlambda)) {
        cvraw[1, j] = coxnet.deviance(predmat[,j],y, weights=weights)
            }
    list(cvraw=cvraw,weights=N,N=N,type.measure=type.measure,grouped=FALSE)
}

getcoef=function(fit,nvars,nx,vnames){
  lmu=fit$lmu
  if(lmu<1){
    ## changed this to a warning message, and return an empty model
    warning("an empty model has been returned; probably a convergence issue")
    coefob=list(a0=fit$a0,beta=zeromat(nvars,as.integer(1),vnames,"s0"),df=0,dim=c(nvars,1),lambda=Inf)
    return(coefob)
  }
  nin=fit$nin[seq(lmu)]
  ninmax=max(nin)
  lam=fit$alm[seq(lmu)]

  stepnames=paste("s",seq(lmu)-1,sep="")
  dd=c(nvars,lmu)
  if(ninmax>0){
           ca=matrix(fit$ca[seq(nx*lmu)],nx,lmu)[seq(ninmax),,drop=FALSE]
           df=apply(abs(ca)>0,2,sum)
           ja=fit$ia[seq(ninmax)]
####confusing but too hard to change
###glmnet builds a list of ever active variables which is nondecreasing
###Since ca was initialized to zero, no harm is done in passing a square matrix
###to new(); then when we do a drop0 it makes it really sparse
           oja=order(ja)
           ja=rep(ja[oja],lmu)
           ia=cumsum(c(1,rep(ninmax,lmu)))
           beta=drop0(new("dgCMatrix",Dim=dd,Dimnames=list(vnames,stepnames),x=as.vector(ca[oja,]),p=as.integer(ia-1),i=as.integer(ja-1)))
         }else {
           beta = zeromat(nvars,lmu,vnames,stepnames)
           df=rep(0,lmu)
         }
  a0=fit$a0
  if(!is.null(a0)){#for Cox model
    a0=a0[seq(lmu)]
  names(a0)=stepnames
  }
  list(a0=a0,beta=beta,df=df,dim=dd,lambda=lam)
}

#' Extract the deviance from a glmnet object
#'
#' Compute the deviance sequence from the glmnet object
#'
#' A glmnet object has components \code{dev.ratio} and \code{nulldev}.  The
#' former is the fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the
#' log-likelihood for the saturated model (a model with a free parameter per
#' observation).  Null deviance is defined to be 2*(loglike_sat
#' -loglike(Null)); The NULL model refers to the intercept model, except for
#' the Cox, where it is the 0 model. Hence dev.ratio=1-deviance/nulldev, and
#' this \code{deviance} method returns (1-dev.ratio)*nulldev.
#'
#' @param object fitted glmnet object
#' @param \dots additional print arguments
#' @return (1-dev.ratio)*nulldev
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet}, \code{predict}, \code{print}, and \code{coef}
#' methods.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}
#' @keywords models regression
#' @examples
#'
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' fit1 = glmnet(x, y)
#' deviance(fit1)
#' @method deviance glmnet
#' @export
deviance.glmnet=function(object,...){
dev=object$dev
nulldev=object$nulldev
(1-dev)*nulldev
}

#' Compute gradient for Cox model
#'
#' Compute the gradient of the log partial likelihood at a particular fit for Cox
#' model.
#'
#' Compute a gradient vector at the fitted vector for the log partial likelihood.
#' This is like a residual vector, and useful for manual screening of
#' predictors for \code{glmnet} in applications where \code{p} is very large
#' (as in GWAS). Uses the Breslow approach to ties.
#'
#' This function is essentially a wrapper: it checks whether the response
#' provided is right-censored or (start, stop] survival data, and calls the
#' appropriate internal routine.
#'
#' @aliases coxgrad
#' @param eta Fit vector (usually from glmnet at a particular lambda).
#' @param y Survival response variable, must be a \code{Surv} or
#' \code{stratifySurv} object.
#' @param w Observation weights (default is all equal to 1).
#' @param std.weights If TRUE (default), observation weights are standardized
#' to sum to 1.
#' @param diag.hessian If \code{TRUE}, compute the diagonal of the Hessian
#' of the log partial likelihood as well. Default is \code{FALSE}.
#'
#' @return A single gradient vector the same length as \code{eta}. If
#' \code{diag.hessian=TRUE}, the diagonal of the Hessian is
#' included as an attribute "diag_hessian".
#'
#' @examples
#' set.seed(1)
#' eta <- rnorm(10)
#' time <- runif(10, min = 1, max = 10)
#' d <- ifelse(rnorm(10) > 0, 1, 0)
#' y <- survival::Surv(time, d)
#' coxgrad(eta, y)
#'
#' # return diagonal of Hessian as well
#' coxgrad(eta, y, diag.hessian = TRUE)
#'
#' # example with (start, stop] data
#' y2 <- survival::Surv(time, time + runif(10), d)
#' coxgrad(eta, y2)
#'
#' # example with strata
#' y2 <- stratifySurv(y, rep(1:2, length.out = 10))
#' coxgrad(eta, y2)
#'
#' @seealso \code{coxnet.deviance}
#' @keywords Cox model
#'
#' @export
coxgrad <- function(eta, y, w, std.weights = TRUE, diag.hessian = FALSE) {
    # if y has 2 columns, it is right-censored data
    # if y has 3 columns, it is (start, stop] data
    # otherwise, throw errors
    if (ncol(y) == 2) {
        return(coxgrad2(eta, y, w, std.weights, diag.hessian))
    } else if (ncol(y) == 3) {
        return(coxgrad3(eta, y, w, std.weights, diag.hessian))
    } else {
        stop("Response y should have 2 or 3 columns")
    }
}

# coxgrad routine for right-censored data
coxgrad2 <- function(eta, y, w, std.weights = TRUE, diag.hessian = FALSE) {
    if (missing(w)) w=rep(1,length(eta))
    if (std.weights) w=w/sum(w)
    nobs <- nrow(y)

    # extract strata (if any)
    if ("strata" %in% names(attributes(y))) {
        strata <- attr(y, "strata")
    } else {
        strata <- rep(1, nobs)
    }
    if (length(strata) != nobs) stop("length of strata != nobs")

    # if all in same strata, do the computations
    # if not, do strata-level computations and concatenate
    if (length(unique(strata)) == 1) {
        time <- y[, "time"]
        d    <- y[, "status"]
        eta <- scale(eta, TRUE, FALSE)  # center eta so exponents are not too large

        # order exp(eta), time, d and w in ascending time order
        # for tied times, all deaths come before censored observations
        if ("stop_time" %in% names(attributes(y))) {
            o <- attr(y, "stop_time")
        } else {
            o <- order(time, d, decreasing = c(FALSE, TRUE))
        }
        exp_eta <- exp(eta)[o]
        time <- time[o]
        d <- d[o]
        w <- w[o]
        rskden <- rev(cumsum(rev(exp_eta*w))) ##reverse order inside;last guy is in all the risk sets

        ### See if there are dups in death times
        dups <- fid(time[d == 1],seq(length(d))[d == 1])
        dd <- d
        ww <- w

        ### next code replaces each sequence of tied death indicators by a new
        ### sequence where only the first is a 1 and the rest are zero. This
        ### makes the accounting in the following step work properly we also
        ### sums the weights in each of the tied death sets, and assign that
        ### weight to the first
        if(!is.null(ties<-dups$index_ties)){
            dd[unlist(ties)]=0
            dd[dups$index_first]=1
            wsum=sapply(ties,function(i,w)sum(w[i]),ww)
            tie1=sapply(ties,function(i)i[1])
            ww[tie1]=wsum
        }

        ### Get counts over risk sets at each death time
        rskcount=cumsum(dd)#this says how many of the risk sets each observation is in; 0 is none
        ### We now form partial sums of the 1/den just at the risk sets
        rskdeninv=cumsum((ww/rskden)[dd==1])
        ### pad with a zero, so we can index it
        rskdeninv=c(0,rskdeninv)

        ### compute gradient for each obs
        grad <- w * (d - exp_eta * rskdeninv[rskcount+1])
        grad[o] <- grad

        # if diag.hessian = TRUE, return the diagonal of the hessian too
        if (diag.hessian) {
            rskdeninv2 <- cumsum((ww/(rskden^2))[dd==1])
            rskdeninv2 <- c(0, rskdeninv2)
            w_exp_eta <- w * exp_eta
            diag_hessian <- w_exp_eta^2 * rskdeninv2[rskcount+1] - w_exp_eta * rskdeninv[rskcount+1]
            diag_hessian[o] <- diag_hessian
            attr(grad, "diag_hessian") <- diag_hessian
        }
        return(grad)
    } else {
        # more than one strata provided: compute strata-level values and
        # concatenate
        overall_grad <- rep(NA, nobs)
        if (diag.hessian) overall_diag_hessian <- rep(NA, nobs)
        for (i in unique(strata)) {
            ii <- which(strata == i)
            strata_res <- coxgrad2(eta[ii], y[ii, , drop = FALSE], w[ii],
                                   std.weights = FALSE, diag.hessian = diag.hessian)
            overall_grad[ii] <- strata_res
            if (diag.hessian) {
                overall_diag_hessian[ii] <- attr(strata_res, "diag_hessian")
            }
        }
        if (diag.hessian) {
            attr(overall_grad, "diag_hessian") <- overall_diag_hessian
        }
        return(overall_grad)
    }
}

# coxgrad routine for (start, stop] data
coxgrad3 <- function(eta, y, w, std.weights = TRUE, diag.hessian = FALSE) {
    if (missing(w)) w=rep(1,length(eta))
    if (std.weights) w=w/sum(w)
    nobs <- nrow(y)

    # extract strata (if any)
    if ("strata" %in% names(attributes(y))) {
        strata <- attr(y, "strata")
    } else {
        strata <- rep(1, nobs)
    }
    if (length(strata) != nobs) stop("length of strata != nobs")

    # if all in same strata, do the computations
    # if not, do strata-level computations and concatenate
    if (length(unique(strata)) == 1) {
        start_time <- y[, "start"]
        stop_time  <- y[, "stop"]
        d          <- y[, "status"]
        eta <- scale(eta, TRUE, FALSE)  # center eta so exponents are not too large

        # get ordering for stop time (ascending, deaths before censored),
        # start time (ascending), and match info if cached
        if ("stop_time" %in% names(attributes(y))) {
            stop_o <- attr(y, "stop_time")
        } else {
            stop_o <- order(stop_time, d, decreasing = c(FALSE, TRUE))
        }
        if ("start_time" %in% names(attributes(y))) {
            start_o <- attr(y, "start_time")
        } else {
            start_o <- order(start_time, decreasing = c(FALSE))
        }
        if ("ss_match" %in% names(attributes(y))) {
            ss_match <- attr(y, "ss_match")
        } else {
            ss_match <- match(start_o, stop_o)
        }
    ## set weights to zero for any observation with start time greater than largest death time
    last_death <- max(stop_time[d==1])
    w[start_time >= last_death] <- 0
    ##

        # keep a set of values which are ordered by start time
        w_exp_eta_start <- (w * exp(eta))[start_o]
        start_time_start <- start_time[start_o]

        # reorder everything by stop time
        exp_eta <- exp(eta)[stop_o]
        start_time <- start_time[stop_o]
        stop_time <- stop_time[stop_o]
        d <- d[stop_o]
        w <- w[stop_o]

        ### See if there are dups in death times
        dups <- fid(stop_time[d == 1],seq(length(d))[d == 1])
        dd <- d
        ww <- w

        ### next code replaces each sequence of tied death indicators by a new
        ### sequence where only the first is a 1 and the rest are zero. This
        ### makes the accounting in the following step work properly we also
        ### sums the weights in each of the tied death sets, and assign that
        ### weight to the first
        if(!is.null(ties<-dups$index_ties)){
            dd[unlist(ties)]=0
            dd[dups$index_first]=1
            wsum=sapply(ties,function(i,w)sum(w[i]),ww)
            tie1=sapply(ties,function(i)i[1])
            ww[tie1]=wsum
        }

        # compute risk set sums rskden[i] = \sum_{j in R_i} w_j exp(eta_j)
        # where i indexes the observations. (In the end, we will only care
        # about the indices i which have actual death times.)
        rskden <- rev(cumsum(rev(exp_eta*w)))
        current_sum <- 0
        death_time <- stop_time[dups$index_first]
        ndeaths <- length(death_time)
        death_idx <- ndeaths; start_idx <- nobs
        while (death_idx > 0 && start_idx > 0) {
            if (start_time_start[start_idx] < death_time[death_idx]) {
                # current start time belongs in risk set ending in stop time,
                # so we should remove the current cumulative sum and consider
                # the next risk set
                stop_idx <- dups$index_first[death_idx]
                rskden[stop_idx] <- rskden[stop_idx] - current_sum
                death_idx <- death_idx - 1
            } else {
                # current start time does not belong in risk set ending in stop
                # time, so we should add it to current_sum and check if the
                # start time before it should also be added
                current_sum <- current_sum + w_exp_eta_start[start_idx]
                start_idx <- start_idx - 1
            }
        }

        # compute the terms rskterm[k] = \sum_{i in C_k} d[i] / rskden[i] and
        # rskterm2[k] = \sum_{i in C_k} d[i] / rskden[i]^2.
        # Here, k indexes the observations, index i runs over the unique death
        # times.
        rskfactor  <- (ww / rskden)[dd == 1]
        rskfactor2 <- (ww / rskden^2)[dd == 1]
        rskdeninv  <- c(0, cumsum(rskfactor))  # pad with 0 so that we can index
        rskdeninv2 <- c(0, cumsum(rskfactor2))

        # this says how many of the risk sets each observation is in; 0 is none
        # (however, if start time is not zero, then we could be including an
        # observation in too many risk sets: we will remove that later.)
        rskcount <- cumsum(dd)
        rskterm  <- rskdeninv[rskcount+1]
        rskterm2 <- rskdeninv2[rskcount+1]
        current_sum <- 0; current_sum2 <- 0
        death_idx <- 1; start_idx <- 1
        while (death_idx <= ndeaths && start_idx <= nobs) {
            if (start_time_start[start_idx] < death_time[death_idx]) {
                # current observation belongs in risk set ending in death time,
                # so we should remove the current cumulative sum and consider
                # the next observation
                stop_idx <- ss_match[start_idx]  # match(start_o[start_idx], stop_o)
                rskterm[stop_idx]  <- rskterm[stop_idx]  - current_sum
                rskterm2[stop_idx] <- rskterm2[stop_idx] - current_sum2
                start_idx <- start_idx + 1
            } else {
                # current observation doesn't belong in risk set ending in death
                # time, so we should add the rskfactor associated with this
                # death time to current_sum and check if the term assoc. with
                # the death time after it should also be added
                current_sum  <- current_sum  + rskfactor[death_idx]
                current_sum2 <- current_sum2 + rskfactor2[death_idx]
                death_idx <- death_idx + 1
            }
        }
        grad <- w * (d - exp_eta * rskterm)
        grad[stop_o] <- grad

        # if diag.hessian = TRUE, return the diagonal of the hessian too
        if (diag.hessian) {
            w_exp_eta <- w * exp_eta
            diag_hessian <- w_exp_eta^2 * rskterm2 - w_exp_eta * rskterm
            diag_hessian[stop_o] <- diag_hessian
            attr(grad, "diag_hessian") <- diag_hessian
        }
        return(grad)
    } else {
        # more than one strata provided: compute strata-level values and
        # concatenate
        overall_grad <- rep(NA, nobs)
        if (diag.hessian) overall_diag_hessian <- rep(NA, nobs)
        for (i in unique(strata)) {
            ii <- which(strata == i)
            strata_res <- coxgrad3(eta[ii], y[ii, , drop = FALSE], w[ii],
                                   std.weights = FALSE, diag.hessian = diag.hessian)
            overall_grad[ii] <- strata_res
            if (diag.hessian) {
                overall_diag_hessian[ii] <- attr(strata_res, "diag_hessian")
            }
        }
        if (diag.hessian) {
            attr(overall_grad, "diag_hessian") <- overall_diag_hessian
        }
        return(overall_grad)
    }
}

#' Helper function for Cox deviance and gradient
#'
#' Helps to find ties in death times of data.
#'
#' @param x Sorted vector of death times.
#' @param index Vector of indices for the death times.
#'
#' @return A list with two arguments.
#' \item{index_first}{A vector of indices for the first observation at each
#' death time as they appear in the sorted list.}
#' \item{index_ties}{If there are no ties at all, this is NULL. If not, this is
#' a list with length equal to the number of unique times with ties. For each
#' time with ties, index_ties gives the indices of the observations with a
#' death at that time.}
#'
#' @examples
#' # Example with no ties
#' glmnet:::fid(c(1, 4, 5, 6), 1:5)
#'
#' # Example with ties
#' glmnet:::fid(c(1, 1, 1, 2, 3, 3, 4, 4, 4), 1:9)
fid <- function(x,index) {
    idup=duplicated(x)
    if(!any(idup)) list(index_first=index,index_ties=NULL)
    else {
        ndup=!idup
        xu=x[ndup]# first death times
        index_first=index[ndup]
        ities=match(x,xu)
        index_ties=split(index,ities)
        nties=sapply(index_ties,length)
        list(index_first=index_first,index_ties=index_ties[nties>1])
    }
}

#' Compute a survival curve from a coxnet object
#'
#' Computes the predicted survivor function for a Cox proportional hazards
#' model with elastic net penalty.
#'
#' To be consistent with other functions in \code{glmnet}, if \code{s}
#' is not specified, survival curves are returned for the entire lambda
#' sequence. This is not recommended usage: it is best to call
#' \code{survfit.coxnet} with a single value of the penalty parameter
#' for the \code{s} option.
#'
#' @param formula A class \code{coxnet} object.
#' @param s Value(s) of the penalty parameter lambda at which the survival
#' curve is required. Default is the entire sequence used to create the model.
#' However, it is recommended that \code{survfit.coxnet} is called for
#' a single penalty parameter.
#' @param ... This is the mechanism for passing additional arguments like
#' (i) x= and y= for the x and y used to fit the model,
#' (ii) weights= and offset= when the model was fit with these options,
#' (iii) arguments for new data (newx, newoffset, newstrata), and
#' (iv) arguments to be passed to survfit.coxph().
#'
#' @return If \code{s} is a single value, an object of class "survfitcox"
#' and "survfit" containing one or more survival curves. Otherwise, a list
#' of such objects, one element for each value in \code{s}.
#' Methods defined for survfit objects are print, summary and plot.
#'
#' @examples
#' set.seed(2)
#' nobs <- 100; nvars <- 15
#' xvec <- rnorm(nobs * nvars)
#' xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
#' x <- matrix(xvec, nrow = nobs)
#' beta <- rnorm(nvars / 3)
#' fx <- x[, seq(nvars / 3)] %*% beta / 3
#' ty <- rexp(nobs, exp(fx))
#' tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
#' y <- survival::Surv(ty, tcens)
#' fit1 <- glmnet(x, y, family = "cox")
#'
#' # survfit object for Cox model where lambda = 0.1
#' sf1 <- survival::survfit(fit1, s = 0.1, x = x, y = y)
#' plot(sf1)
#'
#' # example with new data
#' sf2 <- survival::survfit(fit1, s = 0.1, x = x, y = y, newx = x[1:3, ])
#' plot(sf2)
#'
#' # example with strata
#' y2 <- stratifySurv(y, rep(1:2, length.out = nobs))
#' fit2 <- glmnet(x, y2, family = "cox")
#' sf3 <- survival::survfit(fit2, s = 0.1, x = x, y = y2)
#' sf4 <- survival::survfit(fit2, s = 0.1, x = x, y = y2,
#'                newx = x[1:3, ], newstrata = c(1, 1, 1))
#'
#' @importFrom survival coxph survfit
#' @method survfit coxnet
#' @export
survfit.coxnet <- function(formula, s = NULL, ...) {
  this.call <- match.call()
  object <- formula
  args <- list(...)

  if (!("x" %in% names(args)) || !("y" %in% names(args)))
    stop(paste0("the design matrix x and response y used to fit the model ",
                "need to be passed"))
  y <- args$y

  # if s is NULL, get the whole lambda sequence
  if (is.null(s)) s <- object$lambda

  # check that the required arguments are passed for running coxph
  check_dots(object, ...,
             need = c("offset", "weights"),
             error_start = "used survfit.coxnet()",
             error_end = "in order to run survfit.coxnet")
  # if user wants to run survfit on new data, make sure the arguments required
  # for making new predictions are provided
  if ("newx" %in% names(args)) {
    check_dots(object, ...,
               need = c("offset"),
               error_start = "used survfit.coxnet() with newx argument",
               prefix = "new",
               error_end = "in order to predict on new data")

    if ("strata" %in% names(attributes(y)) &&
        !("newstrata" %in% names(args)))
      stop(paste0("used survfit.coxnet() with newx argument and ",
                  "stratified Cox model was fit, need newstrata argument ",
                  "in order to predict on new data"))
  }

  res <- list()
  for (i in seq_along(s)) {
    # "hack": run coxph with 0 iterations
    coxphmod <- mycoxph(object, s = s[i], ...)

    # If newx is provided, we need to compute the predections at these
    # observations and pass it as an argument to the eventual survfit call
    # as `newdata`. In addition, we have to handle additional options that
    # could have been passed: offset
    if ("newx" %in% names(args)) {
      current_args <- mycoxpred(object, s = s[i], ...)
    } else {
      current_args <- args
    }

    # make the call to survfit.coxph
    current_args$formula <- coxphmod
    current_args$se.fit <- FALSE  # doesn't make sense to compute SEs
    sf <- do.call(survfit, current_args)
    sf$call <- this.call
    res[[i]] <- sf
  }
  if (length(s) > 1) {
    return(res)
  } else {
    return(res[[1]])
  }
}

#' Helper function to fit coxph model for survfit.coxnet
#'
#' This function constructs the coxph call needed to run the "hack" of
#' coxph with 0 iterations. It's a separate function as we have to deal with
#' function options like strata, offset and observation weights.
#'
#' @param object A class \code{coxnet} object.
#' @param s The value of the penalty parameter lambda at which the survival
#' curve is required.
#' @param ... The same ... that was passed to survfit.coxnet.
#'
#' @importFrom survival coxph strata
#' @importFrom stats as.formula
mycoxph <- function(object, s, ...) {
  args <- list(...)
  x <- args$x
  y <- args$y
  glmnet_call_names <- names(object$call)[-1]

  # predict from coxnet model for original data frame x with s, gives fitted
  # linear predictor
  args$object <- object
  args$newx <- x
  args$s <- s
  if ("offset" %in% glmnet_call_names) {
    args$newoffset <- rep(0, length.out = nrow(x))
  }
  eta <- do.call(predict, args)

  # construct list of arguments for coxph() call based on which special
  # arguments were used in the original glmnet() call
  coxphargs <- list(formula = "y ~ X1", data = data.frame(y, eta),
                    init = 1, iter = 0)

  if ("strata" %in% names(attributes(y))) {
    coxphargs$data$strata <- attr(y, "strata")
    coxphargs$formula <- paste(coxphargs$formula, "+ strata(strata)")
  }
  if ("weights" %in% glmnet_call_names) {
    coxphargs$weights <- args$weights
  }
  if ("offset" %in% glmnet_call_names) {
    coxphargs$data$offset <- args$offset
    coxphargs$formula <- paste(coxphargs$formula, "+ offset(offset)")
  }
  coxphargs$formula <- as.formula(coxphargs$formula)
  coxphmod <- do.call(coxph, coxphargs)
  return(coxphmod)
}

#' Helper function to amend ... for new data in survfit.coxnet
#'
#' This function amends the function arguments passed to survfit.coxnet
#' via ... if new data was passed to survfit.coxnet. It's a separate
#' function as we have to deal with function options like newstrata
#' and newoffset.
#'
#' @param object A class \code{coxnet} object.
#' @param s The response for the fitted model.
#' @param ... The same ... that was passed to survfit.coxnet.
#'
#' @importFrom stats predict
mycoxpred <- function(object, s, ...) {
  args <- list(...)

  if ("newoffset" %in% names(args)) {
    new_eta <- predict(object, newx = args$newx, s = s,
                       newoffset = rep(0, nrow(args$newx)))
  } else {
    new_eta <- predict(object, newx = args$newx, s = s)
  }
  new_df <- data.frame(new_eta)
  if ("newoffset" %in% names(args)) new_df$offset <- args$newoffset

  if ("newstrata" %in% names(args)) {
    new_df$strata <- args$newstrata
    args$newstrata <- NULL
  }

  args$newdata <- new_df
  return(args)
}

#' Fit a relaxed glmnet model
#'
#' @param fit For \code{relax.glmnet} a fitted 'glmnet' object
#' @param maxp a limit on how many relaxed coefficients are allowed. Default is
#' 'n-3', where 'n' is the sample size. This may not be sufficient for
#' non-gaussian familes, in which case users should supply a smaller value.
#' This argument can be supplied directly to 'glmnet'.
#' @param path Since \code{glmnet} does not do stepsize optimization, the Newton
#' algorithm can get stuck and not converge, especially with relaxed fits. With \code{path=TRUE},
#' each relaxed fit on a particular set of variables is computed pathwise using the original sequence
#' of lambda values (with a zero attached to the end). Not needed for Gaussian models, and should not
#' be used unless needed, since will lead to longer compute times. Default is \code{path=FALSE}.
#' appropriate subset of variables
#' @param check.args Should \code{relax.glmnet} make sure that all the data
#' dependent arguments used in creating 'fit' have been resupplied. Default is
#' 'TRUE'.
#' @rdname glmnet
#' @export relax.glmnet
relax.glmnet=function(fit,x,..., maxp = n-3, path=FALSE, check.args=TRUE){

    ## maxp could be a sensitive parameter, especially for glms
    ## This next call fails if you don't supply the crucial arguments
    ## that were used in the creation of "fit". Not needed if called relax.glmnet
    if(check.args)check_dots(fit,...,
                need=c("y","weights","offset","lower.limits","upper.limits"),
                error="used relax.glmnet(),")
    d=dim(x)
    n=d[1];p=d[2]
    ## extract coefs, and find out who is nonzero at each lambda
    a0=fit$a0
    beta=fit$beta
    islistbeta=is.list(beta)
    if(islistbeta){
        ngroups=length(beta)
        nzmat=abs(beta[[1]])
        for(k in 2:ngroups)nzmat=nzmat+abs(beta[[k]])
    }
    else    nzmat=abs(beta)
    nzmat=nzmat>0
    ## Here we go to some trouble to make sure we dont fit the same model more than once
    rgauss=sort(abs(rnorm(p)))
    undex=colSums(rgauss*nzmat)#a unique number for each distinct set of nonzeros
    sundex=sort(unique(undex[undex>0]))
    nzindex=match(sundex,undex)# get a match for each unique
    alldex=match(undex,sundex,0)# for putting them back
    ##
    dev=fit$dev.ratio
    nulldev=fit$nulldev
    excluders=seq(p)
    lam0 = if(path)c(fit$lambda,0)else 0
### Fit a coefficient for each non null model
    internal.parms  <- glmnet.control()
    if (internal.parms$itrace) {
        glmnet.control(itrace = 0) ## disable glmnet reporting
        cat("Relaxed Fits\n")
        pb  <- utils::txtProgressBar(min = 0, max = length(sundex), initial = 0, style = 3)
        on.exit(glmnet.control(itrace = internal.parms$itrace))
        on.exit(close(pb), add = TRUE)
    }
    for(i in seq(along=sundex)){
        ind=nzmat[,nzindex[i]]
        wh=alldex==i
        pi=sum(ind)
        if(pi>maxp)next #can only fit maxp coefficients
        exclude=excluders[!ind]
        ifit=update(fit,x=x,lambda=lam0,exclude=exclude,...,relax=FALSE)
        nlam=ifit$dim[2]
        if (islistbeta) {
            a0[, wh] = ifit$a0[,nlam]
            for (i in 1:ngroups) beta[[i]][, wh] = ifit$beta[[i]][,nlam]
        }
        else {
            a0[wh] = ifit$a0[nlam]
            beta[, wh] = ifit$beta[,nlam]
        }
        dev[wh] = ifit$dev.ratio[nlam]
        if (internal.parms$itrace) utils::setTxtProgressBar(pb, i)
    }
    if (internal.parms$itrace) {
        utils::setTxtProgressBar(pb = pb, value = length(sundex))
    }

    fitg=fit
    omit=colSums(nzmat)>maxp
    if(any(omit)){
        if(islistbeta){
            a0=a0[,!omit]
            for(i in 1:ngroups)beta[[i]]=beta[[i]][,!omit]
        }
        else{
            beta=beta[,!omit]
            a0=a0[!omit]
            }
        dev=dev[!omit]
        fit$lambda=fit$lambda[!omit]
        fit$df=fit$df[!omit]
        }
    fit$beta=beta
    fit$a0=a0
    fit$dev.ratio=dev
    class(fitg)=c("relaxed",class(fitg))
    fitg$relaxed=fit
    fitg
    }

cv.glmnet.raw <-
  function (x, y, weights, offset, lambda, type.measure, nfolds, foldid, alignment,grouped, keep,
              parallel, trace.it, glmnet.call, cv.call, ...)
{


  if (trace.it) cat("Training\n")
  glmnet.object = glmnet(x, y, weights = weights, offset = offset,
    lambda = lambda, trace.it=trace.it,...)
  glmnet.object$call = glmnet.call
  subclass=class(glmnet.object)[[1]]
  type.measure=cvtype(type.measure,subclass)
  is.offset = glmnet.object$offset
###Next line is commented out so each call generates its own lambda sequence
# lambda=glmnet.object$lambda
 if (inherits(glmnet.object, "multnet") && !glmnet.object$grouped) {
    nz = predict(glmnet.object, type = "nonzero")
    nz = sapply(nz, function(x) sapply(x, length))
    nz = ceiling(apply(nz, 1, median))
  }
  else nz = sapply(predict(glmnet.object, type = "nonzero"),
         length)
  outlist = as.list(seq(nfolds))
    N=nrow(x)
  if (parallel) {
#  if (parallel && require(foreach)) {
    outlist = foreach(i = seq(nfolds), .packages = c("glmnet")) %dopar%
    {
      which = foldid == i
#      if (is.matrix(y))
      if (length(dim(y))>1)
        y_sub = y[!which, ]
      else y_sub = y[!which]
      if (is.offset)
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      glmnet(x[!which, , drop = FALSE], y_sub, lambda = lambda,
             offset = offset_sub, weights = weights[!which],
             ...)
    }
  }
  else {
      for (i in seq(nfolds)) {
          if (trace.it) cat(sprintf("Fold: %d/%d\n", i, nfolds))
        which = foldid == i
        if (length(dim(y))>1)
            y_sub = y[!which, ]
        else y_sub = y[!which]
      if (is.offset)
        offset_sub = as.matrix(offset)[!which, ]
      else offset_sub = NULL
      outlist[[i]] = glmnet(x[!which, , drop = FALSE],
               y_sub, lambda = lambda, offset = offset_sub,
               weights = weights[!which],trace.it=trace.it, ...)
    }
  }
  lambda = glmnet.object$lambda
  class(outlist)=paste0(subclass,"list")
  predmat=buildPredmat(outlist,lambda,x,offset,foldid,alignment,y=y,weights=weights,
                       grouped=grouped,type.measure=type.measure,family=family(glmnet.object))
### we include type.measure for the special case of coxnet with the deviance vs C-index discrepancy
### family is included for the new GLM crowd
### Next we compute the measures
#    if(subclass=="glmnetfit") attr(predmat,"family")=glmnet.object$family
    fun = paste("cv", subclass, sep = ".")
    cvstuff = do.call(fun, list(predmat,y,type.measure,weights,foldid,grouped))

  grouped=cvstuff$grouped
    if ((N/nfolds < 3) && grouped) {
    warning("Option grouped=FALSE enforced in cv.glmnet, since < 3 observations per fold",
            call. = FALSE)
    grouped = FALSE
    }

  out=cvstats(cvstuff,foldid,nfolds,lambda,nz,grouped)
  cvname = names(cvstuff$type.measure)
  names(cvname)=cvstuff$type.measure# to be compatible with earlier version; silly, I know
  out = c(out,list(call=cv.call,name = cvname, glmnet.fit = glmnet.object))
  if (keep)
    out = c(out, list(fit.preval = predmat, foldid = foldid))
  lamin=with(out,getOptcv.glmnet(lambda, cvm, cvsd, cvname))
  obj = c(out, as.list(lamin))
  class(obj) = "cv.glmnet"
  obj
}

check.exclude <- function(exclude,nvars){
      exclude <- unique(exclude)
      if(length(exclude)> (nvars-2))stop("cannot retain 1 or less variables")
      if(length(exclude)==0)exclude <- NULL
      exclude
      }

#' Compute the roc curve(s) for a binimial glmnet model.
#'
#' @rdname assess.glmnet
#' @export roc.glmnet
roc.glmnet=function(object,newx=NULL,newy,...){
    ### object must be either a glmnet or cv.glmnet object, or else a matrix or a vector of predictions of a glmnet model
    oclass=grep("glmnet",class(object))
    if(length(oclass)){# either a glmnet or cv.glmnet
        if(family(object)!="binomial")stop("roc available only for binomial family")
        predmat=predict(object,newx=newx,...)
    }
    else{
        predmat=object
        if(is.vector(predmat))predmat=as.matrix(predmat)
        }

roc.raw=function(prediction,y){
    op=order(prediction,decreasing=TRUE)
    preds=prediction[op]
    y=y[op]
    noty=1-y
    if(any(duplicated(preds))){
        y=rev(tapply(y,preds,sum))
        noty=rev(tapply(noty,preds,sum))
    }
   data.frame( FPR=cumsum(noty)/sum(noty),TPR=cumsum(y)/sum(y))
}

### newy should be binary, but could be a two-class matrix!
      nc = dim(newy)
    if (is.null(nc)) {
        newy = as.factor(newy)
        ntab = table(newy)
        nc = length(ntab)
        newy = diag(nc)[as.numeric(newy), ]
    }
    else {
         nc = nc[2]
    }
    if(nc!=2)stop("newy should be binary, or a factor with two levels")
    newy=newy[,2]
    n=nrow(predmat)
    if(n!=length(newy))stop("mismatch in size of two arguments")
    out=apply(predmat,2,roc.raw,y=newy)
    if(length(out)==1)out=out[[1]]
    out
}

nonzeroCoef = function (beta, bystep = FALSE) 
{
### bystep = FALSE means which variables were ever nonzero
### bystep = TRUE means which variables are nonzero for each step
  nr=nrow(beta)
  if (nr == 1) {#degenerate case
    if (bystep) 
      apply(beta, 2, function(x) if (abs(x) > 0) 
            1
      else NULL)
    else {
      if (any(abs(beta) > 0)) 
        1
      else NULL
    }
  }
  else {
    beta=abs(beta)>0 # this is sparse
    which=seq(nr)
    ones=rep(1,ncol(beta))
    nz=as.vector((beta%*%ones)>0)
    which=which[nz]
    if (bystep) {
      if(length(which)>0){
        beta=as.matrix(beta[which,,drop=FALSE])
        nzel = function(x, which) if (any(x)) 
          which[x]
        else NULL
        which=apply(beta, 2, nzel, which)
        if(!is.list(which))which=data.frame(which)# apply can return a matrix!!
        which
      }
      else{
        dn=dimnames(beta)[[2]]
        which=vector("list",length(dn))
        names(which)=dn
        which
      }
        
    }
    else which
  }
}

cv.coxnet <- function(predmat,y,type.measure,weights,foldid,grouped){
    ## Note, all the work got done in buildPredmat.coxnetlist for deviance; a special case
devtrue=type.measure=="deviance"
    nfolds = max(foldid)

    if ((length(weights)/nfolds < 10) && !grouped) {
        warning("Option grouped=TRUE enforced for cv.coxnet, since < 10 observations per fold",
                call. = FALSE)
    }

    if(devtrue){
        cvraw=attr(predmat,"cvraw")
        status = y[, "status"]
        N = nfolds - apply(is.na(cvraw), 2, sum)
        weights = as.vector(tapply(weights * status, foldid, sum))
        cvraw = cvraw/weights
    }
    else
    {
        nlambda=ncol(predmat)
        nlams=rep(nlambda,nfolds)
        cvraw = matrix(NA, nfolds, nlambda)
        good = matrix(0, nfolds, nlambda)
        for (i in seq(nfolds)) {
            good[i, seq(nlams[i])] = 1
            which = foldid == i
            for (j in seq(nlams[i])) {
                cvraw[i, j] = Cindex(predmat[which,j],y[which, ], weights[which])
            }
        }
        N = apply(good, 2, sum)
        weights = tapply(weights, foldid, sum)
        }
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=FALSE)

}

#' @method print confusion.table
#' @export
print.confusion.table=function(x,digits = max(3, getOption("digits") - 3), ...){

    ndn=names(dimnames(x))
    rowtot=rowSums(x)
    coltot=colSums(x)
    tot=sum(coltot)
    ncorrect=sum(diag(x))
    correct=(ncorrect)/tot

    x=cbind(x,Total=rowtot)
    x=rbind(x,Total=c(coltot,tot))
    dn=dimnames(x)
    names(dn)=ndn
    dimnames(x)=dn
    print(x,digits=digits,...)
    cat("\n Percent Correct: ",format(round(correct,digits)),"\n")
    invisible()
}

cv.fishnet <-function(predmat,y,type.measure,weights,foldid,grouped){

    devi = function(y, eta) {
    deveta = y * eta - exp(eta)
    devy = y * log(y) - y
    devy[y == 0] = 0
    2 * (devy - deveta)
      }

  N = length(y) - apply(is.na(predmat), 2, sum)
  cvraw = switch(type.measure,
                 mse = (y - exp(predmat))^2,
                 mae = abs(y - exp(predmat)),
                 deviance = devi(y, predmat)
                 )
    list(cvraw=cvraw,weights=weights,N=N,type.measure=type.measure,grouped=grouped)
}

#' @method plot multnet
#' @rdname plot.glmnet
#' @export
plot.multnet=function(x, xvar=c("norm","lambda","dev"),label=FALSE,type.coef=c("coef","2norm"),...){
  xvar=match.arg(xvar)
  type.coef=match.arg(type.coef)
    beta=x$beta
    if(xvar=="norm"){
      cnorm1=function(beta){
        which=nonzeroCoef(beta)
        beta=as.matrix(beta[which,])
        apply(abs(beta),2,sum)
      }
      norm=apply(sapply(x$beta,cnorm1),1,sum)
    } else norm = NULL
    dfmat=x$dfmat
  if(type.coef=="coef"){
    ncl=nrow(dfmat)
    clnames=rownames(dfmat)
    for( i in seq(ncl)){
      plotCoef(beta[[i]],norm,x$lambda,dfmat[i,],x$dev.ratio,label=label,xvar=xvar,ylab=paste("Coefficients: Response",clnames[i]),...)
    }
  }
  else {
    dfseq=round(apply(dfmat,2,mean),1)
    plotCoef(coefnorm(beta,2),norm,x$lambda,dfseq,x$dev.ratio,label=label,xvar=xvar,ylab="Coefficient 2Norms",...)
  }

  }

#' internal glmnet parameters
#'
#' View and/or change the factory default parameters in glmnet
#'
#' If called with no arguments, \code{glmnet.control()} returns a list with the
#' current settings of these parameters. Any arguments included in the call
#' sets those parameters to the new values, and then silently returns. The
#' values set are persistent for the duration of the R session.
#'
#' @param fdev minimum fractional change in deviance for stopping path; factory
#' default = 1.0e-5
#' @param devmax maximum fraction of explained deviance for stopping path;
#' factory default = 0.999
#' @param eps minimum value of lambda.min.ratio (see glmnet); factory default=
#' 1.0e-6
#' @param big large floating point number; factory default = 9.9e35. Inf in
#' definition of upper.limit is set to big
#' @param mnlam minimum number of path points (lambda values) allowed; factory
#' default = 5
#' @param pmin minimum probability for any class. factory default = 1.0e-9.
#' Note that this implies a pmax of 1-pmin.
#' @param exmx maximum allowed exponent. factory default = 250.0
#' @param prec convergence threshold for multi response bounds adjustment
#' solution. factory default = 1.0e-10
#' @param mxit maximum iterations for multiresponse bounds adjustment solution.
#' factory default = 100
#' @param itrace If 1 then progress bar is displayed when running \code{glmnet}
#' and \code{cv.glmnet}. factory default = 0
#' @param epsnr convergence threshold for \code{glmnet.fit}. factory default =
#' 1.0e-6
#' @param mxitnr maximum iterations for the IRLS loop in \code{glmnet.fit}. factory
#' default = 25
#' @param factory If \code{TRUE}, reset all the parameters to the factory
#' default; default is \code{FALSE}
#' @return A list with named elements as in the argument list
#' @author Jerome Friedman, Kenneth Tay, Trevor Hastie\cr Maintainer: Trevor Hastie
#' \email{hastie@@stanford.edu}
#' @seealso \code{glmnet}
#' @keywords models regression
#' @examples
#'
#' glmnet.control(fdev = 0)  #continue along path even though not much changes
#' glmnet.control()  # view current settings
#' glmnet.control(factory = TRUE)  # reset all the parameters to their default
#'
#' @export glmnet.control
glmnet.control <-
  function (fdev = 1e-05, devmax = 0.999, eps = 1e-06, big = 9.9e+35,
            mnlam = 5, pmin = 1e-09, exmx = 250, prec = 1e-10, mxit = 100,
            itrace = 0, epsnr = 1e-06, mxitnr = 25, factory = FALSE)
{
  inquiry=!nargs()
   if (factory)
    invisible(glmnet.control(fdev = 1e-05, devmax = 0.999,
                             eps = 1e-06, big = 9.9e+35, mnlam = 5, pmin = 1e-09,
                             exmx = 250, prec = 1e-10, mxit = 100, itrace = 0,
                             epsnr = 1e-06, mxitnr = 25))
  else {
    if (!missing(fdev))
      chg_fract_dev(as.double(fdev))
    if (!missing(devmax))
      chg_dev_max(as.double(devmax))
    if (!missing(eps))
      chg_min_flmin(as.double(eps))
    if (!missing(big))
      chg_big(as.double(big))
    if (!missing(mnlam))
      chg_min_lambdas(as.integer(mnlam))
    if (!missing(pmin))
      chg_min_null_prob(as.double(pmin))
    if (!missing(exmx))
      chg_max_exp(as.double(exmx))
    if (!missing(prec) | !missing(mxit))
      chg_bnorm(as.double(prec), as.integer(mxit))
    if (!missing(itrace))
      chg_itrace(as.integer(itrace))
    if (!missing(epsnr))
        chg_epsnr(as.double(epsnr))
    if (!missing(mxitnr))
        chg_mxitnr(as.integer(mxitnr))

    value=c(get_int_parms(fdev = double(1),
                     eps = double(1), big = double(1), mnlam = integer(1),
                     devmax = double(1), pmin = double(1), exmx = double(1),
                     itrace = integer(1)),
            get_bnorm(prec = double(1), mxit = integer(1)),
            get_int_parms2(epsnr = double(1), mxitnr = integer(1)))
    if(inquiry)value else invisible(value)
  }
}

# Needs ix and jx for possible subsetting.
lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family,pb){
  nc=dim(y)
  maxit=as.integer(maxit)
  if(is.null(nc)){
    ## Need to construct a y matrix, and include the weights
    y=as.factor(y)
    ntab=table(y)
    minclass=min(ntab)
    if(minclass<=1)stop("one multinomial or binomial class has 1 or 0 observations; not allowed")
    if(minclass<8)warning("one multinomial or binomial class has fewer than 8  observations; dangerous ground")
    classnames=names(ntab)
    nc=as.integer(length(ntab))
    y=diag(nc)[as.numeric(y),]
  }
  else{
    noo=nc[1]
    if(noo!=nobs)stop("x and y have different number of rows in call to glmnet",call.=FALSE)
    nc=as.integer(nc[2])
    classnames=colnames(y)
  }
#Check for size limitations
  maxvars=.Machine$integer.max/(nlam*nc)
  if(nx>maxvars)stop(paste("Integer overflow; num_classes*num_lambda*pmax should not exceed .Machine$integer.max. Reduce pmax to be below", trunc(maxvars)))

  if(!missing(weights)) y=y*weights
### check if any rows of y sum to zero, and if so deal with them
  weights=drop(y%*%rep(1,nc))
  o=weights>0
  if(!all(o)){ #subset the data
    y=y[o,]
    if(is.sparse){ # we have to subset this beast with care
      x=sparseMatrix(i=jx,p=ix-1,x=x@x,dims=c(nobs,nvars))[o,,drop=FALSE]
    }
    else x=x[o,,drop=FALSE]
    nobs=sum(o)
  }else o=NULL


  if(family=="binomial"){
    if(nc>2)stop("More than two classes; use multinomial family instead in call to glmnet",call.=FALSE)
    nc=as.integer(1) # for calling lognet
    y=y[,c(2,1)]#C++ lognet models the first column; we prefer the second (for 0/1 data)
  }


    storage.mode(y)="double"
    if(is.null(offset)){
      offset=y*0 #keeps the shape of y
      is.offset=FALSE
    }
    else{
      offset=as.matrix(offset)
      if(!is.null(o))offset=offset[o,,drop=FALSE]# we might have zero weights
      do=dim(offset)
      if(do[[1]]!=nobs)stop("offset should have the same number of values as observations in binomial/multinomial call to glmnet",call.=FALSE)
      if((do[[2]]==1)&(nc==1))offset=cbind(offset,-offset)
      if((family=="multinomial")&(do[[2]]!=nc))stop("offset should have same shape as y in multinomial call to glmnet",call.=FALSE)
      storage.mode(offset)="double"
      is.offset=TRUE
    }

    # C++ expects offset to have nc number of columns, as documented in Fortran.
    # We do a final modification of offset to take only the first column, since we favor this when fitting.
    if (nc == 1) offset=as.matrix(offset[,1], ncol=1)

    fit=if(is.sparse) splognet_exp(
      parm=alpha,x,y,offset,jd,vp,cl,ne=ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,pb,
                    lmu=integer(1),
                    a0=matrix(0.0, nc, nlam),
                    ca=double(nx*nlam*nc),
                    ia=integer(nx),
                    nin=integer(nlam),
                    nulldev=double(1),
                    dev=double(nlam),
                    alm=double(nlam),
                    nlp=integer(1),
                    jerr=integer(1)
                    )
  else lognet_exp(
          parm=alpha,x,y,offset,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,pb,
          lmu=integer(1),
          a0=matrix(0.0, nc, nlam),
          ca=double(nx*nlam*nc),
          ia=integer(nx),
          nin=integer(nlam),
          nulldev=double(1),
          dev=double(nlam),
          alm=double(nlam),
          nlp=integer(1),
          jerr=integer(1)
          )
if(fit$jerr!=0){
  errmsg=jerr(fit$jerr,maxit,pmax=nx,family)
  if(errmsg$fatal)stop(errmsg$msg,call.=FALSE)
  else warning(errmsg$msg,call.=FALSE)
}
 if(family=="binomial"){
   outlist=getcoef(fit,nvars,nx,vnames)
#   outlist$a0=-outlist$a0#sign flips make 2 target class
#   outlist$beta=-outlist$beta#sign flips make 2 target class
 }
 else  outlist=getcoef.multinomial(fit,nvars,nx,vnames,nc,classnames)
 dev=fit$dev[seq(fit$lmu)]
outlist=c(outlist,list(dev.ratio=dev,nulldev=fit$nulldev,npasses=fit$nlp,jerr=fit$jerr,offset=is.offset,classnames=classnames))
if(family=="multinomial"){
  if(kopt==2)grouped=TRUE else grouped =FALSE
  outlist$grouped=grouped
}
  class(outlist)=switch(family,
         "binomial"="lognet",
         "multinomial"="multnet"
         )
  outlist
}

#' Synthetic dataset with binary response
#'
#' Randomly generated data for binomial regression example.
#'
#' @name BinomialExample
#' @docType data
#' @usage data(BinomialExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{100 by 30 matrix of numeric values.}
#'   \item{y}{Numeric vector of length 100 containing 44 zeros and 56 ones.}
#' }
"BinomialExample"

#' Synthetic dataset with right-censored survival response
#'
#' Randomly generated data for Cox regression example.
#'
#' @name CoxExample
#' @docType data
#' @usage data(CoxExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{1,000 by 30 matrix of numeric values.}
#'   \item{y}{1,000 by 2 matrix with column names "time" and "status". The
#'     first column consists of positive numbers representing time to event,
#'     while the second column represents the status indicator
#'     (0=right-censored, 1=observed).}
#' }
"CoxExample"

#' Synthetic dataset with multiple Gaussian responses
#'
#' Randomly generated data for multi-response Gaussian regression example.
#'
#' @name MultiGaussianExample
#' @docType data
#' @usage data(MultiGaussianExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{100 by 20 matrix of numeric values.}
#'   \item{y}{100 by 4 matrix of numeric values, each column representing
#'     one response vector.}
#' }
"MultiGaussianExample"

#' Synthetic dataset with multinomial response
#'
#' Randomly generated data for multinomial regression example.
#'
#' @name MultinomialExample
#' @docType data
#' @usage data(MultinomialExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{500 by 30 matrix of numeric values.}
#'   \item{y}{Numeric vector of length 500 containing 142 ones, 174 twos
#'     and 184 threes.}
#' }
"MultinomialExample"

#' Synthetic dataset with count response
#'
#' Randomly generated data for Poisson regression example.
#'
#' @name PoissonExample
#' @docType data
#' @usage data(PoissonExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{500 by 20 matrix of numeric values.}
#'   \item{y}{Numeric vector of length 500 consisting of non-negative
#'     integers.}
#' }
"PoissonExample"

#' Synthetic dataset with Gaussian response
#'
#' Randomly generated data for Gaussian regression example.
#'
#' @name QuickStartExample
#' @docType data
#' @usage data(QuickStartExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{100 by 20 matrix of numeric values.}
#'   \item{y}{Numeric vector of length 100.}
#' }
"QuickStartExample"

#' Synthetic dataset with sparse design matrix
#'
#' Randomly generated data for Gaussian regression example with the
#' design matrix x being in sparse matrix format.
#'
#' @name SparseExample
#' @docType data
#' @usage data(SparseExample)
#' @keywords data
#' @format List containing the following elements:
#' \describe{
#'   \item{x}{100 by 20 matrix of numeric values. x is in sparse matrix
#'     format, having class "dgCMatrix".}
#'   \item{y}{Numeric vector of length 100.}
#' }
"SparseExample"

c     mortran 2.0     (version of 7/04/75 mod 7/4/87 (ajc))             
      subroutine coxnet(parm,no,ni,x,y,d,g,w,jd,vp,cl,ne,nx,nlam,flmin,u
     *lam,thr,  maxit,isd,lmu,ca,ia,nin,dev0,dev,alm,nlp,jerr)
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),y(no),d(no),g(no),w(no),vp(ni),ulam(nlam
     *)
      double precision ca(nx,nlam),dev(nlam),alm(nlam),cl(2,ni)         
      integer jd(*),ia(nx),nin(nlam)                                    
      double precision, dimension (:), allocatable :: xs,ww,vq          
      integer, dimension (:), allocatable :: ju                         
      if(maxval(vp) .gt. 0.0)goto 10021                                 
      jerr=10000                                                        
      return                                                            
10021 continue                                                          
      allocate(ww(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(ju(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(vq(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      if(isd .le. 0)goto 10041                                          
      allocate(xs(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
10041 continue                                                          
      call chkvars(no,ni,x,ju)                                          
      if(jd(1).gt.0) ju(jd(2:(jd(1)+1)))=0                              
      if(maxval(ju) .gt. 0)goto 10061                                   
      jerr=7777                                                         
      return                                                            
10061 continue                                                          
      vq=max(0d0,vp)                                                    
      vq=vq*ni/sum(vq)                                                  
      ww=max(0d0,w)                                                     
      sw=sum(ww)                                                        
      if(sw .gt. 0.0)goto 10081                                         
      jerr=9999                                                         
      return                                                            
10081 continue                                                          
      ww=ww/sw                                                          
      call cstandard(no,ni,x,ww,ju,isd,xs)                              
      if(isd .le. 0)goto 10101                                          
      do 10111 j=1,ni                                                   
      cl(:,j)=cl(:,j)*xs(j)                                             
10111 continue                                                          
      continue                                                          
10101 continue                                                          
      call coxnet1(parm,no,ni,x,y,d,g,ww,ju,vq,cl,ne,nx,nlam,flmin,ulam,
     *thr,  isd,maxit,lmu,ca,ia,nin,dev0,dev,alm,nlp,jerr)
      if(jerr.gt.0) return                                              
      dev0=2.0*sw*dev0                                                  
      if(isd .le. 0)goto 10131                                          
      do 10141 k=1,lmu                                                  
      nk=nin(k)                                                         
      ca(1:nk,k)=ca(1:nk,k)/xs(ia(1:nk))                                
10141 continue                                                          
      continue                                                          
10131 continue                                                          
      deallocate(ww,ju,vq)                                              
      if(isd.gt.0) deallocate(xs)                                       
      return                                                            
      end                                                               
      subroutine cstandard(no,ni,x,w,ju,isd,xs)                         
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),w(no),xs(ni)                            
      integer ju(ni)                                                    
      do 10151 j=1,ni                                                   
      if(ju(j).eq.0)goto 10151                                          
      xm=dot_product(w,x(:,j))                                          
      x(:,j)=x(:,j)-xm                                                  
      if(isd .le. 0)goto 10171                                          
      xs(j)=sqrt(dot_product(w,x(:,j)**2))                              
      x(:,j)=x(:,j)/xs(j)                                               
10171 continue                                                          
10151 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      subroutine coxnet1(parm,no,ni,x,y,d,g,q,ju,vp,cl,ne,nx,nlam,flmin,
     *ulam,cthri,  isd,maxit,lmu,ao,m,kin,dev0,dev,alm,nlp,jerr)
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),y(no),q(no),d(no),g(no),vp(ni),ulam(nlam
     *)
      double precision ao(nx,nlam),dev(nlam),alm(nlam),cl(2,ni)         
      integer ju(ni),m(nx),kin(nlam)                                    
      double precision, dimension (:), allocatable :: w,dk,v,xs,wr      
      double precision, dimension (:), allocatable :: a,as,f,dq         
      double precision, dimension (:), allocatable :: e,uu,ga           
      integer, dimension (:), allocatable :: jp,kp,mm,ixx               
      call get_int_parms(sml,eps,big,mnlam,devmax,pmin,exmx,itrace)     
      isd = isd*1                                                       
      sml=sml*100.0                                                     
      devmax=devmax*0.99/0.999                                          
      allocate(e(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(uu(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(f(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(w(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(v(1:ni),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(a(1:ni),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(as(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(xs(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(ga(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(ixx(1:ni),stat=jerr)                                     
      if(jerr.ne.0) return                                              
      allocate(jp(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(kp(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(dk(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(wr(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(dq(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(mm(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      call groups(no,y,d,q,nk,kp,jp,t0,jerr)                            
      if(jerr.ne.0) go to 10180                                         
      alpha=parm                                                        
      oma=1.0-alpha                                                     
      nlm=0                                                             
      ixx=0                                                             
      al=0.0                                                            
      dq=d*q                                                            
      call died(no,nk,dq,kp,jp,dk)                                      
      a=0.0                                                             
      f(1)=0.0                                                          
      fmax=log(huge(f(1))*0.1)                                          
      if(nonzero(no,g) .eq. 0)goto 10201                                
      f=g-dot_product(q,g)                                              
      e=q*exp(sign(min(abs(f),fmax),f))                                 
      goto 10211                                                        
10201 continue                                                          
      f=0.0                                                             
      e=q                                                               
10211 continue                                                          
      continue                                                          
      r0=risk(no,ni,nk,dq,dk,f,e,kp,jp,uu)                              
      rr=-(dot_product(dk(1:nk),log(dk(1:nk)))+r0)                      
      dev0=rr                                                           
      do 10221 i=1,no                                                   
      if((y(i) .ge. t0) .and. (q(i) .gt. 0.0))goto 10241                
      w(i)=0.0                                                          
      wr(i)=w(i)                                                        
10241 continue                                                          
10221 continue                                                          
      continue                                                          
      call outer(no,nk,dq,dk,kp,jp,e,wr,w,jerr,uu)                      
      if(jerr.ne.0) go to 10180                                         
      alf=1.0                                                           
      if(flmin .ge. 1.0)goto 10261                                      
      eqs=max(eps,flmin)                                                
      alf=eqs**(1.0/(nlam-1))                                           
10261 continue                                                          
      m=0                                                               
      mm=0                                                              
      nlp=0                                                             
      nin=nlp                                                           
      mnl=min(mnlam,nlam)                                               
      as=0.0                                                            
      cthr=cthri*dev0                                                   
      do 10271 j=1,ni                                                   
      if(ju(j).eq.0)goto 10271                                          
      ga(j)=abs(dot_product(wr,x(:,j)))                                 
10271 continue                                                          
      continue                                                          
      do 10281 ilm=1,nlam                                               
      if(itrace.ne.0) call setpb(ilm-1)                                 
      al0=al                                                            
      if(flmin .lt. 1.0)goto 10301                                      
      al=ulam(ilm)                                                      
      goto 10291                                                        
10301 if(ilm .le. 2)goto 10311                                          
      al=al*alf                                                         
      goto 10291                                                        
10311 if(ilm .ne. 1)goto 10321                                          
      al=big                                                            
      goto 10331                                                        
10321 continue                                                          
      al0=0.0                                                           
      do 10341 j=1,ni                                                   
      if(ju(j).eq.0)goto 10341                                          
      if(vp(j).gt.0.0) al0=max(al0,ga(j)/vp(j))                         
10341 continue                                                          
      continue                                                          
      al0=al0/max(parm,1.0d-3)                                          
      al=alf*al0                                                        
10331 continue                                                          
10291 continue                                                          
      sa=alpha*al                                                       
      omal=oma*al                                                       
      tlam=alpha*(2.0*al-al0)                                           
      do 10351 k=1,ni                                                   
      if(ixx(k).eq.1)goto 10351                                         
      if(ju(k).eq.0)goto 10351                                          
      if(ga(k).gt.tlam*vp(k)) ixx(k)=1                                  
10351 continue                                                          
      continue                                                          
10360 continue                                                          
      continue                                                          
10371 continue                                                          
      if(nlp .le. maxit)goto 10391                                      
      jerr=-ilm                                                         
      return                                                            
10391 continue                                                          
      if(nin.gt.0) as(m(1:nin))=a(m(1:nin))                             
      call vars(no,ni,x,w,ixx,v)                                        
      continue                                                          
10401 continue                                                          
      nlp=nlp+1                                                         
      dli=0.0                                                           
      do 10411 j=1,ni                                                   
      if(ixx(j).eq.0)goto 10411                                         
      u=a(j)*v(j)+dot_product(wr,x(:,j))                                
      if(abs(u) .gt. vp(j)*sa)goto 10431                                
      at=0.0                                                            
      goto 10441                                                        
10431 continue                                                          
      at=max(cl(1,j),min(cl(2,j),sign(abs(u)-vp(j)*sa,u)/  (v(j)+vp(j)*o
     *mal)))
10441 continue                                                          
      continue                                                          
      if(at .eq. a(j))goto 10461                                        
      del=at-a(j)                                                       
      a(j)=at                                                           
      dli=max(dli,v(j)*del**2)                                          
      wr=wr-del*w*x(:,j)                                                
      f=f+del*x(:,j)                                                    
      if(mm(j) .ne. 0)goto 10481                                        
      nin=nin+1                                                         
      if(nin.gt.nx)goto 10412                                           
      mm(j)=nin                                                         
      m(nin)=j                                                          
10481 continue                                                          
10461 continue                                                          
10411 continue                                                          
10412 continue                                                          
      if(nin.gt.nx)goto 10402                                           
      if(dli.lt.cthr)goto 10402                                         
      if(nlp .le. maxit)goto 10501                                      
      jerr=-ilm                                                         
      return                                                            
10501 continue                                                          
      continue                                                          
10511 continue                                                          
      nlp=nlp+1                                                         
      dli=0.0                                                           
      do 10521 l=1,nin                                                  
      j=m(l)                                                            
      u=a(j)*v(j)+dot_product(wr,x(:,j))                                
      if(abs(u) .gt. vp(j)*sa)goto 10541                                
      at=0.0                                                            
      goto 10551                                                        
10541 continue                                                          
      at=max(cl(1,j),min(cl(2,j),sign(abs(u)-vp(j)*sa,u)/  (v(j)+vp(j)*o
     *mal)))
10551 continue                                                          
      continue                                                          
      if(at .eq. a(j))goto 10571                                        
      del=at-a(j)                                                       
      a(j)=at                                                           
      dli=max(dli,v(j)*del**2)                                          
      wr=wr-del*w*x(:,j)                                                
      f=f+del*x(:,j)                                                    
10571 continue                                                          
10521 continue                                                          
      continue                                                          
      if(dli.lt.cthr)goto 10512                                         
      if(nlp .le. maxit)goto 10591                                      
      jerr=-ilm                                                         
      return                                                            
10591 continue                                                          
      goto 10511                                                        
10512 continue                                                          
      goto 10401                                                        
10402 continue                                                          
      if(nin.gt.nx)goto 10372                                           
      e=q*exp(sign(min(abs(f),fmax),f))                                 
      call outer(no,nk,dq,dk,kp,jp,e,wr,w,jerr,uu)                      
      if(jerr .eq. 0)goto 10611                                         
      jerr=jerr-ilm                                                     
      go to 10180                                                       
10611 continue                                                          
      ix=0                                                              
      do 10621 j=1,nin                                                  
      k=m(j)                                                            
      if(v(k)*(a(k)-as(k))**2.lt.cthr)goto 10621                        
      ix=1                                                              
      goto 10622                                                        
10621 continue                                                          
10622 continue                                                          
      if(ix .ne. 0)goto 10641                                           
      do 10651 k=1,ni                                                   
      if(ixx(k).eq.1)goto 10651                                         
      if(ju(k).eq.0)goto 10651                                          
      ga(k)=abs(dot_product(wr,x(:,k)))                                 
      if(ga(k) .le. sa*vp(k))goto 10671                                 
      ixx(k)=1                                                          
      ix=1                                                              
10671 continue                                                          
10651 continue                                                          
      continue                                                          
      if(ix.eq.1) go to 10360                                           
      goto 10372                                                        
10641 continue                                                          
      goto 10371                                                        
10372 continue                                                          
      if(nin .le. nx)goto 10691                                         
      jerr=-10000-ilm                                                   
      goto 10282                                                        
10691 continue                                                          
      if(nin.gt.0) ao(1:nin,ilm)=a(m(1:nin))                            
      kin(ilm)=nin                                                      
      alm(ilm)=al                                                       
      lmu=ilm                                                           
      dev(ilm)=(risk(no,ni,nk,dq,dk,f,e,kp,jp,uu)-r0)/rr                
      if(ilm.lt.mnl)goto 10281                                          
      if(flmin.ge.1.0)goto 10281                                        
      me=0                                                              
      do 10701 j=1,nin                                                  
      if(ao(j,ilm).ne.0.0) me=me+1                                      
10701 continue                                                          
      continue                                                          
      if(me.gt.ne)goto 10282                                            
      if((dev(ilm)-dev(ilm-mnl+1))/dev(ilm).lt.sml)goto 10282           
      if(dev(ilm).gt.devmax)goto 10282                                  
10281 continue                                                          
10282 continue                                                          
      g=f                                                               
10180 continue                                                          
      deallocate(e,uu,w,dk,v,xs,f,wr,a,as,jp,kp,dq,mm,ga,ixx)           
      return                                                            
      end                                                               
      subroutine groups(no,y,d,q,nk,kp,jp,t0,jerr)                      
      implicit double precision(a-h,o-z)                                
      double precision y(no),d(no),q(no)                                
      integer jp(no),kp(*)                                              
      do 10711 j=1,no                                                   
      jp(j)=j                                                           
10711 continue                                                          
      continue                                                          
      call psort7(y,jp,1,no)                                            
      nj=0                                                              
      do 10721 j=1,no                                                   
      if(q(jp(j)).le.0.0)goto 10721                                     
      nj=nj+1                                                           
      jp(nj)=jp(j)                                                      
10721 continue                                                          
      continue                                                          
      if(nj .ne. 0)goto 10741                                           
      jerr=20000                                                        
      return                                                            
10741 continue                                                          
      j=1                                                               
      continue                                                          
10751 if(d(jp(j)).gt.0.0)goto 10752                                     
      j=j+1                                                             
      if(j.gt.nj)goto 10752                                             
      goto 10751                                                        
10752 continue                                                          
      if(j .lt. nj-1)goto 10771                                         
      jerr=30000                                                        
      return                                                            
10771 continue                                                          
      t0=y(jp(j))                                                       
      j0=j-1                                                            
      if(j0 .le. 0)goto 10791                                           
      continue                                                          
10801 if(y(jp(j0)).lt.t0)goto 10802                                     
      j0=j0-1                                                           
      if(j0.eq.0)goto 10802                                             
      goto 10801                                                        
10802 continue                                                          
      if(j0 .le. 0)goto 10821                                           
      nj=nj-j0                                                          
      do 10831 j=1,nj                                                   
      jp(j)=jp(j+j0)                                                    
10831 continue                                                          
      continue                                                          
10821 continue                                                          
10791 continue                                                          
      jerr=0                                                            
      nk=0                                                              
      yk=t0                                                             
      j=2                                                               
      continue                                                          
10841 continue                                                          
      continue                                                          
10851 if(d(jp(j)).gt.0.0.and.y(jp(j)).gt.yk)goto 10852                  
      j=j+1                                                             
      if(j.gt.nj)goto 10852                                             
      goto 10851                                                        
10852 continue                                                          
      nk=nk+1                                                           
      kp(nk)=j-1                                                        
      if(j.gt.nj)goto 10842                                             
      if(j .ne. nj)goto 10871                                           
      nk=nk+1                                                           
      kp(nk)=nj                                                         
      goto 10842                                                        
10871 continue                                                          
      yk=y(jp(j))                                                       
      j=j+1                                                             
      goto 10841                                                        
10842 continue                                                          
      return                                                            
      end                                                               
      subroutine outer(no,nk,d,dk,kp,jp,e,wr,w,jerr,u)                  
      implicit double precision(a-h,o-z)                                
      double precision d(no),dk(nk),wr(no),w(no)                        
      double precision e(no),u(no),b,c                                  
      integer kp(nk),jp(no)                                             
      call usk(no,nk,kp,jp,e,u)                                         
      b=dk(1)/u(1)                                                      
      c=dk(1)/u(1)**2                                                   
      jerr=0                                                            
      do 10881 j=1,kp(1)                                                
      i=jp(j)                                                           
      w(i)=e(i)*(b-e(i)*c)                                              
      if(w(i) .gt. 0.0)goto 10901                                       
      jerr=-30000                                                       
      return                                                            
10901 continue                                                          
      wr(i)=d(i)-e(i)*b                                                 
10881 continue                                                          
      continue                                                          
      do 10911 k=2,nk                                                   
      j1=kp(k-1)+1                                                      
      j2=kp(k)                                                          
      b=b+dk(k)/u(k)                                                    
      c=c+dk(k)/u(k)**2                                                 
      do 10921 j=j1,j2                                                  
      i=jp(j)                                                           
      w(i)=e(i)*(b-e(i)*c)                                              
      if(w(i) .gt. 0.0)goto 10941                                       
      jerr=-30000                                                       
      return                                                            
10941 continue                                                          
      wr(i)=d(i)-e(i)*b                                                 
10921 continue                                                          
      continue                                                          
10911 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      subroutine vars(no,ni,x,w,ixx,v)                                  
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),w(no),v(ni)                             
      integer ixx(ni)                                                   
      do 10951 j=1,ni                                                   
      if(ixx(j).gt.0) v(j)=dot_product(w,x(:,j)**2)                     
10951 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      subroutine died(no,nk,d,kp,jp,dk)                                 
      implicit double precision(a-h,o-z)                                
      double precision d(no),dk(nk)                                     
      integer kp(nk),jp(no)                                             
      dk(1)=sum(d(jp(1:kp(1))))                                         
      do 10961 k=2,nk                                                   
      dk(k)=sum(d(jp((kp(k-1)+1):kp(k))))                               
10961 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      subroutine usk(no,nk,kp,jp,e,u)                                   
      implicit double precision(a-h,o-z)                                
      double precision e(no),u(nk),h                                    
      integer kp(nk),jp(no)                                             
      h=0.0                                                             
      do 10971 k=nk,1,-1                                                
      j2=kp(k)                                                          
      j1=1                                                              
      if(k.gt.1) j1=kp(k-1)+1                                           
      do 10981 j=j2,j1,-1                                               
      h=h+e(jp(j))                                                      
10981 continue                                                          
      continue                                                          
      u(k)=h                                                            
10971 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      function risk(no,ni,nk,d,dk,f,e,kp,jp,u)                          
      implicit double precision(a-h,o-z)                                
      double precision d(no),dk(nk),f(no)                               
      integer kp(nk),jp(no)                                             
      double precision e(no),u(nk)                                      
      ni = ni*1                                                         
      call usk(no,nk,kp,jp,e,u)                                         
      u=log(u)                                                          
      risk=dot_product(d,f)-dot_product(dk,u)                           
      return                                                            
      end                                                               
      subroutine loglike(no,ni,x,y,d,g,w,nlam,a,flog,jerr)              
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni),y(no),d(no),g(no),w(no),a(ni,nlam),flog(
     *nlam)
      double precision, dimension (:), allocatable :: dk,f,xm,dq,q      
      double precision, dimension (:), allocatable :: e,uu              
      integer, dimension (:), allocatable :: jp,kp                      
      allocate(e(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(q(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(uu(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(f(1:no),stat=jerr)                                       
      if(jerr.ne.0) return                                              
      allocate(dk(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(jp(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(kp(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(dq(1:no),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      allocate(xm(1:ni),stat=jerr)                                      
      if(jerr.ne.0) return                                              
      q=max(0d0,w)                                                      
      sw=sum(q)                                                         
      if(sw .gt. 0.0)goto 11001                                         
      jerr=9999                                                         
      go to 10180                                                       
11001 continue                                                          
      call groups(no,y,d,q,nk,kp,jp,t0,jerr)                            
      if(jerr.ne.0) go to 10180                                         
      fmax=log(huge(e(1))*0.1)                                          
      dq=d*q                                                            
      call died(no,nk,dq,kp,jp,dk)                                      
      gm=dot_product(q,g)/sw                                            
      do 11011 j=1,ni                                                   
      xm(j)=dot_product(q,x(:,j))/sw                                    
11011 continue                                                          
      continue                                                          
      do 11021 lam=1,nlam                                               
      do 11031 i=1,no                                                   
      f(i)=g(i)-gm+dot_product(a(:,lam),(x(i,:)-xm))                    
      e(i)=q(i)*exp(sign(min(abs(f(i)),fmax),f(i)))                     
11031 continue                                                          
      continue                                                          
      flog(lam)=risk(no,ni,nk,dq,dk,f,e,kp,jp,uu)                       
11021 continue                                                          
      continue                                                          
10180 continue                                                          
      deallocate(e,uu,dk,f,jp,kp,dq)                                    
      return                                                            
      end                                                               
      subroutine chkvars(no,ni,x,ju)                                    
      implicit double precision(a-h,o-z)                                
      double precision x(no,ni)                                         
      integer ju(ni)                                                    
      do 11041 j=1,ni                                                   
      ju(j)=0                                                           
      t=x(1,j)                                                          
      do 11051 i=2,no                                                   
      if(x(i,j).eq.t)goto 11051                                         
      ju(j)=1                                                           
      goto 11052                                                        
11051 continue                                                          
11052 continue                                                          
11041 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      function bnorm(b0,al1p,al2p,g,usq,jerr)                           
      implicit double precision(a-h,o-z)                                
      data thr,mxit /1.0d-10,100/                                       
      b=b0                                                              
      zsq=b**2+usq                                                      
      if(zsq .gt. 0.0)goto 11071                                        
      bnorm=0.0                                                         
      return                                                            
11071 continue                                                          
      z=sqrt(zsq)                                                       
      f=b*(al1p+al2p/z)-g                                               
      jerr=0                                                            
      do 11081 it=1,mxit                                                
      b=b-f/(al1p+al2p*usq/(z*zsq))                                     
      zsq=b**2+usq                                                      
      if(zsq .gt. 0.0)goto 11101                                        
      bnorm=0.0                                                         
      return                                                            
11101 continue                                                          
      z=sqrt(zsq)                                                       
      f=b*(al1p+al2p/z)-g                                               
      if(abs(f).le.thr)goto 11082                                       
      if(b .gt. 0.0)goto 11121                                          
      b=0.0                                                             
      goto 11082                                                        
11121 continue                                                          
11081 continue                                                          
11082 continue                                                          
      bnorm=b                                                           
      if(it.ge.mxit) jerr=90000                                         
      return                                                            
      entry chg_bnorm(arg,irg)                                          
      bnorm = 0.0                                                       
      thr=arg                                                           
      mxit=irg                                                          
      return                                                            
      entry get_bnorm(arg,irg)                                          
      bnorm = 0.0                                                       
      arg=thr                                                           
      irg=mxit                                                          
      return                                                            
      end                                                               
      subroutine get_int_parms(sml,eps,big,mnlam,rsqmax,pmin,exmx,itrace
     *)
      implicit double precision(a-h,o-z)                                
      data sml0,eps0,big0,mnlam0,rsqmax0,pmin0,exmx0,itrace0  /1.0d-5,1.
     *0d-6,9.9d35,5,0.999,1.0d-9,250.0,0/
      sml=sml0                                                          
      eps=eps0                                                          
      big=big0                                                          
      mnlam=mnlam0                                                      
      rsqmax=rsqmax0                                                    
      pmin=pmin0                                                        
      exmx=exmx0                                                        
      itrace=itrace0                                                    
      return                                                            
      entry chg_fract_dev(arg)                                          
      sml0=arg                                                          
      return                                                            
      entry chg_dev_max(arg)                                            
      rsqmax0=arg                                                       
      return                                                            
      entry chg_min_flmin(arg)                                          
      eps0=arg                                                          
      return                                                            
      entry chg_big(arg)                                                
      big0=arg                                                          
      return                                                            
      entry chg_min_lambdas(irg)                                        
      mnlam0=irg                                                        
      return                                                            
      entry chg_min_null_prob(arg)                                      
      pmin0=arg                                                         
      return                                                            
      entry chg_max_exp(arg)                                            
      exmx0=arg                                                         
      return                                                            
      entry chg_itrace(irg)                                             
      itrace0=irg                                                       
      return                                                            
      end                                                               
      subroutine get_int_parms2(epsnr,mxitnr)                           
      implicit double precision(a-h,o-z)                                
      data epsnr0,mxitnr0  /1.0d-6,25/                                  
      epsnr=epsnr0                                                      
      mxitnr=mxitnr0                                                    
      return                                                            
      entry chg_epsnr(arg)                                              
      epsnr0=arg                                                        
      return                                                            
      entry chg_mxitnr(irg)                                             
      mxitnr0=irg                                                       
      return                                                            
      end                                                               
      function nonzero(n,v)                                             
      implicit double precision(a-h,o-z)                                
      double precision v(n)                                             
      nonzero=0                                                         
      do 11131 i=1,n                                                    
      if(v(i) .eq. 0.0)goto 11151                                       
      nonzero=1                                                         
      return                                                            
11151 continue                                                          
11131 continue                                                          
      continue                                                          
      return                                                            
      end                                                               
      subroutine psort7(v,a,ii,jj)                                      
      implicit double precision(a-h,o-z)                                
c                                                                       
c     puts into a the permutation vector which sorts v into             
c     increasing order. the array v is not modified.                    
c     only elements from ii to jj are considered.                       
c     arrays iu(k) and il(k) permit sorting up to 2**(k+1)-1 elements   
c                                                                       
c     this is a modification of cacm algorithm #347 by r. c. singleton, 
c     which is a modified hoare quicksort.                              
c                                                                       
      dimension a(jj),v(jj),iu(20),il(20)                               
      integer t,tt                                                      
      integer a                                                         
      double precision v                                                
      m=1                                                               
      i=ii                                                              
      j=jj                                                              
 10   if (i.ge.j) go to 80                                              
 20   k=i                                                               
      ij=(j+i)/2                                                        
      t=a(ij)                                                           
      vt=v(t)                                                           
      if (v(a(i)).le.vt) go to 30                                       
      a(ij)=a(i)                                                        
      a(i)=t                                                            
      t=a(ij)                                                           
      vt=v(t)                                                           
 30   l=j                                                               
      if (v(a(j)).ge.vt) go to 50                                       
      a(ij)=a(j)                                                        
      a(j)=t                                                            
      t=a(ij)                                                           
      vt=v(t)                                                           
      if (v(a(i)).le.vt) go to 50                                       
      a(ij)=a(i)                                                        
      a(i)=t                                                            
      t=a(ij)                                                           
      vt=v(t)                                                           
      go to 50                                                          
 40   a(l)=a(k)                                                         
      a(k)=tt                                                           
 50   l=l-1                                                             
      if (v(a(l)).gt.vt) go to 50                                       
      tt=a(l)                                                           
      vtt=v(tt)                                                         
 60   k=k+1                                                             
      if (v(a(k)).lt.vt) go to 60                                       
      if (k.le.l) go to 40                                              
      if (l-i.le.j-k) go to 70                                          
      il(m)=i                                                           
      iu(m)=l                                                           
      i=k                                                               
      m=m+1                                                             
      go to 90                                                          
 70   il(m)=k                                                           
      iu(m)=j                                                           
      j=l                                                               
      m=m+1                                                             
      go to 90                                                          
 80   m=m-1                                                             
      if (m.eq.0) return                                                
      i=il(m)                                                           
      j=iu(m)                                                           
 90   if (j-i.gt.10) go to 20                                           
      if (i.eq.ii) go to 10                                             
      i=i-1                                                             
 100  i=i+1                                                             
      if (i.eq.j) go to 80                                              
      t=a(i+1)                                                          
      vt=v(t)                                                           
      if (v(a(i)).le.vt) go to 100                                      
      k=i                                                               
 110  a(k+1)=a(k)                                                       
      k=k-1                                                             
      if (vt.lt.v(a(k))) go to 110                                      
      a(k+1)=t                                                          
      go to 100                                                         
      end                                                               

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

elnet_exp <- function(ka, parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr) {
    .Call('_glmnet_elnet_exp', PACKAGE = 'glmnet', ka, parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr)
}

spelnet_exp <- function(ka, parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr) {
    .Call('_glmnet_spelnet_exp', PACKAGE = 'glmnet', ka, parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr)
}

lognet_exp <- function(parm, x, y, g, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, kopt, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr) {
    .Call('_glmnet_lognet_exp', PACKAGE = 'glmnet', parm, x, y, g, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, kopt, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr)
}

splognet_exp <- function(parm, x, y, g, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, kopt, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr) {
    .Call('_glmnet_splognet_exp', PACKAGE = 'glmnet', parm, x, y, g, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, kopt, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr)
}

fishnet_exp <- function(parm, x, y, g, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr) {
    .Call('_glmnet_fishnet_exp', PACKAGE = 'glmnet', parm, x, y, g, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr)
}

spfishnet_exp <- function(parm, x, y, g, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr) {
    .Call('_glmnet_spfishnet_exp', PACKAGE = 'glmnet', parm, x, y, g, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, intr, maxit, pb, lmu, a0, ca, ia, nin, nulldev, dev, alm, nlp, jerr)
}

multelnet_exp <- function(parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, jsd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr) {
    .Call('_glmnet_multelnet_exp', PACKAGE = 'glmnet', parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, jsd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr)
}

multspelnet_exp <- function(parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, jsd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr) {
    .Call('_glmnet_multspelnet_exp', PACKAGE = 'glmnet', parm, x, y, w, jd, vp, cl, ne, nx, nlam, flmin, ulam, thr, isd, jsd, intr, maxit, pb, lmu, a0, ca, ia, nin, rsq, alm, nlp, jerr)
}

get_int_parms <- function(fdev, eps, big, mnlam, devmax, pmin, exmx, itrace) {
    .Call('_glmnet_get_int_parms', PACKAGE = 'glmnet', fdev, eps, big, mnlam, devmax, pmin, exmx, itrace)
}

get_int_parms2 <- function(epsnr, mxitnr) {
    .Call('_glmnet_get_int_parms2', PACKAGE = 'glmnet', epsnr, mxitnr)
}

chg_fract_dev <- function(arg) {
    invisible(.Call('_glmnet_chg_fract_dev', PACKAGE = 'glmnet', arg))
}

chg_dev_max <- function(arg) {
    invisible(.Call('_glmnet_chg_dev_max', PACKAGE = 'glmnet', arg))
}

chg_min_flmin <- function(arg) {
    invisible(.Call('_glmnet_chg_min_flmin', PACKAGE = 'glmnet', arg))
}

chg_big <- function(arg) {
    invisible(.Call('_glmnet_chg_big', PACKAGE = 'glmnet', arg))
}

chg_min_lambdas <- function(irg) {
    invisible(.Call('_glmnet_chg_min_lambdas', PACKAGE = 'glmnet', irg))
}

chg_min_null_prob <- function(arg) {
    invisible(.Call('_glmnet_chg_min_null_prob', PACKAGE = 'glmnet', arg))
}

chg_max_exp <- function(arg) {
    invisible(.Call('_glmnet_chg_max_exp', PACKAGE = 'glmnet', arg))
}

chg_itrace <- function(irg) {
    invisible(.Call('_glmnet_chg_itrace', PACKAGE = 'glmnet', irg))
}

chg_bnorm <- function(arg, irg) {
    invisible(.Call('_glmnet_chg_bnorm', PACKAGE = 'glmnet', arg, irg))
}

get_bnorm <- function(prec, mxit) {
    .Call('_glmnet_get_bnorm', PACKAGE = 'glmnet', prec, mxit)
}

chg_epsnr <- function(arg) {
    invisible(.Call('_glmnet_chg_epsnr', PACKAGE = 'glmnet', arg))
}

chg_mxitnr <- function(irg) {
    invisible(.Call('_glmnet_chg_mxitnr', PACKAGE = 'glmnet', irg))
}

wls_exp <- function(alm0, almc, alpha, m, no, ni, x, r, xv, v, intr, ju, vp, cl, nx, thr, maxit, a, aint, g, ia, iy, iz, mm, nino, rsqc, nlp, jerr) {
    .Call('_glmnet_wls_exp', PACKAGE = 'glmnet', alm0, almc, alpha, m, no, ni, x, r, xv, v, intr, ju, vp, cl, nx, thr, maxit, a, aint, g, ia, iy, iz, mm, nino, rsqc, nlp, jerr)
}

spwls_exp <- function(alm0, almc, alpha, m, no, ni, x, xm, xs, r, xv, v, intr, ju, vp, cl, nx, thr, maxit, a, aint, g, ia, iy, iz, mm, nino, rsqc, nlp, jerr) {
    .Call('_glmnet_spwls_exp', PACKAGE = 'glmnet', alm0, almc, alpha, m, no, ni, x, xm, xs, r, xv, v, intr, ju, vp, cl, nx, thr, maxit, a, aint, g, ia, iy, iz, mm, nino, rsqc, nlp, jerr)
}

#' Simulated data for the glmnet vignette
#'
#' Simple simulated data, used to demonstrate the features of glmnet
#'
#' These datasets are artificial, and are used to test out some of the
#' features of glmnet.
#' @name beta_CVX
#' @aliases x y beta_CVX
#' @format Data objects used to demonstrate features in the glmnet vignette
#' @keywords datasets
#' @useDynLib glmnet
#' @import methods
#' @import Matrix
#' @import foreach
#' @importFrom utils packageDescription
#' @importFrom graphics abline axis matplot points segments text par plot
#' @importFrom stats  approx  coef  median  predict  rmultinom  runif  weighted.mean family rnorm gaussian binomial glm glm.fit
#' @importFrom survival concordance Surv is.Surv
#' @importFrom grDevices rainbow
#' @importFrom Rcpp sourceCpp
#'
#' @examples
#'
#' data(QuickStartExample)
#' x <- QuickStartExample$x; y <- QuickStartExample$y
#' glmnet(x, y)
#'
NULL


#' Internal glmnet functions
#'
#' @description
#' These are not intended for use by users. \code{lambda.interp} does linear
#' interpolation of the lambdas to obtain a prediction at a new point s.
#' \code{glmnet_softmax} does the classification for multinomial models.
#' \code{nonzeroCoef} determines in an efficient manner which variables are
#' nonzero in each fit. \code{jerr} prints out error messages from the C++ routines.
#' \code{plotCoef} is called by the \code{plot} method for \code{glmnet}
#' objects. \code{check_dots} is used in \code{coef} and \code{predict} with
#' argument \code{exact=TRUE}, to make sure user supplies original data used to
#' fit the \code{"glmnet"} object.
#'
#' @name glmnet-internal
#' @aliases auc assess.coxnet auc.mat cvtype cvstats
#' cvcompute getcoef getcoef.multinomial fix.lam error.bars
#' getmin elnet mrelnet lognet fishnet coefnorm coxnet cv.lognet cv.elnet
#' cv.multnet cv.mrelnet cv.coxnet cv.fishnet cv.glmnet.raw cv.relaxed.raw
#' blend.relaxed checkgamma.relax buildPredmat buildPredmat.mrelnetlist
#' buildPredmat.multnetlist buildPredmat.lognetlist buildPredmat.array
#' buildPredmat.coxnetlist buildPredmat.default lambda.interp nonzeroCoef
#' glmnet_softmax getOptcv.glmnet getOptcv.relaxed jerr jerr.elnet jerr.lognet
#' jerr.fishnet jerr.coxnet jerr.mrelnet plotCoef zeromat na.mean check_dots
#' na_sparse_fix prepareX
#'
#' @author Trevor Hastie
#' @keywords internal
NULL





#' Elastic net model paths for some generalized linear models
#'
#' This package fits lasso and elastic-net model paths for regression, logistic
#' and multinomial regression using coordinate descent. The algorithm is
#' extremely fast, and exploits sparsity in the input x matrix where it exists.
#' A variety of predictions can be made from the fitted models.
#'
#' \tabular{ll}{ Package: \tab glmnet\cr Type: \tab Package\cr Version: \tab
#' 1.0\cr Date: \tab 2008-05-14\cr License: \tab What license is it under?\cr }
#' Very simple to use. Accepts \code{x,y} data for regression models, and
#' produces the regularization path over a grid of values for the tuning
#' parameter \code{lambda}. Only 5 functions: \code{glmnet}\cr
#' \code{predict.glmnet}\cr \code{plot.glmnet}\cr \code{print.glmnet}\cr
#' \code{coef.glmnet}
#'
#' @name glmnet-package
#' @docType package
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent (2010), Journal of Statistical Software, Vol. 33(1), 1-22},
#' \doi{10.18637/jss.v033.i01}.\cr
#' Simon, N., Friedman, J., Hastie, T. and Tibshirani, R. (2011)
#' \emph{Regularization Paths for Cox's Proportional
#' Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
#' 39(5), 1-13},
#' \doi{10.18637/jss.v039.i05}.\cr
#' Tibshirani,Robert, Bien, J., Friedman, J., Hastie, T.,Simon, N.,Taylor, J. and
#' Tibshirani, Ryan. (2012) \emph{Strong Rules for Discarding Predictors in
#' Lasso-type Problems, JRSSB, Vol. 74(2), 245-266},
#' \url{https://arxiv.org/abs/1011.2234}.\cr
#' Hastie, T., Tibshirani, Robert and Tibshirani, Ryan (2020) \emph{Best Subset,
#' Forward Stepwise or Lasso? Analysis and Recommendations Based on Extensive Comparisons,
#' Statist. Sc. Vol. 35(4), 579-592},
#' \url{https://arxiv.org/abs/1707.08692}.\cr
#' Glmnet webpage with four vignettes: \url{https://glmnet.stanford.edu}.
#' @keywords models regression package
#' @examples
#'
#' x = matrix(rnorm(100 * 20), 100, 20)
#' y = rnorm(100)
#' g2 = sample(1:2, 100, replace = TRUE)
#' g4 = sample(1:4, 100, replace = TRUE)
#' fit1 = glmnet(x, y)
#' predict(fit1, newx = x[1:5, ], s = c(0.01, 0.005))
#' predict(fit1, type = "coef")
#' plot(fit1, xvar = "lambda")
#' fit2 = glmnet(x, g2, family = "binomial")
#' predict(fit2, type = "response", newx = x[2:5, ])
#' predict(fit2, type = "nonzero")
#' fit3 = glmnet(x, g4, family = "multinomial")
#' predict(fit3, newx = x[1:3, ], type = "response", s = 0.01)
#'
NULL

load(url("https://github.com/DylanDijk/RepoA/blob/main/reprod_features.rda?raw=true"))
load(url("https://github.com/DylanDijk/RepoA/blob/main/reprod_response.rda?raw=true"))

X = reprod_features
y = reprod_response

write.table(X, "X.csv", sep=',', row.names=F, col.names=F)
write.table(y, "y.csv", sep=',', row.names=F, col.names=F)

#' Check if glmnet should call cox.path
#'
#' Helper function to check if glmnet() should call cox.path().
#'
#' For \code{family="cox"}, we only call the original coxnet() function if
#' (i) x is not sparse, (ii) y is right-censored data, and (iii) we are
#' not fitting a stratified Cox model. This function also throws an error
#' if y has a "strata" attribute but is not of type "stratifySurv".
#'
#' @param x Design matrix.
#' @param y Response variable.
#'
#' @return TRUE if cox.path() should be called, FALSE otherwise.
use.cox.path <- function(x, y) {
  y <- response.coxnet(y)
  use_cox_path <- TRUE
  # We only return FALSE if:
  # x is not sparse AND y is right-censored data AND no strata
  # (strata variable being all equal counts as no strata)
  if (!inherits(x, "sparseMatrix") && ncol(y) == 2) {
    if (!("strata" %in% names(attributes(y))) ||
        length(unique(attr(y, "strata"))) == 1)
      use_cox_path <- FALSE
  }
  # if strata provided in y but y not of class stratifySurv, throw error
  if ("strata" %in% names(attributes(y)) && !inherits(y, "stratifySurv"))
    stop(paste0("For fitting stratified Cox models, y must be of class ",
                "stratifySurv, see ?stratifySurv for more details"))

  return(use_cox_path)
}

#' Fit a Cox regression model with elastic net regularization for a path of
#' lambda values
#'
#' Fit a Cox regression model via penalized maximum likelihood for a path of
#' lambda values. Can deal with (start, stop] data and strata, as well as
#' sparse design matrices.
#'
#' Sometimes the sequence is truncated before \code{nlambda} values of lambda
#' have been used. This happens when \code{cox.path} detects that the
#' decrease in deviance is marginal (i.e. we are near a saturated fit).
#'
#' @param x See glmnet help file
#' @param y Survival response variable, must be a \code{Surv} or
#' \code{stratifySurv} object.
#' @param weights See glmnet help file
#' @param offset See glmnet help file
#' @param alpha See glmnet help file
#' @param nlambda See glmnet help file
#' @param lambda.min.ratio See glmnet help file
#' @param lambda See glmnet help file
#' @param standardize See glmnet help file
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than thresh times the null deviance.
#' Default value is \code{1e-10}.
#' @param exclude See glmnet help file
#' @param penalty.factor See glmnet help file
#' @param lower.limits See glmnet help file
#' @param upper.limits See glmnet help file
#' @param maxit See glmnet help file
#' @param trace.it Controls how much information is printed to screen. Default is
#' \code{trace.it=0} (no information printed). If \code{trace.it=1}, a progress
#' bar is displayed. If \code{trace.it=2}, some information about the fitting
#' procedure is printed to the console as the model is being fitted.
#' @param ... Other arguments passed from glmnet (not used right now).
#'
#' @return An object of class "coxnet" and "glmnet".
#' \item{a0}{Intercept value, \code{NULL} for "cox" family.}
#' \item{beta}{A \code{nvars x length(lambda)} matrix of coefficients, stored in
#' sparse matrix format.}
#' \item{df}{The number of nonzero coefficients for each value of lambda.}
#' \item{dim}{Dimension of coefficient matrix.}
#' \item{lambda}{The actual sequence of lambda values used. When alpha=0, the
#' largest lambda reported does not quite give the zero coefficients reported
#' (lambda=inf would in principle). Instead, the largest lambda for alpha=0.001
#' is used, and the sequence of lambda values is derived from this.}
#' \item{dev.ratio}{The fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
#' for the saturated model (a model with a free parameter per observation).
#' Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)). The null model refers to the 0 model.}
#' \item{npasses}{Total passes over the data summed over all lambda values.}
#' \item{jerr}{Error flag, for warnings and errors (largely for internal
#' debugging).}
#' \item{offset}{A logical variable indicating whether an offset was included
#' in the model.}
#' \item{call}{The call that produced this object.}
#' \item{nobs}{Number of observations.}
#'
#' @examples
#' set.seed(2)
#' nobs <- 100; nvars <- 15
#' xvec <- rnorm(nobs * nvars)
#' xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
#' x <- matrix(xvec, nrow = nobs)
#' beta <- rnorm(nvars / 3)
#' fx <- x[, seq(nvars / 3)] %*% beta / 3
#' ty <- rexp(nobs, exp(fx))
#' tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
#' jsurv <- survival::Surv(ty, tcens)
#' fit1 <- glmnet:::cox.path(x, jsurv)
#'
#' # works with sparse x matrix
#' x_sparse <- Matrix::Matrix(x, sparse = TRUE)
#' fit2 <- glmnet:::cox.path(x_sparse, jsurv)
#'
#' # example with (start, stop] data
#' set.seed(2)
#' start_time <- runif(100, min = 0, max = 5)
#' stop_time <- start_time + runif(100, min = 0.1, max = 3)
#' status <- rbinom(n = nobs, prob = 0.3, size = 1)
#' jsurv_ss <- survival::Surv(start_time, stop_time, status)
#' fit3 <- glmnet:::cox.path(x, jsurv_ss)
#'
#' # example with strata
#' jsurv_ss2 <- stratifySurv(jsurv_ss, rep(1:2, each = 50))
#' fit4 <- glmnet:::cox.path(x, jsurv_ss2)
cox.path <- function(x, y, weights=NULL, offset=NULL,
                     alpha=1.0, nlambda=100,
                     lambda.min.ratio=ifelse(nobs<nvars, 1e-2, 1e-4),
                     lambda=NULL, standardize=TRUE,
                     thresh=1e-10, exclude=NULL, penalty.factor=rep(1,nvars),
                     lower.limits=-Inf, upper.limits=Inf, maxit=100000,
                     trace.it=0, ...) {
  ### Prepare all the generic arguments (mimicking top-level glmnet() call)
  if (alpha > 1) {
    warning("alpha > 1; set to 1")
    alpha = 1
  } else if (alpha < 0) {
    warning("alpha < 0; set to 0")
    alpha = 0
  }
  alpha = as.double(alpha)

  this.call <- match.call()

  np = dim(x)
  if (is.null(np) || (np[2] <= 1)) stop("x should be a matrix with 2 or more columns")
  nobs = as.integer(np[1]); nvars = as.integer(np[2])

  # get feature variable names
  vnames <- colnames(x)
  if(is.null(vnames)) vnames <- paste("V",seq(nvars),sep="")

  # check weights
  if(is.null(weights)) weights = rep(1,nobs)
  else if (length(weights) != nobs)
    stop(paste("Number of elements in weights (",length(weights),
               ") not equal to the number of rows of x (",nobs,")",sep=""))
  weights <- as.double(weights)

  # check that response y is a Surv object of the correct length
  y <- response.coxnet(y)
  if (nrow(y) != nobs) stop(paste0("number of observations in y (" , nrow(y),
                                   ") not equal to the number of rows of x (",
                                   nobs, ")"))

  # check offset option
  is.offset <- !(is.null(offset))
  if (is.offset == FALSE) {
    offset <- rep(0, times = nrow(y))
  }

  # check and standardize penalty factors (to sum to nvars)
  if(any(penalty.factor == Inf)) {
    exclude = c(exclude, seq(nvars)[penalty.factor == Inf])
    exclude = sort(unique(exclude))
  }

    ## Compute weighted mean and variance of columns of x, sensitive to sparse matrix
    ## needed to detect constant columns below, and later if standarization
    meansd <- weighted_mean_sd(x, weights)

    ## look for constant variables, and if any, then add to exclude
    const_vars <- meansd$sd == 0
    nzvar <- setdiff(which(!const_vars), exclude)
    # if all the non-excluded variables have zero variance, throw error
    if (length(nzvar) == 0) stop("All used predictors have zero variance")

    ## if any constant vars, add to exclude
    if(any(const_vars)) {
        exclude <- sort(unique(c(which(const_vars),exclude)))
        meansd$sd[const_vars] <- 1.0 ## we divide later, and do not want bad numbers
    }
    if(length(exclude) > 0) {
        jd = match(exclude, seq(nvars), 0)
        if(!all(jd > 0)) stop ("Some excluded variables out of range")
        penalty.factor[jd] = 1 # ow can change lambda sequence
    }
    # check and standardize penalty factors (to sum to nvars)
    vp = pmax(0, penalty.factor)
    if (max(vp) <= 0) stop("All penalty factors are <= 0")
    vp = as.double(vp * nvars / sum(vp))

  ### check on limits
  control <- glmnet.control()
  if (thresh >= control$epsnr)
    warning("thresh should be smaller than glmnet.control()$epsnr",
            call. = FALSE)

  if(any(lower.limits > 0)){ stop("Lower limits should be non-positive") }
  if(any(upper.limits < 0)){ stop("Upper limits should be non-negative") }
  lower.limits[lower.limits == -Inf] = -control$big
  upper.limits[upper.limits == Inf] = control$big
  if (length(lower.limits) < nvars) {
    if(length(lower.limits) == 1) lower.limits = rep(lower.limits, nvars) else
      stop("Require length 1 or nvars lower.limits")
  } else lower.limits = lower.limits[seq(nvars)]
  if (length(upper.limits) < nvars) {
    if(length(upper.limits) == 1) upper.limits = rep(upper.limits, nvars) else
      stop("Require length 1 or nvars upper.limits")
  } else upper.limits = upper.limits[seq(nvars)]

  if (any(lower.limits == 0) || any(upper.limits == 0)) {
    ###Bounds of zero can mess with the lambda sequence and fdev;
    ###ie nothing happens and if fdev is not zero, the path can stop
    fdev <- glmnet.control()$fdev
    if(fdev!= 0) {
      glmnet.control(fdev = 0)
      on.exit(glmnet.control(fdev = fdev))
    }
  }
  ### end check on limits
  ### end preparation of generic arguments

  # standardize x if necessary
    xm <- rep(0.0, times = nvars)
    if (standardize) {
        xs <- meansd$sd
    } else {
        xs <- rep(1.0, times = nvars)
    }
    if (!inherits(x, "sparseMatrix")) {
    x <- scale(x,FALSE,xs)
  } else {
    attr(x, "xm") <- xm
    attr(x, "xs") <- xs
  }
  lower.limits <- lower.limits * xs
  upper.limits <- upper.limits * xs

  if (!("strata" %in% names(attributes(y))))
    y <- stratifySurv(y, rep(1, nobs))

  # Pre-compute and cache some important information: ordering by stop time
  # (ascending, deaths before censored), and for (start, stop] data: ordering
  # by start time and some match information.
  # Information is computed at the strata level
  if (ncol(y) == 2) {
    stop_o <- numeric(nobs)
    for (i in unique(attr(y, "strata"))) {
      ii <- which(attr(y, "strata") == i)
      stop_o[ii] <- order(y[ii, "time"], y[ii, "status"],
                          decreasing = c(FALSE, TRUE))
    }
    attr(y, "stop_time") <- stop_o
  } else {
    stop_o   <- numeric(nobs)
    start_o  <- numeric(nobs)
    ss_match <- numeric(nobs)
    for (i in unique(attr(y, "strata"))) {
      ii <- which(attr(y, "strata") == i)
      stop_o[ii]  <- order(y[ii, "stop"], y[ii, "status"],
                           decreasing = c(FALSE, TRUE))
      start_o[ii] <- order(y[ii, "start"], decreasing = c(FALSE))
      ss_match[ii] <- match(start_o[ii], stop_o[ii])
    }
    attr(y, "stop_time")  <- stop_o
    attr(y, "start_time") <- start_o
    attr(y, "ss_match")   <- ss_match
  }

  # compute null deviance
  # currently using std.weights = FALSE in order to match glmnet output
  nulldev <- coxnet.deviance(y = y, offset = offset, weights = weights,
                             std.weights = FALSE)

  # compute lambda_max and lambda values
  nlam = as.integer(nlambda)
  user_lambda = FALSE   # did user provide their own lambda values?
  if (is.null(lambda)) {
    if (lambda.min.ratio >= 1) stop("lambda.min.ratio should be less than 1")
    lambda_max <- get_cox_lambda_max(x, y, alpha, weights, offset, exclude, vp)
    ulam <- exp(seq(log(lambda_max), log(lambda_max * lambda.min.ratio),
                    length.out = nlam))
  } else {  # user provided lambda values
    user_lambda = TRUE
    if (any(lambda < 0)) stop("lambdas should be non-negative")
    ulam <- as.double(rev(sort(lambda)))
    nlam <- as.integer(length(lambda))
  }

  # start progress bar
  if (trace.it == 1) pb  <- createPB(min = 0, max = nlam, style = 3)

  beta <- matrix(0, nrow = nvars, ncol = nlam)
  dev.ratio <- rep(NA, length = nlam)
  fit <- NULL
  mnl <- min(nlam, control$mnlam)
  for (k in 1:nlam) {
    # get the correct lambda value to fit
    if (k > 1) {
      cur_lambda <- ulam[k]
    } else {
      cur_lambda <- ifelse(user_lambda, ulam[k], control$big)
    }

    if (trace.it == 2) cat("Fitting lambda index", k, ":", ulam[k], fill = TRUE)
    fit <- cox.fit(x, y, weights / sum(weights),
                   lambda = cur_lambda, alpha = alpha,
                   offset = offset, thresh = thresh, maxit = maxit,
                   penalty.factor = vp, exclude = exclude,
                   lower.limits = lower.limits, upper.limits = upper.limits,
                   warm = fit, from.cox.path = TRUE, save.fit = TRUE,
                   trace.it = trace.it)
    if (trace.it == 1) utils::setTxtProgressBar(pb, k)
    # if error code non-zero, a non-fatal error must have occurred
    # print warning, ignore this lambda value and return result
    # for all previous lambda values
    if (fit$jerr != 0) {
      errmsg <- jerr.glmnetfit(fit$jerr, maxit, k)
      warning(errmsg$msg, call. = FALSE)
      k <- k - 1
      break
    }
    beta[, k] <- as.vector(fit$beta)
    dev.ratio[k] <- fit$dev.ratio

    # early stopping if dev.ratio almost 1 or no improvement
    if (k >= mnl && user_lambda == FALSE) {
      if (dev.ratio[k] > control$devmax * 0.99 / 0.999) break
      if (k > 1 && dev.ratio[k] - dev.ratio[k - mnl + 1] <
          control$fdev * 100 * dev.ratio[k]) break
    }
  }
  if (trace.it == 1) {
    utils::setTxtProgressBar(pb, nlam)
    cat("", fill = TRUE)
  }

  # truncate beta, dev.ratio, lambda if necessary
  if (k < nlam) {
    beta <- beta[, 1:k, drop = FALSE]
    dev.ratio <- dev.ratio[1:k]
    ulam <- ulam[1:k]
  }

  # return coefficients to original scale (because of x standardization)
  beta <- beta / xs

  # output
  stepnames <- paste0("s", 0:(length(ulam) - 1))
  out <- list(a0 = NULL)
  out$beta <- Matrix::Matrix(beta, sparse = TRUE,
                             dimnames = list(vnames, stepnames))
  out$df <- as.vector(colSums(abs(beta) > 0))  # as.vector to remove names
  out$dim <- dim(beta)
  out$lambda <- ulam
  out$dev.ratio <- dev.ratio
  out$nulldev <- nulldev
  out$npasses <- fit$npasses
  out$jerr <- fit$jerr
  out$offset <- is.offset
  out$call <- this.call
  out$nobs <- nobs
  class(out) <- c("coxnet", "glmnet")
  out
}

#' Fit a Cox regression model with elastic net regularization for a single
#' value of lambda
#'
#' Fit a Cox regression model via penalized maximum likelihood for a single
#' value of lambda. Can deal with (start, stop] data and strata, as well as
#' sparse design matrices.
#'
#' WARNING: Users should not call \code{cox.fit} directly. Higher-level
#' functions in this package call \code{cox.fit} as a subroutine. If a
#' warm start object is provided, some of the other arguments in the function
#' may be overriden.
#'
#' \code{cox.fit} solves the elastic net problem for a single, user-specified
#' value of lambda. \code{cox.fit} works for Cox regression models, including
#' (start, stop] data and strata. It solves the problem using iteratively
#' reweighted least squares (IRLS). For each IRLS iteration, \code{cox.fit}
#' makes a quadratic (Newton) approximation of the log-likelihood, then calls
#' \code{elnet.fit} to minimize the resulting approximation.
#'
#' In terms of standardization: \code{cox.fit} does not standardize \code{x}
#' and \code{weights}. \code{penalty.factor} is standardized so that they sum
#' up to \code{nvars}.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed that any standardization needed
#' has already been done.
#' @param y Survival response variable, must be a Surv or stratifySurv object.
#' @param weights Observation weights. \code{cox.fit} does NOT standardize
#' these weights.
#' @param lambda A single value for the \code{lambda} hyperparameter.
#' @param alpha See glmnet help file
#' @param offset See glmnet help file
#' @param thresh Convergence threshold for coordinate descent. Each inner
#' coordinate-descent loop continues until the maximum change in the objective
#' after any coefficient update is less than thresh times the null deviance.
#' Default value is \code{1e-10}.
#' @param maxit Maximum number of passes over the data; default is \code{10^5}.
#' (If a warm start object is provided, the number of passes the warm start object
#' performed is included.)
#' @param penalty.factor See glmnet help file
#' @param exclude See glmnet help file
#' @param lower.limits See glmnet help file
#' @param upper.limits See glmnet help file
#' @param warm Either a \code{glmnetfit} object or a list (with name \code{beta}
#' containing coefficients) which can be used as a warm start. Default is
#' \code{NULL}, indicating no warm start. For internal use only.
#' @param from.cox.path Was \code{cox.fit()} called from \code{cox.path()}?
#' Default is FALSE.This has implications for computation of the penalty factors.
#' @param save.fit Return the warm start object? Default is FALSE.
#' @param trace.it Controls how much information is printed to screen. If
#' \code{trace.it=2}, some information about the fitting procedure is printed to
#' the console as the model is being fitted. Default is \code{trace.it=0}
#' (no information printed). (\code{trace.it=1} not used for compatibility with
#' \code{glmnet.path}.)
#'
#' @return An object with class "coxnet", "glmnetfit" and "glmnet". The list
#' returned contains more keys than that of a "glmnet" object.
#' \item{a0}{Intercept value, \code{NULL} for "cox" family.}
#' \item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
#' format.}
#' \item{df}{The number of nonzero coefficients.}
#' \item{dim}{Dimension of coefficient matrix.}
#' \item{lambda}{Lambda value used.}
#' \item{dev.ratio}{The fraction of (null) deviance explained. The deviance
#' calculations incorporate weights if present in the model. The deviance is
#' defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
#' for the saturated model (a model with a free parameter per observation).
#' Hence dev.ratio=1-dev/nulldev.}
#' \item{nulldev}{Null deviance (per observation). This is defined to be
#' 2*(loglike_sat -loglike(Null)). The null model refers to the 0 model.}
#' \item{npasses}{Total passes over the data.}
#' \item{jerr}{Error flag, for warnings and errors (largely for internal
#' debugging).}
#' \item{offset}{A logical variable indicating whether an offset was included
#' in the model.}
#' \item{call}{The call that produced this object.}
#' \item{nobs}{Number of observations.}
#' \item{warm_fit}{If \code{save.fit=TRUE}, output of C++ routine, used for
#' warm starts. For internal use only.}
#' \item{family}{Family used for the model, always "cox".}
#' \item{converged}{A logical variable: was the algorithm judged to have
#' converged?}
#' \item{boundary}{A logical variable: is the fitted value on the boundary of
#' the attainable values?}
#' \item{obj_function}{Objective function value at the solution.}
cox.fit <- function(x, y, weights, lambda, alpha = 1.0, offset = rep(0, nobs),
                    thresh = 1e-10, maxit = 100000,
                    penalty.factor = rep(1.0, nvars), exclude = c(),
                    lower.limits = -Inf, upper.limits = Inf, warm = NULL,
                    from.cox.path = FALSE, save.fit = FALSE, trace.it = 0) {
  this.call <- match.call()
  control <- glmnet.control()

  ### Prepare all generic arguments
  nobs <- nrow(x)
  nvars <- ncol(x)
  is.offset <- !(missing(offset))
  if (is.offset == FALSE) {
    offset <- as.double(rep(0, nobs))
  }
  # add xm and xs attributes if they are missing for sparse x
  # glmnet.fit assumes that x is already standardized. Any standardization
  # the user wants should be done beforehand.
  if (inherits(x, "sparseMatrix")) {
    if ("xm" %in% names(attributes(x)) == FALSE)
      attr(x, "xm") <- rep(0.0, times = nvars)
    if ("xs" %in% names(attributes(x)) == FALSE)
      attr(x, "xs") <- rep(1.0, times = nvars)
  }

  # if calling from cox.path(), we do not need to check on exclude
  # and penalty.factor arguments as they have been prepared by cox.path()
  if (!from.cox.path) {
    # check and standardize penalty factors (to sum to nvars)
    if(any(penalty.factor == Inf)) {
      exclude = c(exclude, seq(nvars)[penalty.factor == Inf])
      exclude = sort(unique(exclude))
    }
    if(length(exclude) > 0) {
      jd = match(exclude, seq(nvars), 0)
      if(!all(jd > 0)) stop ("Some excluded variables out of range")
      penalty.factor[jd] = 1 # ow can change lambda sequence
    }
    vp = pmax(0, penalty.factor)
    vp = as.double(vp * nvars / sum(vp))
  } else {
    vp <- as.double(penalty.factor)
  }

  ### check on limits
  lower.limits[lower.limits == -Inf] = -control$big
  upper.limits[upper.limits == Inf] = control$big
  if (length(lower.limits) < nvars)
    lower.limits = rep(lower.limits, nvars) else
      lower.limits = lower.limits[seq(nvars)]
  if (length(upper.limits) < nvars)
    upper.limits = rep(upper.limits, nvars) else
      upper.limits = upper.limits[seq(nvars)]
  ### end check on limits
  ### end preparation of generic arguments

  # compute null deviance
  if (is.null(warm)) {
    nulldev <- coxnet.deviance(y = y, offset = offset, weights = weights,
                               std.weights = FALSE)
    fit <- NULL
    coefold <- rep(0, nvars)   # initial coefs = 0
    eta <- offset
  } else {
    if (inherits(warm,"glmnetfit")) {
      if (!is(warm$warm_fit,"warmfit")) stop("Invalid warm start object")
      fit <- warm
      nulldev <- fit$nulldev
      coefold <- fit$warm_fit$a  # prev value for coefficients
      eta <- get_eta(x, coefold, 0) + offset
    } else if (inherits(warm,"list") && "beta" %in% names(warm)) {
      fit <- warm
      nulldev <- coxnet.deviance(y = y, offset = offset, weights = weights,
                                 std.weights = FALSE)
      coefold <- fit$beta   # prev value for coefficients
      eta <- get_eta(x, coefold, 0) + offset
      fit$a0 <- 0  # needed for compatibility with elnet.fit()
    } else {
      stop("Invalid warm start object")
    }
  }

  start <- NULL     # current value for coefficients
  obj_val_old <- cox_obj_function(y, eta, weights, lambda, alpha, coefold, vp)
  if (trace.it == 2) {
    cat("Warm Start Objective:", obj_val_old, fill = TRUE)
  }
  conv <- FALSE      # converged?

  # IRLS loop
  for (iter in 1L:control$mxitnr) {
    # compute working response and weights
    coxgrad_results <- coxgrad(eta, y, weights, std.weights = FALSE,
                               diag.hessian = TRUE)
    w <- -attributes(coxgrad_results)$diag_hessian
    z <- (eta - offset) - ifelse(w != 0, -coxgrad_results / w, 0)

    # have to update the weighted residual in our fit object
    # (in theory g and iy should be updated too, but we actually recompute g
    # and iy anyway in wls.f)
    if (!is.null(fit)) {
      fit$warm_fit$r <- w * (z - eta + offset)
    }

    # do WLS with warmstart from previous iteration
    fit <- elnet.fit(x, z, w, lambda, alpha, intercept = FALSE,
                     thresh = thresh, maxit = maxit, penalty.factor = vp,
                     exclude = exclude, lower.limits = lower.limits,
                     upper.limits = upper.limits, warm = fit,
                     from.glmnet.fit = TRUE, save.fit = TRUE)
    if (fit$jerr != 0) return(list(jerr = fit$jerr))

    # update coefficients, eta, mu and obj_val
    start <- fit$warm_fit$a
    eta <- get_eta(x, start, 0) + offset
    obj_val <- cox_obj_function(y, eta, weights, lambda, alpha, start, vp)
    if (trace.it == 2) cat("Iteration", iter, "Objective:", obj_val, fill = TRUE)

    boundary <- FALSE
    halved <- FALSE  # did we have to halve the step size?
    # if objective function is not finite, keep halving the stepsize until it is finite
    # for the halving step, we probably have to adjust fit$g as well?
    if (!is.finite(obj_val) || obj_val > control$big) {
      warning("Infinite objective function!", call. = FALSE)
      if (is.null(coefold))
        stop("no valid set of coefficients has been found: please supply starting values",
             call. = FALSE)
      warning("step size truncated due to divergence", call. = FALSE)
      ii <- 1
      while (!is.finite(obj_val) || obj_val > control$big) {
        if (ii > control$mxitnr)
          stop("inner loop 1; cannot correct step size", call. = FALSE)
        ii <- ii + 1
        start <- (start + coefold)/2
        eta <- get_eta(x, start, 0) + offset
        obj_val <- cox_obj_function(y, eta, weights, lambda, alpha, start, vp)
        if (trace.it == 2) cat("Iteration", iter, " Halved step 1, Objective:",
                               obj_val, fill = TRUE)
      }
      boundary <- TRUE
      halved <- TRUE
    }

    # if we did any halving, we have to update the coefficients, intercept
    # and weighted residual in the warm_fit object
    if (halved) {
      fit$warm_fit$a <- start
      fit$warm_fit$r <- w * (z - eta)
    }

    # test for convergence
    if (abs(obj_val - obj_val_old)/(0.1 + abs(obj_val)) < control$epsnr) {
      conv <- TRUE
      break
    }
    else {
      coefold <- start
      obj_val_old <- obj_val
    }
  }
  # end of IRLS loop

  # checks on convergence and fitted values
  if (!conv)
    warning("cox.fit: algorithm did not converge", call. = FALSE)

  # prepare output object
  if (save.fit == FALSE) {
    fit$warm_fit <- NULL
  }
  # overwrite values from elnet.fit object
  fit$a0 <- list(NULL)
  fit$call <- this.call
  fit$offset <- is.offset
  fit$nulldev <- nulldev
  fit$dev.ratio <- 1 - coxnet.deviance(y = y, pred = eta, weights = weights,
                                       std.weights = FALSE) / nulldev

  # add new key-value pairs to list
  fit$family <- "cox"
  fit$converged <- conv
  fit$boundary <- boundary
  fit$obj_function <- obj_val

  class(fit) <- c("coxnet", "glmnetfit", "glmnet")
  fit
}

#' Elastic net objective function value for Cox regression model
#'
#' Returns the elastic net objective function value for Cox regression model.
#'
#' @param y Survival response variable, must be a \code{Surv} or
#' \code{stratifySurv} object.
#' @param pred Model's predictions for \code{y}.
#' @param weights Observation weights.
#' @param lambda A single value for the \code{lambda} hyperparameter.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' @param coefficients The model's coefficients.
#' @param vp Penalty factors for each of the coefficients.
cox_obj_function <- function(y, pred, weights, lambda, alpha,
                             coefficients, vp) {
  coxnet.deviance(y = y, pred = pred, weights = weights, std.weights = FALSE) +
    lambda * pen_function(coefficients, alpha, vp)
}

#' Get lambda max for Cox regression model
#'
#' Return the lambda max value for Cox regression model, used for computing
#' initial lambda values. For internal use only.
#'
#' This function is called by \code{cox.path} for the value of lambda max.
#'
#' When \code{x} is not sparse, it is expected to already by centered and scaled.
#' When \code{x} is sparse, the function will get its attributes \code{xm} and
#' \code{xs} for its centering and scaling factors. The value of
#' \code{lambda_max} changes depending on whether \code{x} is centered and
#' scaled or not, so we need \code{xm} and \code{xs} to get the correct value.
#'
#' @param x Input matrix, of dimension \code{nobs x nvars}; each row is an
#' observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
#' It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
#' \code{xs(j)} are the centering and scaling factors for variable j respsectively.
#' If it is not a sparse matrix, it is assumed to be standardized.
#' @param y Survival response variable, must be a \code{Surv} or
#' \code{stratifySurv} object.
#' @param alpha The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
#' @param weights Observation weights.
#' @param offset Offset for the model. Default is a zero vector of length
#' \code{nrow(y)}.
#' @param exclude Indices of variables to be excluded from the model.
#' @param vp Separate penalty factors can be applied to each coefficient.
get_cox_lambda_max <- function(x, y, alpha, weights = rep(1, nrow(x)),
                               offset = rep(0, nrow(x)), exclude = c(),
                               vp = rep(1, ncol(x))) {
  nobs <- nrow(x); nvars <- ncol(x)

  # extract strata (if any)
  if ("strata" %in% names(attributes(y))) {
    strata <- attr(y, "strata")
  } else {
    strata <- rep(1, nobs)
  }
  if (length(strata) != nobs) stop("length of strata != nobs")

  # if some penalty factors are zero, we need to compute eta
  vp_zero <- setdiff(which(vp == 0), exclude)
  if (length(vp_zero) > 0) {
    tempx <- x[, vp_zero, drop = FALSE]
    if(inherits(tempx, "sparseMatrix")) {
        attr(tempx, "xm") <- rep(0.0, length(vp_zero))
        attr(tempx, "xs") <- attr(x,"xs")[vp_zero]
      ## coxph cannot handle sparse x. Strata not needed because y is expected to be stratified
      fit <- cox.fit(x = tempx, y = y, offset = offset, weights = weights/sum(weights), lambda = 0)
      fit$beta <- fit$beta/attr(tempx,"xs")# need to put beta on the correct scale for next line to work
      eta <- as.numeric(predict(fit, newx = tempx, newoffset = offset, newstrata = strata))
    } else {
        eps <- glmnet.control()$epsnr

      if (length(unique(strata)) == 1) {
        fit <- survival::coxph(y ~ offset(offset) + tempx, weights = weights, eps = eps)
      } else {
        fit <- survival::coxph(y ~ offset(offset) + tempx + strata(strata),
                               weights = weights, eps = eps)
      }
      eta <- predict(fit, reference="sample") ## Coxph can do strata-specific centering
    }
  } else {
    eta <- offset
  }
  eta <- eta - mean(eta) ## keep numbers small; partial likelihood independent of centering
  ju <- rep(1, nvars)
  ju[exclude] <- 0 # we have already included constant variables in exclude

  # get cox gradient at "null" point
  # note that coxgrad already includes weights, so no need to include them
  # in subsequent computations
  null_grad <- coxgrad(eta, y, weights)

    if (inherits(x, "sparseMatrix")) {
        xm <- attr(x, "xm")
        xs <- attr(x, "xs")
        g <- abs((drop(t(null_grad) %*% x) - sum(null_grad) * xm) / xs)
    } else {
        g <- abs(drop(t(null_grad) %*% x))
    }
  g <- g / ifelse(vp > 0, vp, 1)
  g[ju == 0] <- 0
  lambda_max <- max(g) / max(alpha, 1e-3)
  return(lambda_max)
}

#' @method buildPredmat array
#' @export
buildPredmat.array=function(outlist, lambda, x, offset, foldid, alignment,...){
    if (!is.null(offset)) is.offset = TRUE
    else is.offset = FALSE
    nc=dim(outlist[[1]]$a0)[1]
    nobs=nrow(x)
    predmat = array(NA, c(nobs, nc, length(lambda)))
    nfolds = max(foldid)
    nlams = double(nfolds)
    nlambda = length(lambda)
    for (i in seq(nfolds)) {
        which = foldid == i
        fitobj = outlist[[i]]
        if (is.offset)
            off_sub = offset[which, , drop = FALSE]
        preds = switch(alignment, fraction = predict(fitobj,
                       x[which, , drop = FALSE], newoffset = off_sub,...),
                       lambda = predict(fitobj, x[which, , drop = FALSE],
                       s = lambda, newoffset = off_sub,...))
        nlami = min(dim(preds)[3], nlambda)
        predmat[which, , seq(nlami)] = preds[, , seq(nlami)]
        if (nlami < nlambda)
            predmat[which, , seq(from = nlami, to = nlambda)] = preds[,, nlami]
    }
### fix up dimnames
    rn=rownames(x)
    sn=paste("s",seq(0,length=nlambda),sep="")
    cn=dimnames(preds)[[2]]
    dimnames(predmat)=list(rn,cn,sn)
    predmat
}

#' @method buildPredmat multnetlist
#' @export
buildPredmat.multnetlist=function(outlist, lambda, x, offset, foldid, alignment,...){
    mat=buildPredmat.array(outlist, lambda, x, offset, foldid, alignment,...)
    attr(mat,"classnames")=outlist[[1]]$classnames
    mat
}

#' @method buildPredmat mrelnetlist
#' @export
buildPredmat.mrelnetlist=function(outlist, lambda, x, offset, foldid, alignment,...){
    buildPredmat.array(outlist, lambda, x, offset, foldid, alignment,...)
}

#' @method buildPredmat lognetlist
#' @export
buildPredmat.lognetlist=function(outlist, lambda, x, offset, foldid, alignment,...){
    mat=buildPredmat.default(outlist, lambda, x, offset, foldid, alignment,...)
    attr(mat,"classnames")=outlist[[1]]$classnames
    mat
}


#' @method buildPredmat glmnetfitlist
#' @export
buildPredmat.glmnetfitlist=function(outlist, lambda, x, offset, foldid, alignment,family,...){
    mat=buildPredmat.default(outlist, lambda, x, offset, foldid, alignment,...)
    attr(mat,"family")=family
    mat
}

mrelnet=function(x,is.sparse,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,vnames,maxit,pb){
  maxit=as.integer(maxit)
  weights=as.double(weights)
### compute the null deviance
  y=as.matrix(y)
  nr=ncol(y)
  if(nr>1){
    responseNames=dimnames(y)[[2]]
    if(is.null(responseNames))responseNames=paste("y",seq(nr),sep="")
  }

 storage.mode(y)="double"
   if(is.null(offset)){
    is.offset=FALSE}
  else{
    offset=as.matrix(offset)
    storage.mode(offset)="double"
    if(!all.equal(dim(offset),dim(y)))stop("Offset must match dimension of y")
    y=y-offset
    is.offset=TRUE
    }

  nulldev=0
  for(i in seq(nr)){
  ybar=weighted.mean(y[,i],weights)
  nulldev=nulldev+sum(weights* (y[,i]-ybar)^2)
}

  storage.mode(nr)="integer"

fit=if(is.sparse) multspelnet_exp(
        parm=alpha,x,y,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,maxit,pb,
        lmu=integer(1),
        a0=matrix(0.0,nr,nlam),
        ca=double(nx*nlam*nr),
        ia=integer(nx),
        nin=integer(nlam),
        rsq=double(nlam),
        alm=double(nlam),
        nlp=integer(1),
        jerr=integer(1)
        )
else multelnet_exp(
          parm=alpha,x,y,weights,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,jsd,intr,maxit,pb,
          lmu=integer(1),
          a0=matrix(0.0,nr,nlam),
          ca=double(nx*nlam*nr),
          ia=integer(nx),
          nin=integer(nlam),
          rsq=double(nlam),
          alm=double(nlam),
          nlp=integer(1),
          jerr=integer(1)
          )
if(fit$jerr!=0){
  errmsg=jerr(fit$jerr,maxit,pmax=nx,family="mrelnet")
  if(errmsg$fatal)stop(errmsg$msg,call.=FALSE)
  else warning(errmsg$msg,call.=FALSE)
}
  if(nr>1)
    outlist=getcoef.multinomial(fit,nvars,nx,vnames,nr,responseNames,center.intercept=FALSE)
  else
    outlist=getcoef(fit,nvars,nx,vnames)
  dev=fit$rsq[seq(fit$lmu)]
  outlist=c(outlist,list(dev.ratio=dev,nulldev=nulldev,npasses=fit$nlp,jerr=fit$jerr,offset=is.offset))
  class(outlist)="mrelnet"
  outlist
}

#' plot the cross-validation curve produced by cv.glmnet
#'
#' Plots the cross-validation curve, and upper and lower standard deviation
#' curves, as a function of the \code{lambda} values used. If the object has
#' class \code{"cv.relaxed"} a different plot is produced, showing both
#' \code{lambda} and \code{gamma}
#'
#' A plot is produced, and nothing is returned.
#'
#' @aliases plot.cv.glmnet
#' @param x fitted \code{"cv.glmnet"} object
#' @param sign.lambda Either plot against \code{log(lambda)} (default) or its
#' negative if \code{sign.lambda=-1}.
#' @param \dots Other graphical parameters to plot
#' @author Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
#' Trevor Hastie <hastie@@stanford.edu>
#' @seealso \code{glmnet} and \code{cv.glmnet}.
#' @references Friedman, J., Hastie, T. and Tibshirani, R. (2008)
#' \emph{Regularization Paths for Generalized Linear Models via Coordinate
#' Descent}
#' @keywords models regression
#' @examples
#'
#' set.seed(1010)
#' n = 1000
#' p = 100
#' nzc = trunc(p/10)
#' x = matrix(rnorm(n * p), n, p)
#' beta = rnorm(nzc)
#' fx = (x[, seq(nzc)] %*% beta)
#' eps = rnorm(n) * 5
#' y = drop(fx + eps)
#' px = exp(fx)
#' px = px/(1 + px)
#' ly = rbinom(n = length(px), prob = px, size = 1)
#' cvob1 = cv.glmnet(x, y)
#' plot(cvob1)
#' title("Gaussian Family", line = 2.5)
#' cvob1r = cv.glmnet(x, y, relax = TRUE)
#' plot(cvob1r)
#' frame()
#' set.seed(1011)
#' par(mfrow = c(2, 2), mar = c(4.5, 4.5, 4, 1))
#' cvob2 = cv.glmnet(x, ly, family = "binomial")
#' plot(cvob2)
#' title("Binomial Family", line = 2.5)
#' ## set.seed(1011)
#' ## cvob3 = cv.glmnet(x, ly, family = "binomial", type = "class")
#' ## plot(cvob3)
#' ## title("Binomial Family", line = 2.5)
#'
#' @method plot cv.glmnet
#' @export
plot.cv.glmnet=function(x,sign.lambda=1,...){
    cvobj=x
    xlab = expression(Log(lambda))
#  xlab="log(Lambda)"
  if(sign.lambda<0)xlab=paste("-",xlab,sep="")
  plot.args=list(x=sign.lambda*log(cvobj$lambda),y=cvobj$cvm,ylim=range(cvobj$cvup,cvobj$cvlo),xlab=xlab,ylab=cvobj$name,type="n")
  new.args=list(...)
  if(length(new.args))plot.args[names(new.args)]=new.args
do.call("plot",plot.args)
    error.bars(sign.lambda*log(cvobj$lambda),cvobj$cvup,cvobj$cvlo,width=0.01,
               col="darkgrey")
#               col="antiquewhite2")

    points(sign.lambda*log(cvobj$lambda),cvobj$cvm,pch=20,
          col="red")
axis(side=3,at=sign.lambda*log(cvobj$lambda),labels=paste(cvobj$nz),tick=FALSE,line=0)
abline(v=sign.lambda*log(cvobj$lambda.min),lty=3)
abline(v=sign.lambda*log(cvobj$lambda.1se),lty=3)
  invisible()
}

#' @method predict lognet
#' @export
predict.lognet=function(object,newx,s=NULL,type=c("link","response","coefficients","class","nonzero"),exact=FALSE,newoffset,...){
  type=match.arg(type)
    ### remember that although the C++ lognet makes predictions
  ### for the first class, we make predictions for the second class
  ### to avoid confusion with 0/1 responses.
  ### glmnet flipped the signs of the coefficients
nfit=NextMethod("predict")
  switch(type,
         response={
           pp=exp(-nfit)
           1/(1+pp)
         },
         class={
           cnum=ifelse(nfit>0,2,1)
           clet=object$classnames[cnum]
           if(is.matrix(cnum))clet=array(clet,dim(cnum),dimnames(cnum))
           clet
         },
         nfit
         )
}

\documentclass[a4paper]{article}
\usepackage{graphicx}
% \VignetteIndexEntry{Fitting the Penalized Cox Model}


\title{Coxnet: Regularized Cox Regression}
\author{Noah Simon\\
Jerome Friedman\\
Trevor Hastie\\
Rob Tibshirani}

\begin{document}


\maketitle
\section{Introduction}
We will give a short tutorial on using coxnet. Coxnet is a function which fits the Cox Model regularized by an elastic net penalty. It is used for underdetermined (or nearly underdetermined systems) and chooses a small number of covariates to include in the model. Because the Cox Model
is rarely used for actual prediction, we will rather focus on finding
and interpretating an appropriate model. 
We give a simple example of how to format data and run the Cox Model in
glmnet with cross validation.

\section{Example}
We first load our data and set up the response. In this case $x$ must
be an $n$ by $p$ matrix of covariate values --- each row corresponds to a patient and each column a covariate. $y$ is an $n$ length vector of failure/censoring times,
and status is an $n$ length vector with each entry, a $1$ or a $0$,
indicating whether the corresponding entry in $y$ is indicative
of a failure time or right censoring time ($1$ for failure, $0$ for censoring) 

<<>>=
library("glmnet")
library("survival")
load("VignetteExample.rdata")
attach(patient.data)
@

We then call our functions to fit with the lasso penalty ($\alpha=1$),
and cross validate. We set {\tt maxit = 1000} (increasing the maximum
number of iterations to $1000$) because our data is relatively high
dimensional, so  more iterations are needed for convergence. In
practice, the function will spit out an error if convergence isn't
reached by the maximum number of iterations.

<<>>=
cv.fit <- cv.glmnet(x, Surv(time, status), family="cox", maxit = 1000)
fit <- glmnet(x, Surv(time,status), family =  "cox", maxit = 1000)
@

The {\tt Surv} function packages the survival data into the form
expected by glmnet. Once fit, we can view the optimal $\lambda$ value and a cross
validated error plot to help evaluate our
model.

\begin{center}
\setkeys{Gin}{width = 3 in}
<<fig=TRUE>>=
plot(cv.fit)
@
\end{center}
<<>>=
cv.fit$lambda.min
@ 

The left vertical line in our plot shows us where the CV-error curve
hits its minimum. The right vertical line shows us the most
regularized model with CV-error within $1$ standard deviation of the
minimum. In this case, we see that the minimum was achieved by a
fairly regularized model, however the right line indicates that the
null model (no coefficients included) is within $1$ sd of the
minimum. This might lead us to believe that in actuality the
covariates are not explaining any variability. For the time being we
will concern ourselves with the minimum CV-error model. We can check which covariates our model chose to be active, and see the coefficients of those covariates.

<<>>=
Coefficients <- coef(fit, s = cv.fit$lambda.min)
Active.Index <- which(Coefficients != 0)
Active.Coefficients  <- Coefficients[Active.Index]
@

{\tt coef(fit, s = cv.fit\$lambda.min)} returns the $p$ length coefficient vector of the solution corresponding to $\lambda = ${\tt cv.fit\$lambda.min}.

<<>>=
Active.Index
Active.Coefficients
@

We see that our optimal model chose 2 active covariates ($X80$ and $X394$) each with a small positive effect on hazard.

\end{document}

---
title: "Regularized Cox Regression"
author: 
  - Kenneth Tay
  - Noah Simon
  - Jerome Friedman
  - Trevor Hastie
  - Rob Tibshirani
  - Balasubramanian Narasimhan
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: assets/glmnet_refs.bib
link-citations: true
output:
  pdf_document:
    fig_caption: yes
    toc: yes
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Regularized Cox Regression}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r include=FALSE}
# the code in this chunk enables us to truncate the print output for each
# chunk using the `out.lines` option
# save the built-in output hook
hook_output <- knitr::knit_hooks$get("output")

# set a new output hook to truncate text output
knitr::knit_hooks$set(output = function(x, options) {
  if (!is.null(n <- options$out.lines)) {
    x <- xfun::split_lines(x)
    if (length(x) > n) {
      # truncate the output
      x <- c(head(x, n), "....\n")
    }
    x <- paste(x, collapse = "\n")
  }
  hook_output(x, options)
})
```

## Introduction

This vignette describes how one can use the `glmnet` package to fit regularized Cox models.

The Cox proportional hazards model is commonly used for the study of the relationship beteween predictor variables and survival time. In the usual survival analysis framework, we have data of the form $(y_1, x_1, \delta_1), \ldots, (y_n, x_n, \delta_n)$ where $y_i$, the observed time, is a time of failure if $\delta_i$ is 1 or a right-censored time if $\delta_i$ is 0. We also let $t_1 < t_2 < \ldots < t_m$ be the increasing list of unique failure times, and let $j(i)$ denote the index of the observation failing at time $t_i$.

The Cox model assumes a semi-parametric form for the hazard
$$
h_i(t) = h_0(t) e^{x_i^T \beta},
$$
where $h_i(t)$ is the hazard for patient $i$ at time $t$, $h_0(t)$ is a shared baseline hazard, and $\beta$ is a fixed, length $p$ vector. In the classic setting $n \geq p$, inference is made via the partial likelihood
$$
L(\beta) = \prod_{i=1}^m \frac{e^{x_{j(i)}^T \beta}}{\sum_{j \in R_i} e^{x_j^T \beta}},
$$
where $R_i$ is the set of indices $j$ with $y_j \geq t_i$ (those at risk at time $t_i$).

Note there is no intercept in the Cox model as it is built into the baseline hazard, and like it, would cancel in the partial likelihood.

In `glmnet`, we penalize the negative log of the partial likelihood with an elastic net penalty.

(Credits: The original `"coxnet"` algorithm for right-censored data was developed by Noah Simon, Jerome Friedman, Trevor Hastie and Rob Tibshirani: see @coxnet for details. The other features for Cox models, introduced in v4.1, were developed by Kenneth Tay, Trevor Hastie, Balasubramanian Narasimhan and Rob Tibshirani.)

## Basic usage for right-censored data

We use a pre-generated set of sample data and response. `x` must be an $n\times p$ matrix of covariate values --- each row corresponds to a patient and each column a covariate. `y` is an $n \times 2$  matrix, with a column `"time"` of failure/censoring times, and `"status"` a 0/1 indicator, with 1 meaning the time is a failure time, and 0 a censoring time. The `Surv` function in the `survival` package creates such a response matrix, and it is recommended that the user uses the output of a call to `Surv` for the  response to `glmnet`. (For backward compatibility, `glmnet` can accept a two-column matrix with column names `"time"` and `"status"` for right-censored data.)
```{r}
library(glmnet)
library(survival)
data(CoxExample)
x <- CoxExample$x
y <- CoxExample$y
y[1:5, ]
```

We apply the `glmnet` function to compute the solution path under default settings:
```{r}
fit <- glmnet(x, y, family = "cox")
```

All the standard options such as `alpha`, `weights`, `nlambda` and `standardize` package, and their usage is similar as in the Gaussian case. (See the vignette ["An Introduction to `glmnet`"](https://glmnet.stanford.edu/articles/glmnet.html)  for details, or refer to the help file `help(glmnet)`.)

We can plot the coefficients with the `plot` method:
```{r}
plot(fit)
```

As before, we can extract the coefficients at certain values of $\lambda$:
```{r out.lines = 10}
coef(fit, s = 0.05)
```

Since the Cox Model is not commonly used for prediction, we do not give an illustrative example on prediction. If needed, users can refer to the help file by typing `help(predict.glmnet)`.

### Cross-validation

The function `cv.glmnet` can be used to compute $K$-fold cross-validation (CV) for the Cox model. The usage is similar to that for other families except for two main differences.

First, `type.measure` only supports `"deviance"` (also default) which
gives the partial-likelihood, and `"C"`, which gives the Harrell
*C index*. This is like the area under the curve (AUC) measure of concordance for survival data, but only considers comparable pairs. Pure concordance would record the fraction of pairs for which the order of the death times agree with the order of the predicted risk. However, with survival data, if an observation is right censored at a time *before* another observation's death time, they are not comparable.

The code below illustrates how one can perform cross-validation using the Harell C index. Note that unlike most error measures, a higher C index means better prediction performance.
```{r}
set.seed(1)
cvfit <- cv.glmnet(x, y, family = "cox", type.measure = "C")
```

Once fit, we can view the optimal $\lambda$ value and a cross validated error plot to help evaluate our model.
```{r}
plot(cvfit)
```

As with other families, the left vertical line in our plot shows us where the CV-error curve hits its minimum. The right vertical line shows us the most regularized model with CV-error within 1 standard deviation of the minimum. We also extract such optimal $\lambda$'s:
```{r}
cvfit$lambda.min
cvfit$lambda.1se
```

Second, the option `grouped = TRUE` (default) obtains the CV partial likelihood for the Kth fold by subtraction, i.e. by subtracting the log partial likelihood evaluated on the full dataset from that evaluated on the $(K-1)/K$ dataset. This makes more efficient use of risk sets. With `grouped = FALSE` the log partial likelihood is computed only on the $K$th fold, which is only reasonable if each fold has a large number of observations.

### Handling of ties

`glmnet` supports both the Breslow and Efron approximations for handling ties in survival times. The `cox.ties` argument selects between them.

In the current release (v5.0) the default is `cox.ties = "breslow"`, which matches the pre-v5.0 behavior of `glmnet`. **In v5.1 this default will change to `cox.ties = "efron"` to match the default of `survival::coxph`.** Calls to `glmnet()` with `family = "cox"` that do not set `cox.ties` explicitly will emit a transition warning until the switch occurs. To silence the warning, pass `cox.ties` explicitly:

- `cox.ties = "breslow"` locks in the v5.0 default.
- `cox.ties = "efron"` previews the v5.1 default.

```{r}
# create x matrix
set.seed(1)
nobs <- 100; nvars <- 15
x <- matrix(rnorm(nobs * nvars), nrow = nobs)

# create response with many ties
ty <- rep(rexp(nobs / 5), each = 5)
tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
y <- Surv(ty, tcens)

# Efron: set cox.ties = "efron" in glmnet (matches coxph's default)
glmnet_fit <- glmnet(x, y, family = "cox", lambda = 0, cox.ties = "efron")
coxph_fit <- coxph(y ~ x)
plot(coef(glmnet_fit), coef(coxph_fit))
abline(0, 1)
```

The Breslow approximation (current `glmnet` default):
```{r}
# Breslow: set cox.ties = "breslow" in glmnet, ties = "breslow" in coxph
glmnet_fit <- glmnet(x, y, family = "cox", lambda = 0, cox.ties = "breslow")
coxph_fit <- coxph(y ~ x, ties = "breslow")
plot(coef(glmnet_fit), coef(coxph_fit))
abline(0, 1)
```

## Cox models for start-stop data

Since version 4.1 `glmnet` can fit models where the response is a (start, stop] time interval. As explained in @Therneau2000, the ability to work with start-stop responses opens the door to fitting regularized Cox models with

* time-dependent covariates,
* time-dependent strata,
* left truncation,
* multiple time scales,
* multiple events per subject,
* independent increment, marginal, and conditional models for correlated data, and
* various forms of case-cohort models.

The code below shows how to create a response of this type (using `survival` package's `Surv` function) and how to fit such a model with `glmnet`.
```{r}
# create x matrix
set.seed(2)
nobs <- 100; nvars <- 15
xvec <- rnorm(nobs * nvars)
xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
x <- matrix(xvec, nrow = nobs)  # non-sparse x
x_sparse <- Matrix::Matrix(xvec, nrow = nobs, sparse = TRUE)  # sparse x

# create start-stop data response
beta <- rnorm(5)
fx <- x_sparse[, 1:5] %*% beta / 3
ty <- rexp(nobs, drop(exp(fx)))
tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
starty <- runif(nobs)
yss <- Surv(starty, starty + ty, tcens)

# fit regularized Cox model with start-stop data
fit <- glmnet(x, yss, family = "cox")
```

(Note that the call above would have worked as well if `x` was replaced by `x_sparse`.) `cv.glmnet` works with start-stop data too:
```{r}
cv.fit <- cv.glmnet(x, yss, family = "cox", nfolds = 5)
plot(cv.fit)
```

As a sanity check, the code below shows that fitting start-stop responses
using `glmnet` with `lambda = 0` matches up with `coxph`'s result:
```{r}
glmnet_fit <- glmnet(x, yss, family = "cox", lambda = 0)
coxph_fit <- coxph(yss ~ x)
plot(coef(glmnet_fit), coef(coxph_fit))
abline(0, 1)
```

## Stratified Cox models

One extension of the Cox regression model is to allow for strata that divide the observations into disjoint groups. Each group has its own baseline hazard function, but the groups share the same coefficient vector for the covariates provided by the design matrix `x`.

`glmnet` can fit stratified Cox models with the elastic net
penalty. With `coxph` one can specify strata in the model formula.
Since `glmnet` does not use a model formula, we achieve this by adding
a strata attribute to the `Surv` response object. We achieve this via
the  function `stratifySurv`:
```{r}
strata <- rep(1:5, length.out = nobs)
y2 <- stratifySurv(y, strata)
str(y2[1:6])
```

`stratifySurv` returns an object of class `stratifySurv`. We can then pass this `stratifySurv` object as the response to a `glmnet` call. `glmnet` will fit a stratified Cox model if it detects that the response has class `stratifySurv`.
```{r}
fit <- glmnet(x, y2, family = "cox")
```

This `stratifySurv` object can also be passed to `cv.glmnet` to fit stratified Cox models with cross-validation:
```{r}
cv.fit <- cv.glmnet(x, y2, family = "cox", nfolds = 5)
plot(cv.fit)
```

Note that simply giving the response a `"strata"` attribute is not enough! The response needs to be of class `stratifySurv` in order for subsetting to work correctly. To protect against this, an error will be thrown if the response has a `"strata"` attribute but is not of class `stratifySurv`. Add strata via the `stratifySurv` function.
```{r}
y3 <- y
attr(y3, "strata") <- strata
str(y3[1:6])  # note that the strata attribute is no longer there
```

```{r error=TRUE}
fit <- glmnet(x, y3, family = "cox")
```

## Plotting survival curves

Fitting a regularized Cox model using `glmnet` with `family = "cox"`
returns an object of class `"coxnet"`. Class `"coxnet"` objects have a
`survfit` method which allows the user to visualize the survival
curves from the model. In addition to the `"coxnet"` object, the user
must pass the `x` and `y` objects used to fit the model (for
computation of the baseline hazard), as well as the lambda value for which the survival curve is desired:
```{r}
fit <- glmnet(x, y, family = "cox")
survival::survfit(fit, s = 0.05, x = x, y = y)
```

We are unable to provide standard errors for these survival curves, so we do not present the confidence bounds for them. To plot the survival curve, pass the result of the `survfit` call to `plot`:
```{r}
plot(survival::survfit(fit, s = 0.05, x = x, y = y))
```

As noted in the documentation for `survival::survfit.coxph`, without new data, a curve is produced for a single "pseudo" subject with covariate values equal to the means of the data set, and this resulting  curve(s) almost never make sense. We can get survival curves for individual observations by passing a `newx` argument:
```{r}
survival::survfit(fit, s = 0.05, x = x, y = y, newx = x[1:3, ])
plot(survival::survfit(fit, s = 0.05, x = x, y = y, newx = x[1:3, ]))
```

If the original model was fit with strata, then the `strata` option needs to be specified as well. If `newx` is being passed for such a model, the strata for these new observations need to be passed  via `newstrata`.
```{r}
y2 <- stratifySurv(y, rep(1:2, length.out = nobs))
fit <- glmnet(x, y2, family = "cox")
survival::survfit(fit, s = 0.01, x = x, y = y2)

# survival curve plot for first two individuals in dataset
plot(survival::survfit(fit, s = 0.01, x = x, y = y2, 
                       newx = x[1:2, ], newstrata = strata[1:2]))
```

To be consistent with other methods in `glmnet`, if the `s` parameter is not specified, survival curves are returned for the entire `lambda` sequence. The survival curves are returned as a list, one element for each `lambda` value.
```{r}
sf <- survival::survfit(fit, x = x, y = y2)
length(sf)
length(fit$lambda)
```

The `survfit` method is available for `cv.glmnet` objects as well. By default, the `s` value chosen is the "lambda.1se" value stored in the CV object. The `s` value can also be set to the `"lambda.min"` value stored in the
CV object.
```{r}
cv.fit <- cv.glmnet(x, y2, family = "cox", nfolds = 5)
survival::survfit(cv.fit, x = x, y = y2)
survival::survfit(cv.fit, s = "lambda.min", x = x, y = y2)
```

## References

---
title: "A History of glmnet"
author:
  - Trevor Hastie
  - Balasubramanian Narasimhan
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: assets/glmnet_refs.bib
link-citations: true
output:
  pdf_document:
    fig_caption: yes
    toc: yes
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{A History of glmnet}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, message = FALSE, warning = FALSE,
                      fig.align = "center")
```

## 1. Introduction

`glmnet` is an R package for fitting the lasso and elastic-net
regularization paths of generalized linear models and related
regression problems. It has been on CRAN since 2008. Over the
intervening years it has grown from a single coordinate-descent
Fortran kernel for Gaussian, binomial, multinomial, Poisson, and Cox
responses into a layered system in which *any* GLM family can be
fitted, in which large-scale genomic problems are tractable, and in
which every numerical kernel is now written in modern C++. The
package reached its current shape in identifiable steps, each
due to contributions from several people.

This vignette is a history of those steps and an acknowledgement of
the contributions. While some historical information is in `NEWS.md`,
it misses the details. We hope that this document goes beyond a
listing of enhancements and bug fixes to explain why the package has
its current structure, and attributes contributions accurately with
references to published work where appropriate.

## 2. Origins: coordinate descent and the first release (2008--2010)

The statistical backbone of `glmnet` is older than the package. The
lasso was introduced in @lasso and the elastic-net penalty, which
combines $\ell_1$ and $\ell_2$ regularization and smooths the variable
selection behavior of the lasso in the presence of correlated
features, was introduced in @elasticnet. The *algorithmic* insight,
worked out in @pathwise, that cyclical coordinate descent is
extraordinarily effective when combined with warm starts along a
decreasing sequence of regularization parameters $\lambda_1 >
\lambda_2 > \cdots > \lambda_K$ was implemented in its CRAN debut
(version 1.1-1). The path is computed once; each $\lambda_k$ solution
starts from the previous, and both the set of active variables and the
number of inner passes stay small.

Coordinate descent has a long history, and our team was not the first
to use it in conjunction with the lasso. Tibshirani's Ph.D student
Wenjiang Fu at U. Toronto invented a version of coordinate descent
called the *shooting algorithm* in 1997.  Ingrid Daubechies used
coordinate descent for $\ell_1$ image deblurring in a talk at Stanford
in 2002. In her Ph.D. thesis with Jacquie Meulman at U. Leiden in
2006, Anita van der Kooij used coordinate descent in fitting elastic
net problems. Others have used it is well. The *pathwise* use of
coordinate descent in `glmnet` leads to great efficiency, and allows
for a fairly seamless transition to different loss functions.

The canonical reference for the package itself is
@glmnet, published in the *Journal of Statistical Software* the year
after the initial release. That paper covers Gaussian, two-class
logistic, multinomial, and Poisson regression. The Cox
proportional-hazards model for right-censored survival data was
added in version 1.2 (2010) by Noah Simon, then a PhD student at
Stanford, and documented in @coxnet. At that point the available
families were exactly those exposed through the `family` argument
today: `"gaussian"`, `"binomial"`, `"multinomial"`, `"poisson"`,
`"mgaussian"`, and `"cox"`. Each one had its own Fortran subroutine,
and the dispatch in `glmnet.R` was a chain of `if/else` branches
keyed on the family string:

![Family dispatch in `glmnet` before v4.0. User-facing R in blue,
internal R in orange, Fortran subroutines in
grey.](assets/figures/dispatch_v3.png){width=88%}

A word on the implementation language is warranted, because it
explains most of what came later. The Fortran sources were not written
by hand in Fortran but generated from *Mortran*, a macro language
layered over Fortran 77. In the spirit of open source, the full
Mortran sources live in `inst/mortran/` and were maintained by Jerome
Friedman for over a decade. The combination was extremely fast but it
was difficult to extend and hard for newcomers to read. Every new
feature that touched the inner loop had to be expressed in Mortran or
directly in Fortran, regenerated to Fortran, and then debugged under
the limitations of that toolchain. Furthermore, Fortran was dropping
in popularity and C/C++ was being used more and more for writing fast
code to interface with R. So too with `glmnet`.

## 3. Filling out the matrix (2010--2018)

The years between the first JSS paper and the 2019 v3.0 release were
spent in steady, broadening of what the package could accept as input
and what it could do with it. We summarize the milestones
thematically; the version numbers come from `NEWS.md` and CRAN
archive.
```{r timeline, fig.width=7, fig.height=3.8, out.width="\\textwidth", fig.cap="glmnet release milestones. Top row runs left-to-right; the path folds down at the right and the bottom row runs right-to-left, ending at v5.0. Evenly spaced for legibility rather than to reflect calendar time."}
par(mar = c(0.4, 0.4, 0.4, 0.4))
milestones <- data.frame(
  ver = c("Initial release (2008)", "1.2 (2010)", "1.6 (2011)",
          "2.0-1 (2015)", "3.0 (2019)",
          "4.0 (2020)", "4.1 (2021)", "4.1-3 (2021)", "5.0 (2026)"),
  lbl = c("coordinate descent;\nFortran kernel\n(Friedman, Hastie,\nTibshirani)",
          "coxnet\n(Simon)",
          "strong rules screen\n(Tibshirani et al.)",
          "CV rewrite\n(Hastie)",
          "relax; assess;\nbigGlm\n(Hastie,\nNarasimhan)",
          "GLM extension\n(Tay, Hastie,\nNarasimhan)",
          "(start, stop] Cox;\nstrata; sparse Cox\n(Tay, Narasimhan,\nHastie)",
          "Fortran to C++\n(Yang)",
          "coxdev submodules;\nCox unified\n(Taylor,\nNarasimhan, Hastie)")
)
n <- nrow(milestones); ntop <- 5; nbot <- n - ntop
top_y <- 1.4; bot_y <- -1.4
col_x <- seq_len(ntop)
px <- numeric(n); py <- numeric(n)
px[1:ntop]     <- col_x;                   py[1:ntop]     <- top_y
px[(ntop+1):n] <- rev(col_x[(ntop-nbot+1):ntop]); py[(ntop+1):n] <- bot_y

plot.new(); plot.window(xlim = c(0.4, ntop + 0.45), ylim = c(-2.3, 2.3))

seg <- function(x0, y0, x1, y1)
  segments(x0, y0, x1, y1, lwd = 1.4, col = "grey35")
arr <- function(x0, y0, x1, y1)
  arrows(x0, y0, x1, y1, length = 0.09, angle = 25, lwd = 1.4,
         col = "grey35")
pad <- 0.18
# top row: left -> right
for (i in 1:(ntop-1)) arr(px[i] + pad, py[i], px[i+1] - pad, py[i+1])
# fold down at right: tiny side step to clear labels on the shared column
fold_x <- ntop + 0.3
seg(px[ntop] + pad, py[ntop], fold_x, py[ntop])        # right from last top node
seg(fold_x, py[ntop], fold_x, py[ntop+1])              # down along margin
arr(fold_x, py[ntop+1], px[ntop+1] + pad, py[ntop+1])  # left into first bottom node
# bottom row: right -> left
for (i in (ntop+1):(n-1)) arr(px[i] - pad, py[i], px[i+1] + pad, py[i+1])

points(px, py, pch = 21, bg = "grey90", col = "grey20",
       cex = 2.0, lwd = 1.3)
text(px, py, labels = seq_len(n), cex = 0.72, col = "grey15",
     font = 2)

# version labels: above top row, below bottom row
text(px[1:ntop], py[1:ntop] + 0.28, milestones$ver[1:ntop],
     cex = 0.72, col = "grey25", pos = 3, offset = 0)
text(px[(ntop+1):n], py[(ntop+1):n] - 0.28, milestones$ver[(ntop+1):n],
     cex = 0.72, col = "grey25", pos = 1, offset = 0)

# description labels: below top row, above bottom row (interior)
text(px[1:ntop], py[1:ntop] - 0.28, milestones$lbl[1:ntop],
     cex = 0.66, pos = 1, offset = 0)
text(px[(ntop+1):n], py[(ntop+1):n] + 0.28, milestones$lbl[(ntop+1):n],
     cex = 0.66, pos = 3, offset = 0)
```

**More responses.** The `mgaussian` family for multivariate Gaussian
regression with a grouped-lasso penalty, and a `grouped` option for
the multinomial family, arrived in v1.8 (2012). Sparse `x` via
`Matrix::dgCMatrix` landed even earlier, in v1.6 (2011). This was a
major step making it possible to use `glmnet` with large, sparse
design matrices.


**Strong rules.** Version 1.6 (April 2011) also introduced *strong
rules* screening [@strongrules]: at each $\lambda$ along the path a
cheap inner-product check predicts which variables are likely to
stay inactive, and the coordinate-descent sweep skips them, with a
KKT-condition re-check afterwards to catch any violations. For
sparse problems this screens out the vast majority of predictors and
was a step-change in speed for large $p$. Like several other `glmnet`
milestones, the code landed in CRAN a year before the paper appeared.

**Coefficient constraints.** Upper and lower limits on individual
coefficients, together with `glmnet.control()` for tuning numerical
parameters, appeared in v1.9-1 (2013). The ability to fix a
coefficient in an interval --- or pin it to zero from outside via
`exclude` --- is now routinely relied upon by downstream packages.

**Cross-validation maturity.** The v2.0-1 (2015) release rewrote
`cv.glmnet`: each fold fits on its own $\lambda$ sequence and is then
evaluated at the original path. The alignment subtlety that motivated
this --- paths from different folds do not in general hit the same
$\lambda$ values --- was subsequently addressed again in v2.0-18
(2019) via the `alignment` argument.

**Cox refinements.** A bug affecting ties between the death set and
the risk set was fixed in v2.0-19, and `coxgrad` and an improved
`coxnet.deviance` were added in v2.0-20. These changes prepared the
ground for the larger Cox work that followed in v4.1.

**Fortran maintenance.** Substantial effort was spent in making the
Mortran generated Fortran build cleanly on every CRAN platform:
portable on every Fortran compiler (v2.0-15),
double-precision-consistent and `-Wall`-clean (v2.0-16, 2018), native
Fortran registration (v2.0-8, 2017). These changes were behind the
scenes, less visible to users, but essential for passing CRAN checks
for another decade.

## 4. Usability matures: v3.0 (2019)

Version 3.0, released in November 2019, was the first release in
which the feature set grew faster than the implementation underneath
it. The headline additions, contributed by Trevor Hastie and
Balasubramanian Narasimhan, were: the `relax` argument [@best_subset] to both
`glmnet` and `cv.glmnet`, which refits each active set in the path
without regularization and blends the two fits through a `gamma`
parameter (a dedicated `relax` vignette documents the usage);
`assess.glmnet`, `roc.glmnet`, and `confusion.glmnet` for evaluating
model performance; `makeX` for building input matrices from raw data
frames with one-hot encoding of factors and NA handling; and
`bigGlm` for unpenalized GLM fits using the same algorithmic
machinery. A `trace.it` progress bar made long paths less opaque,
and `print` methods for `cv.glmnet` and the new `relaxed` class were
tidied up.

Importantly, the family dispatch at this point was still entirely
character-based. A call `glmnet(x, y, family = "binomial")` passed
through an `if` branch into `lognet.R`, which called a Fortran
subroutine dedicated to logistic regression. Every family had its
own subroutine. That was about to change.

## 5. GLM extension: v4.0 (2020)

Before Version 4.0, each family reference such as  `"binomial"` was
routed to a dedicated routine.  Version 4.0 (May 2020) generalized the objective 
 
$$
\sum_{i=1}^n w_i\,\text{NLL}_i(\beta_0,\beta) + \lambda\,P_\alpha(\beta),
$$

where $\text{NLL}_i$ is the negative log-likelihood contribution of
observation $i$ under *any* GLM family, and the elastic-net penalty is
$P_\alpha(\beta) = \alpha\|\beta\|_1 +
\tfrac{1-\alpha}{2}\|\beta\|_2^2$. Similar to classical GLMs, 
we use an iteratively reweighted lasso or elastic net algorithm at
each value of `lambda` (along with cyclical coordinate descent) to fit
this model, which is implemented  pathwise down the sequence of
`lambda` values. That is, at each value of `lambda`, we implement the
*proximal Newton* via penalized iterative weighted lease squares

$$
\frac{1}{2n}\sum_{i=1}^n w_i \bigl(y_i - \beta_0 - x_i^\top\beta\bigr)^2
 + \lambda\,P_\alpha(\beta).
$$

  This coordinate-descent kernel that serves this inner loop is
the weighted least squares routine `wls`, which becomes the engine
for every family the package does not handle with a dedicated
subroutine.

The programming was done by Kenneth Tay, then a PhD student at
Stanford, with  supervision from
Hastie and Narasimhan. 
The user-visible
payoff is that the `family` argument can now be an R `family()`
object --- any object of class `"family"` in the sense of `stats::glm`,
including `MASS::negative.binomial`, `statmod::tweedie`, or a
user-defined family. The published reference is @glmnetFlex. Note
that the paper appeared three years after the release; the software
led the publication.

![Family dispatch in `glmnet` v4.0. The new right-hand branch
(`glmnet.path`, `glmnet.fit`, `wls`) handles any `family()` object;
the original per-family subroutines remain for the built-in
string-named families, which stay on the fast
path.](assets/figures/dispatch_v4.png){width=92%}

Two design decisions deserve explicit mention. First, the pre-v4
character-named families were retained and continue to go through
their dedicated subroutines, because a hand-written kernel will
always beat a generic IRLS loop. So the flexibility was added alongside the
old code, not on top of it. Second, the v4.0 warm-start convention
was tightened: the earliest form required passing a full `glmnetfit`
object, which was awkward; in v4.0.1 this was relaxed so a warm start
can be a simple list containing the intercept and coefficient vector.
Consistency with the pre-v4 numerical results --- verified through
KKT-condition checks against the Fortran output --- was the
correctness criterion throughout.

## 6. Cox catches up: v4.1 (2021)

Version 4.1 (January 2021) brought the Cox model to something close
to feature parity with `survival::coxph`, which has served as the
reference implementation throughout `glmnet`'s Cox history. Four
things were added in a single release:

1. *Left truncation and time-varying covariates* via $(start, stop]$
   counting-process input, the same format `survival` has long
   accepted.
2. *Strata*, specified via `stratifySurv()`, with the interpretation
   of `survival::coxph`: a separate baseline hazard per stratum and
   a single shared coefficient vector.
3. *Sparse $x$* for Cox models, which had been a notable gap --- the
   pre-v4.1 Cox path required a dense matrix even when the rest of
   the package was happy with `dgCMatrix`.
4. A `survfit` method for `coxnet` objects so that fitted survival
   curves could be produced without leaving `glmnet`.

Much of this work was also implemented by Tay, building on the v4.0 IRLS machinery
but targeting the Cox deviance specifically. In v4.1-2, following a
suggestion by Rob Tibshirani, the `exclude` argument was extended from
a fixed set of indices to *a function* that is handed `x`, `y`, and
`weights` and returns the indices to drop. This is the user-level hook
by which strong-rules screening can be plugged into the path; see
@strongrules and, for its application at GWAS scale, @snpnet.

## 7. Fortran → C++ port (2021--2022)

During the summer of 2021, first year statistics Ph.D student James
Yang was looking for a fun activity after completing the grueling
qualiying exams. He decided to rewrite the fortran backbone in
`glmnet` in C++!  This caught the attention of Hastie (who became his
advisor) and Narasimhan, because Yang is a very experienced
programmer.  Between v4.1-3 (November 2021) and v4.1-6 (November
2022), Yang replaced essentially all of the Mortran/Fortran in the
package with a header-only C++ implementation built on `Eigen`. This
implementation, `glmnetpp`, is a single template kernel that absorbs
dense and sparse matrices and all the built-in families through
parametrization rather than code duplication.

The staging was deliberate. In v4.1-3 the weighted least squares
kernels (`wls` and its sparse counterpart) and the `elnet` family of
routines were ported first --- these were the engines that the new
`glmnet.fit` path exercised the most, and porting them delivered
roughly 4--8$\times$ speedups on the generic GLM path. In v4.1-4 the
rest of the Fortran was rewritten, leaving only the Cox subroutine.
In v4.1-6 the legacy Fortran that could be removed was removed.

![Family dispatch after the C++ port (v4.1-3 onwards). Coloured
boxes indicate kernels rewritten in C++; only `coxnet()` remained in
Fortran.](assets/figures/dispatch_v4plus.png){width=92%}

Several thousand lines of template C++ replaced several thousand lines
of Fortran, with comparable or better performance because arguments
were no longer copied---R version 3.1+ changed this behavior and `dup
= FALSE` had no effect. This provided a much lower barrier to future
extension, and removed the dependence on a generated-Fortran
toolchain. The Cox subroutine was left alone because its numerical
structure --- stratified risk sets, ties, left truncation --- is
harder to express cleanly as a template, and because James Yang's
priority was to get the rest of the package onto the new
footing. Resolving Cox would take another three years.

## 8. Applications feeding back into the package

`glmnet` is also a piece of infrastructure on which other statistical
software is built, and some of those applications have exerted
pressure on the package's interface. We mention three briefly.

*SNPnet* [@snpnet] applies the lasso at the scale of modern
genome-wide association studies --- hundreds of thousands of samples
by hundreds of thousands of variants --- by aggressively screening
predictors using the strong rules of @strongrules. The work
motivated the v4.1-2 generalization of `exclude` to accept a
function, so that strong-rules screening can be plugged into the
standard `glmnet` call rather than requiring a fork.

*The data-shared lasso* [@datasharedlasso] uses `glmnet` as its
optimization backend to share information across related datasets
while retaining per-dataset coefficients, an idiom close to multi-task
learning.

*Cooperative learning* [@coopLearning] is a framework for
multi-view regression in which multiple data modalities on the same
subjects (genomics, proteomics, imaging, and so on) are jointly
regressed against an outcome under a regularization term that
penalizes disagreement between the views. The companion CRAN
package `multiview` dispatches its fits to `glmnet`.

## 9. Streamlined Cox and v5.0 (2026)

On the eve of v5.0 the package was in an unusual state: every kernel
except the Cox subroutine had been rewritten in C++. The Cox kernel
had survived because of the combinatorial complexity its users
expect of it --- tie handling, stratified risk
sets, left truncation --- and because producing a numerically stable
deviance, gradient, and Hessian in all of these cases in a single
clean implementation is harder than it looks.

The v5.0 release resolves this. Jonathan Taylor joined the team, and contributed
`coxdev`, a standalone C++ library that computes the Cox partial
log-likelihood deviance together with its gradient and Hessian,
handling Breslow and Efron tie methods, strata, and $(start, stop]$
data through a small number of well-tested classes. `coxdev` is
separately developed (a nested git submodule of `glmnetpp`, which is
itself a submodule of `glmnet`), and the computational details are
the subject of a companion vignette. At the same time,
Narasimhan replaced the Fortran code for the Cox regularization path with
C++, making use of the `coxdev` library for the gradient and Hessian
computations, which automatically takes care of all the special cases.
From the point of
view of `glmnet`, the essential change is that Cox is now a
first-class GLM type inside `glmnetpp` --- `glm_type::cox` in the
type enumeration --- and the same IRLS loop that drives the
Gaussian, binomial, and Poisson paths drives the Cox path, with a
thin `CoxDevAdapter` handing the deviance and gradient queries off to
`coxdev`. The last of the Fortran leaves the package in this release.

![Full architecture of `glmnet` v5.0 with the `glmnetpp` and `coxdev`
submodules, the Rcpp boundary, and the `CoxDevAdapter` bridge from
the `glmnetpp` IRLS loop to the `coxdev` deviance and gradient
routines.](assets/figures/dispatch_v5.png){width=100%}

Two user-visible consequences of this unification are worth stating
plainly. First, the Cox model now takes a `cox.ties` argument
with choices `"breslow"` or `"efron"`. The v5.0 default is
`cox.ties = "breslow"`, preserving the pre-v5.0 numerical behavior for
upgraders; v5.1 will switch the default to `"efron"` to match
`survival::coxph`. Calls to `glmnet()` with `family = "cox"` that do
not set `cox.ties` explicitly emit a transition warning until the
switch occurs.  The performance picture is favourable. On the benchmark set
in `tests/benchmark_cox.R`, dense Cox paths are roughly 3 times
faster than the legacy Fortran, sparse Cox paths roughly 90 times
faster, and stratified Cox paths --- which had been the most
penalized case in the old implementation --- roughly 300 times
faster. Full benchmark numbers appear in the NEWS entry for v5.0.

## 10. Looking forward

Development continues along several lines. A Python `glmnet` is
being prepared by Jonathan Taylor that shares a common C++ codebase with the R package,
so that the same kernels serve both languages. The shared codebase
opens the possibility of compiling `glmnet` to WebAssembly for
in-browser use. Upstream work in `coxdev` is aimed at reducing
per-call allocation in the IRLS loop, which is the last remaining
place where the Cox path pays a measurable overhead for its
extensibility.

## 11. Contributors and acknowledgements

`glmnet` is the work of many hands. Jerome Friedman wrote the
original coordinate-descent kernel and the Mortran infrastructure,
and is a co-author of @glmnet, @coxnet, and @pathwise. Trevor
Hastie has been co-author throughout, as well as the package
maintainer. He also wrote the bulk of the R interface code, as well as
methods for plotting, printing, predicting and cross-validation, and
the corresponding code for the relaxed models (with help from Narasimhan).

Robert Tibshirani introduced the lasso [@lasso] and co-authored the
 strong-rules work on which the package depends. Noah
Simon led the original Cox path work and is the first author of
@coxnet. Kenneth Tay is responsible for the v4.0 GLM extension and
much of the v4.1 Cox expansion, and is first author of @glmnetFlex.
Balasubramanian Narasimhan has maintained the package through the
v4.x and v5.0 architectural transitions and is a co-author of
@glmnetFlex. James Yang carried out the v4.1-3 through v4.1-6
port from Fortran to C++ and is the author of `glmnetpp`. Junyang
Qian is first author of @snpnet, whose screening machinery
influenced the v4.1-2 `exclude`-as-function interface. Jonathan
Taylor contributed `coxdev` and with Narasimhan drove the v5.0 unification of the
Cox path onto the C++ engine. Many bug reports and fixes
contributed by users over the years are recorded in `NEWS.md`; the
work of Tomas Kalibera, David Keplinger, and others is gratefully
acknowledged.

## References

---
title: "The `family` Argument for `glmnet`"
author: 
  - Trevor Hastie
  - Kenneth Tay
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: assets/glmnet_refs.bib
link-citations: true
output:
  pdf_document:
    fig_caption: yes
    toc: yes
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{The family Argument for glmnet}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

The `glmnet` package fits a generalized linear model (GLM) via
penalized maximum likelihood. Concretely, it solves the problem

$$ \min_{\beta_0, \beta} \frac{1}{N}\sum_{i=1}^N w_i l_i(y_i, \beta_0 + \beta^T x_i) + \lambda \left[\frac{1 - \alpha}{2}\|\beta\|_2^2 + \alpha \|\beta\|_1 \right] $$
over a grid of values of $\lambda$ covering the entire range. In the equation above, $l_i(y_i, \eta_i)$ is the negative log-likelihood contribution for observation $i$. $\alpha \in [0,1]$ is a tuning parameter which bridges the gap between the lasso ($\alpha = 1$, the default) and ridge regression ($\alpha = 0$), while $\lambda$ controls the overall strength of the penalty.

`glmnet` solves the minimization problem above very efficiently for a limited number of built-in (hardwired) families. To fit these model families, one should specify the `family` argument to `glmnet` as a character string. The families that can be fit efficiently this way are the penalized Gaussian (`"gaussian"`), binomial (`"binomial"`), and Poisson (`"poisson"`) GLMs, along with a few other special cases: the Cox model (`"cox"`), multinomial regression (`"multinomial"`), and multi-response Gaussian (`"mgaussian"`). Details for how to fit these models can be found in the vignette ["An Introduction to `glmnet`"](https://glmnet.stanford.edu/articles/glmnet.html).

Apart from these built-in families, `glmnet` also allows the user to fit a penalized regression model for *any* GLM by allowing the `family` argument to be any legitimate GLM family object, as used by the `stats:glm` function.

### Using class "family" objects for the `family` argument

The `family` argument to `glmnet` can be the result of a call to a `family` function. (To learn more about family functions in R, run `?family` in the R console.) All the functionality of `glmnet` applies to these new families, and hence their addition expands the scope of `glmnet` considerably. In particular,

* All the methods, such as `plot` and `predict`, work as before;
* Large and sparse `x` matrices can be taken as input;
* The user can put upper and lower bound constraints on parameters;
* `cv.glmnet` can be used for selecting the tuning parameters;
* `relax = TRUE` can be specified for fitting unpenalized models to the active sets;
* `offsets` can be provided;
* Penalty strengths, standardization, and other options to `glmnet` work as before.

When the `family` argument is a class `"family"` object, `glmnet` fits the model for each value of `lambda` with a proximal Newton algorithm, also known as iteratively reweighted least squares (IRLS). The outer loop of the IRLS algorithm is coded in R, while the inner loop solves the weighted least squares problem with the elastic net penalty, and is implemented in Fortran. The R code exploits warm starts as it iterates down the path, and so is reasonably efficient.

### More on GLM families

A GLM is linear model for a response variable whose conditional distribution belongs to a one-dimensional exponential family. Apart from Gaussian, Poisson and binomial families, there are other interesting members of this family, e.g. Gamma, inverse Gaussian, negative binomial, to name a few. A GLM consists of 3 parts:

1. A linear predictor: $\eta_i = \sum_{j=1}^p \beta^T x_i$,
2. A link function: $\eta_i = g(\mu_i)$, and
3. A random component: $y_i \sim f(y \mid \mu_i)$.

The user gets to specify the link function $g$ and the family of response distributions $f(\cdot \mid \mu)$, and fitting a GLM  amounts to estimating the parameter $\beta$ by maximum likelihood.

In R, these 3 parts of the GLM are encapsulated in an object of class `family` (run `?family` in the R console for more details). A `family` object is a list of GLM components which allows functions such as `stats:glm` to fit GLMs in R. As an example, the code below shows the constituent parts for the binomial GLM, which is what is used to fit linear logistic regression:
```{r}
fam <- binomial()
class(fam)
names(fam)
```
This is a list of functions and expressions that get used in the *iteratively reweighted least squares* (IRLS) algorithm for fitting the GLM.

`glmnet` can fit penalized GLMs for any family as long as the family can be expressed as a `family` object. In fact, users can make their own families, or customize existing families, just as they can for regular GLMs.

Generally this option should be used only if the desired family is not included in the built-in list. The reason is that the entire path algorithm for the built-in families is implemented in Fortran, and so will be faster.

## Fitting Gaussian, binomial and Poisson GLMs

First, we demonstrate how we can use this new version of `glmnet` to fit ordinary least squares with the elastic net penalty. We set up some fake data:
```{r}
set.seed(1)
x <- matrix(rnorm(500), ncol = 5)
y <- rowSums(x[, 1:2]) + rnorm(100)
```

The function calls below demonstrate how we would fit the model with the old and new `family` parameter options. To fit a linear regression by least squares, we want to use the Gaussian family. There is a *hard-wired* option for this, specified via `family="gaussian"` (which is also the default for `glmnet`). Now we can also use `family = gaussian()` to fit the same model.
```{r message = FALSE}
library(glmnet)
oldfit <- glmnet(x, y, family = "gaussian")
newfit <- glmnet(x, y, family = gaussian())
```

`glmnet` distinguishes these two cases because the first is a character string, while the second is a GLM family object.  Of course if we really wanted to fit this model, we would use the hard-wired version, because it is faster. Here we want to show that they are equivalent, up to machine precision.

There are slight differences in the algorithms used to compute the solutions, so some of the equality tests run using `testthat::expect_equal` might fail. However, these same tests can be made to pass by decreasing the `thresh` option in both function calls:

```{r}
thresh <- 1e-18
oldfit <- glmnet(x, y, family="gaussian", thresh = thresh)
newfit <- glmnet(x, y, family = gaussian(), thresh = thresh)

# tests for equality
library(testthat)
for (key in c("a0", "beta", "df", "dim", "lambda", "dev.ratio", 
              "nulldev", "offset", "nobs")) {
  expect_equal(oldfit[[key]], newfit[[key]])
}
```

Next, we demonstrate the function calls for the binomial and Poisson GLM families. 

```{r}
biny <- ifelse(y > 0, 1, 0)  # binary data
cnty <- ceiling(exp(y))      # count data

# fitting binomial GLMs the old and new way
oldfit <- glmnet(x, biny, family = "binomial")
newfit <- glmnet(x, biny, family = binomial())

# fitting Poisson GLMs the old and new way
oldfit <- glmnet(x, cnty, family = "poisson")
newfit <- glmnet(x, cnty, family = poisson())
```

### Timing comparisons

In the examples above, the new version is simply replicating existing
functionality in `glmnet`. For these GLMs, we recommend specifying the
GLM family as a character string for computational efficiency. The figures below illustrate that existing code for these GLM families is more efficient than the new code, especially for the Gaussian case.

(In the figures, `n` and `p` denote the number of observations and features respectively. Each point is the mean of 5 simulation runs. Note that both the `x` and `y` axes are on a log scale.)

![Timing comparisons for Gaussian family](assets/vignette_gaussian.png){width=90%}

![Timing comparisons for binomial family](assets/vignette_binomial.png){width=90%}

## Fitting other GLMs

The real power of using class `"family"` objects for the `family` argument is in fitting GLMs other than the three in the previous section. For example, performing probit regression with the elastic net penalty is as simple as the code below:
```{r}
newfit <- glmnet(x, biny, family = binomial(link = "probit"))
```

For the *complementary log-log* link we would specify `family = binomial(link = "cloglog")`.

We can fit nonlinear least-squares models by using a different link with the Gaussian family; for example  `family = gaussian(link = "log")`.

For count data, we can fit a quasi-Poisson model that allows for overdispersion:
```{r}
newfit <- glmnet(x, cnty, family = quasipoisson())
```

The negative binomial is often used to model over-dispersed count data
(instead of Poisson regression), and is also easy:
```{r, eval=FALSE}
library(MASS)
newfit <- glmnet(x, cnty, family = negative.binomial(theta = 5))
```

There are many other families, including `quasi` where users can customize their own families. There are additional specialized families such as `statmod:tweedie` for overdispersed count data.

## Class `"glmnetfit"` objects

If `glmnet` is called with a class `"family"` object as its argument, it returns an object with class
```{r}
class(newfit)
```

This is similar to the hard-wired classes; for example a `family = "gaussian"` fit has class 
```{r}
fit <- glmnet(x, y, family = "gaussian")
class(fit)
```

Importantly, both these inherit from class `"glmnet"`, and so all the S3 methods such as `plot`, `predict`, `coef`, and `print` will work out the box.

## Step size halving within iteratively reweighted least squares (IRLS)

For the built-in families, `glmnet` solves the optimization problem
for non-Gaussian families via iteratively reweighted least squares
(IRLS). In each iteration a unit Newton step is taken, and the
algorithm terminates when the unit Newton step fails to decrease the deviance sufficiently. Because the algorithm iss forced to take a unit step, this can result in non-convergence of the algorithm in some cases.

Here is an example of the non-convergence for Poisson data. The `stats:glm` function converges and gives us coefficients that are reasonably close to the truth:
```{r}
set.seed(2020)
n <- 100
p <- 4
x <- matrix(runif(n * p, 5, 10), n)
y <- rpois(n, exp(rowMeans(x)))

# glm fit
glmfit <- glm(y ~ x - 1, family = poisson)
coef(glmfit)
```

Fitting `glmnet` with `lambda = 0` is equivalent to fitting an unregularized GLM. If we use `glmnet` with `family = "poisson"`, we encounter an issue with non-convergence:
```{r}
oldfit <- glmnet(x, y, family = "poisson", standardize = FALSE, 
                 intercept = FALSE, lambda = 0)
coef(oldfit)
```

This divergence happens because the unit Newton step was too large. However, if a `family` object is passed to the `family` argument of `glmnet`, the IRLS algorithm will perform step size halving. After computing the Newton step, the algorithm checks if the new solution has infinite (or astronomically large) objective function value or if it results in invalid $\eta$ or $\mu$. If so, the algorithm halves the step size repeatedly until these invalid conditions no longer hold.

The code below shows that this step size halving avoids the divergence we were experiencing in our running example:
```{r}
glmnet.control(mxitnr = 50)  # increase maximum no. of IRLS iterations allowed
newfit <- glmnet(x, y, family = poisson(), standardize = FALSE, 
                 intercept = FALSE, lambda = 0)
coef(newfit)
```

In the process, `glmnet` warns the user that an infinite objective function value was encountered and that step size halving was done to address the issue. The coefficients are close to those obtained by `stats:glm`, and can be made to be numerically indistinguishable by tightening the convergence criteria in both function calls.
```{r}
thresh <- 1e-15
glmfit <- glm(y ~ x-1, family = poisson,
              control = list(epsilon = thresh, maxit = 100))
newfit <- glmnet(x, y, family = poisson(), standardize = FALSE, intercept = FALSE,
                 lambda = 0, thresh = thresh)

# coef(glmfit) doesn't have intercept but coef(newfit does)
expect_equal(as.numeric(coef(glmfit)), 
             as.numeric(coef(newfit))[2:5])
```

---
title: "The Relaxed Lasso"
author: 
  - Trevor Hastie
  - Balasubramanian Narasimhan
  - Rob Tibshirani
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: assets/glmnet_refs.bib
link-citations: true
output:
  pdf_document:
    fig_caption: yes
    toc: yes
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{The Relaxed Lasso}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r include=FALSE}
# the code in this chunk enables us to truncate the print output for each
# chunk using the `out.lines` option
# save the built-in output hook
hook_output <- knitr::knit_hooks$get("output")

# set a new output hook to truncate text output
knitr::knit_hooks$set(output = function(x, options) {
  if (!is.null(n <- options$out.lines)) {
    x <- xfun::split_lines(x)
    if (length(x) > n) {
      # truncate the output
      x <- c(head(x, n), "....\n")
    }
    x <- paste(x, collapse = "\n")
  }
  hook_output(x, options)
})
```

## Introduction

In this vignette, we describe how the `glmnet` package can be used to fit the *relaxed lasso*.

The idea of the relaxed lasso is to take a `glmnet` fitted object, and then for each lambda, refit the variables in the active set without any penalization. This gives the "relaxed" fit. (We note that there have been other definitions of a relaxed fit, but this is the one we prefer.) This could of course be done for elastic net fits as well as lasso. However, if the number of variables gets too close to the sample size $N$, the relaxed path will be truncated. Furthermore, for binomial and other nonlinear generalized linear models (GLMs) convergence can be an issue with our current implementation if the number of variables is too large, and perversely if the relaxed fit is too strong.

Suppose the `glmnet` fitted linear predictor at $\lambda$ is $\hat\eta_\lambda(x)$ and the relaxed version is $\tilde\eta_\lambda(x)$. We also allow for shrinkage between the two:
$$\tilde \eta_{\lambda,\gamma}=(1-\gamma)\tilde
\eta_\lambda(x)+\gamma\hat\eta_\lambda(x).$$

$\gamma\in[0,1]$ is an additional tuning parameter which can be selected by cross-validation (CV). The debiasing will potentially improve prediction performance, and CV will typically select a model with a smaller number of variables. 

This procedure is very competitive with forward-stepwise and best-subset regression, and has a considerable speed advantage when the number of variables is large. This is especially true for best-subset, but even so for forward stepwise. The latter has to plod through the variables one-at-a-time, while `glmnet` will just plunge in
and find a good active set.

Further details on this form of relaxed fitting can be found in
@best_subset;  more information on glmnet and elastic-net model in
general is given in @glmnet,
@coxnet, @strongrules, and @block.

## Simple relaxed fitting

We demonstrate the most basic relaxed lasso fit as a first example. We load some pre-generated data and fit the relaxed lasso on it by calling `glmnet` with `relax = TRUE`:
```{r out.lines = 15}
library(glmnet)
data(QuickStartExample)
x <- QuickStartExample$x
y <- QuickStartExample$y
fit <- glmnet(x, y, relax = TRUE)
print(fit)
```
In addition to the three columns usually printed for `glmnet` objects
(`Df`, `%Dev` and `Lambda`), there is an extra column `%Dev R` (`R`
stands for "relaxed") which is the percent deviance explained by the
relaxed fit. This is always higher than its neighboring column, which
is the percent deviance exaplined for the penalized fit (on the
training data). Notice that when the `Df` stays the same, the `%Dev R`
does not change, since this typically means the active set is the
same. (The code is also smart enough to only fit such models once, so
in the truncated display shown, 9 lasso models are fit, but only 4
relaxed fits are computed).


The fit object is of class `"relaxed"`, which inherits from class `"glmnet"`. Hence, the usual `plot` method for `"glmnet"` objects can be used. The code below demonstrates some additional flexibility that `"relaxed"` objects have for plotting.
```{r}
par(mfrow = c(1, 3), mar=c(4,4,5.5,1))
plot(fit, main = "gamma = 1")
plot(fit, gamma = 0.5, main = "gamma = 0.5")
plot(fit, gamma = 0, main = "gamma = 0")
```

`gamma = 1` is the traditional `glmnet` fit (also `relax = FALSE`, the default), `gamma = 0` is the unpenalized fit, and `gamma = 0.5` is a mixture of the two (at the coefficient level, and hence also the linear predictors).

We can also select `gamma` using `cv.glmnet`, which by default uses the 5 values `c(0, 0.25, 0.5, 0.75, 1)`. This returns an object of class `"cv.relaxed"`.
```{r}
set.seed(1)
cfit <- cv.glmnet(x, y, relax = TRUE)
plot(cfit)
```

To remove the shading of the standard error bands, pass `se.bands = FALSE`:
```{r}
plot(cfit, se.bands = FALSE)
```

As with regular `"cv.glmnet"` objects, you can make predictions from a relaxed CV object. Just as the `s` option (for `lambda`) admits two special strings `"lambda.1se"` and `"lambda.min"` for special values of `lambda`, the `gamma` option admits two special strings `"gamma.1se"` and `"gamma.min"` for special values of `gamma`. For example, the code below makes predictions for `newx` at the `lambda` and `gamma` values that has the smallest CV error:
```{r}
predict(cfit, newx = x[1:5, ], s = "lambda.min", gamma = "gamma.min")
```

Printing class `"cv.relaxed"` objects gives some basic information on the cross-validation:
```{r}
print(cfit)
```

## More details on relaxed fitting

While we only demonstrate relaxed fits for the default Gaussian family, *any* of the families fit by `glmnet` can also be fit with the `relaxed` option.

Although `glmnet` has a `relax` option, you can also fit relaxed lasso models by post-processing a `glmnet` object with the `relax.glmnet` function.
```{r `relaxed`}
fit <- glmnet(x,y)
fitr <- relax.glmnet(fit, x = x, y = y)
```
This will rarely need to be done; one use case is if the original fit took a long time, and the user wants to avoid refitting it. Note that the arguments are named in the call in order for them to be passed correctly via the `...` argument in `relax.glmnet`.

As mentioned, a `"relaxed"` object inherits from class `"glmnet"`. Apart from the class modification, it has an additional component named `relaxed` which is itself a `glmnet` object, but with the relaxed coefficients. The default behavior of extractor functions like `predict` and `coef`, as well as `plot` will be to present results from the `glmnet` fit, unless a value of `gamma` is given different from the default value `gamma = 1` (see the plots above). The `print` method gives additional info on the relaxed fit.

Likewise, a `cv.relaxed` object inherits from class `cv.glmnet`. Here the `predict` method by default uses the optimal relaxed fit; if predictions from the CV-optimal *original* `glmnet` fit are desired, one can directly use `predict.cv.glmnet`. Similarly, use `print` to print information for cross-validation on the relaxed fit, and `print.cv.glmnet` for information on the cross-validation for the original `glmnet` fit.
```{r}
print(cfit)
print.cv.glmnet(cfit)
```

### Possible convergence issues for relaxed fits

`glmnet` itself is used to fit the relaxed fits by using a single value of zero for `lambda`. However, for nonlinear models such as `family = "binomial"`, `family = "multinomial"` and `family="poisson"`, there can be convergence issues. This is because `glmnet` does not do step size optimization, rather relying on
the pathwise fit to stay in the "quadratic" zone of the log-likelihood. We have an optional `path = TRUE` option for `relax.glmnet`, which actually fits a regurized path toward the `lambda = 0` solution, and thus avoids
the issue. The default is `path = FALSE` since this option adds to the computing time.

##  Application to forward stepwise regression

One use case for a relaxed fit is as a faster version of forward stepwise regression. With a large number `p` of variables, forward stepwise regression can be tedious. On the other hand, because the lasso solves a convex problem, it can plunge in and identify good candidate sets of variables over 100 values of `lambda`, even though `p` could be in the tens of thousands. In a case like this, one can have `cv.glmnet` do the selection of variables.
```{r}
fitr <- cv.glmnet(x, y, gamma = 0, relax = TRUE)
plot(fitr)
```

Notice that we only allow `gamma = 0`, so in this case we are not considering the blended fits.

## References

---
title: "An Introduction to `glmnet`"
author: 
  - Trevor Hastie
  - Junyang Qian
  - Kenneth Tay
date: "`r format(Sys.time(), '%B %d, %Y')`"
bibliography: assets/glmnet_refs.bib
link-citations: true
output:
  pdf_document:
    fig_caption: yes
    toc: yes
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{An Introduction to glmnet}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r include=FALSE}
# the code in this chunk enables us to truncate the print output for each
# chunk using the `out.lines` option
# save the built-in output hook
hook_output <- knitr::knit_hooks$get("output")

# set a new output hook to truncate text output
knitr::knit_hooks$set(output = function(x, options) {
  if (!is.null(n <- options$out.lines)) {
    x <- xfun::split_lines(x)
    if (length(x) > n) {
        
      # truncate the output
      x <- c(head(x, n), "....\n")
    }
    x <- paste(x, collapse = "\n")
  }
  hook_output(x, options)
})
```


## Introduction

Glmnet is a package that fits generalized linear and similar models
via penalized maximum likelihood. The regularization path is computed
for the lasso or elastic net penalty at a grid of values (on the log
scale) for the
regularization parameter lambda. The algorithm is extremely fast, and
can exploit sparsity in the input matrix `x`. It fits linear, logistic
and multinomial, poisson, and Cox regression models. It can also fit
multi-response linear regression, generalized linear models for custom
families, and relaxed lasso regression models. The package includes
methods for prediction and plotting, and  functions for cross-validation.

The authors of glmnet are Jerome Friedman, Trevor Hastie, Rob Tibshirani, Balasubramanian Narasimhan, Kenneth Tay and Noah Simon, with contribution from Junyang Qian, and the R package is maintained by Trevor Hastie. A MATLAB version of glmnet is maintained by Junyang Qian, and a Python version by B. Balakumar (although both are a few versions behind).

This vignette describes basic usage of glmnet in R. There are additional vignettes that should be useful:

* ["Regularized Cox Regression"](https://glmnet.stanford.edu/articles/Coxnet.html) describes how to fit regularized Cox models for survival data with `glmnet`.
* ["GLM `family` functions in `glmnet`"](https://glmnet.stanford.edu/articles/glmnetFamily.html) describes how to fit custom generalized linear models (GLMs) with the elastic net penalty via the `family` argument.
* ["The Relaxed Lasso"](https://glmnet.stanford.edu/articles/relax.html)
  describes how to fit relaxed lasso regression models using the `relax` argument.

`glmnet` solves the problem

$$
\min_{\beta_0,\beta} \frac{1}{N} \sum_{i=1}^{N} w_i l(y_i,\beta_0+\beta^T x_i) + \lambda\left[(1-\alpha)\|\beta\|_2^2/2 + \alpha \|\beta\|_1\right],
$$

over a grid of values of $\lambda$ covering the entire range of possible solutions. Here $l(y_i,\eta_i)$ is the negative log-likelihood contribution for observation $i$; e.g. for the Gaussian case it is $\frac{1}{2}(y_i-\eta_i)^2$. The _elastic net_ penalty is controlled by $\alpha$, and bridges the gap between lasso regression ($\alpha=1$, the default) and ridge regression ($\alpha=0$). The tuning parameter $\lambda$ controls the overall strength of the penalty.

It is known that the ridge penalty shrinks the coefficients of correlated predictors towards each other while the lasso tends to pick one of them and discard the others. The elastic net penalty mixes these two: if predictors are correlated in groups, an $\alpha=0.5$ tends to either select or leave out the entire group of features. This is a  higher level parameter, and users might pick a value upfront or experiment with a few different values. One use of $\alpha$ is for numerical stability; for example, the elastic net with $\alpha = 1 - \epsilon$ for some small $\epsilon > 0$ performs much like the lasso, but removes any degeneracies and wild behavior caused by extreme correlations.

The `glmnet` algorithms use cyclical coordinate descent, which successively optimizes the objective function over each parameter with others fixed, and cycles repeatedly until convergence. The package also makes use of the strong rules for efficient restriction of the active set. Due to highly efficient updates and techniques such as warm starts and active-set convergence, our algorithms can compute the solution path very quickly.

The code can handle sparse input-matrix formats, as well as range constraints on coefficients. The core of `glmnet` is a set of Fortran subroutines, which make for very fast execution.

The theory and algorithms in this implementation are described in @glmnet, @coxnet, @strongrules and @block. 

## Installation

Like many other R packages, the simplest way to obtain `glmnet` is to install it directly from CRAN. Type the following command in R console:

```{r, eval=FALSE}
install.packages("glmnet", repos = "https://cran.us.r-project.org")
```

Users may change the `repos` argument depending on their locations and preferences. Other arguments such as the directories to install the packages at can be altered in the command. For more details, see `help(install.packages)`. Alternatively, users can download the package source  from [CRAN](https://cran.r-project.org/package=glmnet) and type Unix commands to install it to the desired location.

## Quick Start

The purpose of this section is to give users a general sense of the package. We will briefly go over the main functions, basic operations and outputs. After this section, users may have a better idea of what functions are available, which ones to use, or at least where to seek help.

First, we load the `glmnet` package:
```{r}
library(glmnet)
```

The default model used in the package is the Guassian linear model or "least squares", which we will demonstrate in this section. We load a set of data created beforehand for illustration:
```{r}
data(QuickStartExample)
x <- QuickStartExample$x
y <- QuickStartExample$y
```
The command loads an input matrix `x` and a response vector `y` from this saved R data archive.

We fit the model using the most basic call to  `glmnet`.
```{r}
fit <- glmnet(x, y)
```
`fit` is an object of class `glmnet` that contains all the relevant information of the fitted model for further use. We do not encourage users to extract the components directly. Instead, various methods are provided for the object such as `plot`, `print`, `coef` and `predict` that enable us to execute those tasks more elegantly.

We can visualize the coefficients by executing the `plot` method:
```{r}
plot(fit)
```

Each curve corresponds to a variable. It shows the path of its coefficient against the $\ell_1$-norm of the whole coefficient vector as $\lambda$ varies. The axis above indicates the number of nonzero coefficients at the current $\lambda$, which is the effective degrees of freedom (_df_) for the lasso. Users may also wish to annotate the curves: this can be done by setting `label = TRUE` in the plot command.

A summary of the `glmnet` path at each step is displayed if we just enter the object name or use the `print` function:
```{r out.lines = 10}
print(fit)
```
It shows from left to right the number of nonzero coefficients (`Df`),
the percent (of null) deviance explained (`%dev`) and the value of
$\lambda$ (`Lambda`). Although `glmnet` fits the model for 100 values
of `lambda` by default, it stops early if `%dev` does not change
sufficently from one lambda to the next (typically near the end of the
path.) Here we have truncated the prinout for brevity. 

We can obtain the model coefficients at one or more $\lambda$'s within the range of the sequence:
```{r out.lines = 10}
coef(fit, s = 0.1)
```
(Why `s` and not `lambda`? In case we want to allow one to specify the model size in other ways in the future.) Users can also make predictions at specific $\lambda$'s with new input data:
```{r}
set.seed(29)
nx <- matrix(rnorm(5 * 20), 5, 20)
predict(fit, newx = nx, s = c(0.1, 0.05))
```

The function `glmnet` returns a sequence of models for the users to choose from. In many cases, users may prefer the software to select one of them. Cross-validation is perhaps the simplest and most widely used method for that task. `cv.glmnet` is the main function to do cross-validation here, along with various supporting methods such as plotting and prediction.
```{r}
cvfit <- cv.glmnet(x, y)
```

`cv.glmnet` returns a `cv.glmnet` object, a list with all the ingredients of the cross-validated fit. As with `glmnet`, we do not encourage users to extract the components directly except for viewing the selected values of $\lambda$. The package provides well-designed functions for potential tasks. For example, we can plot the object:
```{r}
plot(cvfit)
```

This plots the cross-validation curve (red dotted line) along with upper and lower standard deviation curves along the $\lambda$ sequence (error bars). Two special values along the $\lambda$ sequence are indicated by the vertical dotted lines. `lambda.min` is the value of $\lambda$ that gives minimum mean cross-validated error, while `lambda.1se` is the value of $\lambda$ that gives the most regularized model such that the cross-validated error is within one standard error of the minimum.

We can use the following code to get the value of `lambda.min` and the model coefficients at that value of $\lambda$:
```{r out.lines = 10}
cvfit$lambda.min
coef(cvfit, s = "lambda.min")
```

To get the corresponding values at `lambda.1se`, simply replace
`lambda.min` with `lambda.1se` above, or omit the `s` argument, since
`lambda.1se` is the default.

Note that the coefficients are represented in the sparse matrix format. This is because the solutions along the regularization path are often sparse, and hence it is more efficient in time and space to use a sparse format. If you prefer non-sparse format, pipe the output through `as.matrix()`.

Predictions can be made based on the fitted `cv.glmnet` object as well. The code below gives predictions for the new input matrix `newx` at `lambda.min`:
```{r}
predict(cvfit, newx = x[1:5,], s = "lambda.min")
```

This concludes `glmnet` 101. With the tools introduced so far, users are able to fit the entire elastic net family, including ridge regression, using squared-error loss. There are many more arguments in the package that give users a great deal of flexibility. To learn more, move on to later sections.

## Linear Regression: `family = "gaussian"` (default)

`"gaussian"` is the default `family` argument for the function `glmnet`. Suppose we have observations $x_i \in \mathbb{R}^p$ and the responses $y_i \in \mathbb{R}, i = 1, \ldots, N$. The objective function for the Gaussian family is
$$
\min_{(\beta_0, \beta) \in \mathbb{R}^{p+1}}\frac{1}{2N} \sum_{i=1}^N (y_i -\beta_0-x_i^T \beta)^2+\lambda \left[ (1-\alpha)\|\beta\|_2^2/2 + \alpha\|\beta\|_1\right],
$$
where $\lambda \geq 0$ is a complexity parameter and $0 \leq \alpha \leq 1$ is a compromise between ridge regression ($\alpha = 0$) and lasso regression ($\alpha = 1$).

`glmnet` applies coordinate descent to solve the problem. Specifically, suppose we have current estimates $\tilde{\beta_0}$ and $\tilde{\beta}_\ell$ $\forall \ell\in 1,\ldots,p$. By computing the gradient at $\beta_j = \tilde{\beta}_j$ and simple calculus, the update is
$$
\tilde{\beta}_j \leftarrow \frac{S(\frac{1}{N}\sum_{i=1}^N x_{ij}(y_i-\tilde{y}_i^{(j)}),\lambda \alpha)}{1+\lambda(1-\alpha)},
$$
where $\tilde{y}_i^{(j)} = \tilde{\beta}_0 + \sum_{\ell \neq j} x_{i\ell} \tilde{\beta}_\ell$, and $S(z, \gamma)$ is the soft-thresholding operator with value $\text{sign}(z)(|z|-\gamma)_+$.

This formula above applies when the `x` variables are standardized to have unit variance (the default); it is slightly more complicated when they are not. Note that for `family = "gaussian"`, `glmnet` standardizes $y$ to have unit variance before computing its `lambda` sequence (and then unstandardizes the resulting coefficients). If you wish to reproduce or compare results with other software, it is best to supply a standardized $y$ first (Using the "1/N" variance formula).

### Commonly used function arguments

`glmnet` provides various arguments for users to customize the fit: we introduce some commonly used arguments here. (For more information, type `?glmnet`.)

* `alpha` is for the elastic net mixing parameter $\alpha$, with range $\alpha \in [0,1]$. $\alpha = 1$ is lasso regression (default) and $\alpha = 0$ is ridge regression.

* `weights` is for the observation weights, default is 1 for each observation. (Note: `glmnet` rescales the weights internally to sum to N, the sample size.)

* `nlambda` is the number of $\lambda$ values in the sequence (default is 100).

* `lambda` can be provided if the user wants to specify the lambda sequence, but typical usage is for the program to construct the lambda sequence on its own. When automatically generated, the $\lambda$ sequence is determined by `lambda.max` and `lambda.min.ratio`. The latter is the ratio of smallest value of the generated $\lambda$ sequence (say `lambda.min`) to `lambda.max`.  The program generates `nlambda` values linear on the log scale from `lambda.max` down to `lambda.min`. `lambda.max` is not user-specified but is computed from the input $x$ and $y$: it is the smallest value for `lambda` such that all the coefficients are zero.  For `alpha = 0` (ridge) `lambda.max` would be $\infty$: in this case we pick a value corresponding to a small value for `alpha` close to zero.)

* `standardize` is a logical flag for `x` variable standardization prior to fitting the model sequence. The coefficients are always returned on the original scale. Default is `standardize = TRUE`.

As an example, we set $\alpha = 0.2$ (more like a ridge regression), and give double weight to the latter half of the observations. We set `nlambda` to 20 so that the model fit is only compute for 20 values of $\lambda$. In practice, we recommend `nlambda` to be 100 (default) or more. In most cases, it does not come with extra cost because of the warm-starts used in the algorithm, and for nonlinear models leads to better convergence properties.
```{r}
wts <-  c(rep(1,50), rep(2,50))
fit <- glmnet(x, y, alpha = 0.2, weights = wts, nlambda = 20)
```

We can then print the `glmnet` object:
```{r}
print(fit)
```
This displays the call that produced the object `fit` and a three-column matrix with columns `Df` (the number of nonzero coefficients), `%dev` (the percent deviance explained) and `Lambda` (the corresponding value of $\lambda$). (The `digits` argument can used to specify significant digits in the printout.)

Here the actual number of $\lambda$'s is less than that specified in the call. This is because of the algorithm's stopping criteria. According to the default internal settings, the computations stop if either the fractional change in deviance down the path is less than $10^{-5}$ or the fraction of explained deviance reaches $0.999$. From the last few lines of the output, we see the fraction of deviance does not change much and therefore the computation ends before the all 20 models are fit. The internal parameters governing the stopping criteria can be changed. For details, see the Appendix section or type `help(glmnet.control)`.

### Predicting and plotting with `glmnet` objects

We can extract the coefficients and make predictions for a `glmnet` object at certain values of $\lambda$. Two commonly used arguments are:

* `s` for specifiying the value(s) of $\lambda$ at which to extract coefficients/predictions.

* `exact` for indicating whether the exact values of coefficients are desired or not. If `exact = TRUE` and predictions are to be made at values of `s` not included in the original fit, these values of `s` are merged with `object$lambda` and the model is refit before predictions are made. If `exact = FALSE` (default), then the `predict` function uses linear interpolation to make predictions for values of `s` that do not coincide with lambdas used in the fitting algorithm.

Here is a simple example illustrating the use of both these function arguments:
```{r}
fit <- glmnet(x, y)
any(fit$lambda == 0.5)  # 0.5 not in original lambda sequence
coef.apprx <- coef(fit, s = 0.5, exact = FALSE)
coef.exact <- coef(fit, s = 0.5, exact = TRUE, x=x, y=y)
cbind2(coef.exact[which(coef.exact != 0)], 
       coef.apprx[which(coef.apprx != 0)])
```
The left and right columns show the coefficients for `exact = TRUE` and `exact = FALSE` respectively. (For brevity we only show the non-zero coefficients.) We see from the above that 0.5 is not in the sequence and that hence there are some small differences in coefficient values. Linear interpolation is usually accurate enough if there are no special requirements. Notice that with `exact = TRUE` we have to supply by named argument any data that was used in creating the original fit, in this case `x` and `y`.

Users can make predictions from the fitted `glmnet` object. In addition to the arguments in `coef`, the primary argument is `newx`, a matrix of new values for `x` at which predictions are desired. The `type` argument allows users to choose the type of prediction returned:

* "link" returns the fitted values (i.e. $\hat\beta_0 + x_i^T\hat\beta$)

* "response" gives the same output as "link" for "gaussian" family.

* "coefficients" returns the model codfficients.

* "nonzero" retuns a list of the indices of the nonzero coefficients for each value of `s`.

For example, the following code gives the fitted values for the first 5 observations at $\lambda = 0.05$:
```{r}
predict(fit, newx = x[1:5,], type = "response", s = 0.05)
```

If multiple values of `s` are supplied, a matrix of predictions is
produced. If no value of `s` is supplied, a matrix of predictions is
supplied, with columns corresponding to all the lambdas used in the fit.

We can plot the fitted object as in the Quick Start section. Here we walk through more arguments for the `plot` function. The `xvar` argument allows users to decide what is plotted on the `x`-axis. `xvar` allows three measures: "norm" for the $\ell_1$-norm of the coefficients (default), "lambda" for the log-lambda value and "dev" for %deviance explained. Users can also label the curves with the variable index numbers simply by setting `label = TRUE`.

For example, let's plot `fit` against the log-lambda value and with each curve labeled:
```{r}
plot(fit, xvar = "lambda", label = TRUE)
```

Now when we plot against %deviance we get a very different
picture. This is percent deviance explained on the training data, and
is a measure of complexity of the model.  We see that toward the end of the path, %deviance is not changing much but the coefficients are "blowing up" a bit. This enables us focus attention on the parts of the fit that matter. This will especially be true for other models, such as logistic regression.
```{r}
plot(fit, xvar = "dev", label = TRUE)
```

### Cross-validation

K-fold cross-validation can be performed using the `cv.glmnet` function. In addition to all the `glmnet` parameters, `cv.glmnet` has its special parameters including `nfolds` (the number of folds), `foldid` (user-supplied folds), and `type.measure`(the loss used for cross-validation):

* "deviance" or "mse" for squared loss, and

* "mae" uses mean absolute error.

As an example,
```{r}
cvfit <- cv.glmnet(x, y, type.measure = "mse", nfolds = 20)
```
does 20-fold cross-validation based on mean squared error criterion (the default for "gaussian" family). Printing the resulting object gives some basic information on the cross-validation performed:
```{r}
print(cvfit)
```

`cv.glmnet` also supports parallel computing. To make it work, users must register parallel beforehand. We give a simple example of comparison here. Unfortunately, the package `doMC` is not available on Windows platforms (it is on others), so we cannot run the code here, but we present timing information recorded during one of our test runs.
```{r, eval=FALSE}
library(doMC)
registerDoMC(cores = 2)
X <- matrix(rnorm(1e4 * 200), 1e4, 200)
Y <- rnorm(1e4)
```

```{r, eval=FALSE}
system.time(cv.glmnet(X, Y))
```
```{r, echo=FALSE}
structure(c(2.44, 0.08, 2.518, 0, 0), class = "proc_time", .Names = c("user.self",
"sys.self", "elapsed", "user.child", "sys.child"))
```
```{r, eval=FALSE}
system.time(cv.glmnet(X, Y, parallel = TRUE))
```
```{r, echo=FALSE}
structure(c(0.508999999999999, 0.057, 1.56699999999999, 1.941,
0.1), class = "proc_time", .Names = c("user.self", "sys.self",
"elapsed", "user.child", "sys.child"))
```

As suggested from the above, parallel computing can significantly speed up the computation process especially for large-scale problems.

The `coef` and `predict` methods for `cv.glmnet` objects are similar to those for a `glmnet` object, except that two special strings are also supported by `s` (the values of $\lambda$ requested):

* "lambda.min": the $\lambda$ at which the smallest MSE is achieved.

* "lambda.1se": the largest $\lambda$ at which the MSE is within one standard error of the smallest MSE (default).

```{r out.lines = 10}
cvfit$lambda.min
predict(cvfit, newx = x[1:5,], s = "lambda.min")
coef(cvfit, s = "lambda.min")
```

Users can explicitly control the fold that each observation is assigned to via the `foldid` argument. This is useful, for example, in using cross-validation to select a value for $\alpha$:
```{r}
foldid <- sample(1:10, size = length(y), replace = TRUE)
cv1  <- cv.glmnet(x, y, foldid = foldid, alpha = 1)
cv.5 <- cv.glmnet(x, y, foldid = foldid, alpha = 0.5)
cv0  <- cv.glmnet(x, y, foldid = foldid, alpha = 0)
```

There are no built-in plot functions to put them all on the same plot, so we are on our own here:
```{r}
par(mfrow = c(2,2))
plot(cv1); plot(cv.5); plot(cv0)
plot(log(cv1$lambda)   , cv1$cvm , pch = 19, col = "red",
     xlab = "log(Lambda)", ylab = cv1$name)
points(log(cv.5$lambda), cv.5$cvm, pch = 19, col = "grey")
points(log(cv0$lambda) , cv0$cvm , pch = 19, col = "blue")
legend("topleft", legend = c("alpha= 1", "alpha= .5", "alpha 0"),
       pch = 19, col = c("red","grey","blue"))
```

We see that the lasso (`alpha=1`) does about the best here. We also see that the range of lambdas used differs with `alpha`.

### Other function arguments

In this section we breifly describe some other useful arguments when
calling  `glmnet`: `upper.limits`, `lower.limits`, `penalty.factor`, `exclude` and `intercept`.

Suppose we want to fit our model but limit the coefficients to be bigger than -0.7 and less than 0.5. This can be achieved by specifying the `upper.limits` and `lower.limits` arguments:
```{r}
tfit <- glmnet(x, y, lower.limits = -0.7, upper.limits = 0.5)
plot(tfit)
```

Often we want the coefficients to be positive: to do so, we just need to specify `lower.limits = 0`. (Note, the lower limit must be no bigger than zero, and the upper limit no smaller than zero.) These bounds can be a vector, with different values for each coefficient. If given as a scalar, the same number gets recycled for all.

The `penalty.factor` argument allows users to apply separate penalty factors to each coefficient. This is very useful when we have prior knowledge or preference over the variables. Specifically, if $v_j$ denotes the penalty factor for the $j$th variable, the penalty term becomes
$$
\lambda \sum_{j=1}^p \boldsymbol{v_j} P_\alpha(\beta_j) = \lambda \sum_{j=1}^p \boldsymbol{v_j} \left[ (1-\alpha)\frac{1}{2} \beta_j^2 + \alpha |\beta_j| \right].
$$

The default is 1 for each coefficient, i.e. coefficients are penalized
equally. Note that any variable with `penalty.factor` equal to zero is
not penalized at all! This is useful in the case where some variables
are always to be included unpenalized in the model, such as the demographic
variables sex and age in medical studies. Note the penalty factors are internally rescaled to sum to `nvars`, the number of variables in the given `x` matrix.

Here is an example where we set the penalty factors for variables 1, 3 and 5 to be zero:
```{r}
p.fac <- rep(1, 20)
p.fac[c(1, 3, 5)] <- 0
pfit <- glmnet(x, y, penalty.factor = p.fac)
plot(pfit, label = TRUE)
```

We see from the labels that the three variables with zero penalty factors always stay in the model, while the others follow typical regularization paths and shrunk to zero eventually.

`exclude` allows one to block certain variables from being the model at all. Of course, one could simply subset these out of `x`, but sometimes `exclude` is more useful, since it returns a full vector of coefficients, just with the excluded ones set to zero.

The `intercept` argument allows the user to decide if an intercept should be included in the model or not (it is never penalized). The default is `intercept = TRUE`. If `intercept = FALSE` the intercept is forced to be zero.

## Linear Regression: `family = "mgaussian"` (multi-response)

The multi-response Gaussian family is useful when there are a number of (correlated) responses, also known as the "multi-task learning" problem. Here, a variable is either included in the model for all the responses, or excluded for all the responses. Most of the arguments for this family are the same as that for `family = "gaussian"`, so we focus on the differences with the single response model.

As the name suggests, the response $y$ is not a vector but a matrix of quantitative responses. As a result, the coefficients at each value of lambda are also a matrix.

`glmnet` solves the problem
$$
\min_{(\beta_0, \beta) \in \mathbb{R}^{(p+1)\times K}}\frac{1}{2N} \sum_{i=1}^N \|y_i -\beta_0-\beta^T x_i\|^2_F+\lambda \left[ (1-\alpha)\|\beta\|_F^2/2 + \alpha\sum_{j=1}^p\|\beta_j\|_2\right].
$$
Here $\beta_j$ is the $j$th row of the $p\times K$ coefficient matrix
$\beta$, and we replace the absolute penalty on each single
coefficient by a group-lasso penalty on each coefficient $K$-vector
$\beta_j$ for a single predictor (i.e. column of the `x` matrix). The
group lasso penalty behaves like the lasso, but on the whole group of
coefficients for each response: they are either all zero, or else none
are zero, but are shrunk by an amount depending on $\lambda$.

We use a set of data generated beforehand for illustration. We fit a regularized multi-response Gaussian model to the data, with an object `mfit` returned.
```{r}
data(MultiGaussianExample)
x <- MultiGaussianExample$x
y <- MultiGaussianExample$y
mfit <- glmnet(x, y, family = "mgaussian")
```
The `standardize.response` argument is only for `mgaussian` family. If `standardize.response = TRUE`, the response variables are standardized (default is `FALSE`).

As before, we can use the `plot` method to visualize the coefficients:
```{r}
plot(mfit, xvar = "lambda", label = TRUE, type.coef = "2norm")
```

Note that we set `type.coef = "2norm"`. Under this setting, a single curve is plotted per variable, with value equal to the $\ell_2$ norm of the variable's coefficient vector. The default setting is `type.coef = "coef"`, where a coefficient plot is created for each response (multiple figures). `xvar` and `label` are two other arguments which have the same functionality as in the single-response case.

We can extract the coefficients and make predictions at requested
values of $\lambda$ by using the `coef` and `predict` methods
respectively, as before. Here is an example of a `predict` call:
```{r}
predict(mfit, newx = x[1:5,], s = c(0.1, 0.01))
```
The prediction result is saved in a three-dimensional array with the
first two dimensions being the prediction matrix for each response
variable and the third corresponding to the response variables.

## Logistic Regression: `family = "binomial"`

Logistic regression is a widely-used model when the response is binary. Suppose the response variable takes values in $\mathcal{G}=\{1,2\}$. Denote $y_i = I(g_i=1)$. We model

$$\mbox{Pr}(G=2|X=x)=\frac{e^{\beta_0+\beta^Tx}}{1+e^{\beta_0+\beta^Tx}},$$

which can be written in the following form:

$$\log\frac{\mbox{Pr}(G=2|X=x)}{\mbox{Pr}(G=1|X=x)}=\beta_0+\beta^Tx,$$
the so-called "logistic" or log-odds transformation.

The objective function for  logistic regression is the penalized negative binomial log-likelihood, and is
$$
\min_{(\beta_0, \beta) \in \mathbb{R}^{p+1}} -\left[\frac{1}{N} \sum_{i=1}^N y_i \cdot (\beta_0 + x_i^T \beta) - \log (1+e^{(\beta_0+x_i^T \beta)})\right] + \lambda \big[ (1-\alpha)\|\beta\|_2^2/2 + \alpha\|\beta\|_1\big].
$$
Logistic regression is often plagued with degeneracies when $p > N$ and exhibits wild behavior even when $N$ is close to $p$; the elastic net penalty alleviates these issues, and regularizes and selects variables as well.

We use a "proximal Newton" algorithm for optimizing this
criterion. This makes repeated use of a quadratic approximation to the
log-likelihood, and then weighted coordinate descent on the resulting
penalized weighted least-squares problem. These constitute an outer
and inner loop, also known as iteratively reweighted penalized least squares.

For illustration purposes, we load the pre-generated input matrix `x`
and the response vector `y` from the data file. The input matrix $x$
is the same as for other families. For binomial logistic regression,
the response variable $y$ should be either a binary vector, a factor
with two levels, or a two-column matrix of counts or proportions. The
latter is useful for grouped binomial data, or in applications where
we have "soft" class membership, such as occurs in the EM algorithm.
```{r}
data(BinomialExample)
x <- BinomialExample$x
y <- BinomialExample$y
```

Other optional arguments of `glmnet` for binomial regression are almost same as those for Gaussian family. Don't forget to set `family` option to "binomial":
```{r}
fit <- glmnet(x, y, family = "binomial")
```

As before, we can print and plot the fitted object, extract the coefficients at specific $\lambda$'s and also make predictions. For plotting, the optional arguments such as `xvar` and `label` work in the same way as for `family = "gaussian"`. Prediction is a little different for `family = "binomial"`, mainly in the function argument `type`:

* "link" gives the linear predictors.

* "response" gives the fitted probabilities.

* "class" produces the class label corresponding to the maximum probability.

As with `family = "gaussian"`, "coefficients" computes the coefficients at values of `s` and "nonzero" retuns a list of the indices of the nonzero coefficients for each value of `s`. Note that the results ("link", "response", "coefficients", "nonzero") are returned only for the class corresponding to the second level of the factor response.

In the following example, we make prediction of the class labels at $\lambda = 0.05, 0.01$.
```{r}
predict(fit, newx = x[1:5,], type = "class", s = c(0.05, 0.01))
```

For logistic regression, `cv.glmnet` has similar arguments and usage as Gaussian. `nfolds`, `weights`, `lambda`, `parallel` are all available to users. There are some differences in `type.measure`: "deviance" and "mse" do not both mean squared loss. Rather,

* "mse" uses squared loss.

* "deviance" uses actual deviance.

* "mae" uses mean absolute error.

* "class" gives misclassification error.

* "auc" (for two-class logistic regression ONLY) gives area under the ROC curve.

For example, the code below uses misclassification error as the criterion for 10-fold cross-validation:
```{r}
cvfit <- cv.glmnet(x, y, family = "binomial", type.measure = "class")
```

As before, we can plot the object and show the optimal values of $\lambda$.
```{r}
plot(cvfit)
cvfit$lambda.min
cvfit$lambda.1se
```

`coef` and `predict` for the `cv.glmnet` object for `family = "binomial"` are simliar to the Gaussian case and we omit the details.

Like other generalized linear models, `glmnet` allows for an
"offset". This is a fixed vector of $N$ numbers that is added into the
linear predictor. For example, you may have fitted some other logistic
regression using other variables (and data), and now you want to see
if the present variables can add further predictive power. To do this,
you can use the predicted logit from the other model as an offset in
the `glmnet` call. Offsets are also useful in Poisson models, which we
discuss later.

## Multinomial Regression: `family = "multinomial"`

The multinomial model extends the binomial when the number of classes
is more than two. Suppose the response variable has $K$ levels ${\cal G}=\{1,2,\ldots,K\}$. Here we model
$$\mbox{Pr}(G=k|X=x)=\frac{e^{\beta_{0k}+\beta_k^Tx}}{\sum_{\ell=1}^Ke^{\beta_{0\ell}+\beta_\ell^Tx}}.$$
There is a linear predictor for each class!

Let ${Y}$ be the $N \times K$ indicator response matrix, with elements $y_{i\ell} = I(g_i=\ell)$. Then the elastic net penalized negative log-likelihood function becomes
$$
\ell(\{\beta_{0k},\beta_{k}\}_1^K) = -\left[\frac{1}{N} \sum_{i=1}^N \Big(\sum_{k=1}^Ky_{il} (\beta_{0k} + x_i^T \beta_k)- \log \big(\sum_{\ell=1}^K e^{\beta_{0\ell}+x_i^T \beta_\ell}\big)\Big)\right] +\lambda \left[ (1-\alpha)\|\beta\|_F^2/2 + \alpha\sum_{j=1}^p\|\beta_j\|_q\right].
$$
Here we really abuse notation! $\beta$ is a $p\times K$ matrix of coefficients. $\beta_k$ refers to the $k$th column (for outcome category $k$), and $\beta_j$ the $j$th row (vector of $K$ coefficients for variable $j$). The last penalty term is $\|\beta_j\|_q$. We support two options for $q$: $q\in \{1,2\}$. When $q=1$, this is a lasso penalty on each of the parameters. When $q=2$, this is a grouped-lasso penalty on all the $K$ coefficients for a particular variable, which makes them all be zero or nonzero together.

The standard Newton algorithm can be tedious here. Instead, for $q=1$ we use a so-called partial Newton algorithm by making a partial quadratic approximation to the log-likelihood, allowing only $(\beta_{0k}, \beta_k)$ to vary for a single class at a time. For each value of $\lambda$, we first cycle over all classes indexed by $k$, computing each time a partial quadratic approximation about the parameters of the current class. Then, the inner procedure is almost the same as for the binomial case. When $q=2$, we use a different approach that we will not explain here.

For the `family = "multinomial"` case, usage is similar to that for `family = "binomial"`. In this section we describe the differences. First, we load a set of generated data:
```{r}
data(MultinomialExample)
x <- MultinomialExample$x
y <- MultinomialExample$y
```

The response variable can be a `nc >= 2` level factor, or an `nc`-column matrix of counts or proportions. Internally glmnet will make the rows of this matrix sum to 1, and absorb the total mass into the weight for that observation. `offset` should be a `nobs x nc` matrix if one is provided.

A special option for multinomial regression is `type.multinomial`, which allows the usage of a grouped lasso penalty ($q = 2$) if `type.multinomial = "grouped"`. The default is `type.multinomial = "ungrouped"` ($q = 1$).
```{r}
fit <- glmnet(x, y, family = "multinomial", type.multinomial = "grouped")
plot(fit, xvar = "lambda", label = TRUE, type.coef = "2norm")
```

For the `plot` method, the function arguments are `xvar`, `label` and `type.coef`, in addition to other ordinary graphical parameters. `xvar` and `label` are the same as other families while `type.coef` is only for multinomial regression and multi-response Gaussian model. It can produce a figure of coefficients for each response variable if `type.coef = "coef"` or a figure showing the $\ell_2$-norm in one figure if `type.coef = "2norm"`.

We can also do cross-validation and plot the returned object. Note
that although `type.multinomial` is not a named argument in
`cv.glmnet`, in fact any argument that can be passed to `glmnet` is
valid in the argument list of `cv.glmnet`. Such arguments are passed
via the `...` argument directly to the calls to `glmnet` inside the
`cv.glmnet` function.
```{r}
cvfit <- cv.glmnet(x, y, family = "multinomial", type.multinomial = "grouped")
plot(cvfit)
```

Users may wish to predict at the optimally selected $\lambda$:
```{r}
predict(cvfit, newx = x[1:10,], s = "lambda.min", type = "class")
```

## Poisson Regression: `family = "poisson"`

Poisson regression is used to model count data under the assumption of Poisson error, or otherwise non-negative data where the mean and variance are proportional. Like the Gaussian and binomial models, the Poisson distribution is a member of the exponential family of distributions. We usually model its positive mean on the log scale:   $\log \mu(x) = \beta_0+\beta' x$.

The log-likelihood for observations $\{x_i,y_i\}_1^N$ is given by
$$
l(\beta|X, Y) = \sum_{i=1}^N \left(y_i (\beta_0+\beta^T x_i) - e^{\beta_0+\beta^Tx_i}\right).
$$
As before, we optimize the penalized log-likelihood:
$$
\min_{\beta_0,\beta} -\frac1N l(\beta|X, Y)  + \lambda \left((1-\alpha) \sum_{i=1}^N \beta_i^2/2 +\alpha \sum_{i=1}^N |\beta_i|\right).
$$

`glmnet` uses an outer Newton loop and an inner weighted least-squares loop (as in logistic regression) to optimize this criterion.

First, we load a pre-generated set of Poisson data:
```{r}
data(PoissonExample)
x <- PoissonExample$x
y <- PoissonExample$y
```

We apply the function `glmnet` with `family = "poisson"`:
```{r}
fit <- glmnet(x, y, family = "poisson")
```
The optional input arguments of `glmnet` for `"poisson"` family are similar to those for other families.

`offset` is a particularly useful argument for Poisson models. When dealing with rate data in Poisson models, the counts collected are often based on different exposures such as length of time observed, area and years. A poisson rate $\mu(x)$ is relative to a unit exposure time, so if an observation $y_i$ was exposed for $E_i$ units of time, then the expected count would be $E_i\mu(x)$, and the log mean would be $\log(E_i)+\log(\mu(x))$. In a case like this, we would supply an *offset* $\log(E_i)$ for each observation. Hence `offset` is a vector of length $N$ that is included in the linear predictor. (_Warning_: if `offset` is supplied in `glmnet`, offsets must also also be supplied to `predict` via the `newoffset` argument to make reasonable predictions.)

Again, we plot the coefficients to have a first sense of the result.
```{r}
plot(fit)
```

As before, we can extract the coefficients and make predictions at certain $\lambda$'s using `coef` and `predict` respectively. The optional input arguments are similar to those for other families. For the `predict` method, the argument `type` has the same meaning as that for `family = "binomial"`, except that "response" gives the fitted mean (rather than fitted probabilities in the binomial case). For example, we can do the following:
```{r out.lines = 7}
coef(fit, s = 1)
predict(fit, newx = x[1:5,], type = "response", s = c(0.1,1))
```

We may also use cross-validation to find the optimal $\lambda$'s and thus make inferences.
```{r}
cvfit <- cv.glmnet(x, y, family = "poisson")
```
Options are almost the same as the Gaussian family except that for `type.measure`:

* "deviance" (default) gives the deviance.
* "mse" is for mean squared error.
* "mae" is for mean absolute error.

## Cox Regression: `family = "cox"`

The Cox proportional hazards model is commonly used for the study of
the relationship beteween predictor variables and survival time. We
have another vignette (["Regularized Cox
Regression"](https://glmnet.stanford.edu/articles/Coxnet.html))
dedicated solely to fitting regularized Cox models with the `glmnet`
package; please consult that vignette for details.

## Programmable GLM families: `family = family()`

Since version 4.0, `glmnet` has the facility to fit any GLM family by
specifying a `family` object, as used by `stats::glm`. For these more
general families, the outer Newton loop is performed in R, while the
inner elastic-net loop is performed in Fortran, for each value of
lambda. The price for this generality is a small hit in speed. 
For details, see the vignette 
["GLM `family` functions in `glmnet`"](https://glmnet.stanford.edu/articles/glmnetFamily.html)

##  Assessing models on test data

Once we have fit a series of models using `glmnet`, we often assess their performance on a set of evaluation or test data. We usually go through the process of building a prediction matrix, deciding on the performance measure, and computing these measures for a series of values for `lambda` (and `gamma` for relaxed fits). `glmnet` provides three functions (`assess.glmnet`, `roc.glmnet` and `confusion.glmnet`) that make these tasks easier.

### Performance measures

The function `assess.glmnet` computes the same performance measures produced by `cv.glmnet`, but on a validation or test dataset. 
```{r}
data(BinomialExample)
x <- BinomialExample$x
y <- BinomialExample$y
itrain <- 1:70
fit <- glmnet(x[itrain, ], y[itrain], family = "binomial", nlambda = 5)
assess.glmnet(fit, newx = x[-itrain, ], newy = y[-itrain])
```

This produces a list with *all* the measures suitable for a binomial model, computed for the entire sequence of lambdas in the fit object. Here the function identifies the model family from the `fit` object.

A second use case builds the prediction matrix before calling `assess.glmnet`:
```{r, eval=FALSE}
pred <- predict(fit, newx = x[-itrain, ])
assess.glmnet(pred, newy = y[-itrain], family = "binomial")
```

Here we have to provide the `family` as an argument; the results (not shown) are the same. Users can see the various measures suitable for each family via 
```{r}
glmnet.measures()
```

`assess.glmnet` can also take the result of `cv.glmnet` as input. In this case the predictions are made at the optimal values for the parameter(s).
```{r out.lines = 11}
cfit <- cv.glmnet(x[itrain, ], y[itrain], family = "binomial", nlambda = 30)
assess.glmnet(cfit, newx = x[-itrain, ], newy = y[-itrain])
```

This uses the default value of `s = "lambda.1se"`, just like `predict` would have done. Users can provide additional arguments that get passed on to `predict`. For example, the code below shows the performance measures for `s = "lambda.min"`:
```{r out.lines = 11}
assess.glmnet(cfit, newx = x[-itrain, ],newy = y[-itrain], s = "lambda.min")
```

### Prevalidation

One interesting use case for `assess.glmnet` is to get the results of cross-validation using other measures. By specifying `keep = TRUE` in the `cv.glmnet` call, a matrix of prevalidated predictions are stored in the returned output as the `fit.preval` component. We can then use this component in the call to `assess.glmnet`:
```{r out.lines = 11}
cfit <- cv.glmnet(x, y, family = "binomial", keep = TRUE, nlambda = 30)
assess.glmnet(cfit$fit.preval, newy = y, family = "binomial")
```

Users can verify that the first measure here `deviance` is identical to the component `cvm` on the `cfit` object.

### ROC curves for binomial data

In the special case of binomial models, users often would like to see the ROC curve for validation or test data. Here the function `roc.glmnet` provides the goodies. Its first argument is as in `assess.glmnet`. Here we illustrate one use case, using the prevlidated CV fit. 
```{r}
cfit <- cv.glmnet(x, y, family = "binomial", type.measure = "auc", 
                  keep = TRUE)
rocs <- roc.glmnet(cfit$fit.preval, newy = y)
```

`roc.glmnet` returns a list of cross-validated ROC data, one for each model along the path. The code below demonstrates how one can plot the output. The first line identifies the `lambda` value giving the best area under the curve (AUC). Then we plot all the ROC curves in grey and the "winner" in red.
```{r}
best <- cvfit$index["min",]
plot(rocs[[best]], type = "l")
invisible(sapply(rocs, lines, col="grey"))
lines(rocs[[best]], lwd = 2,col = "red")
```

### Confusion matrices for classification

For binomial and multinomial models, we often wish to examine the classification performance on new data. The function `confusion.glmnet` will do that for us.
```{r}
data(MultinomialExample)
x <- MultinomialExample$x
y <- MultinomialExample$y
set.seed(101)
itrain <- sample(1:500, 400, replace = FALSE)
cfit <- cv.glmnet(x[itrain, ], y[itrain], family = "multinomial")
cnf <- confusion.glmnet(cfit, newx = x[-itrain, ], newy = y[-itrain])
```

`confusion.glmnet` produces a table of class "confusion.table" which inherits from class "table", and we also provide a `print` method for it.
```{r}
print(cnf)
```

The first argument to `confusion.glmnet` should be a `glmnet` or `cv.glmnet` object (from which predictions can be made), or a matrix/array of predictions, such as the *kept* `"fit.preval"` component in the output of a `cv.glmnet` call with `keep = TRUE`. When a matrix/array of predictions is provided, we need to specify the `family` option, otherwise *confusion* can exist between "binomial" and "multinomial" prediction matrices.

When predictions for more than one model in the path is provided, `confusion.glmnet` returns a list of confusion tables. For example, the prevalidated predictions from `cv.glmnet` are for the whole `lambda` path, and so we are returned a list of confusion tables. In the code below, we identify and print the one achieving the smallest classification error.
```{r}
cfit <- cv.glmnet(x, y, family = "multinomial", type = "class", keep = TRUE)
cnf <- confusion.glmnet(cfit$fit.preval, newy = y, family = "multinomial")
best <- cfit$index["min",]
print(cnf[[best]])
```

## Filtering variables

The `exclude` argument to `glmnet` accepts a vector of indices, indicating which variables should be excluded from the fit, i.e. get zeros for their
coefficients. From version 4.1-2, the `exclude` argument can accept a *function*. The idea is that variables can be filtered based on some of their properties (e.g. too sparse) before any models are fit. When performing cross-validation (CV), this filtering should be done separately inside each fold of `cv.glmnet`.

Here is a typical filter function: it excludes variables that are more than 80% sparse.
```{r}
filter <- function(x, ...) which(colMeans(x == 0) > 0.8)
```
This function gets invoked inside the call to `glmnet`, and uses the supplied `x` to generate the indices.

We give a simple example using this:
```{r}
set.seed(101)
n <-500; p <- 50
x <- matrix(rnorm(n * p), n, p)
x[sample(seq(length(x)), 4 * n * p / 5)] <- 0
y <- rnorm(n) + x %*% (rnorm(p) / 5) > 0
excl <- filter(x)
print(excl)
fit.orig <- glmnet(x, y, family = "binomial", exclude = excl)
fit.new  <- glmnet(x, y, family = "binomial", exclude = filter)
all.equal(fit.orig, fit.new)
```
The fits are the same (the `call` component is all that differs).

The nice thing with this approach is that `cv.glmnet` does the right thing: it applies the filter function separately to each training-fold `x`, and hence accounts for any bias that may be incurred by the filtering.
```{r}
cvfit.filt <- cv.glmnet(x, y, family = "binomial", exclude = filter)
```
 
The most general form of the filter function is
```{r, eval=FALSE}
filter <- function(x, y, weights, ...) {}
```

Yes, users can use `y` as well in their filtering criterion, and the observation weights if that makes sense. As we did above, some of
the arguments can be omitted, but the `...` must always be there. We will give some useful examples of filter functions a bit later.

An important point to note is that any other arguments in a user's filter function are *ignored*. So, what we should if we want one or more additional
parameters to guide the filtering, such as the choice of `0.8` in our filter function above? The safest way to achieve this is to write a filter *generator* function, that produces a filter function as its output. We give an example:

```{r}
sparsity <- function(fraction = 0.7) {
  function(x, ...) which(colMeans(x == 0) > fraction)
}
sparsity(0.5)
```

Here, `sparsity()` is a function that produces a function. The argument `fraction` is found in the environment of that function, and
hence is *built in*. We can now run `cv.glmnet` as follows:

```{r}
foldid <- sample(rep(1:10,length.out = length(y)))
cvfit.filt1 <- cv.glmnet(x, y, family = "binomial", foldid = foldid, 
                         exclude = filter)
cvfit.filt2 <- cv.glmnet(x, y, family = "binomial", foldid = foldid, 
                         exclude = sparsity(0.8))
all.equal(cvfit.filt1, cvfit.filt2)
```

These agree in the important parts.

We end this section with some useful filtering functions. In genomics, where the `x` matrix can be very wide, we often filter features based on
variance. We will use expression arrays as an example. All else being equal, we expect the variance of the gene expression of a gene to be
higher if there is something *going on* across the samples, and we guess that that *something* is related to the response. Here is a function that efficiently computes the column-wise variance for a wide matrix, followed by a filter function that uses it.

```{r}
uvar <- function(x, means = FALSE) {
  # if means = TRUE, the means and variances are returned, 
  # otherwise just the variances
  m <- colMeans(x)
  n <- nrow(x)
  x <- x - outer(rep(1,n),m)
  v <- colSums(x^2) / (n - 1)
  if (means) list(mean = m, var = v) else v
}

vfilter <- function(q = 0.3) {
  function(x,...) {
    v <- uvar(x)
    which(v < quantile(v, q))
  }
}
```

Here, our filter function `vfilter()` is a function generator as discussed above, and will exclude the fraction `q` of variables with the lowest variance.
	
We can also use `y` as part of our filtering. This final example builds on the example above. Here the response `y` is binary, taking two values. We want to filter out variables with small absolute values for the two-sample t-statistic with groups defined by `y`. Again, we present an efficient function for computing the t-statistics, and then the filter. (Notice that for convenience, our function `uvar` above can optionally return a mean vector and a variance vector.)

```{r}
ut.test <- function(x, y, s0 = 0) {
  ni <- table(y); n <- sum(ni)
  if(length(ni) != 2) stop("Only two-sample t-test here")
  index <- seq(n)
  mv <- tapply(index, y, function(i, x) uvar(x[i, ], means = TRUE), x = x)
  ss <- ((ni[1] - 1) * mv[[1]]$var  + (ni[2] - 1) * mv[[2]]$var)
  sd <- sqrt(ss * (1 / ni[[1]] + 1 / ni[[2]]) / (n - 2))
  numer <- mv[[1]]$mean - mv[[2]]$mean
  numer / (sd + s0)
}

tfilter <- function(q = 0.3, s0 = 0) {
  function(x, y, ...) {
    abs_tstats <- abs(ut.test(x, y, s0 = s0))
    which(abs_tstats < quantile(abs_tstats, q))
  }
}
```

One might think that unsupervised filtering using just `x` before fitting the model is fair game, and perhaps need not be accounted for in cross-validation. This is an interesting issue, and one can argue that there could be some bias if it was not accounted for. But filtering using `x` and `y` *definitely* introduces potential bias. However, it can still give good results, as long as it it correctly accounted for when running CV.

```{r}
cvfit.filt3 <- cv.glmnet(x, y, family = "binomial", foldid = foldid, 
                         exclude = tfilter(0.4))
```

## Other Package Features

In this section, we describe other features in the `glmnet` package that might be of interest to users.

### Sparse matrix support

Our package supports sparse input matrices, which allow the efficient storage and operation of large matrices having only a few nonzero entries. The usage of sparse matrices (inherits from class `"sparseMatrix"` as in the `Matrix` package) in `glmnet` is the same as if a regular matrix is provided.

We load a set of sample data created beforehand. It loads `x`, a 100*20 sparse input matrix and `y`, the response vector.
```{r}
data(SparseExample)
x <- SparseExample$x
y <- SparseExample$y
class(x)
```
Users can create a sparse matrix with the function `sparseMatrix` by
providing the locations and values of the nonzero
entries. Alternatively, the `Matrix` function from the `Matrix`
package can also be used to contruct a sparse matrix by setting
`sparse = TRUE`, but this defeats the purpose somewhat if the matrix
is large.

We can fit the model the same way as before.
```{r}
fit <- glmnet(x, y)
```

We can also do the cross-validation and plot the resulting object.
```{r}
cvfit = cv.glmnet(x, y)
plot(cvfit)
```

The usage of other functions are similar and we do not expand here.

Note that sparse matrices can also be used for `newx`, the new input matrix in the `predict` function. For example,
```{r}
i <- sample(1:5, size = 25, replace = TRUE)
j <- sample(1:20, size = 25, replace = TRUE)
x <- rnorm(25)
nx <- sparseMatrix(i = i, j = j, x = x, dims = c(5, 20))
predict(cvfit, newx = nx, s = "lambda.min")
```

### Fitting big and/or sparse unpenalized generalized linear models

The `glmnet` package includes a function `bigGlm` for fitting a single _unpenalized_ generalized linear model (GLM), but allowing all the options of `glmnet`. In other words, the user can set coefficient upper and/or lower bounds, and can provide the `x` matrix in sparse matrix format. This is not too much more than fitting a model with a single value of `lambda = 0` (with some protection from edge cases). `predict` and `print` methods can be called on the output.
```{r}
data(BinomialExample)
x <- BinomialExample$x
y <- BinomialExample$y
fit <- bigGlm(x, y, family = "binomial", lower.limits = -1)
print(fit)
```

### Creating `x` from mixed variables and/or missing data

The `glmnet` package includes a function `makeX` that makes it easy to create the model matrix `x` needed as input to `glmnet`. It takes as input a data frame, which can contain vectors, matrices and factors. Some of the features are:

* Factors are *one-hot* encoded to form indicator matrices.
* Missing values in the resultant matrix can be replaced by the column means.
* The `sparse` option returns a matrix in column-sparse format. This is useful if the data are large, and factors have many levels.
* Two data frames can be provided, `train` and `test`. This ensures the factor levels correspond, and also imputes missing data in the test data from means in the training data.
  
Our first example demonstrates how `makeX` works with factors:
```{r}
set.seed(101)
X <- matrix(rnorm(5), nrow = 5)
X2 <- sample(letters[1:3], 5, replace = TRUE)
X3 <- sample(LETTERS[1:3], 5, replace = TRUE)
df <- data.frame(X, X2, X3)
makeX(df)
```

Include the option `sparse = TRUE` if a sparse output is desired:
```{r}
makeX(df, sparse = TRUE)
```

Next, let us add some missing values to our data matrix. By default, `makeX` leaves `NA`s as is:
```{r}
Xn  <- X ; Xn[3,1] <- NA
X2n <- X2; X2n[1]  <- NA
X3n <- X3; X3n[5]  <- NA
dfn <- data.frame(Xn, X2n, X3n)
dfn
makeX(dfn)
```

We can impute the missing values with column means by passing the option `na.impute = TRUE`:
```{r}
makeX(dfn, na.impute = TRUE, sparse = TRUE)
```

Finally if a test set is available as well, both the training and test sets can be passed to `makeX` at the same time so that all the levels of factors present in the training and test sets will be represented correctly in the output matrix. In the example below, the third column of the training set only contains "B" and "C" while that of the training set only contains "A" and "C". By passing both data sets to `makeX` at the same time, this third column is correctly expanded into 3 feature columns for both the training and test sets.
```{r}
set.seed(102)
X <- matrix(rnorm(5), nrow = 5)
X2 <- sample(letters[1:3], 5, replace = TRUE)
X3 <- sample(LETTERS[1:3], 5, replace = TRUE)
Xn  <- X ; Xn[5,1] <- NA
X2n <- X2; X2n[1]  <- NA
X3n <- X3; X3n[2]  <- NA
dftn <- data.frame(Xn, X2n, X3n)
dftn
makeX(dfn, dftn, sparse = TRUE)
```

###  Progress bar

Ever run a job on a big dataset, and wonder how long it will take? `glmnet` and `cv.glmnet` come equipped with a progress bar, which can by displayed by passing `trace.it = TRUE` to these functions.
```{r, eval=FALSE}
fit <- glmnet(x, y, trace.it = TRUE)
```

``##``

`` |==================================           |65%``

This display changes in place as the fit is produced. The progress bar is also very helpful with `cv.glmnet`:
```{r, eval=FALSE}
fit <- cv.glmnet(x, y, trace.it = TRUE)
```
``##``

`` Training``

``   |=============================================| 100%``

`` Fold: 1/10``

``   |=============================================| 100%``

`` Fold: 2/10``

``   |=============================================| 100%``

`` Fold: 3/10``

``   |=============================                |  70%``

Tracing of the folds works a little differently when distributed computing is used. 
 
If the user wants `glmnet` and `cv.glmnet` to always print the progress bar, this can be achieved (for a session) via a call to `glmnet.control` with the `itrace` argument:
```{r, eval=FALSE}
glmnet.control(itrace = 1)
```

To reset it, one makes a similar call and sets `itrace = 0`.

## Appendix 0: Convergence Criteria

Glmnet uses a convergence criterion that focuses not on coefficient change but rather the impact of the change on the fitted values, and hence the loss part of the objective. The net result is a weighted norm of the coefficient change vector.

For Gaussian models it uses the following. Suppose observation $i$ has weight $w_i$. Let $v_j$ be the (weighted)
sum-of-squares for variable $x_j$:
$$v_j=\sum_{i=1}^Nw_ix_{ij}^2.$$
If there is an intercept in the model, these $x_j$ will be centered by the weighted mean, and hence this would be a weighted variance. After $\hat\beta_j^o$ has been updated to $\hat\beta_j^n$, we compute $\Delta_j=v_j(\hat\beta_j^o-\hat\beta_j^n)^2$. After a complete cycle of coordinate descent, we look at $\Delta_{max}=\max_j\Delta_j$. Why this measure? We can write
$$\Delta_j=\frac1N\sum_{i=1}^N w_i(x_{ij}\hat\beta_j^o-x_{ij}\hat\beta_j^n)^2,$$
which measures the weighted sum of squares of changes in fitted values for this term. This measures the impact of the change in this coefficient on the fit. If the largest such change is negligible, we stop.

For logistic regression and other non-Gaussian models it is similar for the inner loop, only now the weights for each observation are more complex. For example, for logistic regression the weights are those that arise from the current Newton step, i.e. $w_i^*=w_i\hat p_i(1-\hat p_i)$, where the $\hat p_i$'s are the fitted probabilities as we entered the current inner loop. The intuition is the same: it measures the impact of the coefficient change on the current weighted least squares loss, or quadratic approximation to the log-likelihood loss.

What about outer-loop convergence?

* If the argument `family` was a character string, we use the same measure, except now $\hat\beta^o$ is the coefficient vector before we entered this inner loop, and $\hat\beta^n$ the converged solution for this inner loop. Hence if this Newton step had no impact, we declare outer-loop convergence.

* If the argument `family` was a class "family" object, outer-loop convergence is determined by the change in the objective function value. If the fractional change in the objective function value is less than the `epsnr` control parameter, we declare outer-loop convergence. `epsnr` can be changed via a call to `glmnet.control`.

## Appendix 1: Internal Parameters

Our package has a set of internal parameters which control some aspects of the computation of the path. The *factory default* settings are expected to serve in most cases, and users do not need to make changes unless there are special requirements.

There are several parameters that users can change:

* `fdev` - minimum fractional change in deviance for stopping path; factory default = 1.0e-5.

* `devmax` - maximum fraction of explained deviance for stopping path; factory default = 0.999.

* `eps` - minimum value of `lambda.min.ratio` (see `glmnet` documentation); factory default= 1.0e-6.

* `big` - large floating point number; factory default = 9.9e35. Inf in definition of `upper.limits` is set to `big`.

* `mnlam` - minimum number of path points (lambda values) allowed; factory default = 5.

* `pmin` - minimum null probability for any class; factory default = 1.0e-5.

* `exmx` - maximum allowed exponent; factory default = 250.0.

* `prec` - convergence threshold for multi-response bounds adjustment solution; factory default = 1.0e-10.

* `mxit` - maximum iterations for multi-response bounds adjustment solution; factory default = 100.

* `epsnr`: convergence threshold for the iteratively reweighted least squares loop (see "The `family` Argument for `glmnet`" vignette); factory default = 1e-08.

* `mxitnr`: maximum iterations for the iteratively reweighted least squares loop for each value of $\lambda$ (see "The `family` Argument for `glmnet`" vignette); factory default = 25.

* `factory` - If `TRUE`, reset all the parameters to the factory default; default is `FALSE`.

We illustrate how to change these control parameters through an example. Note that any changes made hold for the duration of the R session, or unless they are changed by the user with a subsequent call to `glmnet.control`.
```{r}
data(QuickStartExample)
x <- QuickStartExample$x
y <- QuickStartExample$y
fit <- glmnet(x, y)
length(fit$lambda)  # number of lambda values fit
```

We can change the minimum fractional change in deviance for stopping path and compare the results. By setting `fdev` to be larger than the default, we see that the computation stopped earlier in the path.
```{r}
glmnet.control(fdev = 0.1)
fit <- glmnet(x, y)
length(fit$lambda)  # number of lambda values fit
```

Users can reset to the default settings with the following code:
```{r}
glmnet.control(factory = TRUE)
```

To view current settings, call `glmnet.control` without any arguments:
```{r out.lines = 8}
glmnet.control()
```

## Appendix 2: Comparison with Other Packages

Some may want to use `glmnet` to solve the lasso or elastic net problem at a single $\lambda$. We compare here the solution by `glmnet` with other packages (such as CVX), and also as an illustration of parameter settings in this situation.

(__Warning__: Though such problems can be solved by `glmnet`, it is __not recommended__ and is not the spirit of the package. `glmnet` fits the __entire__ solution path for the lasso or elastic net problems efficiently with various techniques such as using warm starts and strong rules. Those advantages will disappear if the $\lambda$ sequence is forced to be only one value.)

We illustrate with a typical example in linear models for the purpose of comparison. Given $X$ and $Y$, we want to find $\beta$ such that
$$
\min_{\beta} \|Y - X\beta\|_2^2 + \lambda_0 \|\beta\|_1,
$$
where, say, $\lambda_0 = 8$.

We first solve this using `glmnet`. Notice that there is no intercept term in the objective function, and the columns of $X$ are not necessarily standardized. Corresponding parameters have to be set to make it work correctly. In addition, there is a $1/(2n)$ factor before the quadratic term by default, so we need to adjust $\lambda$ accordingly. For the purpose of comparison, we set `thresh = 1e-20`. However, this is not necessary in many practical applications.
```{r, echo=FALSE}
data(QuickStartExample)
x <- QuickStartExample$x
y <- QuickStartExample$y
```
```{r}
np <- dim(x); n <- np[1]; p <-np[2]

fit <- glmnet(x, y, intercept = F, standardize = F, 
              lambda = 8 / (2 * n), thresh = 1e-20)
```

We then extract the coefficients (with no intercept):
```{r,eval=FALSE}
beta_glmnet <- as.matrix(predict(fit, type = "coefficients")[-1,])
```

Alternatively, a more stable and __strongly recommended__ way to perform this task is to first fit the entire lasso or elastic net path without specifying `lambda`, but then provide the requested $\lambda_0$ to a `predict` call to extract the corresponding coefficients. (Remember to set `exact = TRUE` in the `predict` call to get the exact solution. Otherwise, it will be approximated by linear interpolation.)

```{r}
fit <- glmnet(x, y, intercept = F, standardize = F, thresh = 1e-20)
beta_glmnet <- as.matrix(predict(fit, s = 8 / (2 * n), 
                                 type = "coefficients", 
                                 exact = TRUE, x = x, y = y)[-1,])
```

Next, we use CVX, a general convex optimization solver, to solve this
specific lasso problem. CVX is implemented in the CVXR package on
CRAN.  

```{r, eval=FALSE}
library(CVXR)
beta <- Variable(p)
loss <- sum((y-x%*%beta)^2)/(2*n)
lassoPenalty <- function(beta,lambda)lambda*p_norm(beta,1)
obj <- loss + lassoPenalty(beta, lambda = 8/(2*n))
prob <- Problem(Minimize(obj))
result <- solve(prob)
beta_CVX <- result$getValue(beta)
```

For convenience, the results were saved in `CVXResult.RData`, and we simply load in the results.
```{r}
data(CVXResults)
```

Finally, we solve the same problem with the `lars` package:
```{r, message=FALSE}
library(lars)
fit_lars <- lars(x, y, type = "lasso", intercept = F, normalize = F)
beta_lars <- predict(fit_lars, s = 8 / 2, type = "coefficients", 
                     mode = "lambda")$coefficients
```

The results are listed below up to 6 decimal digits (due to convergence thresholds). We see that all three packages give the same result.
```{r}
cmp <- round(cbind(beta_glmnet, beta_lars, beta_CVX), digits = 6)
colnames(cmp) <- c("beta_glmnet", "beta_lars", "beta_CVX")
cmp
```

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxpath.R
\name{cox.path}
\alias{cox.path}
\title{Fit a Cox regression model with elastic net regularization for a path of
lambda values}
\usage{
cox.path(
  x,
  y,
  weights = NULL,
  offset = NULL,
  alpha = 1,
  nlambda = 100,
  lambda.min.ratio = ifelse(nobs < nvars, 0.01, 1e-04),
  lambda = NULL,
  standardize = TRUE,
  thresh = 1e-10,
  exclude = NULL,
  penalty.factor = rep(1, nvars),
  lower.limits = -Inf,
  upper.limits = Inf,
  maxit = 1e+05,
  trace.it = 0,
  ...
)
}
\arguments{
\item{x}{See glmnet help file}

\item{y}{Survival response variable, must be a \code{Surv} or
\code{stratifySurv} object.}

\item{weights}{See glmnet help file}

\item{offset}{See glmnet help file}

\item{alpha}{See glmnet help file}

\item{nlambda}{See glmnet help file}

\item{lambda.min.ratio}{See glmnet help file}

\item{lambda}{See glmnet help file}

\item{standardize}{See glmnet help file}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than thresh times the null deviance.
Default value is \code{1e-10}.}

\item{exclude}{See glmnet help file}

\item{penalty.factor}{See glmnet help file}

\item{lower.limits}{See glmnet help file}

\item{upper.limits}{See glmnet help file}

\item{maxit}{See glmnet help file}

\item{trace.it}{Controls how much information is printed to screen. Default is
\code{trace.it=0} (no information printed). If \code{trace.it=1}, a progress
bar is displayed. If \code{trace.it=2}, some information about the fitting
procedure is printed to the console as the model is being fitted.}

\item{...}{Other arguments passed from glmnet (not used right now).}
}
\value{
An object of class "coxnet" and "glmnet".
\item{a0}{Intercept value, \code{NULL} for "cox" family.}
\item{beta}{A \code{nvars x length(lambda)} matrix of coefficients, stored in
sparse matrix format.}
\item{df}{The number of nonzero coefficients for each value of lambda.}
\item{dim}{Dimension of coefficient matrix.}
\item{lambda}{The actual sequence of lambda values used. When alpha=0, the
largest lambda reported does not quite give the zero coefficients reported
(lambda=inf would in principle). Instead, the largest lambda for alpha=0.001
is used, and the sequence of lambda values is derived from this.}
\item{dev.ratio}{The fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
for the saturated model (a model with a free parameter per observation).
Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)). The null model refers to the 0 model.}
\item{npasses}{Total passes over the data summed over all lambda values.}
\item{jerr}{Error flag, for warnings and errors (largely for internal
debugging).}
\item{offset}{A logical variable indicating whether an offset was included
in the model.}
\item{call}{The call that produced this object.}
\item{nobs}{Number of observations.}
}
\description{
Fit a Cox regression model via penalized maximum likelihood for a path of
lambda values. Can deal with (start, stop] data and strata, as well as
sparse design matrices.
}
\details{
Sometimes the sequence is truncated before \code{nlambda} values of lambda
have been used. This happens when \code{cox.path} detects that the
decrease in deviance is marginal (i.e. we are near a saturated fit).
}
\examples{
set.seed(2)
nobs <- 100; nvars <- 15
xvec <- rnorm(nobs * nvars)
xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
x <- matrix(xvec, nrow = nobs)
beta <- rnorm(nvars / 3)
fx <- x[, seq(nvars / 3)] \%*\% beta / 3
ty <- rexp(nobs, exp(fx))
tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
jsurv <- survival::Surv(ty, tcens)
fit1 <- glmnet:::cox.path(x, jsurv)

# works with sparse x matrix
x_sparse <- Matrix::Matrix(x, sparse = TRUE)
fit2 <- glmnet:::cox.path(x_sparse, jsurv)

# example with (start, stop] data
set.seed(2)
start_time <- runif(100, min = 0, max = 5)
stop_time <- start_time + runif(100, min = 0.1, max = 3)
status <- rbinom(n = nobs, prob = 0.3, size = 1)
jsurv_ss <- survival::Surv(start_time, stop_time, status)
fit3 <- glmnet:::cox.path(x, jsurv_ss)

# example with strata
jsurv_ss2 <- stratifySurv(jsurv_ss, rep(1:2, each = 50))
fit4 <- glmnet:::cox.path(x, jsurv_ss2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{pen_function}
\alias{pen_function}
\title{Elastic net penalty value}
\usage{
pen_function(coefficients, alpha = 1, vp = 1)
}
\arguments{
\item{coefficients}{The model's coefficients (excluding intercept).}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.}

\item{vp}{Penalty factors for each of the coefficients.}
}
\description{
Returns the elastic net penalty value without the \code{lambda} factor.
}
\details{
The penalty is defined as
\deqn{(1-\alpha)/2 \sum vp_j \beta_j^2 + \alpha \sum vp_j |\beta|.}
Note the omission of the multiplicative \code{lambda} factor.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{get_start}
\alias{get_start}
\title{Get null deviance, starting mu and lambda max}
\usage{
get_start(
  x,
  y,
  weights,
  family,
  intercept,
  is.offset,
  offset,
  exclude,
  vp,
  alpha
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed to be standardized.}

\item{y}{Quantitative response variable.}

\item{weights}{Observation weights.}

\item{family}{A description of the error distribution and link function to be
used in the model. This is the result of a call to a family function.
(See \code{\link[stats:family]{family}} for details on family functions.)}

\item{intercept}{Does the model we are fitting have an intercept term or not?}

\item{is.offset}{Is the model being fit with an offset or not?}

\item{offset}{Offset for the model. If \code{is.offset=FALSE}, this should be
a zero vector of the same length as \code{y}.}

\item{exclude}{Indices of variables to be excluded from the model.}

\item{vp}{Separate penalty factors can be applied to each coefficient.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.}
}
\description{
Return the null deviance, starting mu and lambda max values for
initialization. For internal use only.
}
\details{
This function is called by \code{glmnet.path} for null deviance, starting mu
and lambda max values. It is also called by \code{glmnet.fit} when used
without warmstart, but they only use the null deviance and starting mu values.

When \code{x} is not sparse, it is expected to already by centered and scaled.
When \code{x} is sparse, the function will get its attributes \code{xm} and
\code{xs} for its centering and scaling factors.

Note that whether \code{x} is centered & scaled or not, the values of \code{mu}
and \code{nulldev} don't change. However, the value of \code{lambda_max} does
change, and we need \code{xm} and \code{xs} to get the correct value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cindex.R
\name{Cindex}
\alias{Cindex}
\title{compute C index for a Cox model}
\usage{
Cindex(pred, y, weights = rep(1, nrow(y)))
}
\arguments{
\item{pred}{Predictions from a \code{"coxnet"} object}

\item{y}{a survival response object - a matrix with two columns "time" and
"status"; see documentation for "glmnet"}

\item{weights}{optional observation weights}
}
\description{
Computes Harrel's C index for predictions from a \code{"coxnet"} object.
}
\details{
Computes the concordance index, taking into account censoring.
}
\examples{

set.seed(10101)
N = 1000
p = 30
nzc = p/3
x = matrix(rnorm(N * p), N, p)
beta = rnorm(nzc)
fx = x[, seq(nzc)] \%*\% beta/3
hx = exp(fx)
ty = rexp(N, hx)
tcens = rbinom(n = N, prob = 0.3, size = 1)  # censoring indicator
y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
fit = glmnet(x, y, family = "cox")
pred = predict(fit, newx = x)
apply(pred, 2, Cindex, y=y)
cv.glmnet(x, y, family = "cox", type.measure = "C")

}
\references{
Harrel Jr, F. E. and Lee, K. L. and Mark, D. B. (1996)
\emph{Tutorial in biostatistics: multivariable prognostic models: issues in
developing models, evaluating assumptions and adequacy, and measuring and
reducing error}, Statistics in Medicine, 15, pages 361--387.
}
\seealso{
\code{cv.glmnet}
}
\author{
Trevor Hastie  \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{Cox}
\keyword{cross-validation}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{CoxExample}
\alias{CoxExample}
\title{Synthetic dataset with right-censored survival response}
\format{
List containing the following elements:
\describe{
\item{x}{1,000 by 30 matrix of numeric values.}
\item{y}{1,000 by 2 matrix with column names "time" and "status". The
first column consists of positive numbers representing time to event,
while the second column represents the status indicator
(0=right-censored, 1=observed).}
}
}
\usage{
data(CoxExample)
}
\description{
Randomly generated data for Cox regression example.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survfit.cv.glmnet.R
\name{survfit.cv.glmnet}
\alias{survfit.cv.glmnet}
\title{Compute a survival curve from a cv.glmnet object}
\usage{
\method{survfit}{cv.glmnet}(formula, s = c("lambda.1se", "lambda.min"), ...)
}
\arguments{
\item{formula}{A class \code{cv.glmnet} object. The object should have
been fit with \code{family = "cox"}.}

\item{s}{Value(s) of the penalty parameter lambda at which predictions
are required. Default is the value s="lambda.1se" stored on the CV object.
Alternatively s="lambda.min" can be used. If s is numeric, it is taken
as the value(s) of lambda to be used.}

\item{...}{Other arguments to be passed to \code{survfit.coxnet}.}
}
\value{
If \code{s} is a single value, an object of class "survfitcox"
and "survfit" containing one or more survival curves. Otherwise, a list
of such objects, one element for each value in \code{s}.
Methods defined for survfit objects are print, summary and plot.
}
\description{
Computes the predicted survivor function for a Cox proportional hazards
model with elastic net penalty from a cross-validated glmnet model.
}
\details{
This function makes it easier to use the results of cross-validation
to compute a survival curve.
}
\examples{
set.seed(2)
nobs <- 100; nvars <- 15
xvec <- rnorm(nobs * nvars)
x <- matrix(xvec, nrow = nobs)
beta <- rnorm(nvars / 3)
fx <- x[, seq(nvars / 3)] \%*\% beta / 3
ty <- rexp(nobs, exp(fx))
tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
y <- survival::Surv(ty, tcens)
cvfit <- cv.glmnet(x, y, family = "cox")
# default: s = "lambda.1se"
survival::survfit(cvfit, x = x, y = y)

# s = "lambda.min"
survival::survfit(cvfit, s = "lambda.min", x = x, y = y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeX.R
\name{makeX}
\alias{makeX}
\title{convert a data frame to a data matrix with one-hot encoding}
\usage{
makeX(train, test = NULL, na.impute = FALSE, sparse = FALSE, ...)
}
\arguments{
\item{train}{Required argument. A dataframe consisting of vectors, matrices
and factors}

\item{test}{Optional argument. A dataframe matching 'train' for use as
testing data}

\item{na.impute}{Logical, default \code{FALSE}. If \code{TRUE}, missing
values for any column in the resultant 'x' matrix are replaced by the means
of the nonmissing values derived from 'train'}

\item{sparse}{Logical, default \code{FALSE}. If \code{TRUE} then the
returned matrice(s) are converted to matrices of class "CsparseMatrix".
Useful if some factors have a large number of levels, resulting in very big
matrices, mostly zero}

\item{...}{additional arguments, currently unused}
}
\value{
If only 'train' was provided, the function returns a matrix 'x'. If
missing values were imputed, this matrix has an attribute containing its
column means (before imputation). If 'test' was provided as well, a list
with two components is returned: 'x' and 'xtest'.
}
\description{
Converts a data frame to a data matrix suitable for input to \code{glmnet}.
Factors are converted to dummy matrices via "one-hot" encoding. Options deal
with missing values and sparsity.
}
\details{
The main function is to convert factors to dummy matrices via "one-hot"
encoding. Having the 'train' and 'test' data present is useful if some
factor levels are missing in either. Since a factor with k levels leads to a
submatrix with 1/k entries zero, with large k the \code{sparse=TRUE} option
can be helpful; a large matrix will be returned, but stored in sparse matrix
format. Finally, the function can deal with missing data. The current
version has the option to replace missing observations with the mean from
the training data. For dummy submatrices, these are the mean proportions at
each level.
}
\examples{

set.seed(101)
### Single data frame
X = matrix(rnorm(20), 10, 2)
X3 = sample(letters[1:3], 10, replace = TRUE)
X4 = sample(LETTERS[1:3], 10, replace = TRUE)
df = data.frame(X, X3, X4)
makeX(df)
makeX(df, sparse = TRUE)

### Single data freame with missing values
Xn = X
Xn[3, 1] = NA
Xn[5, 2] = NA
X3n = X3
X3n[6] = NA
X4n = X4
X4n[9] = NA
dfn = data.frame(Xn, X3n, X4n)

makeX(dfn)
makeX(dfn, sparse = TRUE)
makeX(dfn, na.impute = TRUE)
makeX(dfn, na.impute = TRUE, sparse = TRUE)

### Test data as well
X = matrix(rnorm(10), 5, 2)
X3 = sample(letters[1:3], 5, replace = TRUE)
X4 = sample(LETTERS[1:3], 5, replace = TRUE)
dft = data.frame(X, X3, X4)

makeX(df, dft)
makeX(df, dft, sparse = TRUE)

### Missing data in test as well
Xn = X
Xn[3, 1] = NA
Xn[5, 2] = NA
X3n = X3
X3n[1] = NA
X4n = X4
X4n[2] = NA
dftn = data.frame(Xn, X3n, X4n)

makeX(dfn, dftn)
makeX(dfn, dftn, sparse = TRUE)
makeX(dfn, dftn, na.impute = TRUE)
makeX(dfn, dftn, sparse = TRUE, na.impute = TRUE)

}
\seealso{
\code{glmnet}
}
\author{
Trevor Hastie\cr Maintainer: Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.cv.glmnet.R, R/predict.cv.relaxed.R
\name{predict.cv.glmnet}
\alias{predict.cv.glmnet}
\alias{coef.cv.glmnet}
\alias{coef.cv.relaxed}
\alias{predict.cv.relaxed}
\title{make predictions from a "cv.glmnet" object.}
\usage{
\method{predict}{cv.glmnet}(object, newx, s = c("lambda.1se", "lambda.min"), ...)

\method{predict}{cv.relaxed}(
  object,
  newx,
  s = c("lambda.1se", "lambda.min"),
  gamma = c("gamma.1se", "gamma.min"),
  ...
)
}
\arguments{
\item{object}{Fitted \code{"cv.glmnet"} or \code{"cv.relaxed"} object.}

\item{newx}{Matrix of new values for \code{x} at which predictions are to be
made. Must be a matrix; can be sparse as in \code{Matrix} package. See
documentation for \code{predict.glmnet}.}

\item{s}{Value(s) of the penalty parameter \code{lambda} at which
predictions are required. Default is the value \code{s="lambda.1se"} stored
on the CV \code{object}. Alternatively \code{s="lambda.min"} can be used. If
\code{s} is numeric, it is taken as the value(s) of \code{lambda} to be
used. (For historical reasons we use the symbol 's' rather than 'lambda' to
reference this parameter)}

\item{\dots}{Not used. Other arguments to predict.}

\item{gamma}{Value (single) of 'gamma' at which predictions are to be made}
}
\value{
The object returned depends on the \dots{} argument which is passed
on to the \code{predict} method for \code{glmnet} objects.
}
\description{
This function makes predictions from a cross-validated glmnet model, using
the stored \code{"glmnet.fit"} object, and the optimal value chosen for
\code{lambda} (and \code{gamma} for a 'relaxed' fit.
}
\details{
This function makes it easier to use the results of cross-validation to make
a prediction.
}
\examples{

x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
cv.fit = cv.glmnet(x, y)
predict(cv.fit, newx = x[1:5, ])
coef(cv.fit)
coef(cv.fit, s = "lambda.min")
predict(cv.fit, newx = x[1:5, ], s = c(0.001, 0.002))
cv.fitr = cv.glmnet(x, y, relax = TRUE)
predict(cv.fit, newx = x[1:5, ])
coef(cv.fit)
coef(cv.fit, s = "lambda.min", gamma = "gamma.min")
predict(cv.fit, newx = x[1:5, ], s = c(0.001, 0.002), gamma = "gamma.min")

}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent, Journal of Statistical Software, Vol. 33, Issue 1, Feb 2010}\cr
\url{https://www.jstatsoft.org/v33/i01/}
\url{https://arxiv.org/abs/1707.08692}\cr Hastie, T., Tibshirani, Robert,
Tibshirani, Ryan (2019) \emph{Extended Comparisons of Best Subset Selection,
Forward Stepwise Selection, and the Lasso}
}
\seealso{
\code{glmnet}, and \code{print}, and \code{coef} methods, and
\code{cv.glmnet}.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.glmnet.R, R/predict.glmnet.R,
%   R/predict.relaxed.R
\name{coef.glmnet}
\alias{coef.glmnet}
\alias{predict.glmnet}
\alias{coef.relaxed}
\alias{predict.relaxed}
\alias{predict.elnet}
\alias{predict.lognet}
\alias{predict.multnet}
\alias{predict.mrelnet}
\alias{predict.fishnet}
\alias{predict.coxnet}
\title{Extract coefficients from a glmnet object}
\usage{
\method{coef}{glmnet}(object, s = NULL, exact = FALSE, ...)

\method{predict}{glmnet}(
  object,
  newx,
  s = NULL,
  type = c("link", "response", "coefficients", "nonzero", "class"),
  exact = FALSE,
  newoffset,
  ...
)

\method{predict}{relaxed}(
  object,
  newx,
  s = NULL,
  gamma = 1,
  type = c("link", "response", "coefficients", "nonzero", "class"),
  exact = FALSE,
  newoffset,
  ...
)
}
\arguments{
\item{object}{Fitted \code{"glmnet"} model object or a \code{"relaxed"}
model (which inherits from class "glmnet").}

\item{s}{Value(s) of the penalty parameter \code{lambda} at which
predictions are required. Default is the entire sequence used to create the
model.}

\item{exact}{This argument is relevant only when predictions are made at
values of \code{s} (lambda) \emph{different} from those used in the fitting
of the original model. Not available for \code{"relaxed"} objects. If
\code{exact=FALSE} (default), then the predict function uses linear
interpolation to make predictions for values of \code{s} (lambda) that do
not coincide with those used in the fitting algorithm. While this is often a
good approximation, it can sometimes be a bit coarse.  With
\code{exact=TRUE}, these different values of \code{s} are merged (and
sorted) with \code{object$lambda}, and the model is refit before predictions
are made. In this case, it is required to supply the original data \code{x=}
and \code{y=} as additional named arguments to \code{predict()} or
\code{coef()}.  The workhorse \code{predict.glmnet()} needs to \code{update}
the model, and so needs the data used to create it. The same is true of
\code{weights}, \code{offset}, \code{penalty.factor}, \code{lower.limits},
\code{upper.limits} if these were used in the original call. Failure to do
so will result in an error.}

\item{\dots}{This is the mechanism for passing arguments like \code{x=} when
\code{exact=TRUE}; see\code{exact} argument.}

\item{newx}{Matrix of new values for \code{x} at which predictions are to be
made. Must be a matrix; can be sparse as in \code{Matrix} package. This
argument is not used for \code{type=c("coefficients","nonzero")}}

\item{type}{Type of prediction required. Type \code{"link"} gives the linear
predictors for \code{"binomial"}, \code{"multinomial"}, \code{"poisson"} or
\code{"cox"} models; for \code{"gaussian"} models it gives the fitted
values. Type \code{"response"} gives the fitted probabilities for
\code{"binomial"} or \code{"multinomial"}, fitted mean for \code{"poisson"}
and the fitted relative-risk for \code{"cox"}; for \code{"gaussian"} type
\code{"response"} is equivalent to type \code{"link"}. Type
\code{"coefficients"} computes the coefficients at the requested values for
\code{s}.  Note that for \code{"binomial"} models, results are returned only
for the class corresponding to the second level of the factor response.
Type \code{"class"} applies only to \code{"binomial"} or
\code{"multinomial"} models, and produces the class label corresponding to
the maximum probability. Type \code{"nonzero"} returns a list of the indices
of the nonzero coefficients for each value of \code{s}.}

\item{newoffset}{If an offset is used in the fit, then one must be supplied
for making predictions (except for \code{type="coefficients"} or
\code{type="nonzero"})}

\item{gamma}{Single value of \code{gamma} at which predictions are required,
for "relaxed" objects.}
}
\value{
The object returned depends on type.
}
\description{
Similar to other predict methods, this functions predicts fitted values,
logits, coefficients and more from a fitted \code{"glmnet"} object.
}
\details{
The shape of the objects returned are different for \code{"multinomial"}
objects. This function actually calls \code{NextMethod()}, and the
appropriate predict method is invoked for each of the three model types.
\code{coef(...)} is equivalent to \code{predict(type="coefficients",...)}
}
\examples{
x=matrix(rnorm(100*20),100,20)
y=rnorm(100)
g2=sample(1:2,100,replace=TRUE)
g4=sample(1:4,100,replace=TRUE)
fit1=glmnet(x,y)
predict(fit1,newx=x[1:5,],s=c(0.01,0.005))
predict(fit1,type="coef")
fit2=glmnet(x,g2,family="binomial")
predict(fit2,type="response",newx=x[2:5,])
predict(fit2,type="nonzero")
fit3=glmnet(x,g4,family="multinomial")
predict(fit3,newx=x[1:3,],type="response",s=0.01)
}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent (2010), Journal of Statistical Software, Vol. 33(1), 1-22},
\doi{10.18637/jss.v033.i01}.\cr
Simon, N., Friedman, J., Hastie, T. and Tibshirani, R. (2011)
\emph{Regularization Paths for Cox's Proportional
Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
39(5), 1-13},
\doi{10.18637/jss.v039.i05}.\cr
Glmnet webpage with four vignettes, \url{https://glmnet.stanford.edu}.
}
\seealso{
\code{glmnet}, and \code{print}, and \code{coef} methods, and
\code{cv.glmnet}.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet.control.R
\name{glmnet.control}
\alias{glmnet.control}
\title{internal glmnet parameters}
\usage{
glmnet.control(
  fdev = 1e-05,
  devmax = 0.999,
  eps = 1e-06,
  big = 9.9e+35,
  mnlam = 5,
  pmin = 1e-09,
  exmx = 250,
  prec = 1e-10,
  mxit = 100,
  itrace = 0,
  epsnr = 1e-06,
  mxitnr = 25,
  factory = FALSE
)
}
\arguments{
\item{fdev}{minimum fractional change in deviance for stopping path; factory
default = 1.0e-5}

\item{devmax}{maximum fraction of explained deviance for stopping path;
factory default = 0.999}

\item{eps}{minimum value of lambda.min.ratio (see glmnet); factory default=
1.0e-6}

\item{big}{large floating point number; factory default = 9.9e35. Inf in
definition of upper.limit is set to big}

\item{mnlam}{minimum number of path points (lambda values) allowed; factory
default = 5}

\item{pmin}{minimum probability for any class. factory default = 1.0e-9.
Note that this implies a pmax of 1-pmin.}

\item{exmx}{maximum allowed exponent. factory default = 250.0}

\item{prec}{convergence threshold for multi response bounds adjustment
solution. factory default = 1.0e-10}

\item{mxit}{maximum iterations for multiresponse bounds adjustment solution.
factory default = 100}

\item{itrace}{If 1 then progress bar is displayed when running \code{glmnet}
and \code{cv.glmnet}. factory default = 0}

\item{epsnr}{convergence threshold for \code{glmnet.fit}. factory default =
1.0e-6}

\item{mxitnr}{maximum iterations for the IRLS loop in \code{glmnet.fit}. factory
default = 25}

\item{factory}{If \code{TRUE}, reset all the parameters to the factory
default; default is \code{FALSE}}
}
\value{
A list with named elements as in the argument list
}
\description{
View and/or change the factory default parameters in glmnet
}
\details{
If called with no arguments, \code{glmnet.control()} returns a list with the
current settings of these parameters. Any arguments included in the call
sets those parameters to the new values, and then silently returns. The
values set are persistent for the duration of the R session.
}
\examples{

glmnet.control(fdev = 0)  #continue along path even though not much changes
glmnet.control()  # view current settings
glmnet.control(factory = TRUE)  # reset all the parameters to their default

}
\seealso{
\code{glmnet}
}
\author{
Jerome Friedman, Kenneth Tay, Trevor Hastie\cr Maintainer: Trevor Hastie
\email{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxpath.R
\name{get_cox_lambda_max}
\alias{get_cox_lambda_max}
\title{Get lambda max for Cox regression model}
\usage{
get_cox_lambda_max(
  x,
  y,
  alpha,
  weights = rep(1, nrow(x)),
  offset = rep(0, nrow(x)),
  exclude = c(),
  vp = rep(1, ncol(x))
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed to be standardized.}

\item{y}{Survival response variable, must be a \code{Surv} or
\code{stratifySurv} object.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.}

\item{weights}{Observation weights.}

\item{offset}{Offset for the model. Default is a zero vector of length
\code{nrow(y)}.}

\item{exclude}{Indices of variables to be excluded from the model.}

\item{vp}{Separate penalty factors can be applied to each coefficient.}
}
\description{
Return the lambda max value for Cox regression model, used for computing
initial lambda values. For internal use only.
}
\details{
This function is called by \code{cox.path} for the value of lambda max.

When \code{x} is not sparse, it is expected to already by centered and scaled.
When \code{x} is sparse, the function will get its attributes \code{xm} and
\code{xs} for its centering and scaling factors. The value of
\code{lambda_max} changes depending on whether \code{x} is centered and
scaled or not, so we need \code{xm} and \code{xs} to get the correct value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet-package.R
\name{beta_CVX}
\alias{beta_CVX}
\alias{x}
\alias{y}
\title{Simulated data for the glmnet vignette}
\format{
Data objects used to demonstrate features in the glmnet vignette
}
\description{
Simple simulated data, used to demonstrate the features of glmnet
}
\details{
These datasets are artificial, and are used to test out some of the
features of glmnet.
}
\examples{

data(QuickStartExample)
glmnet(x, y)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.glmnet.R
\name{print.glmnet}
\alias{print.glmnet}
\alias{print.relaxed}
\alias{print.bigGlm}
\title{print a glmnet object}
\usage{
\method{print}{glmnet}(x, digits = max(3, getOption("digits") - 3), ...)
}
\arguments{
\item{x}{fitted glmnet object}

\item{digits}{significant digits in printout}

\item{\dots}{additional print arguments}
}
\value{
The matrix above is silently returned
}
\description{
Print a summary of the glmnet path at each step along the path.
}
\details{
The call that produced the object \code{x} is printed, followed by a
three-column matrix with columns \code{Df}, \verb{\%Dev} and \code{Lambda}.
The \code{Df} column is the number of nonzero coefficients (Df is a
reasonable name only for lasso fits). \verb{\%Dev} is the percent deviance
explained (relative to the null deviance).  In the case of a 'relaxed' fit,
an additional column is inserted, \verb{\%Dev R} which gives the percent
deviance explained by the relaxed model. For a "bigGlm" model, a simpler
summary is printed.
}
\examples{

x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
fit1 = glmnet(x, y)
print(fit1)
}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008). Regularization Paths for Generalized Linear Models via Coordinate Descent
}
\seealso{
\code{glmnet}, \code{predict} and \code{coef} methods.
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxpath.R
\name{use.cox.path}
\alias{use.cox.path}
\title{Check if glmnet should call cox.path}
\usage{
use.cox.path(x, y)
}
\arguments{
\item{x}{Design matrix.}

\item{y}{Response variable.}
}
\value{
TRUE if cox.path() should be called, FALSE otherwise.
}
\description{
Helper function to check if glmnet() should call cox.path().
}
\details{
For \code{family="cox"}, we only call the original coxnet() function if
(i) x is not sparse, (ii) y is right-censored data, and (iii) we are
not fitting a stratified Cox model. This function also throws an error
if y has a "strata" attribute but is not of type "stratifySurv".
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{get_eta}
\alias{get_eta}
\title{Helper function to get etas (linear predictions)}
\usage{
get_eta(x, beta, a0)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed to be standardized.}

\item{beta}{Feature coefficients.}

\item{a0}{Intercept.}
}
\description{
Given x, coefficients and intercept, return linear predictions. Wrapper that
works with both regular and sparse x. Only works for single set of coefficients
and intercept.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv.glmnet.R
\name{cv.glmnet}
\alias{cv.glmnet}
\title{Cross-validation for glmnet}
\usage{
cv.glmnet(
  x,
  y,
  weights = NULL,
  offset = NULL,
  lambda = NULL,
  type.measure = c("default", "mse", "deviance", "class", "auc", "mae", "C"),
  nfolds = 10,
  foldid = NULL,
  alignment = c("lambda", "fraction"),
  grouped = TRUE,
  keep = FALSE,
  parallel = FALSE,
  gamma = c(0, 0.25, 0.5, 0.75, 1),
  relax = FALSE,
  trace.it = 0,
  ...
)
}
\arguments{
\item{x}{\code{x} matrix as in \code{glmnet}.}

\item{y}{response \code{y} as in \code{glmnet}.}

\item{weights}{Observation weights; defaults to 1 per observation}

\item{offset}{Offset vector (matrix) as in \code{glmnet}}

\item{lambda}{Optional user-supplied lambda sequence; default is
\code{NULL}, and \code{glmnet} chooses its own sequence}

\item{type.measure}{loss to use for cross-validation. Currently five
options, not all available for all models. The default is
\code{type.measure="deviance"}, which uses squared-error for gaussian models
(a.k.a \code{type.measure="mse"} there), deviance for logistic and poisson
regression, and partial-likelihood for the Cox model.
\code{type.measure="class"} applies to binomial and multinomial logistic
regression only, and gives misclassification error.
\code{type.measure="auc"} is for two-class logistic regression only, and
gives area under the ROC curve. \code{type.measure="mse"} or
\code{type.measure="mae"} (mean absolute error) can be used by all models
except the \code{"cox"}; they measure the deviation from the fitted mean to
the response.
\code{type.measure="C"} is Harrel's concordance measure, only available for \code{cox} models.}

\item{nfolds}{number of folds - default is 10. Although \code{nfolds} can be
as large as the sample size (leave-one-out CV), it is not recommended for
large datasets. Smallest value allowable is \code{nfolds=3}}

\item{foldid}{an optional vector of values between 1 and \code{nfold}
identifying what fold each observation is in. If supplied, \code{nfold} can
be missing.}

\item{alignment}{This is an experimental argument, designed to fix the
problems users were having with CV, with possible values \code{"lambda"}
(the default) else \code{"fraction"}. With \code{"lambda"} the \code{lambda}
values from the master fit (on all the data) are used to line up the
predictions from each of the folds. In some cases this can give strange
values, since the effective \code{lambda} values in each fold could be quite
different. With \code{"fraction"} we line up the predictions in each fold
according to the fraction of progress along the regularization. If in the
call a \code{lambda} argument is also provided, \code{alignment="fraction"}
is ignored (with a warning).}

\item{grouped}{This is an experimental argument, with default \code{TRUE},
and can be ignored by most users. For all models except the \code{"cox"},
this refers to computing \code{nfolds} separate statistics, and then using
their mean and estimated standard error to describe the CV curve. If
\code{grouped=FALSE}, an error matrix is built up at the observation level
from the predictions from the \code{nfold} fits, and then summarized (does
not apply to \code{type.measure="auc"}). For the \code{"cox"} family,
\code{grouped=TRUE} obtains the CV partial likelihood for the Kth fold by
\emph{subtraction}; by subtracting the log partial likelihood evaluated on
the full dataset from that evaluated on the on the (K-1)/K dataset. This
makes more efficient use of risk sets. With \code{grouped=FALSE} the log
partial likelihood is computed only on the Kth fold}

\item{keep}{If \code{keep=TRUE}, a \emph{prevalidated} array is returned
containing fitted values for each observation and each value of
\code{lambda}. This means these fits are computed with this observation and
the rest of its fold omitted. The \code{folid} vector is also returned.
Default is keep=FALSE.  If \code{relax=TRUE}, then a list of such arrays is
returned, one for each value of 'gamma'. Note: if the value 'gamma=1' is
omitted, this case is included in the list since it corresponds to the
original 'glmnet' fit.}

\item{parallel}{If \code{TRUE}, use parallel \code{foreach} to fit each
fold.  Must register parallel before hand, such as \code{doMC} or others.
See the example below.}

\item{gamma}{The values of the parameter for mixing the relaxed fit with the
regularized fit, between 0 and 1; default is \code{gamma = c(0, 0.25, 0.5,
0.75, 1)}}

\item{relax}{If \code{TRUE}, then CV is done with respect to the mixing
parameter \code{gamma} as well as \code{lambda}. Default is
\code{relax=FALSE}}

\item{trace.it}{If \code{trace.it=1}, then progress bars are displayed;
useful for big models that take a long time to fit. Limited tracing if
\code{parallel=TRUE}}

\item{\dots}{Other arguments that can be passed to \code{glmnet}}
}
\value{
an object of class \code{"cv.glmnet"} is returned, which is a list
with the ingredients of the cross-validation fit.  If the object was created
with \code{relax=TRUE} then this class has a prefix class of
\code{"cv.relaxed"}.  \item{lambda}{the values of \code{lambda} used in the
fits.} \item{cvm}{The mean cross-validated error - a vector of length
\code{length(lambda)}.} \item{cvsd}{estimate of standard error of
\code{cvm}.} \item{cvup}{upper curve = \code{cvm+cvsd}.} \item{cvlo}{lower
curve = \code{cvm-cvsd}.} \item{nzero}{number of non-zero coefficients at
each \code{lambda}.} \item{name}{a text string indicating type of measure
(for plotting purposes).} \item{glmnet.fit}{a fitted glmnet object for the
full data.} \item{lambda.min}{value of \code{lambda} that gives minimum
\code{cvm}.} \item{lambda.1se}{largest value of \code{lambda} such that
error is within 1 standard error of the minimum.} \item{fit.preval}{if
\code{keep=TRUE}, this is the array of prevalidated fits. Some entries can
be \code{NA}, if that and subsequent values of \code{lambda} are not reached
for that fold} \item{foldid}{if \code{keep=TRUE}, the fold assignments used}
\item{index}{a one column matrix with the indices of \code{lambda.min} and \code{lambda.1se} in the sequence of coefficients, fits etc.}
\item{relaxed}{if \code{relax=TRUE}, this additional item has the CV info
for each of the mixed fits. In particular it also selects \code{lambda,
gamma} pairs corresponding to the 1se rule, as well as the minimum error. It also has a component \code{index}, a two-column matrix  which contains the \code{lambda} and \code{gamma} indices corresponding to the "min" and "1se" solutions.}
}
\description{
Does k-fold cross-validation for glmnet, produces a plot, and returns a
value for \code{lambda} (and \code{gamma} if \code{relax=TRUE})
}
\details{
The function runs \code{glmnet} \code{nfolds}+1 times; the first to get the
\code{lambda} sequence, and then the remainder to compute the fit with each
of the folds omitted. The error is accumulated, and the average error and
standard deviation over the folds is computed.  Note that \code{cv.glmnet}
does NOT search for values for \code{alpha}. A specific value should be
supplied, else \code{alpha=1} is assumed by default. If users would like to
cross-validate \code{alpha} as well, they should call \code{cv.glmnet} with
a pre-computed vector \code{foldid}, and then use this same fold vector in
separate calls to \code{cv.glmnet} with different values of \code{alpha}.
Note also that the results of \code{cv.glmnet} are random, since the folds
are selected at random. Users can reduce this randomness by running
\code{cv.glmnet} many times, and averaging the error curves.

If \code{relax=TRUE} then the values of \code{gamma} are used to mix the
fits. If \eqn{\eta} is the fit for lasso/elastic net, and \eqn{\eta_R} is
the relaxed fit (with unpenalized coefficients), then a relaxed fit mixed by
\eqn{\gamma} is \deqn{\eta(\gamma)=(1-\gamma)\eta_R+\gamma\eta.} There is
practically no extra cost for having a lot of values for \code{gamma}.
However, 5 seems sufficient for most purposes. CV then selects both
\code{gamma} and \code{lambda}.
}
\examples{

set.seed(1010)
n = 1000
p = 100
nzc = trunc(p/10)
x = matrix(rnorm(n * p), n, p)
beta = rnorm(nzc)
fx = x[, seq(nzc)] \%*\% beta
eps = rnorm(n) * 5
y = drop(fx + eps)
px = exp(fx)
px = px/(1 + px)
ly = rbinom(n = length(px), prob = px, size = 1)
set.seed(1011)
cvob1 = cv.glmnet(x, y)
plot(cvob1)
coef(cvob1)
predict(cvob1, newx = x[1:5, ], s = "lambda.min")
title("Gaussian Family", line = 2.5)
set.seed(1011)
cvob1a = cv.glmnet(x, y, type.measure = "mae")
plot(cvob1a)
title("Gaussian Family", line = 2.5)
set.seed(1011)
par(mfrow = c(2, 2), mar = c(4.5, 4.5, 4, 1))
cvob2 = cv.glmnet(x, ly, family = "binomial")
plot(cvob2)
title("Binomial Family", line = 2.5)
frame()
set.seed(1011)
cvob3 = cv.glmnet(x, ly, family = "binomial", type.measure = "class")
plot(cvob3)
title("Binomial Family", line = 2.5)
\dontrun{
cvob1r = cv.glmnet(x, y, relax = TRUE)
plot(cvob1r)
predict(cvob1r, newx = x[, 1:5])
set.seed(1011)
cvob3a = cv.glmnet(x, ly, family = "binomial", type.measure = "auc")
plot(cvob3a)
title("Binomial Family", line = 2.5)
set.seed(1011)
mu = exp(fx/10)
y = rpois(n, mu)
cvob4 = cv.glmnet(x, y, family = "poisson")
plot(cvob4)
title("Poisson Family", line = 2.5)

# Multinomial
n = 500
p = 30
nzc = trunc(p/10)
x = matrix(rnorm(n * p), n, p)
beta3 = matrix(rnorm(30), 10, 3)
beta3 = rbind(beta3, matrix(0, p - 10, 3))
f3 = x \%*\% beta3
p3 = exp(f3)
p3 = p3/apply(p3, 1, sum)
g3 = glmnet:::rmult(p3)
set.seed(10101)
cvfit = cv.glmnet(x, g3, family = "multinomial")
plot(cvfit)
title("Multinomial Family", line = 2.5)
# Cox
beta = rnorm(nzc)
fx = x[, seq(nzc)] \%*\% beta/3
hx = exp(fx)
ty = rexp(n, hx)
tcens = rbinom(n = n, prob = 0.3, size = 1)  # censoring indicator
y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
foldid = sample(rep(seq(10), length = n))
fit1_cv = cv.glmnet(x, y, family = "cox", foldid = foldid)
plot(fit1_cv)
title("Cox Family", line = 2.5)
# Parallel
require(doMC)
registerDoMC(cores = 4)
x = matrix(rnorm(1e+05 * 100), 1e+05, 100)
y = rnorm(1e+05)
system.time(cv.glmnet(x, y))
system.time(cv.glmnet(x, y, parallel = TRUE))
}

}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}, \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}\cr
\emph{Journal of Statistical Software, Vol. 33(1), 1-22 Feb 2010}\cr
\url{https://www.jstatsoft.org/v33/i01/}\cr Simon, N., Friedman, J., Hastie,
T., Tibshirani, R. (2011) \emph{Regularization Paths for Cox's Proportional
Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
39(5) 1-13}\cr \url{https://www.jstatsoft.org/v39/i05/}
}
\seealso{
\code{glmnet} and \code{plot}, \code{predict}, and \code{coef}
methods for \code{"cv.glmnet"} and \code{"cv.relaxed"} objects.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Noah Simon
helped develop the 'coxnet' function.\cr Jeffrey Wong and B. Narasimhan
helped with the parallel option\cr Maintainer: Trevor Hastie
\email{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survfit.coxnet.R
\name{survfit.coxnet}
\alias{survfit.coxnet}
\title{Compute a survival curve from a coxnet object}
\usage{
\method{survfit}{coxnet}(formula, s = NULL, ...)
}
\arguments{
\item{formula}{A class \code{coxnet} object.}

\item{s}{Value(s) of the penalty parameter lambda at which the survival
curve is required. Default is the entire sequence used to create the model.
However, it is recommended that \code{survfit.coxnet} is called for
a single penalty parameter.}

\item{...}{This is the mechanism for passing additional arguments like
(i) x= and y= for the x and y used to fit the model,
(ii) weights= and offset= when the model was fit with these options,
(iii) arguments for new data (newx, newoffset, newstrata), and
(iv) arguments to be passed to survfit.coxph().}
}
\value{
If \code{s} is a single value, an object of class "survfitcox"
and "survfit" containing one or more survival curves. Otherwise, a list
of such objects, one element for each value in \code{s}.
Methods defined for survfit objects are print, summary and plot.
}
\description{
Computes the predicted survivor function for a Cox proportional hazards
model with elastic net penalty.
}
\details{
To be consistent with other functions in \code{glmnet}, if \code{s}
is not specified, survival curves are returned for the entire lambda
sequence. This is not recommended usage: it is best to call
\code{survfit.coxnet} with a single value of the penalty parameter
for the \code{s} option.
}
\examples{
set.seed(2)
nobs <- 100; nvars <- 15
xvec <- rnorm(nobs * nvars)
xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
x <- matrix(xvec, nrow = nobs)
beta <- rnorm(nvars / 3)
fx <- x[, seq(nvars / 3)] \%*\% beta / 3
ty <- rexp(nobs, exp(fx))
tcens <- rbinom(n = nobs, prob = 0.3, size = 1)
y <- survival::Surv(ty, tcens)
fit1 <- glmnet(x, y, family = "cox")

# survfit object for Cox model where lambda = 0.1
sf1 <- survival::survfit(fit1, s = 0.1, x = x, y = y)
plot(sf1)

# example with new data
sf2 <- survival::survfit(fit1, s = 0.1, x = x, y = y, newx = x[1:3, ])
plot(sf2)

# example with strata
y2 <- stratifySurv(y, rep(1:2, length.out = nobs))
fit2 <- glmnet(x, y2, family = "cox")
sf3 <- survival::survfit(fit2, s = 0.1, x = x, y = y2)
sf4 <- survival::survfit(fit2, s = 0.1, x = x, y = y2,
               newx = x[1:3, ], newstrata = c(1, 1, 1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{PoissonExample}
\alias{PoissonExample}
\title{Synthetic dataset with count response}
\format{
List containing the following elements:
\describe{
\item{x}{500 by 20 matrix of numeric values.}
\item{y}{Numeric vector of length 500 consisting of non-negative
integers.}
}
}
\usage{
data(PoissonExample)
}
\description{
Randomly generated data for Poisson regression example.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{glmnet.path}
\alias{glmnet.path}
\title{Fit a GLM with elastic net regularization for a path of lambda values}
\usage{
glmnet.path(
  x,
  y,
  weights = NULL,
  lambda = NULL,
  nlambda = 100,
  lambda.min.ratio = ifelse(nobs < nvars, 0.01, 1e-04),
  alpha = 1,
  offset = NULL,
  family = gaussian(),
  standardize = TRUE,
  intercept = TRUE,
  thresh = 1e-10,
  maxit = 1e+05,
  penalty.factor = rep(1, nvars),
  exclude = integer(0),
  lower.limits = -Inf,
  upper.limits = Inf,
  trace.it = 0
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. Can be a sparse matrix.}

\item{y}{Quantitative response variable.}

\item{weights}{Observation weights. Default is 1 for each observation.}

\item{lambda}{A user supplied lambda sequence. Typical usage is to have the
program compute its own lambda sequence based on \code{nlambda} and
\code{lambda.min.ratio}. Supplying a value of lambda overrides this.}

\item{nlambda}{The number of lambda values, default is 100.}

\item{lambda.min.ratio}{Smallest value for lambda as a fraction of lambda.max,
the (data derived) entry value (i.e. the smallest value for which all
coefficients are zero). The default depends on the sample size \code{nobs}
relative to the number of variables \code{nvars}. If \code{nobs >= nvars}, the
default is 0.0001, close to zero. If \code{nobs < nvars}, the default is 0.01.
A very small value of \code{lambda.min.ratio} will lead to a saturated fit
in the \code{nobs < nvars} case. This is undefined for some families of
models, and the function will exit gracefully when the percentage deviance
explained is almost 1.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
\code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.}

\item{offset}{A vector of length \code{nobs} that is included in the linear
predictor. Useful for the "poisson" family (e.g. log of exposure time), or
for refining a model by starting at a current fit. Default is NULL. If
supplied, then values must also be supplied to the \code{predict} function.}

\item{family}{A description of the error distribution and link function to be
used in the model. This is the result of a call to a family function. Default
is \code{gaussian()}. (See \code{\link[stats:family]{family}} for details on
family functions.)}

\item{standardize}{Logical flag for x variable standardization, prior to
fitting the model sequence. The coefficients are always returned on the
original scale. Default is \code{standardize=TRUE}. If variables are in the
same units already, you might not wish to standardize.}

\item{intercept}{Should intercept be fitted (default=TRUE) or set to zero (FALSE)?}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than thresh times the null deviance.
Default value is \code{1e-10}.}

\item{maxit}{Maximum number of passes over the data; default is \code{10^5}.}

\item{penalty.factor}{Separate penalty factors can be applied to each
coefficient. This is a number that multiplies \code{lambda} to allow differential
shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
variable is always included in the model. Default is 1 for all variables (and
implicitly infinity for variables listed in exclude). Note: the penalty
factors are internally rescaled to sum to \code{nvars}.}

\item{exclude}{Indices of variables to be excluded from the model. Default is
none. Equivalent to an infinite penalty factor.}

\item{lower.limits}{Vector of lower limits for each coefficient; default
\code{-Inf}. Each of these must be non-positive. Can be presented as a single
value (which will then be replicated), else a vector of length \code{nvars}.}

\item{upper.limits}{Vector of upper limits for each coefficient; default
\code{Inf}. See \code{lower.limits}.}

\item{trace.it}{Controls how much information is printed to screen. Default is
\code{trace.it=0} (no information printed). If \code{trace.it=1}, a progress
bar is displayed. If \code{trace.it=2}, some information about the fitting
procedure is printed to the console as the model is being fitted.}
}
\value{
An object with class "glmnetfit" and "glmnet".
\item{a0}{Intercept sequence of length \code{length(lambda)}.}
\item{beta}{A \code{nvars x length(lambda)} matrix of coefficients, stored in
sparse matrix format.}
\item{df}{The number of nonzero coefficients for each value of lambda.}
\item{dim}{Dimension of coefficient matrix.}
\item{lambda}{The actual sequence of lambda values used. When alpha=0, the
largest lambda reported does not quite give the zero coefficients reported
(lambda=inf would in principle). Instead, the largest lambda for alpha=0.001
is used, and the sequence of lambda values is derived from this.}
\item{dev.ratio}{The fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
for the saturated model (a model with a free parameter per observation).
Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
\item{npasses}{Total passes over the data summed over all lambda values.}
\item{jerr}{Error flag, for warnings and errors (largely for internal
debugging).}
\item{offset}{A logical variable indicating whether an offset was included
in the model.}
\item{call}{The call that produced this object.}
\item{family}{Family used for the model.}
\item{nobs}{Number of observations.}
}
\description{
Fit a generalized linear model via penalized maximum likelihood for a path of
lambda values. Can deal with any GLM family.
}
\details{
\code{glmnet.path} solves the elastic net problem for a path of lambda values.
It generalizes \code{glmnet::glmnet} in that it works for any GLM family.

Sometimes the sequence is truncated before \code{nlambda} values of lambda
have been used. This happens when \code{glmnet.path} detects that the decrease
in deviance is marginal (i.e. we are near a saturated fit).
}
\examples{
set.seed(1)
x <- matrix(rnorm(100 * 20), nrow = 100)
y <- ifelse(rnorm(100) > 0, 1, 0)

# binomial with probit link
fit1 <- glmnet:::glmnet.path(x, y, family = binomial(link = "probit"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survfit.coxnet.R
\name{mycoxpred}
\alias{mycoxpred}
\title{Helper function to amend ... for new data in survfit.coxnet}
\usage{
mycoxpred(object, s, ...)
}
\arguments{
\item{object}{A class \code{coxnet} object.}

\item{s}{The response for the fitted model.}

\item{...}{The same ... that was passed to survfit.coxnet.}
}
\description{
This function amends the function arguments passed to survfit.coxnet
via ... if new data was passed to survfit.coxnet. It's a separate
function as we have to deal with function options like newstrata
and newoffset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{obj_function}
\alias{obj_function}
\title{Elastic net objective function value}
\usage{
obj_function(y, mu, weights, family, lambda, alpha, coefficients, vp)
}
\arguments{
\item{y}{Quantitative response variable.}

\item{mu}{Model's predictions for \code{y}.}

\item{weights}{Observation weights.}

\item{family}{A description of the error distribution and link function to be
used in the model. This is the result of a call to a family function.}

\item{lambda}{A single value for the \code{lambda} hyperparameter.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.}

\item{coefficients}{The model's coefficients (excluding intercept).}

\item{vp}{Penalty factors for each of the coefficients.}
}
\description{
Returns the elastic net objective function value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmult.R
\name{rmult}
\alias{rmult}
\title{Generate multinomial samples from a probability matrix}
\usage{
rmult(p)
}
\arguments{
\item{p}{matrix of probabilities, with number of columns the number of classes}
}
\value{
a vector of class memberships
}
\description{
Generate multinomial samples
}
\details{
Simple function that calls the \code{rmultinom} function. It generates a class label
for each row of its input matrix of class probabilities.
}
\author{
Trevor Hastie \cr Maintainer: Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{glmnet.fit}
\alias{glmnet.fit}
\title{Fit a GLM with elastic net regularization for a single value of lambda}
\usage{
glmnet.fit(
  x,
  y,
  weights,
  lambda,
  alpha = 1,
  offset = rep(0, nobs),
  family = gaussian(),
  intercept = TRUE,
  thresh = 1e-10,
  maxit = 1e+05,
  penalty.factor = rep(1, nvars),
  exclude = c(),
  lower.limits = -Inf,
  upper.limits = Inf,
  warm = NULL,
  from.glmnet.path = FALSE,
  save.fit = FALSE,
  trace.it = 0
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed that any standardization needed
has already been done.}

\item{y}{Quantitative response variable.}

\item{weights}{Observation weights. \code{glmnet.fit} does NOT standardize
these weights.}

\item{lambda}{A single value for the \code{lambda} hyperparameter.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
\code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.}

\item{offset}{A vector of length \code{nobs} that is included in the linear
predictor. Useful for the "poisson" family (e.g. log of exposure time), or
for refining a model by starting at a current fit. Default is NULL. If
supplied, then values must also be supplied to the \code{predict} function.}

\item{family}{A description of the error distribution and link function to be
used in the model. This is the result of a call to a family function. Default
is \code{gaussian()}. (See \code{\link[stats:family]{family}} for details on
family functions.)}

\item{intercept}{Should intercept be fitted (default=TRUE) or set to zero (FALSE)?}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than thresh times the null deviance.
Default value is \code{1e-10}.}

\item{maxit}{Maximum number of passes over the data; default is \code{10^5}.
(If a warm start object is provided, the number of passes the warm start object
performed is included.)}

\item{penalty.factor}{Separate penalty factors can be applied to each
coefficient. This is a number that multiplies \code{lambda} to allow differential
shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
variable is always included in the model. Default is 1 for all variables (and
implicitly infinity for variables listed in exclude). Note: the penalty
factors are internally rescaled to sum to \code{nvars}.}

\item{exclude}{Indices of variables to be excluded from the model. Default is
none. Equivalent to an infinite penalty factor.}

\item{lower.limits}{Vector of lower limits for each coefficient; default
\code{-Inf}. Each of these must be non-positive. Can be presented as a single
value (which will then be replicated), else a vector of length \code{nvars}.}

\item{upper.limits}{Vector of upper limits for each coefficient; default
\code{Inf}. See \code{lower.limits}.}

\item{warm}{Either a \code{glmnetfit} object or a list (with names \code{beta}
and \code{a0} containing coefficients and intercept respectively) which can
be used as a warm start. Default is \code{NULL}, indicating no warm start.
For internal use only.}

\item{from.glmnet.path}{Was \code{glmnet.fit()} called from \code{glmnet.path()}?
Default is FALSE.This has implications for computation of the penalty factors.}

\item{save.fit}{Return the warm start object? Default is FALSE.}

\item{trace.it}{Controls how much information is printed to screen. If
\code{trace.it=2}, some information about the fitting procedure is printed to
the console as the model is being fitted. Default is \code{trace.it=0}
(no information printed). (\code{trace.it=1} not used for compatibility with
\code{glmnet.path}.)}
}
\value{
An object with class "glmnetfit" and "glmnet". The list
returned contains more keys than that of a "glmnet" object.
\item{a0}{Intercept value.}
\item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
format.}
\item{df}{The number of nonzero coefficients.}
\item{dim}{Dimension of coefficient matrix.}
\item{lambda}{Lambda value used.}
\item{dev.ratio}{The fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
for the saturated model (a model with a free parameter per observation).
Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
\item{npasses}{Total passes over the data.}
\item{jerr}{Error flag, for warnings and errors (largely for internal
debugging).}
\item{offset}{A logical variable indicating whether an offset was included
in the model.}
\item{call}{The call that produced this object.}
\item{nobs}{Number of observations.}
\item{warm_fit}{If \code{save.fit=TRUE}, output of FORTRAN routine, used for
warm starts. For internal use only.}
\item{family}{Family used for the model.}
\item{converged}{A logical variable: was the algorithm judged to have
converged?}
\item{boundary}{A logical variable: is the fitted value on the boundary of
the attainable values?}
\item{obj_function}{Objective function value at the solution.}
}
\description{
Fit a generalized linear model via penalized maximum likelihood for a single
value of lambda. Can deal with any GLM family.
}
\details{
WARNING: Users should not call \code{glmnet.fit} directly. Higher-level functions
in this package call \code{glmnet.fit} as a subroutine. If a warm start object
is provided, some of the other arguments in the function may be overriden.

\code{glmnet.fit} solves the elastic net problem for a single, user-specified
value of lambda. \code{glmnet.fit} works for any GLM family. It solves the
problem using iteratively reweighted least squares (IRLS). For each IRLS
iteration, \code{glmnet.fit} makes a quadratic (Newton) approximation of the
log-likelihood, then calls \code{elnet.fit} to minimize the resulting
approximation.

In terms of standardization: \code{glmnet.fit} does not standardize \code{x}
and \code{weights}. \code{penalty.factor} is standardized so that they sum up
to \code{nvars}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.glmnet.R, R/plot.mrelnet.R,
%   R/plot.multnet.R, R/plot.relaxed.R
\name{plot.glmnet}
\alias{plot.glmnet}
\alias{plot.multnet}
\alias{plot.mrelnet}
\alias{plot.relaxed}
\title{plot coefficients from a "glmnet" object}
\usage{
\method{plot}{glmnet}(x, xvar = c("norm", "lambda", "dev"), label = FALSE, ...)

\method{plot}{mrelnet}(
  x,
  xvar = c("norm", "lambda", "dev"),
  label = FALSE,
  type.coef = c("coef", "2norm"),
  ...
)

\method{plot}{multnet}(
  x,
  xvar = c("norm", "lambda", "dev"),
  label = FALSE,
  type.coef = c("coef", "2norm"),
  ...
)

\method{plot}{relaxed}(x, xvar = c("lambda", "dev"), label = FALSE, gamma = 1, ...)
}
\arguments{
\item{x}{fitted \code{"glmnet"} model}

\item{xvar}{What is on the X-axis. \code{"norm"} plots against the L1-norm
of the coefficients, \code{"lambda"} against the log-lambda sequence, and
\code{"dev"} against the percent deviance explained.}

\item{label}{If \code{TRUE}, label the curves with variable sequence
numbers.}

\item{\dots}{Other graphical parameters to plot}

\item{type.coef}{If \code{type.coef="2norm"} then a single curve per
variable, else if \code{type.coef="coef"}, a coefficient plot per response}

\item{gamma}{Value of the mixing parameter for a "relaxed" fit}
}
\description{
Produces a coefficient profile plot of the coefficient paths for a fitted
\code{"glmnet"} object.
}
\details{
A coefficient profile plot is produced. If \code{x} is a multinomial model,
a coefficient plot is produced for each class.
}
\examples{
x=matrix(rnorm(100*20),100,20)
y=rnorm(100)
g2=sample(1:2,100,replace=TRUE)
g4=sample(1:4,100,replace=TRUE)
fit1=glmnet(x,y)
plot(fit1)
plot(fit1,xvar="lambda",label=TRUE)
fit3=glmnet(x,g4,family="multinomial")
plot(fit3,pch=19)
}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}
}
\seealso{
\code{glmnet}, and \code{print}, \code{predict} and \code{coef}
methods.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{elnet.fit}
\alias{elnet.fit}
\title{Solve weighted least squares (WLS) problem for a single lambda value}
\usage{
elnet.fit(
  x,
  y,
  weights,
  lambda,
  alpha = 1,
  intercept = TRUE,
  thresh = 1e-07,
  maxit = 1e+05,
  penalty.factor = rep(1, nvars),
  exclude = c(),
  lower.limits = -Inf,
  upper.limits = Inf,
  warm = NULL,
  from.glmnet.fit = FALSE,
  save.fit = FALSE
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed that any standardization needed
has already been done.}

\item{y}{Quantitative response variable.}

\item{weights}{Observation weights. \code{elnet.fit} does NOT standardize
these weights.}

\item{lambda}{A single value for the \code{lambda} hyperparameter.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.
The penalty is defined as \deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.}
\code{alpha=1} is the lasso penalty, and \code{alpha=0} the ridge penalty.}

\item{intercept}{Should intercept be fitted (default=TRUE) or set to zero (FALSE)?}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than thresh times the null deviance.
Default value is \code{1e-7}.}

\item{maxit}{Maximum number of passes over the data; default is \code{10^5}.
(If a warm start object is provided, the number of passes the warm start object
performed is included.)}

\item{penalty.factor}{Separate penalty factors can be applied to each
coefficient. This is a number that multiplies \code{lambda} to allow differential
shrinkage. Can be 0 for some variables, which implies no shrinkage, and that
variable is always included in the model. Default is 1 for all variables (and
implicitly infinity for variables listed in exclude). Note: the penalty
factors are internally rescaled to sum to \code{nvars}.}

\item{exclude}{Indices of variables to be excluded from the model. Default is
none. Equivalent to an infinite penalty factor.}

\item{lower.limits}{Vector of lower limits for each coefficient; default
\code{-Inf}. Each of these must be non-positive. Can be presented as a single
value (which will then be replicated), else a vector of length \code{nvars}.}

\item{upper.limits}{Vector of upper limits for each coefficient; default
\code{Inf}. See \code{lower.limits}.}

\item{warm}{Either a \code{glmnetfit} object or a list (with names \code{beta}
and \code{a0} containing coefficients and intercept respectively) which can
be used as a warm start. Default is \code{NULL}, indicating no warm start.
For internal use only.}

\item{from.glmnet.fit}{Was \code{elnet.fit()} called from \code{glmnet.fit()}?
Default is FALSE.This has implications for computation of the penalty factors.}

\item{save.fit}{Return the warm start object? Default is FALSE.}
}
\value{
An object with class "glmnetfit" and "glmnet". The list returned has
the same keys as that of a \code{glmnet} object, except that it might have an
additional \code{warm_fit} key.
\item{a0}{Intercept value.}
\item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
format.}
\item{df}{The number of nonzero coefficients.}
\item{dim}{Dimension of coefficient matrix.}
\item{lambda}{Lambda value used.}
\item{dev.ratio}{The fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
for the saturated model (a model with a free parameter per observation).
Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)). The null model refers to the intercept model.}
\item{npasses}{Total passes over the data.}
\item{jerr}{Error flag, for warnings and errors (largely for internal
debugging).}
\item{offset}{Always FALSE, since offsets do not appear in the WLS problem.
Included for compability with glmnet output.}
\item{call}{The call that produced this object.}
\item{nobs}{Number of observations.}
\item{warm_fit}{If \code{save.fit=TRUE}, output of C++ routine, used for
warm starts. For internal use only.}
}
\description{
Solves the weighted least squares (WLS) problem for a single lambda value. Internal
function that users should not call directly.
}
\details{
WARNING: Users should not call \code{elnet.fit} directly. Higher-level functions
in this package call \code{elnet.fit} as a subroutine. If a warm start object
is provided, some of the other arguments in the function may be overriden.

\code{elnet.fit} is essentially a wrapper around a C++ subroutine which
minimizes

\deqn{1/2 \sum w_i (y_i - X_i^T \beta)^2 + \sum \lambda \gamma_j
[(1-\alpha)/2 \beta^2+\alpha|\beta|],}

over \eqn{\beta}, where \eqn{\gamma_j} is the relative penalty factor on the
jth variable. If \code{intercept = TRUE}, then the term in the first sum is
\eqn{w_i (y_i - \beta_0 - X_i^T \beta)^2}, and we are minimizing over both
\eqn{\beta_0} and \eqn{\beta}.

None of the inputs are standardized except for \code{penalty.factor}, which
is standardized so that they sum up to \code{nvars}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deviance.glmnet.R
\name{deviance.glmnet}
\alias{deviance.glmnet}
\title{Extract the deviance from a glmnet object}
\usage{
\method{deviance}{glmnet}(object, ...)
}
\arguments{
\item{object}{fitted glmnet object}

\item{\dots}{additional print arguments}
}
\value{
(1-dev.ratio)*nulldev
}
\description{
Compute the deviance sequence from the glmnet object
}
\details{
A glmnet object has components \code{dev.ratio} and \code{nulldev}.  The
former is the fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the
log-likelihood for the saturated model (a model with a free parameter per
observation).  Null deviance is defined to be 2*(loglike_sat
-loglike(Null)); The NULL model refers to the intercept model, except for
the Cox, where it is the 0 model. Hence dev.ratio=1-deviance/nulldev, and
this \code{deviance} method returns (1-dev.ratio)*nulldev.
}
\examples{

x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
fit1 = glmnet(x, y)
deviance(fit1)
}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}
}
\seealso{
\code{glmnet}, \code{predict}, \code{print}, and \code{coef}
methods.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxgrad.R
\name{coxgrad}
\alias{coxgrad}
\title{Compute gradient for Cox model}
\usage{
coxgrad(eta, y, w, std.weights = TRUE, diag.hessian = FALSE)
}
\arguments{
\item{eta}{Fit vector (usually from glmnet at a particular lambda).}

\item{y}{Survival response variable, must be a \code{Surv} or
\code{stratifySurv} object.}

\item{w}{Observation weights (default is all equal to 1).}

\item{std.weights}{If TRUE (default), observation weights are standardized
to sum to 1.}

\item{diag.hessian}{If \code{TRUE}, compute the diagonal of the Hessian
of the log partial likelihood as well. Default is \code{FALSE}.}
}
\value{
A single gradient vector the same length as \code{eta}. If
\code{diag.hessian=TRUE}, the diagonal of the Hessian is
included as an attribute "diag_hessian".
}
\description{
Compute the gradient of the log partial likelihood at a particular fit for Cox
model.
}
\details{
Compute a gradient vector at the fitted vector for the log partial likelihood.
This is like a residual vector, and useful for manual screening of
predictors for \code{glmnet} in applications where \code{p} is very large
(as in GWAS). Uses the Breslow approach to ties.

This function is essentially a wrapper: it checks whether the response
provided is right-censored or (start, stop] survival data, and calls the
appropriate internal routine.
}
\examples{
set.seed(1)
eta <- rnorm(10)
time <- runif(10, min = 1, max = 10)
d <- ifelse(rnorm(10) > 0, 1, 0)
y <- survival::Surv(time, d)
coxgrad(eta, y)

# return diagonal of Hessian as well
coxgrad(eta, y, diag.hessian = TRUE)

# example with (start, stop] data
y2 <- survival::Surv(time, time + runif(10), d)
coxgrad(eta, y2)

# example with strata
y2 <- stratifySurv(y, rep(1:2, length.out = 10))
coxgrad(eta, y2)

}
\seealso{
\code{coxnet.deviance}
}
\keyword{Cox}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SparseExample}
\alias{SparseExample}
\title{Synthetic dataset with sparse design matrix}
\format{
List containing the following elements:
\describe{
\item{x}{100 by 20 matrix of numeric values. x is in sparse matrix
format, having class "dgCMatrix".}
\item{y}{Numeric vector of length 100.}
}
}
\usage{
data(SparseExample)
}
\description{
Randomly generated data for Gaussian regression example with the
design matrix x being in sparse matrix format.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet.measures.R
\name{glmnet.measures}
\alias{glmnet.measures}
\title{Display the names of the measures used in CV for different "glmnet" families}
\usage{
glmnet.measures(
  family = c("all", "gaussian", "binomial", "poisson", "multinomial", "cox", "mgaussian",
    "GLM")
)
}
\arguments{
\item{family}{If a "glmnet" family is supplied, a list of the names of
measures available for that family are produced. Default is "all", in which
case the names of measures for all families are produced.}
}
\description{
Produces a list of names of measures
}
\details{
Try it and see. A very simple function to provide information
}
\seealso{
\code{cv.glmnet} and \code{assess.glmnet}.
}
\author{
Trevor Hastie\cr Maintainer: Trevor Hastie
\email{hastie@stanford.edu}
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet.R, R/relax.glmnet.R
\name{glmnet}
\alias{glmnet}
\alias{relax.glmnet}
\title{fit a GLM with lasso or elasticnet regularization}
\usage{
glmnet(
  x,
  y,
  family = c("gaussian", "binomial", "poisson", "multinomial", "cox", "mgaussian"),
  weights = NULL,
  offset = NULL,
  alpha = 1,
  nlambda = 100,
  lambda.min.ratio = ifelse(nobs < nvars, 0.01, 1e-04),
  lambda = NULL,
  standardize = TRUE,
  intercept = TRUE,
  thresh = 1e-07,
  dfmax = nvars + 1,
  pmax = min(dfmax * 2 + 20, nvars),
  exclude = NULL,
  penalty.factor = rep(1, nvars),
  lower.limits = -Inf,
  upper.limits = Inf,
  maxit = 1e+05,
  type.gaussian = ifelse(nvars < 500, "covariance", "naive"),
  type.logistic = c("Newton", "modified.Newton"),
  standardize.response = FALSE,
  type.multinomial = c("ungrouped", "grouped"),
  relax = FALSE,
  trace.it = 0,
  ...
)

relax.glmnet(fit, x, ..., maxp = n - 3, path = FALSE, check.args = TRUE)
}
\arguments{
\item{x}{input matrix, of dimension nobs x nvars; each row is an observation
vector. Can be in sparse matrix format (inherit from class
\code{"sparseMatrix"} as in package \code{Matrix})}

\item{y}{response variable. Quantitative for \code{family="gaussian"}, or
\code{family="poisson"} (non-negative counts). For \code{family="binomial"}
should be either a factor with two levels, or a two-column matrix of counts
or proportions (the second column is treated as the target class; for a
factor, the last level in alphabetical order is the target class). For
\code{family="multinomial"}, can be a \code{nc>=2} level factor, or a matrix
with \code{nc} columns of counts or proportions. For either
\code{"binomial"} or \code{"multinomial"}, if \code{y} is presented as a
vector, it will be coerced into a factor. For \code{family="cox"}, preferably
a \code{Surv} object from the survival package: see Details section for
more information. For \code{family="mgaussian"}, \code{y} is a matrix
of quantitative responses.}

\item{family}{Either a character string representing
one of the built-in families, or else a \code{glm()} family object. For more
information, see Details section below or the documentation for response
type (above).}

\item{weights}{observation weights. Can be total counts if responses are
proportion matrices. Default is 1 for each observation}

\item{offset}{A vector of length \code{nobs} that is included in the linear
predictor (a \code{nobs x nc} matrix for the \code{"multinomial"} family).
Useful for the \code{"poisson"} family (e.g. log of exposure time), or for
refining a model by starting at a current fit. Default is \code{NULL}. If
supplied, then values must also be supplied to the \code{predict} function.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0\le\alpha\le 1}.
The penalty is defined as
\deqn{(1-\alpha)/2||\beta||_2^2+\alpha||\beta||_1.} \code{alpha=1} is the
lasso penalty, and \code{alpha=0} the ridge penalty.}

\item{nlambda}{The number of \code{lambda} values - default is 100.}

\item{lambda.min.ratio}{Smallest value for \code{lambda}, as a fraction of
\code{lambda.max}, the (data derived) entry value (i.e. the smallest value
for which all coefficients are zero). The default depends on the sample size
\code{nobs} relative to the number of variables \code{nvars}. If \code{nobs
> nvars}, the default is \code{0.0001}, close to zero.  If \code{nobs <
nvars}, the default is \code{0.01}.  A very small value of
\code{lambda.min.ratio} will lead to a saturated fit in the \code{nobs <
nvars} case. This is undefined for \code{"binomial"} and
\code{"multinomial"} models, and \code{glmnet} will exit gracefully when the
percentage deviance explained is almost 1.}

\item{lambda}{A user supplied \code{lambda} sequence. Typical usage is to
have the program compute its own \code{lambda} sequence based on
\code{nlambda} and \code{lambda.min.ratio}. Supplying a value of
\code{lambda} overrides this. WARNING: use with care. Avoid supplying a
single value for \code{lambda} (for predictions after CV use
\code{predict()} instead).  Supply instead a decreasing sequence of
\code{lambda} values. \code{glmnet} relies on its warms starts for speed,
and its often faster to fit a whole path than compute a single fit.}

\item{standardize}{Logical flag for x variable standardization, prior to
fitting the model sequence. The coefficients are always returned on the
original scale. Default is \code{standardize=TRUE}.  If variables are in the
same units already, you might not wish to standardize. See details below for
y standardization with \code{family="gaussian"}.}

\item{intercept}{Should intercept(s) be fitted (default=TRUE) or set to zero
(FALSE)}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than \code{thresh} times the null
deviance. Defaults value is \code{1E-7}.}

\item{dfmax}{Limit the maximum number of variables in the model. Useful for
very large \code{nvars}, if a partial path is desired.}

\item{pmax}{Limit the maximum number of variables ever to be nonzero}

\item{exclude}{Indices of variables to be excluded from the model. Default
is none. Equivalent to an infinite penalty factor (next item).}

\item{penalty.factor}{Separate penalty factors can be applied to each
coefficient. This is a number that multiplies \code{lambda} to allow
differential shrinkage. Can be 0 for some variables, which implies no
shrinkage, and that variable is always included in the model. Default is 1
for all variables (and implicitly infinity for variables listed in
\code{exclude}). Note: the penalty factors are internally rescaled to sum to
nvars, and the lambda sequence will reflect this change.}

\item{lower.limits}{Vector of lower limits for each coefficient; default
\code{-Inf}. Each of these must be non-positive. Can be presented as a
single value (which will then be replicated), else a vector of length
\code{nvars}}

\item{upper.limits}{Vector of upper limits for each coefficient; default
\code{Inf}. See \code{lower.limits}}

\item{maxit}{Maximum number of passes over the data for all lambda values;
default is 10^5.}

\item{type.gaussian}{Two algorithm types are supported for (only)
\code{family="gaussian"}. The default when \code{nvar<500} is
\code{type.gaussian="covariance"}, and saves all inner-products ever
computed. This can be much faster than \code{type.gaussian="naive"}, which
loops through \code{nobs} every time an inner-product is computed. The
latter can be far more efficient for \code{nvar >> nobs} situations, or when
\code{nvar > 500}.}

\item{type.logistic}{If \code{"Newton"} then the exact hessian is used
(default), while \code{"modified.Newton"} uses an upper-bound on the
hessian, and can be faster.}

\item{standardize.response}{This is for the \code{family="mgaussian"}
family, and allows the user to standardize the response variables}

\item{type.multinomial}{If \code{"grouped"} then a grouped lasso penalty is
used on the multinomial coefficients for a variable. This ensures they are
all in our out together. The default is \code{"ungrouped"}}

\item{relax}{If \code{TRUE} then for each \emph{active set} in the path of
solutions, the model is refit without any regularization. See \code{details}
for more information. This argument is new, and users may experience convergence issues
with small datasets, especially with non-gaussian families. Limiting the
value of 'maxp' can alleviate these issues in some cases.}

\item{trace.it}{If \code{trace.it=1}, then a progress bar is displayed;
useful for big models that take a long time to fit.}

\item{...}{Additional argument used in \code{relax.glmnet}. These include
some of the original arguments to 'glmnet', and each must be named if used.}

\item{fit}{For \code{relax.glmnet} a fitted 'glmnet' object}

\item{maxp}{a limit on how many relaxed coefficients are allowed. Default is
'n-3', where 'n' is the sample size. This may not be sufficient for
non-gaussian familes, in which case users should supply a smaller value.
This argument can be supplied directly to 'glmnet'.}

\item{path}{Since \code{glmnet} does not do stepsize optimization, the Newton
algorithm can get stuck and not converge, especially with relaxed fits. With \code{path=TRUE},
each relaxed fit on a particular set of variables is computed pathwise using the original sequence
of lambda values (with a zero attached to the end). Not needed for Gaussian models, and should not
be used unless needed, since will lead to longer compute times. Default is \code{path=FALSE}.
appropriate subset of variables}

\item{check.args}{Should \code{relax.glmnet} make sure that all the data
dependent arguments used in creating 'fit' have been resupplied. Default is
'TRUE'.}
}
\value{
An object with S3 class \code{"glmnet","*" }, where \code{"*"} is
\code{"elnet"}, \code{"lognet"}, \code{"multnet"}, \code{"fishnet"}
(poisson), \code{"coxnet"} or \code{"mrelnet"} for the various types of
models. If the model was created with \code{relax=TRUE} then this class has
a prefix class of \code{"relaxed"}.  \item{call}{the call that produced this
object} \item{a0}{Intercept sequence of length \code{length(lambda)}}
\item{beta}{For \code{"elnet"}, \code{"lognet"}, \code{"fishnet"} and
\code{"coxnet"} models, a \code{nvars x length(lambda)} matrix of
coefficients, stored in sparse column format (\code{"CsparseMatrix"}). For
\code{"multnet"} and \code{"mgaussian"}, a list of \code{nc} such matrices,
one for each class.} \item{lambda}{The actual sequence of \code{lambda}
values used. When \code{alpha=0}, the largest lambda reported does not quite
give the zero coefficients reported (\code{lambda=inf} would in principle).
Instead, the largest \code{lambda} for \code{alpha=0.001} is used, and the
sequence of \code{lambda} values is derived from this.} \item{dev.ratio}{The
fraction of (null) deviance explained (for \code{"elnet"}, this is the
R-square). The deviance calculations incorporate weights if present in the
model. The deviance is defined to be 2*(loglike_sat - loglike), where
loglike_sat is the log-likelihood for the saturated model (a model with a
free parameter per observation). Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)); The NULL model refers to the intercept
model, except for the Cox, where it is the 0 model.} \item{df}{The number of
nonzero coefficients for each value of \code{lambda}. For \code{"multnet"},
this is the number of variables with a nonzero coefficient for \emph{any}
class.} \item{dfmat}{For \code{"multnet"} and \code{"mrelnet"} only. A
matrix consisting of the number of nonzero coefficients per class}
\item{dim}{dimension of coefficient matrix (ices)} \item{nobs}{number of
observations} \item{npasses}{total passes over the data summed over all
lambda values} \item{offset}{a logical variable indicating whether an offset
was included in the model} \item{jerr}{error flag, for warnings and errors
(largely for internal debugging).} \item{relaxed}{If \code{relax=TRUE}, this
additional item is another glmnet object with different values for
\code{beta} and \code{dev.ratio}}
}
\description{
Fit a generalized linear model via penalized maximum likelihood.  The
regularization path is computed for the lasso or elasticnet penalty at a
grid of values for the regularization parameter lambda. Can deal with all
shapes of data, including very large sparse data matrices. Fits linear,
logistic and multinomial, poisson, and Cox regression models.
}
\details{
The sequence of models implied by \code{lambda} is fit by coordinate
descent. For \code{family="gaussian"} this is the lasso sequence if
\code{alpha=1}, else it is the elasticnet sequence.

The objective function for \code{"gaussian"} is \deqn{1/2 RSS/nobs +
\lambda*penalty,} and for the other models it is \deqn{-loglik/nobs +
\lambda*penalty.} Note also that for \code{"gaussian"}, \code{glmnet}
standardizes y to have unit variance (using 1/n rather than 1/(n-1) formula)
before computing its lambda sequence (and then unstandardizes the resulting
coefficients); if you wish to reproduce/compare results with other software,
best to supply a standardized y. The coefficients for any predictor
variables with zero variance are set to zero for all values of lambda.
\subsection{Details on \code{family} option}{

From version 4.0 onwards, glmnet supports both the original built-in families,
as well as \emph{any} family object as used by \code{stats:glm()}.
This opens the door to a wide variety of additional models. For example
\code{family=binomial(link=cloglog)} or \code{family=negative.binomial(theta=1.5)} (from the MASS library).
Note that the code runs faster for the built-in families.

The built in families are specifed via a character string. For all families,
the object produced is a lasso or elasticnet regularization path for fitting the
generalized linear regression paths, by maximizing the appropriate penalized
log-likelihood (partial likelihood for the "cox" model). Sometimes the
sequence is truncated before \code{nlambda} values of \code{lambda} have
been used, because of instabilities in the inverse link functions near a
saturated fit. \code{glmnet(...,family="binomial")} fits a traditional
logistic regression model for the log-odds.
\code{glmnet(...,family="multinomial")} fits a symmetric multinomial model,
where each class is represented by a linear model (on the log-scale). The
penalties take care of redundancies. A two-class \code{"multinomial"} model
will produce the same fit as the corresponding \code{"binomial"} model,
except the pair of coefficient matrices will be equal in magnitude and
opposite in sign, and half the \code{"binomial"} values.
Two useful additional families are the \code{family="mgaussian"} family and
the \code{type.multinomial="grouped"} option for multinomial fitting. The
former allows a multi-response gaussian model to be fit, using a "group
-lasso" penalty on the coefficients for each variable. Tying the responses
together like this is called "multi-task" learning in some domains. The
grouped multinomial allows the same penalty for the
\code{family="multinomial"} model, which is also multi-responsed. For both
of these the penalty on the coefficient vector for variable j is
\deqn{(1-\alpha)/2||\beta_j||_2^2+\alpha||\beta_j||_2.} When \code{alpha=1}
this is a group-lasso penalty, and otherwise it mixes with quadratic just
like elasticnet. A small detail in the Cox model: if death times are tied
with censored times, we assume the censored times occurred just
\emph{before} the death times in computing the Breslow approximation; if
users prefer the usual convention of \emph{after}, they can add a small
number to all censoring times to achieve this effect.
}

\subsection{Details on response for \code{family="cox"}}{

For Cox models, the response should preferably be a \code{Surv} object,
created by the \code{Surv()} function in \pkg{survival} package. For
right-censored data, this object should have type "right", and for
(start, stop] data, it should have type "counting". To fit stratified Cox
models, strata should be added to the response via the \code{stratifySurv()}
function before passing the response to \code{glmnet()}. (For backward
compatibility, right-censored data can also be passed as a
two-column matrix with columns named 'time' and 'status'. The
latter is a binary variable, with '1' indicating death, and '0' indicating
right censored.)
}

\subsection{Details on \code{relax} option}{

If \code{relax=TRUE}
a duplicate sequence of models is produced, where each active set in the
elastic-net path is refit without regularization. The result of this is a
matching \code{"glmnet"} object which is stored on the original object in a
component named \code{"relaxed"}, and is part of the glmnet output.
Generally users will not call \code{relax.glmnet} directly, unless the
original 'glmnet' object took a long time to fit. But if they do, they must
supply the fit, and all the original arguments used to create that fit. They
can limit the length of the relaxed path via 'maxp'.
}
}
\examples{

# Gaussian
x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
fit1 = glmnet(x, y)
print(fit1)
coef(fit1, s = 0.01)  # extract coefficients at a single value of lambda
predict(fit1, newx = x[1:10, ], s = c(0.01, 0.005))  # make predictions

# Relaxed
fit1r = glmnet(x, y, relax = TRUE)  # can be used with any model

# multivariate gaussian
y = matrix(rnorm(100 * 3), 100, 3)
fit1m = glmnet(x, y, family = "mgaussian")
plot(fit1m, type.coef = "2norm")

# binomial
g2 = sample(c(0,1), 100, replace = TRUE)
fit2 = glmnet(x, g2, family = "binomial")
fit2n = glmnet(x, g2, family = binomial(link=cloglog))
fit2r = glmnet(x,g2, family = "binomial", relax=TRUE)
fit2rp = glmnet(x,g2, family = "binomial", relax=TRUE, path=TRUE)

# multinomial
g4 = sample(1:4, 100, replace = TRUE)
fit3 = glmnet(x, g4, family = "multinomial")
fit3a = glmnet(x, g4, family = "multinomial", type.multinomial = "grouped")
# poisson
N = 500
p = 20
nzc = 5
x = matrix(rnorm(N * p), N, p)
beta = rnorm(nzc)
f = x[, seq(nzc)] \%*\% beta
mu = exp(f)
y = rpois(N, mu)
fit = glmnet(x, y, family = "poisson")
plot(fit)
pfit = predict(fit, x, s = 0.001, type = "response")
plot(pfit, y)

# Cox
set.seed(10101)
N = 1000
p = 30
nzc = p/3
x = matrix(rnorm(N * p), N, p)
beta = rnorm(nzc)
fx = x[, seq(nzc)] \%*\% beta/3
hx = exp(fx)
ty = rexp(N, hx)
tcens = rbinom(n = N, prob = 0.3, size = 1)  # censoring indicator
y = cbind(time = ty, status = 1 - tcens)  # y=Surv(ty,1-tcens) with library(survival)
fit = glmnet(x, y, family = "cox")
plot(fit)

# Cox example with (start, stop] data
set.seed(2)
nobs <- 100; nvars <- 15
xvec <- rnorm(nobs * nvars)
xvec[sample.int(nobs * nvars, size = 0.4 * nobs * nvars)] <- 0
x <- matrix(xvec, nrow = nobs)
start_time <- runif(100, min = 0, max = 5)
stop_time <- start_time + runif(100, min = 0.1, max = 3)
status <- rbinom(n = nobs, prob = 0.3, size = 1)
jsurv_ss <- survival::Surv(start_time, stop_time, status)
fit <- glmnet(x, jsurv_ss, family = "cox")

# Cox example with strata
jsurv_ss2 <- stratifySurv(jsurv_ss, rep(1:2, each = 50))
fit <- glmnet(x, jsurv_ss2, family = "cox")

# Sparse
n = 10000
p = 200
nzc = trunc(p/10)
x = matrix(rnorm(n * p), n, p)
iz = sample(1:(n * p), size = n * p * 0.85, replace = FALSE)
x[iz] = 0
sx = Matrix(x, sparse = TRUE)
inherits(sx, "sparseMatrix")  #confirm that it is sparse
beta = rnorm(nzc)
fx = x[, seq(nzc)] \%*\% beta
eps = rnorm(n)
y = fx + eps
px = exp(fx)
px = px/(1 + px)
ly = rbinom(n = length(px), prob = px, size = 1)
system.time(fit1 <- glmnet(sx, y))
system.time(fit2n <- glmnet(x, y))

}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent (2010), Journal of Statistical Software, Vol. 33(1), 1-22},
\url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}.\cr
Simon, N., Friedman, J., Hastie, T. and Tibshirani, R. (2011)
\emph{Regularization Paths for Cox's Proportional
Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
39(5), 1-13}, \url{https://www.jstatsoft.org/v39/i05/}.\cr Tibshirani,
Robert, Bien, J., Friedman, J., Hastie, T.,Simon, N.,Taylor, J. and
Tibshirani, Ryan. (2012) \emph{Strong Rules for Discarding Predictors in
Lasso-type Problems, JRSSB, Vol. 74(2), 245-266},
\url{https://statweb.stanford.edu/~tibs/ftp/strong.pdf}.\cr
Hastie, T., Tibshirani, Robert and Tibshirani, Ryan. \emph{Extended
Comparisons of Best Subset Selection, Forward Stepwise Selection, and the
Lasso (2017), Stanford Statistics Technical Report},
\url{https://arxiv.org/abs/1707.08692}.\cr
Glmnet webpage with four vignettes, \url{https://glmnet.stanford.edu}.
}
\seealso{
\code{print}, \code{predict}, \code{coef} and \code{plot} methods,
and the \code{cv.glmnet} function.
}
\author{
Jerome Friedman, Trevor Hastie, Balasubramanian Narasimhan, Noah
Simon, Kenneth Tay and Rob Tibshirani\cr Maintainer: Trevor Hastie
\email{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.cv.glmnet.R, R/plot.cv.relaxed.R
\name{plot.cv.glmnet}
\alias{plot.cv.glmnet}
\alias{plot.cv.relaxed}
\title{plot the cross-validation curve produced by cv.glmnet}
\usage{
\method{plot}{cv.glmnet}(x, sign.lambda = 1, ...)

\method{plot}{cv.relaxed}(x, se.bands = TRUE, ...)
}
\arguments{
\item{x}{fitted \code{"cv.glmnet"} object}

\item{sign.lambda}{Either plot against \code{log(lambda)} (default) or its
negative if \code{sign.lambda=-1}.}

\item{\dots}{Other graphical parameters to plot}

\item{se.bands}{Should shading be produced to show standard-error bands;
default is \code{TRUE}}
}
\description{
Plots the cross-validation curve, and upper and lower standard deviation
curves, as a function of the \code{lambda} values used. If the object has
class \code{"cv.relaxed"} a different plot is produced, showing both
\code{lambda} and \code{gamma}
}
\details{
A plot is produced, and nothing is returned.
}
\examples{

set.seed(1010)
n = 1000
p = 100
nzc = trunc(p/10)
x = matrix(rnorm(n * p), n, p)
beta = rnorm(nzc)
fx = (x[, seq(nzc)] \%*\% beta)
eps = rnorm(n) * 5
y = drop(fx + eps)
px = exp(fx)
px = px/(1 + px)
ly = rbinom(n = length(px), prob = px, size = 1)
cvob1 = cv.glmnet(x, y)
plot(cvob1)
title("Gaussian Family", line = 2.5)
cvob1r = cv.glmnet(x, y, relax = TRUE)
plot(cvob1r)
frame()
set.seed(1011)
par(mfrow = c(2, 2), mar = c(4.5, 4.5, 4, 1))
cvob2 = cv.glmnet(x, ly, family = "binomial")
plot(cvob2)
title("Binomial Family", line = 2.5)
## set.seed(1011)
## cvob3 = cv.glmnet(x, ly, family = "binomial", type = "class")
## plot(cvob3)
## title("Binomial Family", line = 2.5)

}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}
}
\seealso{
\code{glmnet} and \code{cv.glmnet}.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxgrad.R
\name{fid}
\alias{fid}
\title{Helper function for Cox deviance and gradient}
\usage{
fid(x, index)
}
\arguments{
\item{x}{Sorted vector of death times.}

\item{index}{Vector of indices for the death times.}
}
\value{
A list with two arguments.
\item{index_first}{A vector of indices for the first observation at each
death time as they appear in the sorted list.}
\item{index_ties}{If there are no ties at all, this is NULL. If not, this is
a list with length equal to the number of unique times with ties. For each
time with ties, index_ties gives the indices of the observations with a
death at that time.}
}
\description{
Helps to find ties in death times of data.
}
\examples{
# Example with no ties
glmnet:::fid(c(1, 4, 5, 6), 1:5)

# Example with ties
glmnet:::fid(c(1, 1, 1, 2, 3, 3, 4, 4, 4), 1:9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assess.glmnet.R, R/confusion.glmnet.R,
%   R/roc.glmnet.R
\name{assess.glmnet}
\alias{assess.glmnet}
\alias{confusion.glmnet}
\alias{roc.glmnet}
\title{assess performance of a 'glmnet' object using test data.}
\usage{
assess.glmnet(
  object,
  newx = NULL,
  newy,
  weights = NULL,
  family = c("gaussian", "binomial", "poisson", "multinomial", "cox", "mgaussian"),
  ...
)

confusion.glmnet(
  object,
  newx = NULL,
  newy,
  family = c("binomial", "multinomial"),
  ...
)

roc.glmnet(object, newx = NULL, newy, ...)
}
\arguments{
\item{object}{Fitted \code{"glmnet"} or \code{"cv.glmnet"}, \code{"relaxed"}
or \code{"cv.relaxed"} object, or a matrix of predictions (for
\code{roc.glmnet} or \code{assess.glmnet}). For \code{roc.glmnet} the model
must be a 'binomial', and for \code{confusion.glmnet} must be either
'binomial' or 'multinomial'}

\item{newx}{If predictions are to made, these are the 'x' values. Required
for \code{confusion.glmnet}}

\item{newy}{required argument for all functions; the new response values}

\item{weights}{For observation weights for the test observations}

\item{family}{The family of the model, in case predictions are passed in as
'object'}

\item{...}{additional arguments to \code{predict.glmnet} when "object" is a
"glmnet" fit, and predictions must be made to produce the statistics.}
}
\value{
\code{assess.glmnet} produces a list of vectors of measures.
\code{roc.glmnet} a list of 'roc' two-column matrices, and
\code{confusion.glmnet} a list of tables. If a single prediction is
provided, or predictions are made from a CV object, the latter two drop the
list status and produce a single matrix or table.
}
\description{
Given a test set, produce summary performance measures for the glmnet
model(s)
}
\details{
\code{assess.glmnet} produces all the different performance measures
provided by \code{cv.glmnet} for each of the families. A single vector, or a
matrix of predictions can be provided, or fitted model objects or CV
objects. In the case when the predictions are still to be made, the
\code{...} arguments allow, for example, 'offsets' and other prediction
parameters such as values for 'gamma' for 'relaxed' fits.  \code{roc.glmnet}
produces for a single vector a two column matrix with columns TPR and FPR
(true positive rate and false positive rate). This object can be plotted to
produce an ROC curve. If more than one predictions are called for, then a
list of such matrices is produced.  \code{confusion.glmnet} produces a
confusion matrix tabulating the classification results. Again, a single
table or a list, with a print method.
}
\examples{

data(QuickStartExample)
set.seed(11)
train = sample(seq(length(y)),70,replace=FALSE)
fit1 = glmnet(x[train,], y[train])
assess.glmnet(fit1, newx = x[-train,], newy = y[-train])
preds = predict(fit1, newx = x[-train, ], s = c(1, 0.25))
assess.glmnet(preds, newy = y[-train], family = "gaussian")
fit1c = cv.glmnet(x, y, keep = TRUE)
fit1a = assess.glmnet(fit1c$fit.preval, newy=y,family="gaussian")
plot(fit1c$lambda, log="x",fit1a$mae,xlab="Log Lambda",ylab="Mean Absolute Error")
abline(v=fit1c$lambda.min, lty=2, col="red")
data(BinomialExample)
fit2 = glmnet(x[train,], y[train], family = "binomial")
assess.glmnet(fit2,newx = x[-train,], newy=y[-train], s=0.1)
plot(roc.glmnet(fit2, newx = x[-train,], newy=y[-train])[[10]])
fit2c = cv.glmnet(x, y, family = "binomial", keep=TRUE)
idmin = match(fit2c$lambda.min, fit2c$lambda)
plot(roc.glmnet(fit2c$fit.preval, newy = y)[[idmin]])
data(MultinomialExample)
set.seed(103)
train = sample(seq(length(y)),100,replace=FALSE)
fit3 = glmnet(x[train,], y[train], family = "multinomial")
confusion.glmnet(fit3, newx = x[-train, ], newy = y[-train], s = 0.01)
fit3c = cv.glmnet(x, y, family = "multinomial", type.measure="class", keep=TRUE)
idmin = match(fit3c$lambda.min, fit3c$lambda)
confusion.glmnet(fit3c$fit.preval, newy = y, family="multinomial")[[idmin]]

}
\seealso{
\code{cv.glmnet}, \code{glmnet.measures} and \code{vignette("relax",package="glmnet")}
}
\author{
Trevor Hastie and Rob Tibshirani\cr Maintainer: Trevor Hastie
\href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{classification}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxpath.R
\name{cox.fit}
\alias{cox.fit}
\title{Fit a Cox regression model with elastic net regularization for a single
value of lambda}
\usage{
cox.fit(
  x,
  y,
  weights,
  lambda,
  alpha = 1,
  offset = rep(0, nobs),
  thresh = 1e-10,
  maxit = 1e+05,
  penalty.factor = rep(1, nvars),
  exclude = c(),
  lower.limits = -Inf,
  upper.limits = Inf,
  warm = NULL,
  from.cox.path = FALSE,
  save.fit = FALSE,
  trace.it = 0
)
}
\arguments{
\item{x}{Input matrix, of dimension \code{nobs x nvars}; each row is an
observation vector. If it is a sparse matrix, it is assumed to be unstandardized.
It should have attributes \code{xm} and \code{xs}, where \code{xm(j)} and
\code{xs(j)} are the centering and scaling factors for variable j respsectively.
If it is not a sparse matrix, it is assumed that any standardization needed
has already been done.}

\item{y}{Survival response variable, must be a Surv or stratifySurv object.}

\item{weights}{Observation weights. \code{cox.fit} does NOT standardize
these weights.}

\item{lambda}{A single value for the \code{lambda} hyperparameter.}

\item{alpha}{See glmnet help file}

\item{offset}{See glmnet help file}

\item{thresh}{Convergence threshold for coordinate descent. Each inner
coordinate-descent loop continues until the maximum change in the objective
after any coefficient update is less than thresh times the null deviance.
Default value is \code{1e-10}.}

\item{maxit}{Maximum number of passes over the data; default is \code{10^5}.
(If a warm start object is provided, the number of passes the warm start object
performed is included.)}

\item{penalty.factor}{See glmnet help file}

\item{exclude}{See glmnet help file}

\item{lower.limits}{See glmnet help file}

\item{upper.limits}{See glmnet help file}

\item{warm}{Either a \code{glmnetfit} object or a list (with name \code{beta}
containing coefficients) which can be used as a warm start. Default is
\code{NULL}, indicating no warm start. For internal use only.}

\item{from.cox.path}{Was \code{cox.fit()} called from \code{cox.path()}?
Default is FALSE.This has implications for computation of the penalty factors.}

\item{save.fit}{Return the warm start object? Default is FALSE.}

\item{trace.it}{Controls how much information is printed to screen. If
\code{trace.it=2}, some information about the fitting procedure is printed to
the console as the model is being fitted. Default is \code{trace.it=0}
(no information printed). (\code{trace.it=1} not used for compatibility with
\code{glmnet.path}.)}
}
\value{
An object with class "coxnet", "glmnetfit" and "glmnet". The list
returned contains more keys than that of a "glmnet" object.
\item{a0}{Intercept value, \code{NULL} for "cox" family.}
\item{beta}{A \code{nvars x 1} matrix of coefficients, stored in sparse matrix
format.}
\item{df}{The number of nonzero coefficients.}
\item{dim}{Dimension of coefficient matrix.}
\item{lambda}{Lambda value used.}
\item{dev.ratio}{The fraction of (null) deviance explained. The deviance
calculations incorporate weights if present in the model. The deviance is
defined to be 2*(loglike_sat - loglike), where loglike_sat is the log-likelihood
for the saturated model (a model with a free parameter per observation).
Hence dev.ratio=1-dev/nulldev.}
\item{nulldev}{Null deviance (per observation). This is defined to be
2*(loglike_sat -loglike(Null)). The null model refers to the 0 model.}
\item{npasses}{Total passes over the data.}
\item{jerr}{Error flag, for warnings and errors (largely for internal
debugging).}
\item{offset}{A logical variable indicating whether an offset was included
in the model.}
\item{call}{The call that produced this object.}
\item{nobs}{Number of observations.}
\item{warm_fit}{If \code{save.fit=TRUE}, output of C++ routine, used for
warm starts. For internal use only.}
\item{family}{Family used for the model, always "cox".}
\item{converged}{A logical variable: was the algorithm judged to have
converged?}
\item{boundary}{A logical variable: is the fitted value on the boundary of
the attainable values?}
\item{obj_function}{Objective function value at the solution.}
}
\description{
Fit a Cox regression model via penalized maximum likelihood for a single
value of lambda. Can deal with (start, stop] data and strata, as well as
sparse design matrices.
}
\details{
WARNING: Users should not call \code{cox.fit} directly. Higher-level
functions in this package call \code{cox.fit} as a subroutine. If a
warm start object is provided, some of the other arguments in the function
may be overriden.

\code{cox.fit} solves the elastic net problem for a single, user-specified
value of lambda. \code{cox.fit} works for Cox regression models, including
(start, stop] data and strata. It solves the problem using iteratively
reweighted least squares (IRLS). For each IRLS iteration, \code{cox.fit}
makes a quadratic (Newton) approximation of the log-likelihood, then calls
\code{elnet.fit} to minimize the resulting approximation.

In terms of standardization: \code{cox.fit} does not standardize \code{x}
and \code{weights}. \code{penalty.factor} is standardized so that they sum
up to \code{nvars}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.cv.glmnet.R
\name{print.cv.glmnet}
\alias{print.cv.glmnet}
\alias{print.cv.relaxed}
\title{print a cross-validated glmnet object}
\usage{
\method{print}{cv.glmnet}(x, digits = max(3, getOption("digits") - 3), ...)
}
\arguments{
\item{x}{fitted 'cv.glmnet' object}

\item{digits}{significant digits in printout}

\item{\dots}{additional print arguments}
}
\description{
Print a summary of the results of cross-validation for a glmnet model.
}
\details{
A summary of the cross-validated fit is produced, slightly different for a
'cv.relaxed' object than for a 'cv.glmnet' object.  Note that a 'cv.relaxed'
object inherits from class 'cv.glmnet', so by directly invoking
\code{print.cv.glmnet(object)} will print the summary as if
\code{relax=TRUE} had not been used.
}
\examples{

x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
fit1 = cv.glmnet(x, y)
print(fit1)
fit1r = cv.glmnet(x, y, relax = TRUE)
print(fit1r)
## print.cv.glmnet(fit1r)  ## CHECK WITH TREVOR
}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}\cr \url{https://arxiv.org/abs/1707.08692}\cr Hastie, T.,
Tibshirani, Robert, Tibshirani, Ryan (2019) \emph{Extended Comparisons of
Best Subset Selection, Forward Stepwise Selection, and the Lasso}
}
\seealso{
\code{glmnet}, \code{predict} and \code{coef} methods.
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeX.R
\name{na.replace}
\alias{na.replace}
\title{Replace the missing entries in a matrix columnwise with the entries in a
supplied vector}
\usage{
na.replace(x, m = rowSums(x, na.rm = TRUE))
}
\arguments{
\item{x}{A matrix with potentially missing values, and also potentially in
sparse matrix format (i.e. inherits from "sparseMatrix")}

\item{m}{Optional argument. A vector of values used to replace the missing
entries, columnwise. If missing, the column means of 'x' are used}
}
\value{
A version of 'x' is returned with the missing values replaced.
}
\description{
Missing entries in any given column of the matrix are replaced by the column
means or the values in a supplied vector.
}
\details{
This is a simple imputation scheme. This function is called by \code{makeX}
if the \code{na.impute=TRUE} option is used, but of course can be used on
its own. If 'x' is sparse, the result is sparse, and the replacements are
done so as to maintain sparsity.
}
\examples{

set.seed(101)
### Single data frame
X = matrix(rnorm(20), 10, 2)
X[3, 1] = NA
X[5, 2] = NA
X3 = sample(letters[1:3], 10, replace = TRUE)
X3[6] = NA
X4 = sample(LETTERS[1:3], 10, replace = TRUE)
X4[9] = NA
dfn = data.frame(X, X3, X4)

x = makeX(dfn)
m = rowSums(x, na.rm = TRUE)
na.replace(x, m)

x = makeX(dfn, sparse = TRUE)
na.replace(x, m)

}
\seealso{
\code{makeX} and \code{glmnet}
}
\author{
Trevor Hastie\cr Maintainer: Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratifySurv.R
\name{stratifySurv}
\alias{stratifySurv}
\title{Add strata to a Surv object}
\usage{
stratifySurv(y, strata)
}
\arguments{
\item{y}{A Surv object.}

\item{strata}{A vector of length equal to the number of observations in
y, indicating strata membership.}
}
\value{
An object of class \code{stratifySurv} (in addition to all the
classes \code{y} belonged to).
}
\description{
Helper function to add strata as an attribute to a Surv object. The
output of this function can be used as the response in \code{glmnet()}
for fitting stratified Cox models.
}
\details{
When fitting a stratified Cox model with \code{glmnet()}, strata should
be added to a \code{Surv} response with this helper function. Note that
it is not sufficient to add strata as an attribute to the \code{Surv}
response manually: if the result does not have class \code{stratifySurv},
subsetting of the response will not work properly.
}
\examples{
y <- survival::Surv(1:10, rep(0:1, length.out = 10))
strata <- rep(1:3, length.out = 10)
y2 <- stratifySurv(y, strata)  # returns stratifySurv object

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MultiGaussianExample}
\alias{MultiGaussianExample}
\title{Synthetic dataset with multiple Gaussian responses}
\format{
List containing the following elements:
\describe{
\item{x}{100 by 20 matrix of numeric values.}
\item{y}{100 by 4 matrix of numeric values, each column representing
one response vector.}
}
}
\usage{
data(MultiGaussianExample)
}
\description{
Randomly generated data for multi-response Gaussian regression example.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxnet.deviance.R
\name{coxnet.deviance}
\alias{coxnet.deviance}
\title{Compute deviance for Cox model}
\usage{
coxnet.deviance(
  pred = NULL,
  y,
  x = NULL,
  offset = NULL,
  weights = NULL,
  std.weights = TRUE,
  beta = NULL
)
}
\arguments{
\item{pred}{Fit vector or matrix (usually from glmnet at a particular
lambda or a sequence of lambdas).}

\item{y}{Survival response variable, must be a \code{Surv} or
\code{stratifySurv} object.}

\item{x}{Optional \code{x} matrix, to be supplied if \code{pred = NULL}.}

\item{offset}{Optional offset vector.}

\item{weights}{Observation weights (default is all equal to 1).}

\item{std.weights}{If TRUE (default), observation weights are standardized
to sum to 1.}

\item{beta}{Optional coefficient vector/matrix, to be supplied if
\code{pred = NULL}.}
}
\value{
A vector of deviances, one for each column of predictions.
}
\description{
Compute the deviance (-2 log partial likelihood) for Cox model.
}
\details{
Computes the deviance for a single set of predictions, or for a matrix
of predictions. The user can either supply the predictions
directly through the \code{pred} option, or by supplying the \code{x} matrix
and \code{beta} coefficients. Uses the Breslow approach to ties.

The function first checks if \code{pred} is passed: if so, it is used as
the predictions. If \code{pred} is not passed but \code{x} and \code{beta}
are passed, then these values are used to compute the predictions. If
neither \code{x} nor \code{beta} are passed, then the predictions are all
taken to be 0.

\code{coxnet.deviance()} is a wrapper: it calls the appropriate internal
routine based on whether the response is right-censored data or
(start, stop] survival data.
}
\examples{
set.seed(1)
eta <- rnorm(10)
time <- runif(10, min = 1, max = 10)
d <- ifelse(rnorm(10) > 0, 1, 0)
y <- survival::Surv(time, d)
coxnet.deviance(pred = eta, y = y)

# if pred not provided, it is set to zero vector
coxnet.deviance(y = y)

# example with x and beta
x <- matrix(rnorm(10 * 3), nrow = 10)
beta <- matrix(1:3, ncol = 1)
coxnet.deviance(y = y, x = x, beta = beta)

# example with (start, stop] data
y2 <- survival::Surv(time, time + runif(10), d)
coxnet.deviance(pred = eta, y = y2)

# example with strata
y2 <- stratifySurv(y, rep(1:2, length.out = 10))
coxnet.deviance(pred = eta, y = y2)

}
\seealso{
\code{coxgrad}
}
\keyword{Cox}
\keyword{model}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{predict.glmnetfit}
\alias{predict.glmnetfit}
\title{Get predictions from a \code{glmnetfit} fit object}
\usage{
\method{predict}{glmnetfit}(
  object,
  newx,
  s = NULL,
  type = c("link", "response", "coefficients", "nonzero"),
  exact = FALSE,
  newoffset,
  ...
)
}
\arguments{
\item{object}{Fitted "glmnetfit" object.}

\item{newx}{Matrix of new values for \code{x} at which predictions are to be
made. Must be a matrix. This argument is not used for \code{type =
c("coefficients","nonzero")}.}

\item{s}{Value(s) of the penalty parameter lambda at which predictions are
required. Default is the entire sequence used to create the model.}

\item{type}{Type of prediction required. Type "link" gives the linear
predictors (eta scale); Type "response" gives the fitted values (mu scale).
Type "coefficients" computes the coefficients at the requested values for s.
Type "nonzero" returns a list of the indices of the nonzero coefficients for
each value of s.}

\item{exact}{This argument is relevant only when predictions are made at values
of \code{s} (lambda) \emph{different} from those used in the fitting of the
original model. If \code{exact=FALSE} (default), then the predict function
uses linear interpolation to make predictions for values of \code{s} (lambda)
that do not coincide with those used in the fitting algorithm. While this is
often a good approximation, it can sometimes be a bit coarse. With
\code{exact=TRUE}, these different values of \code{s} are merged (and sorted)
with \code{object$lambda}, and the model is refit before predictions are made.
In this case, it is required to supply the original data x= and y= as additional
named arguments to predict() or coef(). The workhorse \code{predict.glmnet()}
needs to update the model, and so needs the data used to create it. The same
is true of weights, offset, penalty.factor, lower.limits, upper.limits if
these were used in the original call. Failure to do so will result in an error.}

\item{newoffset}{If an offset is used in the fit, then one must be supplied for
making predictions (except for type="coefficients" or type="nonzero").}

\item{...}{This is the mechanism for passing arguments like \code{x=} when
\code{exact=TRUE}; see \code{exact} argument.}
}
\value{
The object returned depends on type.
}
\description{
Gives fitted values, linear predictors, coefficients and number of non-zero
coefficients from a fitted \code{glmnetfit} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet-package.R
\name{glmnet-internal}
\alias{glmnet-internal}
\alias{auc}
\alias{assess.coxnet}
\alias{auc.mat}
\alias{cvtype}
\alias{cvstats}
\alias{cvcompute}
\alias{getcoef}
\alias{getcoef.multinomial}
\alias{fix.lam}
\alias{error.bars}
\alias{getmin}
\alias{elnet}
\alias{mrelnet}
\alias{lognet}
\alias{fishnet}
\alias{coefnorm}
\alias{coxnet}
\alias{cv.lognet}
\alias{cv.elnet}
\alias{cv.multnet}
\alias{cv.mrelnet}
\alias{cv.coxnet}
\alias{cv.fishnet}
\alias{cv.glmnet.raw}
\alias{cv.relaxed.raw}
\alias{blend.relaxed}
\alias{checkgamma.relax}
\alias{buildPredmat}
\alias{buildPredmat.mrelnetlist}
\alias{buildPredmat.multnetlist}
\alias{buildPredmat.lognetlist}
\alias{buildPredmat.array}
\alias{buildPredmat.coxnetlist}
\alias{buildPredmat.default}
\alias{lambda.interp}
\alias{nonzeroCoef}
\alias{glmnet_softmax}
\alias{getOptcv.glmnet}
\alias{getOptcv.relaxed}
\alias{jerr}
\alias{jerr.elnet}
\alias{jerr.lognet}
\alias{jerr.fishnet}
\alias{jerr.coxnet}
\alias{jerr.mrelnet}
\alias{plotCoef}
\alias{zeromat}
\alias{na.mean}
\alias{check_dots}
\alias{na_sparse_fix}
\alias{prepareX}
\title{Internal glmnet functions}
\description{
These are not intended for use by users. \code{lambda.interp} does linear
interpolation of the lambdas to obtain a prediction at a new point s.
\code{glmnet_softmax} does the classification for multinomial models.
\code{nonzeroCoef} determines in an efficient manner which variables are
nonzero in each fit. \code{jerr} prints out error messages from the C++ routines.
\code{plotCoef} is called by the \code{plot} method for \code{glmnet}
objects. \code{check_dots} is used in \code{coef} and \code{predict} with
argument \code{exact=TRUE}, to make sure user supplies original data used to
fit the \code{"glmnet"} object.
}
\author{
Trevor Hastie
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{BinomialExample}
\alias{BinomialExample}
\title{Synthetic dataset with binary response}
\format{
List containing the following elements:
\describe{
\item{x}{100 by 30 matrix of numeric values.}
\item{y}{Numeric vector of length 100 containing 44 zeros and 56 ones.}
}
}
\usage{
data(BinomialExample)
}
\description{
Randomly generated data for binomial regression example.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{dev_function}
\alias{dev_function}
\title{Elastic net deviance value}
\usage{
dev_function(y, mu, weights, family)
}
\arguments{
\item{y}{Quantitative response variable.}

\item{mu}{Model's predictions for \code{y}.}

\item{weights}{Observation weights.}

\item{family}{A description of the error distribution and link function to be
used in the model. This is the result of a call to a family function.}
}
\description{
Returns the elastic net deviance value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/response.coxnet.R
\name{response.coxnet}
\alias{response.coxnet}
\title{Make response for coxnet}
\usage{
response.coxnet(y)
}
\arguments{
\item{y}{Response variable. Either a class "Surv" object or a two-column
matrix with columns named 'time' and 'status'.}
}
\value{
A class "Surv" object.
}
\description{
Internal function to make the response y passed to glmnet suitable
for coxnet (i.e. glmnet with family = "cox"). Sanity checks are performed
here too.
}
\details{
If y is a class "Surv" object, this function returns y with no changes. If
y is a two-column matrix with columns named 'time' and 'status', it is
converted into a "Surv" object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnet-package.R
\docType{package}
\name{glmnet-package}
\alias{glmnet-package}
\title{Elastic net model paths for some generalized linear models}
\description{
This package fits lasso and elastic-net model paths for regression, logistic
and multinomial regression using coordinate descent. The algorithm is
extremely fast, and exploits sparsity in the input x matrix where it exists.
A variety of predictions can be made from the fitted models.
}
\details{
\tabular{ll}{ Package: \tab glmnet\cr Type: \tab Package\cr Version: \tab
1.0\cr Date: \tab 2008-05-14\cr License: \tab What license is it under?\cr }
Very simple to use. Accepts \code{x,y} data for regression models, and
produces the regularization path over a grid of values for the tuning
parameter \code{lambda}. Only 5 functions: \code{glmnet}\cr
\code{predict.glmnet}\cr \code{plot.glmnet}\cr \code{print.glmnet}\cr
\code{coef.glmnet}
}
\examples{

x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
g2 = sample(1:2, 100, replace = TRUE)
g4 = sample(1:4, 100, replace = TRUE)
fit1 = glmnet(x, y)
predict(fit1, newx = x[1:5, ], s = c(0.01, 0.005))
predict(fit1, type = "coef")
plot(fit1, xvar = "lambda")
fit2 = glmnet(x, g2, family = "binomial")
predict(fit2, type = "response", newx = x[2:5, ])
predict(fit2, type = "nonzero")
fit3 = glmnet(x, g4, family = "multinomial")
predict(fit3, newx = x[1:3, ], type = "response", s = 0.01)

}
\references{
Friedman, J., Hastie, T. and Tibshirani, R. (2008)
\emph{Regularization Paths for Generalized Linear Models via Coordinate
Descent}, \url{https://web.stanford.edu/~hastie/Papers/glmnet.pdf}\cr
\emph{Journal of Statistical Software, Vol. 33(1), 1-22 Feb 2010}\cr
\url{https://www.jstatsoft.org/v33/i01/}\cr Simon, N., Friedman, J., Hastie,
T., Tibshirani, R. (2011) \emph{Regularization Paths for Cox's Proportional
Hazards Model via Coordinate Descent, Journal of Statistical Software, Vol.
39(5) 1-13}\cr \url{https://www.jstatsoft.org/v39/i05/}\cr Tibshirani,
Robert., Bien, J., Friedman, J.,Hastie, T.,Simon, N.,Taylor, J. and
Tibshirani, Ryan. (2012) \emph{Strong Rules for Discarding Predictors in
Lasso-type Problems, JRSSB, vol 74},\cr
\url{https://statweb.stanford.edu/~tibs/ftp/strong.pdf}\cr
\emph{Glmnet webpage with four vignettes}
\url{https://glmnet.stanford.edu}
}
\author{
Jerome Friedman, Trevor Hastie and Rob Tibshirani\cr Maintainer:
Trevor Hastie \href{mailto:hastie@stanford.edu}{hastie@stanford.edu}
}
\keyword{models}
\keyword{package}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survfit.coxnet.R
\name{mycoxph}
\alias{mycoxph}
\title{Helper function to fit coxph model for survfit.coxnet}
\usage{
mycoxph(object, s, ...)
}
\arguments{
\item{object}{A class \code{coxnet} object.}

\item{s}{The value of the penalty parameter lambda at which the survival
curve is required.}

\item{...}{The same ... that was passed to survfit.coxnet.}
}
\description{
This function constructs the coxph call needed to run the "hack" of
coxph with 0 iterations. It's a separate function as we have to deal with
function options like strata, offset and observation weights.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bigGlm.R
\name{bigGlm}
\alias{bigGlm}
\title{fit a glm with all the options in \code{glmnet}}
\usage{
bigGlm(x, ..., path = FALSE)
}
\arguments{
\item{x}{input matrix}

\item{...}{Most other arguments to glmnet that make sense}

\item{path}{Since \code{glmnet} does not do stepsize optimization, the Newton
algorithm can get stuck and not converge, especially with unpenalized fits. With \code{path=TRUE},
the  fit computed with pathwise lasso regularization. The current implementation does this twice:
the first time to get the lambda sequence, and the second time with a zero attached to the end).
Default is \code{path=FALSE}.}
}
\value{
It returns an object of class "bigGlm" that inherits from class
"glmnet". That means it can be predicted from, coefficients extracted via
\code{coef}. It has its own print method.
}
\description{
Fit a generalized linear model as in \code{glmnet} but unpenalized. This
allows all the features of \code{glmnet} such as sparse x, bounds on
coefficients, offsets, and so on.
}
\details{
This is essentially the same as fitting a "glmnet" model with a single value
\code{lambda=0}, but it avoids some edge cases. CAVEAT: If the user tries a
problem with N smaller than or close to p for some models, it is likely to
fail (and maybe not gracefully!) If so, use the \code{path=TRUE} argument.
}
\examples{

# Gaussian
x = matrix(rnorm(100 * 20), 100, 20)
y = rnorm(100)
fit1 = bigGlm(x, y)
print(fit1)

fit2=bigGlm(x,y>0,family="binomial")
print(fit2)
fit2p=bigGlm(x,y>0,family="binomial",path=TRUE)
print(fit2p)

}
\seealso{
\code{print}, \code{predict}, and \code{coef} methods.
}
\author{
Trevor Hastie\cr Maintainer: Trevor Hastie
\email{hastie@stanford.edu}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmnetFlex.R
\name{weighted_mean_sd}
\alias{weighted_mean_sd}
\title{Helper function to compute weighted mean and standard deviation}
\usage{
weighted_mean_sd(x, weights = rep(1, nrow(x)))
}
\arguments{
\item{x}{Observation matrix.}

\item{weights}{Optional weight vector.}
}
\value{
A list with components.
\item{mean}{vector of weighted means of columns of x}
\item{sd}{vector of weighted standard deviations of columns of x}
}
\description{
Helper function to compute weighted mean and standard deviation.
Deals gracefully whether x is sparse matrix or not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxpath.R
\name{cox_obj_function}
\alias{cox_obj_function}
\title{Elastic net objective function value for Cox regression model}
\usage{
cox_obj_function(y, pred, weights, lambda, alpha, coefficients, vp)
}
\arguments{
\item{y}{Survival response variable, must be a \code{Surv} or
\code{stratifySurv} object.}

\item{pred}{Model's predictions for \code{y}.}

\item{weights}{Observation weights.}

\item{lambda}{A single value for the \code{lambda} hyperparameter.}

\item{alpha}{The elasticnet mixing parameter, with \eqn{0 \le \alpha \le 1}.}

\item{coefficients}{The model's coefficients.}

\item{vp}{Penalty factors for each of the coefficients.}
}
\description{
Returns the elastic net objective function value for Cox regression model.
}
